@@ -28,7 +28,7 @@ select distinct category, sname from supplier;
 select distinct category, sname from supplier order by sname ;
 select category, sname from supplier order by sname ;
 
-select count( distinct category ) from supplier;
+select distinct count( category ) from supplier;
  
 drop table supplier;
 
