@@ -13,7 +13,7 @@ class Cego
 public:
 
    Cego();
-   ~Cego();
+   virtual ~Cego();
 
    void parse();
    ListT<Chain>& getTokenList() { return _tokenList; }
@@ -22,80 +22,189 @@ protected:
 
    virtual char nextChar() = 0;
    virtual void backChar() = 0;
-   virtual void execSync() = 0;
-   virtual void execListTableSet() = 0;
-   virtual void execListTable() = 0;
-   virtual void execListProc() = 0;
-   virtual void execListView() = 0;
-   virtual void execListIndex() = 0;
-   virtual void execListBTree() = 0;
-   virtual void execListKey() = 0;
-   virtual void execListCounter() = 0;
-   virtual void execListSysObj() = 0;
-   virtual void execListTmpObj() = 0;
-   virtual void execListCheck() = 0;
+   virtual void setSyncMethod() = 0;
+   virtual void setListTableSetMethod() = 0;
+   virtual void setListTableMethod() = 0;
+   virtual void setListProcMethod() = 0;
+   virtual void setListViewMethod() = 0;
+   virtual void setListIndexMethod() = 0;
+   virtual void setListBTreeMethod() = 0;
+   virtual void setListKeyMethod() = 0;
+   virtual void setListCounterMethod() = 0;
+   virtual void setListSysObjMethod() = 0;
+   virtual void setListTmpObjMethod() = 0;
+   virtual void setListCheckMethod() = 0;
+   virtual void setListTriggerMethod() = 0;
+   virtual void setListAliasMethod() = 0;
    virtual void setTableSetOpt() = 0;
-   virtual void execTableInfo() = 0;
-   virtual void execShowUptime() = 0;
-   virtual void execShowPool() = 0;
-   virtual void execShowSystemSpace() = 0;
-   virtual void execTableDesc() = 0;
-   virtual void execIndexDesc() = 0;
-   virtual void execBTreeDesc() = 0;
-   virtual void execViewDesc() = 0;
-   virtual void execKeyDesc() = 0;
-   virtual void execCheckDesc() = 0;
-   virtual void execProcDesc() = 0;
-   virtual void execProcShow() = 0;
-   virtual void execViewShow() = 0;
-   virtual void execCheckShow() = 0;
-   virtual void execSetTableSet() = 0;
-   virtual void execAuthUser() = 0;
-   virtual void execEnableAppendMode() = 0;
-   virtual void execDisableAppendMode() = 0;
-   virtual void execEnableAutoCommit() = 0;
-   virtual void execDisableAutoCommit() = 0;
-   virtual void execSetIsolation() = 0;
-   virtual void execEnableUpdateSync() = 0;
-   virtual void execDisableUpdateSync() = 0;
-   virtual void execEnableBTreeCache() = 0;
-   virtual void execDisableBTreeCache() = 0;
-   virtual void execTableReorganize() = 0;
-   virtual void execIndexReorganize() = 0;
-   virtual void execSelect() = 0;
-   virtual void execSelectPlan() = 0;
-   virtual void execQuery() = 0;
-   virtual void execViewCreate() = 0;
-   virtual void execViewLoad() = 0;
-   virtual void execProcCreate() = 0;
-   virtual void execProcLoad() = 0;
-   virtual void execProcCall() = 0;
-   virtual void execFuncCall() = 0;
-   virtual void execUserTableCreate() = 0;
-   virtual void execPrimaryIndexCreate() = 0;
-   virtual void execIndexCreate() = 0;
-   virtual void execTableDrop() = 0;
-   virtual void execIndexDrop() = 0;
-   virtual void execViewDrop() = 0;
-   virtual void execProcDrop() = 0;
-   virtual void execFKeyDrop() = 0;
-   virtual void execCheckDrop() = 0;
-   virtual void execBTreeDrop() = 0;
+   virtual void setTableInfoMethod() = 0;
+   virtual void setTableSizeMethod() = 0;
+   virtual void setTupleInfoMethod() = 0;
+   virtual void setShowUptimeMethod() = 0;
+   virtual void setShowPoolMethod() = 0;
+   virtual void setShowSystemSpaceMethod() = 0;
+   virtual void setTableDescMethod() = 0;
+   virtual void setIndexDescMethod() = 0;
+   virtual void setBTreeDescMethod() = 0;
+   virtual void setViewDescMethod() = 0;
+   virtual void setKeyDescMethod() = 0;
+   virtual void setCheckDescMethod() = 0;
+   virtual void setAliasDescMethod() = 0;
+   virtual void setProcDescMethod() = 0;
+   virtual void setProcShowMethod() = 0;
+   virtual void setViewShowMethod() = 0;
+   virtual void setCheckShowMethod() = 0;
+   virtual void setTriggerShowMethod() = 0;
+   virtual void setSetTableSetMethod() = 0;
+   virtual void setAuthUserMethod() = 0;
+   virtual void setEnableGraceModeMethod() = 0;
+   virtual void setDisableGraceModeMethod() = 0;
+   virtual void setEnableAppendModeMethod() = 0;
+   virtual void setDisableAppendModeMethod() = 0;
+   virtual void setEnableAutoCommitMethod() = 0;
+   virtual void setDisableAutoCommitMethod() = 0;
+   virtual void setSetIsolationMethod() = 0;
+   virtual void setTableReorganizeMethod() = 0;
+   virtual void setIndexReorganizeMethod() = 0;
+   virtual void setSelectMethod() = 0;
+   virtual void setSelectPlanMethod() = 0;
+   virtual void setQueryMethod() = 0;
+   virtual void setViewCreateMethod() = 0;
+   virtual void setViewLoadMethod() = 0;
+   virtual void setProcCreateMethod() = 0;
+   virtual void setProcLoadMethod() = 0;
+   virtual void setProcCallMethod() = 0;
+   virtual void setFuncCallMethod() = 0;
+   virtual void setUserTableCreateMethod() = 0;
+   virtual void setPrimaryIndexCreateMethod() = 0;
+   virtual void setIndexCreateMethod() = 0;
+   virtual void setCachedOption() = 0;
+   virtual void setAliasCreateMethod() = 0;
+   virtual void setAliasAttr() = 0;
+   virtual void setQueryCondLoadMethod() = 0;
+   virtual void setTriggerCreateMethod() = 0;
+   virtual void setTriggerLoadMethod() = 0;
+   virtual void setTableTruncateMethod() = 0;
+   virtual void setTableDropMethod() = 0;
+   virtual void setIndexDropMethod() = 0;
+   virtual void setViewDropMethod() = 0;
+   virtual void setProcDropMethod() = 0;
+   virtual void setFKeyDropMethod() = 0;
+   virtual void setCheckDropMethod() = 0;
+   virtual void setTriggerDropMethod() = 0;
+   virtual void setBTreeDropMethod() = 0;
+   virtual void setAliasDropMethod() = 0;
    virtual void dropOpt1() = 0;
    virtual void dropOpt2() = 0;
-   virtual void execForeignKeyCreate() = 0;
-   virtual void execCheckCreate() = 0;
-   virtual void execAssignStatement() = 0;
-   virtual void execPrint() = 0;
-   virtual void execIndexCheck() = 0;
-   virtual void execCreateCounter() = 0;
-   virtual void execDropCounter() = 0;
-   virtual void execSetCounter() = 0;
-   virtual void execCounterId() = 0;
-   virtual void jdbcInfo() = 0;
+   virtual void setForeignKeyCreateMethod() = 0;
+   virtual void setCheckCreateMethod() = 0;
+   virtual void setAssignStatementMethod() = 0;
+   virtual void setPrintMethod() = 0;
+   virtual void setIndexCheckMethod() = 0;
+   virtual void setCreateCounterMethod() = 0;
+   virtual void setDropCounterMethod() = 0;
+   virtual void setSetCounterMethod() = 0;
+   virtual void setCounterId() = 0;
+   virtual void setJDBCInfoMethod() = 0;
    virtual void jdbcArg1() = 0;
    virtual void jdbcArg2() = 0;
+   virtual void triggerStore() = 0;
+   virtual void triggerHead() = 0;
+   virtual void setTriggerBefore() = 0;
+   virtual void setTriggerAfter() = 0;
+   virtual void addInsertTrigger() = 0;
+   virtual void addUpdateTrigger() = 0;
+   virtual void addDeleteTrigger() = 0;
+   virtual void caseClause() = 0;
+   virtual void caseConditionList1() = 0;
+   virtual void caseConditionList2() = 0;
+   virtual void caseCondition() = 0;
+   virtual void casePrepare() = 0;
+   virtual void procStoreFetchArg1() = 0;
+   virtual void procStoreFetchArg2() = 0;
+   virtual void setPrimaryColumn() = 0;
+   virtual void setOrdinaryColumn() = 0;
+   virtual void setColumnDesc() = 0;
+   virtual void setNotNullOpt() = 0;
+   virtual void setNullOpt() = 0;
+   virtual void defaultOpt() = 0;
+   virtual void defaultNullOpt() = 0;
+   virtual void createSimpleAttrList() = 0;
+   virtual void createSimpleAttr() = 0;
+   virtual void setUniqueIndex() = 0;
+   virtual void setAVLIndex() = 0;
+   virtual void setBTreeIndex() = 0;
+   virtual void startStore() = 0;
+   virtual void commitStore() = 0;
+   virtual void rollbackStore() = 0;
+   virtual void lockStore() = 0;
+   virtual void unlockStore() = 0;
+   virtual void insertStore() = 0;
+   virtual void insertBySelectStore() = 0;
+   virtual void insertPrepare() = 0;
+   virtual void insertValueSpecStore() = 0;
+   virtual void insertArg() = 0;
+   virtual void insertExprValue() = 0;
+   virtual void updateStore() = 0;
+   virtual void updateAssignment() = 0;
+   virtual void returnOnFirstTrue() = 0;
+   virtual void returnOnFirstFalse() = 0;
+   virtual void returnVarAssignment() = 0;
+   virtual void deleteStore() = 0;
+   virtual void setUpdDelAlias() = 0;
+   virtual void selectStore() = 0;
+   virtual void noFromOption() = 0;
+   virtual void selectSelectionStore1() = 0;
+   virtual void selectSelectionStore2() = 0;
+   virtual void selectionList1() = 0;
+   virtual void selectionList2() = 0;
+   virtual void selectItem() = 0;
+   virtual void selectTableListStore() = 0;
+   virtual void selectJoinSpecStore() = 0;
+   virtual void selectInnerJoinStore1() = 0;
+   virtual void selectInnerJoinStore2() = 0;
+   virtual void selectLeftOuterJoinStore1() = 0;
+   virtual void selectLeftOuterJoinStore2() = 0;
+   virtual void selectRightOuterJoinStore1() = 0;
+   virtual void selectRightOuterJoinStore2() = 0;
+   virtual void selectTable1() = 0;
+   virtual void selectTable2() = 0;
+   virtual void selectStackJoinTable() = 0;
+   virtual void aliasOpt() = 0;
+   virtual void noAliasOpt() = 0;
+   virtual void selectGroupClause() = 0;
+   virtual void selectEmptyGroupClause() = 0;
+   virtual void selectGroupList1() = 0;
+   virtual void selectGroupList2() = 0;
+   virtual void selectHavingClause() = 0;
+   virtual void selectEmptyHavingClause() = 0;
+   virtual void selectOrderingClause() = 0;
+   virtual void selectEmptyOrderingClause() = 0;
+   virtual void selectOrderingList1() = 0;
+   virtual void selectOrderingList2() = 0;
+   virtual void selectOrderingAsc() = 0;
+   virtual void selectOrderingDesc() = 0;
+   virtual void selectDistinctOpt() = 0;
+   virtual void selectNoDistinctOpt() = 0;
+   virtual void selectLimitOpt1() = 0;
+   virtual void selectLimitOpt2() = 0;
    virtual void unionAllStore() = 0;
+   virtual void wcWhereClause() = 0;
+   virtual void wcEmptyClause() = 0;
+   virtual void alterStore() = 0;
+   virtual void alterDropCol() = 0;
+   virtual void alterAddCol() = 0;
+   virtual void alterModCol() = 0;
+   virtual void alterModDefault() = 0;
+   virtual void alterRenameCol() = 0;
+   virtual void renameTable() = 0;
+   virtual void renameIndex() = 0;
+   virtual void renameBTree() = 0;
+   virtual void renameKey() = 0;
+   virtual void renameProcedure() = 0;
+   virtual void renameView() = 0;
+   virtual void renameCheck() = 0;
+   virtual void renameTrigger() = 0;
    virtual void procHead() = 0;
    virtual void procStore() = 0;
    virtual void procReturnOpt() = 0;
@@ -128,13 +237,22 @@ protected:
    virtual void procElsIfCondition() = 0;
    virtual void procWhileStatement() = 0;
    virtual void procWhileCondition() = 0;
-   virtual void procCondition1() = 0;
-   virtual void procCondition2() = 0;
-   virtual void procCondition3() = 0;
-   virtual void procPredicate1() = 0;
-   virtual void procPredicate2() = 0;
-   virtual void procPredicate3() = 0;
-   virtual void procPredicate4() = 0;
+   virtual void wcConditionAnd() = 0;
+   virtual void wcConditionOr() = 0;
+   virtual void wcConditionPredicate() = 0;
+   virtual void wcPredicateQueryExists() = 0;
+   virtual void wcPredicateIn() = 0;
+   virtual void wcPredicateNotIn() = 0;
+   virtual void wcPredicateQueryIn() = 0;
+   virtual void wcPredicateQueryNotIn() = 0;
+   virtual void wcPredicateExprComp() = 0;
+   virtual void wcPredicateBetween() = 0;
+   virtual void wcPredicateNullComp() = 0;
+   virtual void wcPredicateNotNullComp() = 0;
+   virtual void wcPredicateLikeComp() = 0;
+   virtual void wcPredicateNotLikeComp() = 0;
+   virtual void wcPredicateExprOnly() = 0;
+   virtual void wcPredicateNotPredicate() = 0;
    virtual void procExpr1() = 0;
    virtual void procExpr2() = 0;
    virtual void procExpr3() = 0;
@@ -156,106 +274,14 @@ protected:
    virtual void procFactor12() = 0;
    virtual void procFactor13() = 0;
    virtual void procFactor14() = 0;
-   virtual void caseClause() = 0;
-   virtual void caseConditionList1() = 0;
-   virtual void caseConditionList2() = 0;
-   virtual void caseCondition() = 0;
-   virtual void casePrepare() = 0;
-   virtual void procStoreFetchArg1() = 0;
-   virtual void procStoreFetchArg2() = 0;
-   virtual void setPrimaryColumn() = 0;
-   virtual void setOrdinaryColumn() = 0;
-   virtual void setColumnDesc() = 0;
-   virtual void setNotNullOpt() = 0;
-   virtual void setNullOpt() = 0;
-   virtual void defaultOpt() = 0;
-   virtual void defaultNullOpt() = 0;
-   virtual void createSimpleAttrList() = 0;
-   virtual void createSimpleAttr() = 0;
-   virtual void setUniqueIndex() = 0;
-   virtual void setAVLIndex() = 0;
-   virtual void setBTreeIndex() = 0;
-   virtual void startStore() = 0;
-   virtual void commitStore() = 0;
-   virtual void rollbackStore() = 0;
-   virtual void insertStore() = 0;
-   virtual void insertBySelectStore() = 0;
-   virtual void insertPrepare() = 0;
-   virtual void insertValueSpecStoreExecute() = 0;
-   virtual void insertArg() = 0;
-   virtual void insertExprValue() = 0;
-   virtual void selectStore() = 0;
-   virtual void selectSelectionStore1() = 0;
-   virtual void selectSelectionStore2() = 0;
-   virtual void selectionList1() = 0;
-   virtual void selectionList2() = 0;
-   virtual void selectItem() = 0;
-   virtual void selectTableListStore() = 0;
-   virtual void selectJoinSpecStore() = 0;
-   virtual void selectInnerJoinStore() = 0;
-   virtual void selectLeftOuterJoinStore() = 0;
-   virtual void selectRightOuterJoinStore() = 0;
-   virtual void selectTable1() = 0;
-   virtual void selectTable2() = 0;
-   virtual void selectStackJoinTable() = 0;
-   virtual void aliasOpt() = 0;
-   virtual void noAliasOpt() = 0;
-   virtual void selectGroupClause() = 0;
-   virtual void selectEmptyGroupClause() = 0;
-   virtual void selectGroupList1() = 0;
-   virtual void selectGroupList2() = 0;
-   virtual void selectHavingClause() = 0;
-   virtual void selectEmptyHavingClause() = 0;
-   virtual void selectOrderingClause() = 0;
-   virtual void selectEmptyOrderingClause() = 0;
-   virtual void selectOrderingList1() = 0;
-   virtual void selectOrderingList2() = 0;
-   virtual void selectOrderingAsc() = 0;
-   virtual void selectOrderingDesc() = 0;
-   virtual void selectDistinctOpt() = 0;
-   virtual void selectLimitOpt1() = 0;
-   virtual void selectLimitOpt2() = 0;
-   virtual void updateStore() = 0;
-   virtual void updateAssignment() = 0;
-   virtual void returnVarAssignment() = 0;
-   virtual void deleteStore() = 0;
-   virtual void setUpdDelAlias() = 0;
-   virtual void alterStore() = 0;
-   virtual void alterDropCol() = 0;
-   virtual void alterAddCol() = 0;
-   virtual void alterModCol() = 0;
-   virtual void alterModDefault() = 0;
-   virtual void alterRenameCol() = 0;
-   virtual void renameTable() = 0;
-   virtual void renameIndex() = 0;
-   virtual void renameBTree() = 0;
-   virtual void renameKey() = 0;
-   virtual void renameProcedure() = 0;
-   virtual void renameView() = 0;
-   virtual void renameCheck() = 0;
    virtual void functionNative() = 0;
    virtual void functionNextCount() = 0;
    virtual void functionSetCount() = 0;
+   virtual void functionGetCount() = 0;
    virtual void functionUserDef1() = 0;
    virtual void functionUserDef2() = 0;
-   virtual void functionExprList1() = 0;
-   virtual void functionExprList2() = 0;
-   virtual void wcWhereClause() = 0;
-   virtual void wcEmptyClause() = 0;
-   virtual void wcConditionAnd() = 0;
-   virtual void wcConditionOr() = 0;
-   virtual void wcConditionPredicate() = 0;
-   virtual void wcPredicateQueryExists() = 0;
-   virtual void wcPredicateQueryIn() = 0;
-   virtual void wcPredicateQueryNotIn() = 0;
-   virtual void wcPredicateExprComp() = 0;
-   virtual void wcPredicateBetween() = 0;
-   virtual void wcPredicateNullComp() = 0;
-   virtual void wcPredicateNotNullComp() = 0;
-   virtual void wcPredicateLikeComp() = 0;
-   virtual void wcPredicateNotLikeComp() = 0;
-   virtual void wcPredicateCondition() = 0;
-   virtual void wcPredicateNotPredicate() = 0;
+   virtual void storeExprList1() = 0;
+   virtual void storeExprList2() = 0;
    virtual void miscIntType() = 0;
    virtual void miscLongType() = 0;
    virtual void miscStringType() = 0;
@@ -267,9 +293,10 @@ protected:
    virtual void miscSmallIntType() = 0;
    virtual void miscTinyIntType() = 0;
    virtual void miscDecimalType() = 0;
-   virtual void miscFixedType() = 0;
    virtual void miscBlobType() = 0;
    virtual void miscClobType() = 0;
+   virtual void miscDecimalDim1() = 0;
+   virtual void miscDecimalDim2() = 0;
    virtual void miscEqualComp() = 0;
    virtual void miscNotEqualComp() = 0;
    virtual void miscLessComp() = 0;
@@ -292,16 +319,12 @@ protected:
    virtual void miscFloatValue() = 0;
    virtual void miscDoubleValue() = 0;
    virtual void miscDecimalValue() = 0;
-   virtual void miscFixedValue() = 0;
-   virtual void miscFormatDate() = 0;
    virtual void miscSysDate() = 0;
    virtual void miscNull() = 0;
    virtual void miscTrueValue() = 0;
    virtual void miscFalseValue() = 0;
    virtual void miscBlobRef() = 0;
    virtual void miscClobRef() = 0;
-   virtual void miscDateFormat() = 0;
-   virtual void miscDateValue() = 0;
    virtual void setObject1() = 0;
    virtual void setObject2() = 0;
    virtual void setObject3() = 0;
@@ -340,13 +363,15 @@ protected:
       UPDATE,
       UNIQUE,
       UNION,
+      TRUNCATE,
       TRUNC,
       TRUEVAL,
       TRIM,
       TO,
-      STR2DATE,
       STR2LONG,
       STR2INT,
+      TUPLEINFO,
+      TRIGGER,
       TRANSACTION,
       TMPOBJ,
       TINYINT,
@@ -354,6 +379,7 @@ protected:
       THEN,
       TABLESET,
       TABLE,
+      TABLESIZE,
       TABLEINFO,
       SYSOBJ,
       SYSTEMSPACE,
@@ -368,6 +394,7 @@ protected:
       SETCOUNT,
       SET,
       SELECT,
+      SCANDATE,
       RTRIM,
       ROWLIMIT,
       ROUND,
@@ -406,8 +433,11 @@ protected:
       LTRIM,
       LOWER,
       LONG,
-      LOCKS,
+      UNLOCK,
+      LOCK,
       LOAD,
+      LMOD,
+      LDIV,
       LIST,
       LIKE,
       LENGTH,
@@ -417,8 +447,10 @@ protected:
       JDBC,
       ISOLATION,
       IS,
+      INT2ASC,
       INSERT,
-      INT2DATE,
+      NEWDATE,
+      LONG2DATE,
       INTO,
       INT,
       INNER,
@@ -428,12 +460,14 @@ protected:
       IDENTIFIED,
       HAVING,
       GROUP,
+      GRACE,
       GETPOS,
+      GETCOUNT,
       FROM,
       FOREIGN,
       FOR,
       FLOAT,
-      FIXED,
+      FIRST,
       FETCH,
       FALSEVAL,
       EXISTS,
@@ -450,27 +484,29 @@ protected:
       DEFAULT,
       DECIMAL,
       DATETIME,
-      DATE2INT,
+      DATE2LONG,
       DATE2STR,
-      DATE,
       CURSOR,
       CREATE,
       COUNTER,
       COUNT,
       CONSTRAINT,
+      CONDITION,
       COLUMN,
       COMMIT,
       CLOSE,
       CLOB2STR,
+      CLOBREF,
       CLOBSIZE,
       CLOB,
       CHECK,
       CASE,
       CALL,
-      CACHE,
+      CACHED,
       BY,
       BTREE,
       BOOL,
+      BLOBREF,
       BLOBSIZE,
       BLOB,
       BITXOR,
@@ -479,16 +515,20 @@ protected:
       BIGINT,
       BETWEEN,
       BEGIN,
+      BEFORE,
       AVLTREE,
       AVG,
       AUTOCOMMIT,
       AUTHORIZE,
+      ASC2INT,
       ASC,
       AS,
       APPEND,
       ALTER,
       ALL,
       AND,
+      ALIAS,
+      AFTER,
       ADD,
       STRINGVAL,
       ENDTOKEN
@@ -503,11 +543,92 @@ private:
    enum Production {
       Start,
       TableSetOpt,
+      CachedOpt,
+      AliasAttrList,
+      AliasAttr,
       DropOpt,
       CounterId,
       Statement,
       JdbcArgList,
+      TriggerDef,
+      TriggerHead,
+      TriggerEvent,
+      TriggerCondition,
+      TriggerOp,
+      CaseClause,
+      CaseConditionList,
+      CaseCondition,
+      CasePrepare,
+      FetchArgList,
+      ColumnList,
+      KeyColumn,
+      ColumnDesc,
+      NullOpt,
+      DefaultNullOpt,
+      DefaultOpt,
+      DefaultDefOpt,
+      SimpleAttrList,
+      SimpleAttr,
+      IndexOption,
+      IndexType,
+      BtreeType,
+      StartStatement,
+      CommitStatement,
+      RollbackStatement,
+      LockStatement,
+      UnlockStatement,
+      InsertStatement,
+      InsertArgSpec,
+      InsertPrepare,
+      InsertValueSpecList,
+      InsertValueSpec,
+      InsertArgList,
+      InsertValueList,
+      UpdateStatement,
+      UpdateList,
+      Assignment,
+      UpdReturnOpt,
+      NoUpdateReturnOpt,
+      ReturnVarList,
+      ReturnVar,
+      DeleteStatement,
+      UpdDelAlias,
+      SelectStatement,
+      FromOption,
+      NoFromOption,
+      Selection,
+      SelectionList,
+      SelectItem,
+      TableSource,
+      JoinSpecList,
+      JoinSpec,
+      InnerJoin,
+      TableList,
+      Table,
+      JoinTable,
+      AliasOpt,
+      NoAliasOpt,
+      GroupClause,
+      NoGroup,
+      GroupList,
+      HavingClause,
+      NoHaving,
+      OrderingClause,
+      NoOrdering,
+      OrderingList,
+      OrderingOpt,
+      DefaultOrderOpt,
+      DistinctOpt,
+      NoDistinctOpt,
+      LimitOpt,
+      NoLimit,
       UnionAllOpt,
+      WhereClause,
+      NoWhere,
+      AlterStatement,
+      AlterList,
+      AlterOption,
+      RenameStatement,
       ProcHead,
       ProcDef,
       ReturnOpt,
@@ -533,90 +654,20 @@ private:
       ProcElsIfCondition,
       ProcStatement,
       ProcWhileCondition,
-      ProcCondition,
-      ProcPredicate,
+      Condition,
+      Predicate,
       Expr,
       Term,
       Factor,
-      CaseClause,
-      CaseConditionList,
-      CaseCondition,
-      CasePrepare,
-      FetchArgList,
-      ColumnList,
-      KeyColumn,
-      ColumnDesc,
-      NullOpt,
-      DefaultNullOpt,
-      DefaultOpt,
-      DefaultDefOpt,
-      SimpleAttrList,
-      SimpleAttr,
-      IndexOption,
-      IndexType,
-      BtreeType,
-      StartStatement,
-      CommitStatement,
-      RollbackStatement,
-      InsertStatement,
-      InsertArgSpec,
-      InsertPrepare,
-      InsertValueSpecList,
-      InsertValueSpec,
-      InsertArgList,
-      InsertValueList,
-      SelectStatement,
-      Selection,
-      SelectionList,
-      SelectItem,
-      TableSource,
-      JoinSpecList,
-      JoinSpec,
-      InnerJoin,
-      TableList,
-      Table,
-      JoinTableOn,
-      AliasOpt,
-      NoAliasOpt,
-      GroupClause,
-      NoGroup,
-      GroupList,
-      HavingClause,
-      NoHaving,
-      OrderingClause,
-      NoOrdering,
-      OrderingList,
-      OrderingOpt,
-      DefaultOrderOpt,
-      DistinctOpt,
-      LimitOpt,
-      NoLimit,
-      UpdateStatement,
-      UpdateList,
-      Assignment,
-      UpdReturnOpt,
-      ReturnVarList,
-      ReturnVar,
-      DeleteStatement,
-      UpdDelAlias,
-      AlterStatement,
-      AlterList,
-      AlterOption,
-      RenameStatement,
       Function,
       ExpressionList,
-      WhereClause,
-      NoWhere,
-      QueryCondition,
-      QueryPredicate,
       DataTypeProd,
+      DecimalDim,
       Comparison,
       Attribute,
       SignedInt,
       SignedFloat,
       Constant,
-      DateFormat,
-      DateValue,
       ObjSpec
    };
    void loadScanner();
@@ -882,6 +933,38 @@ private:
    void loadParser259();
    void loadParser260();
    void loadParser261();
+   void loadParser262();
+   void loadParser263();
+   void loadParser264();
+   void loadParser265();
+   void loadParser266();
+   void loadParser267();
+   void loadParser268();
+   void loadParser269();
+   void loadParser270();
+   void loadParser271();
+   void loadParser272();
+   void loadParser273();
+   void loadParser274();
+   void loadParser275();
+   void loadParser276();
+   void loadParser277();
+   void loadParser278();
+   void loadParser279();
+   void loadParser280();
+   void loadParser281();
+   void loadParser282();
+   void loadParser283();
+   void loadParser284();
+   void loadParser285();
+   void loadParser286();
+   void loadParser287();
+   void loadParser288();
+   void loadParser289();
+   void loadParser290();
+   void loadParser291();
+   void loadParser292();
+   void loadParser293();
 
    class ScannerStateEntry {
 
@@ -978,8 +1061,8 @@ class ScannerTransEntry {
       int _num;
    };
 
-   ActionEntry _actionMap[868][185];
-   int _jumpMap[868][118];
+   ActionEntry _actionMap[938][202];
+   int _jumpMap[938][129];
 
    class ProdEntry {
 
