@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -31,12 +31,12 @@
 #include "CegoDataType.h"
 #include "CegoTableManager.h"
 #include "CegoTransactionManager.h"
+#include "CegoDatabaseManager.h"
 #include "CegoField.h"
 #include "CegoQuery.h"
 #include "CegoSelect.h"
-#include "CegoCondDesc.h"
 #include "CegoAttrDesc.h"
-#include "CegoPredDesc.h"
+#include "CegoPredicate.h"
 #include "CegoDbHandler.h"
 #include "CegoProcVar.h"
 #include "CegoProcBlock.h"
@@ -44,28 +44,33 @@
 #include "CegoExpr.h"
 #include "CegoTerm.h"
 #include "CegoFactor.h"
-#include "CegoProcCond.h"
-#include "CegoProcPred.h"
 #include "CegoAlterDesc.h"
 #include "CegoProcException.h"
 #include "CegoCaseCond.h"
+#include "CegoTrigger.h"
+#include "CegoAttrAlias.h"
 
 class CegoDbThreadPool;
 
 class CegoAction : public Cego {
-    
-public:
-    
 
-    CegoAction(CegoDistManager *pTabMng, CegoDbThreadPool *pDbPool = 0 );
+public:
+
+    CegoAction(CegoDistManager *pTabMng, CegoDbThreadPool *pDbPool = 0);
     ~CegoAction();
 
     void setTableSet(const Chain& tableSet);
     const Chain& getTableSet();
 
+    void setGraceMode(bool isGrace);
+    
     void setDbHandle(CegoDbHandler *pDbHandle);
-    CegoSelect* getSelect();
-    CegoProcedure* getProcedure();
+    void setLogToFile(bool logToFile);
+	
+    CegoSelect* getSelect(bool setToNull);
+    CegoProcedure* getProcedure(bool setToNull);
+    CegoTrigger* getTrigger(bool setToNull);
+    CegoCondition* getCondition(bool setToNull);
 
     void cleanUp();
     void setCommandChain(char* pC);
@@ -74,103 +79,126 @@ public:
     void backChar();
 
     void printTokenList();
-
+    
     /////////////////////////////////////////////////
     /////////// start of semantic actions ///////////
     /////////////////////////////////////////////////
 
-    void execSync();
-    void execShowDataBase();
-    void execListTableSet();
-    void execListTable();
-    void execListView();
-    void execListProc();
-    void execListIndex();
-    void execListBTree();
-    void execListKey();
-    void execListCounter();
-    void execListSysObj();
-    void execListTmpObj();
-    void execListCheck();
+    void setSyncMethod() { _execMethod = EXECSYNC; };
+    void setListTableSetMethod() { _execMethod = EXECLISTTABLESET; };
+    void setListTableMethod() { _execMethod = EXECLISTTABLE; };
+    void setListViewMethod() { _execMethod = EXECLISTVIEW; };
+    void setListProcMethod() { _execMethod = EXECLISTPROC; };
+    void setListTriggerMethod() { _execMethod = EXECLISTTRIGGER; };
+    void setListAliasMethod() { _execMethod = EXECLISTALIAS; };
+    void setListIndexMethod() { _execMethod = EXECLISTINDEX; };
+    void setListBTreeMethod() { _execMethod = EXECLISTBTREE; };
+    void setListKeyMethod() { _execMethod = EXECLISTKEY; };
+    void setListCounterMethod() { _execMethod = EXECLISTCOUNTER; };
+    void setListSysObjMethod() { _execMethod = EXECLISTSYSOBJ; };
+    void setListTmpObjMethod() { _execMethod = EXECLISTTMPOBJ; };
+    void setListCheckMethod() { _execMethod = EXECLISTCHECK; };
+
 
     void setTableSetOpt();
 
-    void execTableInfo();
-    void execShowUptime();
-    void execShowPool();
-    void execShowSystemSpace();
-    void execTableDesc();
-    void execIndexDesc();
-    void execBTreeDesc();
-    void execViewDesc();
-    void execKeyDesc();
-    void execCheckDesc();
-    void execProcDesc();
+    void setTableInfoMethod() { _execMethod = EXECTABLEINFO; };
+    void setTableSizeMethod() { _execMethod = EXECTABLESIZE; };
+    void setTupleInfoMethod() { _execMethod = EXECTUPLEINFO; };
+    void setShowUptimeMethod() { _execMethod = EXECSHOWUPTIME; };
+    void setShowPoolMethod() { _execMethod = EXECSHOWPOOL; };
+    void setShowSystemSpaceMethod() { _execMethod = EXECSHOWSYSTEMSPACE; };
+    void setTableDescMethod() { _execMethod = EXECTABLEDESC; };
+    void setIndexDescMethod() { _execMethod = EXECINDEXDESC; };
+    void setBTreeDescMethod() { _execMethod = EXECBTREEDESC; };
+    void setViewDescMethod() { _execMethod = EXECVIEWDESC; };
+    void setKeyDescMethod() { _execMethod = EXECKEYDESC; };
+    void setCheckDescMethod() { _execMethod = EXECCHECKDESC; };
+    void setAliasDescMethod() { _execMethod = EXECALIASDESC; };
+    void setProcDescMethod() { _execMethod = EXECPROCDESC; };
 
-    void execProcShow();
-    void execViewShow();
-    void execCheckShow();
+    void setProcShowMethod() { _execMethod = EXECPROCSHOW; };
+    void setViewShowMethod() { _execMethod = EXECVIEWSHOW; };
+    void setCheckShowMethod() { _execMethod = EXECCHECKSHOW; };
+    void setTriggerShowMethod() { _execMethod = EXECTRIGGERSHOW; };
 
-    void execSetTableSet();
-    void execAuthUser();
+    void setSetTableSetMethod() { _execMethod = EXECSETTABLESET; };
+    void setAuthUserMethod() { _execMethod = EXECAUTHUSER; };
 
-    void execEnableAppendMode();
-    void execDisableAppendMode();
+    void setEnableGraceModeMethod() { _execMethod = EXECENABLEGRACEMODE; };
+    void setDisableGraceModeMethod() { _execMethod = EXECDISABLEGRACEMODE; };
 
-    void execEnableBTreeCache();
-    void execDisableBTreeCache();
+    void setEnableAppendModeMethod() { _execMethod = EXECENABLEAPPENDMODE; };
+    void setDisableAppendModeMethod() { _execMethod = EXECDISABLEAPPENDMODE; };
     
-    void execEnableAutoCommit();
-    void execDisableAutoCommit();
+    void setEnableAutoCommitMethod() { _execMethod = EXECENABLEAUTOCOMMIT; };
+    void setDisableAutoCommitMethod(){ _execMethod = EXECDISABLEAUTOCOMMIT; };
 
-    void execSetIsolation();
+    void setSetIsolationMethod() { _execMethod = EXECSETISOLATION; };
 
-    void execEnableUpdateSync();
-    void execDisableUpdateSync();
+    void setTableReorganizeMethod() { _execMethod = EXECTABLEREORGANIZE; };
+    void setIndexReorganizeMethod() { _execMethod = EXECINDEXREORGANIZE; };
+    void setSelectMethod() { _execMethod = EXECSELECT; };
+    void setSelectPlanMethod() { _execMethod = EXECSELECTPLAN; };
+    void setQueryMethod() { _execMethod = EXECQUERY; };
 
-    void execTableReorganize();
-    void execIndexReorganize();
-    void execSelect();
-    void execSelectPlan();
-    void execQuery();
+    void setViewCreateMethod() { _execMethod = EXECVIEWCREATE; };
+    void setViewLoadMethod() { _execMethod = EXECVIEWLOAD; };
+    void setViewDropMethod() { _execMethod = EXECVIEWDROP; };
 
-    void execViewCreate();
-    void execViewLoad();
-    void execViewDrop();
+    void setQueryCondLoadMethod() { _execMethod = EXECQUERYCONDLOAD; };
 
-    void execProcCreate();
-    void execProcLoad();
-    void execProcCall();
-    void execFuncCall();
+    
+    void setTriggerCreateMethod() { _execMethod = EXECTRIGGERCREATE; };
+    void setTriggerLoadMethod() { _execMethod = EXECTRIGGERLOAD; };
+    void setTriggerDropMethod() { _execMethod = EXECTRIGGERDROP; };
 
-    void execUserTableCreate();
-    void execPrimaryIndexCreate();
-    void execIndexCreate();
-    void execTableDrop();
-    void execIndexDrop();
-    void execProcDrop();
-    void execFKeyDrop();
-    void execCheckDrop();
-    void execBTreeDrop();
+    void setProcCreateMethod() { _execMethod = EXECPROCCREATE; };
+    void setProcLoadMethod() { _execMethod = EXECPROCLOAD; };
+    void setProcCallMethod() { _execMethod = EXECPROCCALL; };
+    void setFuncCallMethod() { _execMethod = EXECFUNCCALL; };
 
-    void dropOpt1();
-    void dropOpt2();
-
-    void execForeignKeyCreate();
-    void execCheckCreate();
-    void execAssignStatement();
-    void execPrint();
-    void execIndexCheck();
-    void execCreateCounter();
-    void execDropCounter();
-    void execSetCounter();
-    void execCounterId();
+    void setUserTableCreateMethod() { _execMethod = EXECUSERTABLECREATE; };
+    void setPrimaryIndexCreateMethod() { _execMethod = EXECPRIMARYINDEXCREATE; };
+    void setIndexCreateMethod() { _execMethod = EXECINDEXCREATE; };
+    void setTableTruncateMethod() { _execMethod = EXECTABLETRUNCATE; };
+    void setTableDropMethod() { _execMethod = EXECTABLEDROP; };
+    void setIndexDropMethod() { _execMethod = EXECINDEXDROP; };
+    void setProcDropMethod() { _execMethod = EXECPROCDROP; };
+    void setFKeyDropMethod() { _execMethod = EXECFKEYDROP; };
+    void setCheckDropMethod() { _execMethod = EXECCHECKDROP; };
+    void setBTreeDropMethod() { _execMethod = EXECBTREEDROP; };
+    void setAliasDropMethod() { _execMethod = EXECALIASDROP; };
+    
+    void setForeignKeyCreateMethod() { _execMethod = EXECFOREIGNKEYCREATE; };
+    void setCheckCreateMethod() { _execMethod = EXECCHECKCREATE; };
+    void setAssignStatementMethod() { _execMethod = EXECASSIGNSTATEMENT; };
+    void setPrintMethod() { _execMethod = EXECPRINT; };
+    void setIndexCheckMethod() { _execMethod = EXECINDEXCHECK; };
+    void setCreateCounterMethod() { _execMethod = EXECCREATECOUNTER; };
+    void setDropCounterMethod() { _execMethod = EXECDROPCOUNTER; };
+    void setSetCounterMethod() { _execMethod = EXECSETCOUNTER; };
 
+    void setAliasCreateMethod() { _execMethod = EXECALIASCREATE; };
+    
     // JDBC info calls 
-    void jdbcInfo();
+    void setJDBCInfoMethod() { _execMethod = EXECJDBCINFO; };
     void jdbcArg1();
     void jdbcArg2();
+    void dropOpt1();
+    void dropOpt2();
+    void setCounterId();
 
+    // trigger actions
+    void triggerHead();
+    void triggerStore();
+    void setTriggerBefore();
+    void setTriggerAfter();
+    
+    void addInsertTrigger();
+    void addUpdateTrigger();
+    void addDeleteTrigger();
+    
     // procedure actions
 
     void procHead();
@@ -197,13 +225,6 @@ public:
     void procElsIfCondition();
     void procWhileStatement();
     void procWhileCondition();
-    void procCondition1();
-    void procCondition2();
-    void procCondition3();
-    void procPredicate1();
-    void procPredicate2();
-    void procPredicate3();
-    void procPredicate4();
 
     void procExceptionStatement();
     void procExceptionCondition();
@@ -248,9 +269,12 @@ public:
     void procBlockStatement();
 
     void setUniqueIndex();
+    void setCachedOption();
     void setAVLIndex();
     void setBTreeIndex();
 
+    void setAliasAttr();
+    
     void setPrimaryColumn();
     void setOrdinaryColumn();
     void setColumnDesc();
@@ -266,6 +290,8 @@ public:
     void startStore();
     void commitStore();
     void rollbackStore();
+    void lockStore();
+    void unlockStore();
 
     // query actions
 
@@ -273,11 +299,13 @@ public:
     void insertPrepare();
     void insertBySelectStore();
     void insertArg();
-    void insertValueSpecStoreExecute();     
+    void insertValueSpecStore();     
     void insertExprValue();    
 
     void unionAllStore();
     void selectStore();
+    void noFromOption();
+    
     void selectSelectionStore1();
     void selectSelectionStore2();
     void selectionList1();
@@ -290,9 +318,12 @@ public:
     void selectTable2();
     void selectStackJoinTable();
 
-    void selectInnerJoinStore();
-    void selectLeftOuterJoinStore();
-    void selectRightOuterJoinStore();
+    void selectInnerJoinStore1();
+    void selectInnerJoinStore2();
+    void selectLeftOuterJoinStore1();
+    void selectLeftOuterJoinStore2();
+    void selectRightOuterJoinStore1();
+    void selectRightOuterJoinStore2();
 
     void aliasOpt();
     void noAliasOpt();
@@ -302,9 +333,10 @@ public:
 
     void selectGroupList1();
     void selectGroupList2();
+    
     void selectHavingClause();
     void selectEmptyHavingClause();
-
+    
     void selectOrderingClause();
     void selectEmptyOrderingClause();
     void selectOrderingList1();
@@ -313,12 +345,15 @@ public:
     void selectOrderingDesc();
 
     void selectDistinctOpt();
+    void selectNoDistinctOpt();
     void selectLimitOpt1();
     void selectLimitOpt2();
 
     void updateStore();
     void updateAssignment();
     void returnVarAssignment();
+    void returnOnFirstTrue();
+    void returnOnFirstFalse();
     
     void deleteStore();
 
@@ -336,16 +371,18 @@ public:
     void renameProcedure();
     void renameView();
     void renameCheck();
+    void renameTrigger();
 
     void functionNative();
     void functionNextCount();
     void functionSetCount();
+    void functionGetCount();
 
     void functionUserDef1();
     void functionUserDef2();
 
-    void functionExprList1();
-    void functionExprList2();
+    void storeExprList1();
+    void storeExprList2();
     
     void wcWhereClause();
     void wcEmptyClause();
@@ -356,6 +393,9 @@ public:
 
     void wcPredicateQueryExists();
 
+    void wcPredicateIn();
+    void wcPredicateNotIn();
+
     void wcPredicateQueryIn();
     void wcPredicateQueryNotIn();
 
@@ -368,8 +408,7 @@ public:
     void wcPredicateNotNullComp();
     void wcPredicateLikeComp();
     void wcPredicateNotLikeComp();
-
-    void wcPredicateCondition();
+    void wcPredicateExprOnly();
     void wcPredicateNotPredicate();
 
     void miscIntType();
@@ -383,10 +422,12 @@ public:
     void miscSmallIntType();
     void miscTinyIntType();
     void miscDecimalType();
-    void miscFixedType();
     void miscBlobType();
     void miscClobType();
 
+    void miscDecimalDim1();
+    void miscDecimalDim2();
+    
     void miscEqualComp();
     void miscNotEqualComp();
     void miscLessComp();
@@ -415,15 +456,10 @@ public:
     void miscFloatValue();
     void miscDoubleValue();
     void miscDecimalValue();
-    void miscFixedValue();
 
     void miscNull();
     void miscSysDate();
-    void miscFormatDate();
-
-    void miscDateFormat();
-    void miscDateValue();
-
+    
     void miscTrueValue();
     void miscFalseValue();
     void miscBlobRef();
@@ -435,8 +471,199 @@ public:
 
     void setUpdDelAlias();
 
-private:
+    void execute();
 
+    bool processBatchFile(const Chain& batchFileName, bool ignoreError, bool consoleOut, Chain& errorMsg);
+    
+private:
+    
+    enum ExecutionMethod {
+	EXECSYNC,
+	EXECLISTTABLESET,
+	EXECLISTTABLE,
+	EXECLISTVIEW,
+	EXECLISTPROC,
+	EXECLISTINDEX,
+	EXECLISTBTREE,
+	EXECLISTKEY,
+	EXECLISTCOUNTER,
+	EXECLISTSYSOBJ,
+	EXECLISTTMPOBJ,
+	EXECLISTCHECK,
+	EXECLISTTRIGGER,
+	EXECLISTALIAS,
+	EXECTABLEINFO,
+	EXECTABLESIZE,
+	EXECTUPLEINFO,
+	EXECSHOWUPTIME,
+	EXECSHOWPOOL,
+	EXECSHOWSYSTEMSPACE,
+	EXECTABLEDESC,
+	EXECINDEXDESC,
+	EXECBTREEDESC,
+	EXECVIEWDESC,
+	EXECKEYDESC,
+	EXECALIASDESC,
+	EXECCHECKDESC,
+	EXECPROCDESC,
+	EXECPROCSHOW,
+	EXECVIEWSHOW,
+	EXECCHECKSHOW,
+	EXECTRIGGERSHOW,
+	EXECSETTABLESET,
+	EXECAUTHUSER,
+	EXECENABLEGRACEMODE,
+	EXECDISABLEGRACEMODE,
+	EXECENABLEAPPENDMODE,
+	EXECDISABLEAPPENDMODE,
+	EXECENABLEAUTOCOMMIT,
+	EXECDISABLEAUTOCOMMIT,
+	EXECSETISOLATION,
+	EXECTABLEREORGANIZE,
+	EXECINDEXREORGANIZE,
+	EXECSELECT,
+	EXECSELECTPLAN,
+	EXECQUERY,
+	EXECVIEWCREATE,
+	EXECVIEWLOAD,
+	EXECVIEWDROP,
+	EXECQUERYCONDLOAD,
+	EXECTRIGGERCREATE,
+	EXECTRIGGERLOAD,
+	EXECTRIGGERDROP,
+	EXECPROCCREATE,
+	EXECPROCLOAD,
+	EXECPROCCALL,
+	EXECFUNCCALL,
+	EXECUSERTABLECREATE,
+	EXECPRIMARYINDEXCREATE,
+	EXECINDEXCREATE,
+	EXECTABLEDROP,
+	EXECTABLETRUNCATE,
+	EXECINDEXDROP,
+	EXECPROCDROP,
+	EXECFKEYDROP,
+	EXECCHECKDROP,
+	EXECALIASDROP,
+	EXECBTREEDROP,
+	EXECFOREIGNKEYCREATE,
+	EXECCHECKCREATE,
+	EXECALIASCREATE,
+	EXECASSIGNSTATEMENT,
+	EXECPRINT,
+	EXECINDEXCHECK,
+	EXECCREATECOUNTER,
+	EXECDROPCOUNTER,
+	EXECSETCOUNTER,
+	EXECJDBCINFO
+    };
+
+    ExecutionMethod _execMethod;
+    
+    /////////////////////////////////////////
+    /////////// execution methods ///////////
+    /////////////////////////////////////////
+
+    void execSync();
+    void execListTableSet();
+    void execListTable();
+    void execListView();
+    void execListProc();
+    void execListIndex();
+    void execListBTree();
+    void execListKey();
+    void execListCounter();
+    void execListSysObj();
+    void execListTmpObj();
+    void execListCheck();
+    void execListTrigger();
+    void execListAlias();
+
+    void execTableInfo();
+    void execTableSize();
+    void execTupleInfo();
+    void execShowUptime();
+    void execShowPool();
+    void execShowSystemSpace();
+    void execTableDesc();
+    void execIndexDesc();
+    void execBTreeDesc();
+    void execViewDesc();
+    void execKeyDesc();
+    void execCheckDesc();
+    void execAliasDesc();
+    void execProcDesc();
+
+    void execProcShow();
+    void execViewShow();
+    void execCheckShow();
+    void execTriggerShow();
+
+    void execSetTableSet();
+    void execAuthUser();
+
+    void execEnableGraceMode();
+    void execDisableGraceMode();
+
+    void execEnableAppendMode();
+    void execDisableAppendMode();
+
+    void execEnableBTreeCache();
+    void execDisableBTreeCache();
+    
+    void execEnableAutoCommit();
+    void execDisableAutoCommit();
+
+    void execSetIsolation();
+
+    void execTableReorganize();
+    void execIndexReorganize();
+    void execSelect();
+    void execSelectPlan();
+    void execQuery();
+
+    void execViewCreate();
+    void execViewLoad();
+
+    void execQueryCondLoad();
+    void execViewDrop();
+    
+
+    void execTriggerCreate();
+    void execTriggerLoad();
+    void execTriggerDrop();
+
+    void execProcCreate();
+    void execProcLoad();
+    void execProcCall();
+    void execFuncCall();
+
+    void execUserTableCreate();
+    void execPrimaryIndexCreate();
+    void execIndexCreate();
+    void execTableDrop();
+    void execTableTruncate();
+    void execIndexDrop();
+    void execProcDrop();
+    void execFKeyDrop();
+    void execCheckDrop();
+    void execAliasDrop();
+    void execBTreeDrop();
+
+    void execForeignKeyCreate();
+    void execCheckCreate();
+    void execAliasCreate();
+    void execAssignStatement();
+    void execPrint();
+    void execIndexCheck();
+    void execCreateCounter();
+    void execDropCounter();
+    void execSetCounter();
+
+    // JDBC info calls 
+    void execJDBCInfo();
+
+    
     void reallocateStringBuf();
     Chain getPlanString(Element *pPlan, const Chain& title, int indent = 0);
     Chain getJoinPlanString(Element *pJoin, const Chain& title, int indent = 0);
@@ -446,21 +673,21 @@ private:
 			 const ListT<CegoBTreeObject>& btreeList,
 			 const ListT<CegoKeyObject>& keyList,
 			 const ListT<CegoCheckObject>& checkList,
+			 const ListT<CegoTriggerObject>& triggerList,
+			 const ListT<CegoAliasObject>& aliasList,
 			 CegoTableObject& oe,
-			 ListT< ListT<CegoFieldValue> > &fa);
+			 ListT< ListT<CegoFieldValue> > &fa, bool withSize);
 
 
-    int _i;
     char *_pC;
 
     int _stringBufLen;
     char *_stringBuf; // [MAXSTRINGLEN];
 
-    int _aliasCount;
-
     Chain _fieldName;
     CegoDataType _dataType;
     int _dataLen;
+    int _dataDim;
 
     CegoDistManager* _pTabMng;
     
@@ -471,13 +698,14 @@ private:
 
     ListT<CegoField> _fal;
     ListT<CegoContentObject*> _coList;
-    ListT<CegoProcCond*> *_pCondList;
+    ListT<CegoPredicate*> *_pPredList;
     ListT<CegoProcBlock*> *_pIfBlockList;
     ListT<CegoField> _fieldList;
     ListT<CegoAlterDesc> _alterList;
     ListT<CegoField> _idxList;
     ListT<Chain> _jdbcArgList;
     ListT<CegoAttrDesc*>* _pGroupList;
+    
     ListT<CegoExpr*>* _pOrderingList;
     ListT<CegoOrderNode::Ordering>* _pOrderingOptList;
 
@@ -487,27 +715,27 @@ private:
     ListT<ListT<CegoExpr*> > _exprListArray;
     ListT<CegoReturnVar*> _returnVarList;
     
+    ListT<CegoAttrAlias> _aliasList;
+    
     // stack variables
 
     StackT< ListT<CegoField> > _fieldListStack;
-    StackT<CegoPredDesc*> _predDescStack;
+    StackT<CegoPredicate*> _predDescStack;
     StackT<CegoAttrDesc*> _attrDescStack;
-    StackT<CegoCondDesc*> _condDescStack;
-    StackT<CegoHavingDesc*> _havingDescStack;
+    StackT<CegoCondition*> _condDescStack;
     StackT<CegoCaseCond*> _caseCondStack;
+
     StackT<ListT<CegoContentObject*> > _coListStack;
     StackT<ListT<CegoExpr*> > _exprListStack;
     StackT<ListT<CegoAttrDesc*>*> _groupClauseStack;
     StackT<ListT<CegoExpr*>*> _orderingClauseStack;
     StackT<CegoProcBlock*> _blockStack;
-    StackT<CegoProcCond*> _condStack;
-    StackT<CegoProcPred*> _predStack;
     StackT<CegoTerm*> _termStack;
     StackT<CegoFactor*> _factorStack;
     StackT<CegoExpr*> _exprStack;
     StackT<CegoFunction*> _functionStack;
     StackT<CegoSelect*> _unionStack;
-    StackT<ListT<CegoProcCond*>* > _condListStack;
+    StackT<ListT<CegoPredicate*>* > _predListStack;
     StackT<ListT<CegoProcBlock*>* > _ifBlockListStack;
     StackT<CegoComparison> _compStack;
     StackT<bool> _distinctStack;
@@ -520,40 +748,50 @@ private:
     StackT<Chain> _objNameStack;
     StackT<Chain> _objTableSetStack;
 
-    CegoCaseCond *_pCaseCond;
+    CegoQuery* _pQuery;
     CegoSelect *_pSelect;
+    CegoProcedure* _pProc;
+    CegoTrigger* _pTrigger;
+    CegoCondition* _pCond;
     
     bool _isUnique;
+    bool _isCached;
     bool _isBTree;
 
     bool _initialAssignment;
     bool _ifExistsOpt;
-
+    bool _returnOnFirst;
+    
     CegoDbHandler* _pDbHandle;
-
+    bool _logToFile;
+    
     Chain _tableSet;
     Chain _defTableSet;
     Chain _updDelAlias;
 
+    // trigger declarations
+
+    bool _isTriggerBefore;
+    bool _isTriggerOnUpdate;
+    bool _isTriggerOnInsert;
+    bool _isTriggerOnDelete;
+        
     // procedure declarations
 
-    CegoProcBlock *_pBlock;
-    Chain _procName;
+    CegoProcBlock* _pBlock;
+
+    Chain _triggerTableSet;
     Chain _procTableSet;
     
     CegoDataType _returnType;
     int _returnTypeLen;
+    int _returnTypeDim;
+    
     CegoProcedure::ProcType _procType;
-    
-    CegoProcedure* _pProc;
-    CegoQuery *_pQuery;
-    
+      
     CegoProcBlock* _pMasterBlock;
 
-    bool _procContext;
     bool _isNullable;
-    bool _distinctOpt;
-    bool _distinctAgg;
     
     Chain _dateFormat;
     Chain _dateValue;
@@ -567,6 +805,8 @@ private:
     
     CegoDbThreadPool* _pDbPool;
 
+    bool _isGrace;
+    
     unsigned long _modId;
 };
 
