@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -21,11 +21,13 @@
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+#include <lfcbase/SigHandler.h>
+
 #include "CegoAdm.h"
 #include "CegoAdminHandler.h"
 #include "CegoDefs.h"
 
-class CegoAdmAction : public CegoAdm {
+class CegoAdmAction : public CegoAdm, public SigHandler {
     
 public:
 
@@ -34,8 +36,11 @@ public:
     CegoAdmAction(CegoAdminHandler *pAH, CegoModule *pModule, ResultMode resultMode, bool rawMode);
     ~CegoAdmAction();
 
+    void sigCatch(int sig);
+	
     void setCommandChain(char* pC);
     char nextChar();
+    void readChain();
     void backChar();
 
     void setPitFormat(const Chain& pitFormat);
@@ -51,6 +56,7 @@ public:
     void lockInfoAction();
     void lockStatAction();
     void poolInfoAction();
+    void parameterInfoAction();
     void poolEntryListAction();
     void listTableSetAction();
     void tableSetInfoAction();
@@ -61,7 +67,10 @@ public:
     void transactionInfoAction(); 
     void threadInfoAction();
     void dbThreadInfoAction();    
-    void dbThreadLastQueryAction();    
+    void dbThreadQueryHistAction();
+    void dbQueryHistLastAction();
+    void dbQueryHistCostAction();
+    
     void admThreadInfoAction();
     void logThreadInfoAction();
     void copyInfoAction();
@@ -76,6 +85,9 @@ public:
     void msgAction();
     void cleanupAction();
     void forceloadAction();
+    void cpDumpAction();
+    void noInitAction();
+    
     void xmlModeAction();
     void binModeAction();
     void plainModeAction();
@@ -95,12 +107,16 @@ public:
     void switchTableSetAction();
     void copyTableSetAction();
 
+    void restoreTableSetAction();
     void recoverTableSetAction();
     void setLogMngAction();
+    void setBackupMngAction();
     void showLogMngAction();
+    void showBackupMngAction();
     void setTSInitFileAction();
     void setCheckpointAction();
 
+    void setTSLogUserAction();
     void setTSSysSizeAction();
     void setTSTmpSizeAction();
     void setTSAppSizeAction();
@@ -111,6 +127,9 @@ public:
     void setTSLogSizeAction();
     void setTSSortAreaSizeAction();
 
+    void setNumQueryLastAction();
+    void setNumQueryCostAction();
+	
     void switchSecondaryAction();
     void switchMediatorAction();
 
@@ -122,7 +141,9 @@ public:
     void correctTableSetAction();
 
     void beginBackupAction();
+    void executeBackupAction();
     void endBackupAction();
+    void listBackupAction();
 
     void syncTableSetAction();
 
@@ -175,6 +196,9 @@ public:
     void enableArchLogAction();
     void disableArchLogAction();
 
+    void enableAppendModeAction();
+    void disableAppendModeAction();
+
     void enableAutoCorrectAction();
     void disableAutoCorrectAction();
 
@@ -185,15 +209,25 @@ public:
     void disableQueryCacheAction();
 
     void listQueryCacheAction();
+    void showQueryCacheAction();
     void cleanQueryCacheAction();
-    void setQueryCacheEntryAction();
-    void setQueryCacheSizeAction();
+    void setQueryCacheParamAction();
+
+    void setQueryCacheMaxEntry();
+    void setQueryCacheMaxSize();
+    void setQueryCacheHashRange();
+    void setQueryCacheThreshold();
 
     void listTableCacheAction();
+    void showTableCacheAction();
     void cleanTableCacheAction();
-    void setTableCacheEntryAction();
-    void setTableCacheSizeAction();
+    void setTableCacheParamAction();
 
+    void setTableCacheMaxEntry();
+    void setTableCacheMaxSize();
+    void setTableCacheHashRange();
+    void setTableCacheFilter();
+    
     void storePrimaryAttr();
     void storeSecondaryAttr();
     void storeSysSizeAttr();
@@ -210,16 +244,20 @@ public:
     void dumpUIndex();
 
     void cleanUp();
+    void enableAbort(bool abortEnabled) { _abortEnabled = abortEnabled; }
 
 private:
 
+    void reallocateStringBuf();
     void handleMedResult(CegoAdminHandler::ResultType res);
 
-    int _i;
+    bool _abortEnabled;
+    bool _isAborted;
+    
     char *_pC;
 
     int _stringBufLen;
-    char _stringBuf[MAXSTRINGLEN];
+    char *_stringBuf; // [MAXSTRINGLEN];
 
     CegoModule *_pModule;
     CegoAdminHandler* _pAH;
@@ -243,9 +281,12 @@ private:
     Chain _syncCmd;
     bool _keepTicket;
     int _timeout;
-    bool _doLogging;
+
     bool _doCleanup;
     bool _doForceload;
+    bool _doCPDump;
+    bool _doNoInit;
+    
     Chain _xportMode;
 
     Chain _buMsg;
@@ -254,23 +295,15 @@ private:
     Chain _filterPerm;
     Chain _rightPerm;
 
+    // query and table cache params
+    int _maxEntry;
+    int _maxSize;
+    int _threshold;
+    int _hashRange;
+    Chain _cacheFilter;
+    
     unsigned long _modId;
 
     ListT<CegoField> _admSchema;
 };
-
 #endif
-
-
-
-
-
-
-
-
-
-
-
-
-
-
