@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2010-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -45,7 +45,7 @@ class CegoAdmNet {
 			 ListT<Chain>& fileInfoList,
 			 ListT<Chain>& logInfoList);
 
-    void startTableSet(const Chain& tableSet, bool doCleanup, bool forceload);
+    void startTableSet(const Chain& tableSet, bool doCleanup, bool forceload, bool cpDump, bool noInit);
     void stopTableSet(const Chain& tableSet);
 
     void getThreadInfo(ListT<Chain>& threadInfoList);
@@ -78,7 +78,7 @@ class CegoAdmNet {
     void relocateSecondary(const Chain& tableSet, const Chain& secondary);
     void relocateMediator(const Chain& tableSet, const Chain& mediator);
 
-    void recoverTableSet(const Chain& tableSet, int pit);
+    void recoverTableSet(const Chain& tableSet, const Chain& pit);
 
     void disconnect();
 
@@ -93,7 +93,5 @@ private:
     NetHandler* _pN;
     CegoAdminHandler* _pAH;
     CegoModule* _pModule;
-
 };
-
 #endif
