@@ -6,7 +6,7 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke       
+// (C)opyright 2000-2025 Bjoern Lemke 
 //
 // IMPLEMENTATION MODULE
 //
@@ -28,21 +28,16 @@
 #include <string.h>
 #include <stdlib.h>
 
-CegoAggregation::CegoAggregation()
+CegoAggregation::CegoAggregation(bool isDistinct)
 {
     _type = COUNT;
     _pExpr = 0;
+    _isDistinct = isDistinct;
 }
 
-CegoAggregation::CegoAggregation(char* buf, CegoDistManager *pGTM, int tabSetId)
+CegoAggregation::CegoAggregation(char* buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId)
 {
-    decode(buf, pGTM, tabSetId);
-}
-
-CegoAggregation::CegoAggregation(Element* pAE, CegoDistManager *pGTM)
-{
-    _pExpr=0;
-    fromElement(pAE, pGTM);
+    decode(buf, pGTM, pBlock, tabSetId);
 }
 
 CegoAggregation::CegoAggregation(AggType type, CegoExpr* pExpr, bool isDistinct)
@@ -58,6 +53,13 @@ CegoAggregation::~CegoAggregation()
 	delete _pExpr;
 }
 
+void CegoAggregation::cleanUp()
+{
+    if ( _pExpr )
+	_pExpr->cleanUp();
+}
+
+/*
 void CegoAggregation::setBlock(CegoProcBlock *pBlock)
 {
     if ( _pExpr )
@@ -67,7 +69,18 @@ void CegoAggregation::setBlock(CegoProcBlock *pBlock)
 void CegoAggregation::setFieldListArray(ListT<CegoField> **pFLA)
 {
     if ( _pExpr )
+    {
 	_pExpr->setFieldListArray(pFLA);
+    }
+}
+*/
+
+void CegoAggregation::setTabSetId(int tabSetId)
+{
+    if ( _pExpr )
+    {
+	_pExpr->setTabSetId(tabSetId);
+    }
 }
 
 void CegoAggregation::setAggregationId(int aid)
@@ -104,7 +117,7 @@ CegoAggregation& CegoAggregation::operator = ( const CegoAggregation& ag)
     return (*this);
 }
    
-const CegoAggregation::AggType CegoAggregation::getType() const
+const CegoAggregation::AggType& CegoAggregation::getType() const
 {
     return _type;
 }
@@ -114,13 +127,18 @@ CegoExpr* CegoAggregation::getExpr() const
     return _pExpr;
 }
 
+bool CegoAggregation::isDistinct() const
+{
+    return _isDistinct;
+}
+
 CegoAggregation* CegoAggregation::clone(bool isAttrRef)
 {
     CegoAggregation *pAgg;
     if ( _pExpr )
 	pAgg = new  CegoAggregation(_type, _pExpr->clone(isAttrRef), _isDistinct );
     else
-	pAgg = new  CegoAggregation();
+	pAgg = new  CegoAggregation(_isDistinct);
 
     pAgg->setAggregationId ( _aid );
     return ( pAgg ) ; 
@@ -138,7 +156,7 @@ ListT<CegoAttrDesc*> CegoAggregation::getAttrRefList() const
     return attrList;
 }
 
-void CegoAggregation::encode(char *buf)
+void CegoAggregation::encode(char *buf, CegoProcBlock *pBlock)
 {
     char* pBuf = (char*)buf;
     
@@ -155,14 +173,13 @@ void CegoAggregation::encode(char *buf)
     
     if ( _pExpr )
     {
-	_pExpr->encode(pBuf);
-	pBuf = pBuf + _pExpr->getEncodingLength();
+	_pExpr->encode(pBuf, pBlock);
+	pBuf = pBuf + _pExpr->getEncodingLength(pBlock);
     }
 }
 
-void CegoAggregation::decode(char *buf, CegoDistManager *pGTM, int tabSetId)
-{
-    
+void CegoAggregation::decode(char *buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId)
+{   
     if ( _pExpr )
     {
 	delete _pExpr;
@@ -174,145 +191,88 @@ void CegoAggregation::decode(char *buf, CegoDistManager *pGTM, int tabSetId)
     memcpy( &_type, pBuf,  sizeof(CegoAggregation::AggType));
     pBuf = pBuf + sizeof(CegoAggregation::AggType);
     
-    char c;
-    memcpy( pBuf, &c, sizeof(char));
+    char c = 0;
+    memcpy( &c, pBuf, sizeof(char));
     pBuf = pBuf + sizeof(char);
     
     if ( c == 1 )
     {
-	_pExpr = new CegoExpr(pBuf, pGTM, tabSetId);	
+	_pExpr = new CegoExpr(pBuf, pGTM, pBlock, tabSetId);	
     }
 }
 
-int CegoAggregation::getEncodingLength() const
+int CegoAggregation::getEncodingLength(CegoProcBlock *pBlock) const
 {
     int size = sizeof(CegoAggregation::AggType);
     size += sizeof(char);
     if ( _pExpr )
-	size += _pExpr->getEncodingLength();
+	size += _pExpr->getEncodingLength(pBlock);
     return size;
 }
 
-Element* CegoAggregation::toElement() const
-{
-    Element *pAggElement = new Element( XML_AGGREGATION_ELEMENT ); 
-
-    switch (_type)
-    {
-    case CegoAggregation::MIN:
-	pAggElement->setAttribute(XML_AGG_ATTR, XML_MIN_VALUE);
-	break;
-    case CegoAggregation::MAX:
-	pAggElement->setAttribute(XML_AGG_ATTR, XML_MAX_VALUE);
-	break;
-    case CegoAggregation::AVG:
-	pAggElement->setAttribute(XML_AGG_ATTR, XML_AVG_VALUE);
-	break;
-    case CegoAggregation::SUM:
-	pAggElement->setAttribute(XML_AGG_ATTR, XML_SUM_VALUE);
-	break;
-    case CegoAggregation::COUNT:
-	pAggElement->setAttribute(XML_AGG_ATTR, XML_COUNT_VALUE);
-	break;
-    }
-
-    if ( _pExpr )
-    {
-	pAggElement->addContent(_pExpr->toElement());
-    }
-
-    return pAggElement;
-}
-
-void CegoAggregation::fromElement(Element *pAggElement, CegoDistManager *pGTM)
-{
-    
-    if ( _pExpr )
-	delete _pExpr;
-    _pExpr = 0;
-
-    ListT<Element*> fl = pAggElement->getChildren(XML_FACTOR_ELEMENT);
-    Element **pFE = fl.First();
-    if ( pFE )
-    {
-	_pExpr = new CegoExpr(*pFE, pGTM);
-    }
-   
-    Chain aggString = pAggElement->getAttributeValue( XML_AGG_ATTR );
-
-    if ( aggString == Chain(XML_MIN_VALUE) )
-	_type = CegoAggregation::MIN;
-    else if ( aggString == Chain(XML_MAX_VALUE) )
-	_type = CegoAggregation::MAX;
-    else if ( aggString == Chain(XML_AVG_VALUE) )
-	_type = CegoAggregation::AVG;
-    else if ( aggString == Chain(XML_SUM_VALUE) )
-	_type = CegoAggregation::SUM;
-    else if ( aggString == Chain(XML_COUNT_VALUE) )
-	_type = CegoAggregation::COUNT;
-}
-
-Chain CegoAggregation::getId() const
+Chain CegoAggregation::getId(CegoProcBlock *pBlock) const
 {
     Chain s;
     switch (_type)
     {
     case CegoAggregation::MIN:
-	s =  Chain("min(") + _pExpr->getId() + Chain(")");
+	s =  Chain("min(") + _pExpr->getId(pBlock) + Chain(")");
 	break;
     case CegoAggregation::MAX:
-	s =  Chain("max(") + _pExpr->getId() + Chain(")");
+	s =  Chain("max(") + _pExpr->getId(pBlock) + Chain(")");
 	break;
     case CegoAggregation::AVG:
-	s =  Chain("avg(") + _pExpr->getId() + Chain(")");
+	s =  Chain("avg(") + _pExpr->getId(pBlock) + Chain(")");
 	break;
     case CegoAggregation::SUM:
-	s =  Chain("sum(") + _pExpr->getId() + Chain(")");
+	s =  Chain("sum(") + _pExpr->getId(pBlock) + Chain(")");
 	break;
     case CegoAggregation::COUNT:
     {       
+	Chain distOpt;
+	if ( _isDistinct )
+	    distOpt = Chain("d");	    
+	
 	if ( _pExpr )
 	{ 
-	    Chain distOpt;
-	    if ( _isDistinct )
-		distOpt = Chain("d");	    
-	    s = Chain("count(") + distOpt + _pExpr->getId() + Chain(")");
+	    s = distOpt + Chain("count(") + _pExpr->getId(pBlock) + Chain(")");
 	}
 	else
-	    s = Chain("count(*)");
+	    s = distOpt + Chain("count(*)");
     }
     }
     return s;
 }
 
-Chain CegoAggregation::toChain() const
+Chain CegoAggregation::toChain(int defTabSetId) const
 {
     Chain s;
     switch (_type)
     {
     case CegoAggregation::MIN:
-	s =  Chain("min(") + _pExpr->toChain() + Chain(")");
+	s =  Chain("min(") + _pExpr->toChain(defTabSetId) + Chain(")");
 	break;
     case CegoAggregation::MAX:
-	s =  Chain("max(") + _pExpr->toChain() + Chain(")");
+	s =  Chain("max(") + _pExpr->toChain(defTabSetId) + Chain(")");
 	break;
     case CegoAggregation::AVG:
-	s =  Chain("avg(") + _pExpr->toChain() + Chain(")");
+	s =  Chain("avg(") + _pExpr->toChain(defTabSetId) + Chain(")");
 	break;
     case CegoAggregation::SUM:
-	s =  Chain("sum(") + _pExpr->toChain() + Chain(")");
+	s =  Chain("sum(") + _pExpr->toChain(defTabSetId) + Chain(")");
 	break;
     case CegoAggregation::COUNT:
     {       
+	Chain distOpt;
+	if ( _isDistinct )
+	    distOpt = Chain(" distinct ");	    
+	
 	if ( _pExpr )
 	{ 
-	    Chain distOpt;
-	    if ( _isDistinct )
-		distOpt = Chain(" distinct ");	    
-	    s = Chain("count(") + distOpt + _pExpr->toChain() + Chain(")");
+	    s = distOpt + Chain("count(") + _pExpr->toChain(defTabSetId) + Chain(")");
 	}
 	else
-	    s = Chain("count(*)");
+	    s = distOpt + Chain("count(*)");
     }
     }
     return s;
@@ -325,6 +285,6 @@ Chain CegoAggregation::dbFormat(CegoDatabaseFormater *pForm) const
 
 ostream& operator << (ostream& s, const CegoAggregation& ag)
 {
-    s << ag.toChain();
+    s << ag.toChain(0);
     return s;
 }
