@@ -8,7 +8,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke  
+// (C)opyright 2000-2025 Bjoern Lemke  
 //
 // INTERFACE MODULE
 //
@@ -37,22 +37,28 @@ public:
     
     enum AggType { MIN, MAX, AVG, SUM, COUNT };
     
-    CegoAggregation();
-    CegoAggregation(char* buf, CegoDistManager *pGTM, int tabSetId);
-    CegoAggregation(Element* pAE, CegoDistManager *pGTM);
+    CegoAggregation(bool isDistinct);
+    CegoAggregation(char* buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId);
     CegoAggregation(AggType type, CegoExpr* pExpr, bool isDistinct = false);
     ~CegoAggregation();
 
-    void setFieldListArray(ListT<CegoField> **pFLA);
+    void cleanUp();
 
+    /*
+    void setFieldListArray(ListT<CegoField> **pFLA);
     void setBlock(CegoProcBlock *pBlock);
+    */
 
     CegoFieldValue& getFieldValue();
     void setFieldValue(const CegoFieldValue& fv);
     
-    const AggType getType() const;
+    const AggType& getType() const;
     CegoExpr* getExpr() const;
 
+    bool isDistinct() const;
+
+    void setTabSetId(int tabSetId);
+    
     void setAggregationId(int aid);
     const int getAggregationId() const;
 
@@ -64,15 +70,12 @@ public:
 
     ListT<CegoAttrDesc*> getAttrRefList() const;
 
-    void encode(char *buf);
-    void decode(char *buf, CegoDistManager *pGTM, int tabSetId);
-    int getEncodingLength() const;
-
-    Element* toElement() const;
-    void fromElement(Element *pAggElement, CegoDistManager *pGTM);
-
-    Chain getId() const;
-    Chain toChain() const;
+    void encode(char *buf, CegoProcBlock *pBlock);
+    void decode(char *buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId);
+    int getEncodingLength(CegoProcBlock *pBlock) const;
+    
+    Chain getId(CegoProcBlock *pBlock) const;
+    Chain toChain(int defTabSetId) const;
     Chain dbFormat(CegoDatabaseFormater *pForm) const;
     
     friend ostream& operator << (ostream& s, const CegoAggregation& ag);
