@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -50,7 +50,11 @@ Chain CegoAttrCond::getId() const
 
 void CegoAttrCond::add(const CegoAttrComp& attrComp)
 {
-    _attrCompSet.Insert(attrComp);
+    CegoAttrComp *pAC;
+    if ( (pAC = _attrCompSet.Find(attrComp)) == 0 )
+    {	
+	_attrCompSet.Insert(attrComp);
+    }
 }
 
 int CegoAttrCond::numComp() const
@@ -84,12 +88,12 @@ int CegoAttrCond::getStrength() const
     return 0;
 }
 
-SetT<CegoAttrComp>& CegoAttrCond::getAttrCompSet()
+const TreeT<CegoAttrComp>& CegoAttrCond::getAttrCompSet() const
 {
     return _attrCompSet;
 }
 
-CegoAttrCond CegoAttrCond::getFilterCond(const ListT<CegoField>& fl) const
+CegoAttrCond CegoAttrCond::getFilterCond(const ListT<CegoField>& fl, bool ignoreNullComp) const
 {
     CegoAttrCond ac;
 
@@ -101,7 +105,9 @@ CegoAttrCond CegoAttrCond::getFilterCond(const ListT<CegoField>& fl) const
 	{
 	    if ( pF->getTableAlias() == pAC->getTableName() && pF->getAttrName() == pAC->getAttrName() )
 	    {
-		ac.add(*pAC);
+		if ( ! ( ignoreNullComp && pAC->isNullComp() ) )
+		    ac.add(*pAC);
+		/* else ignore comp */
 	    }
 	    pAC = _attrCompSet.Next();
 	}
@@ -112,42 +118,35 @@ CegoAttrCond CegoAttrCond::getFilterCond(const ListT<CegoField>& fl) const
 }
 
 CegoAttrCond CegoAttrCond::getIndexCond(const ListT<CegoField>& fl) const
-{
+{ 
     CegoAttrCond ac;
-
-    bool isPrimary=true;
-    bool goOn=true;
+ 
+    int pos = 0;
     CegoField *pF = fl.First();
-    while ( pF && goOn )
-    {	
-	bool attrFound=false;
-	CegoAttrComp *pAC = _attrCompSet.First();
-	while ( pAC )
-	{
-	    
-	    if ( pF->getAttrName() == pAC->getAttrName() )
-	    {
-		if ( isPrimary || pAC->getComparison() == EQUAL )
-		{
-		    ac.add(*pAC);
-		    attrFound=true;
-		}
-	    }
-	    pAC = _attrCompSet.Next();
-	}
-
-	isPrimary = false;
-
-	if ( attrFound )
-	    pF = fl.Next();
-	else
-	    goOn=false;	    
+ 
+    while ( pF  )
+    {
+        CegoAttrComp *pAC = _attrCompSet.First();
+        while ( pAC )
+        {
+            if ( pF->getAttrName() == pAC->getAttrName()
+                 && ( pAC->getCompMode() == CegoAttrComp::VAL
+                      || pAC->getCompMode() == CegoAttrComp::ATTR
+                      || pAC->getCompMode() == CegoAttrComp::BTWN ) )
+            {
+                pAC->setPos(pos);
+                ac.add(*pAC);
+            }
+            pAC = _attrCompSet.Next();
+        }
+        pF = fl.Next();
+        pos++;
     }
     return ac;
 }
 
 bool CegoAttrCond::setup(const ListT<CegoField>& fl)
-{
+{   
     CegoAttrComp *pAC = _attrCompSet.First();
     while ( pAC )
     {
@@ -164,64 +163,134 @@ bool CegoAttrCond::setup(ListT<CegoField>** pJoinBuf, int offset)
     CegoAttrComp *pAC = _attrCompSet.First();
     while ( pAC )
     {
-	if ( pAC->getCompMode() == CegoAttrComp::ATTR )
+	pAC->reset();
+	if ( pAC->getCompMode() == CegoAttrComp::ATTR || pAC->getCompMode() == CegoAttrComp::BTWN )
+	{	  	    
 	    if ( pAC->setup(pJoinBuf, offset) == false )
+	    {
 		return false;
+	    }	   
+	}
 	pAC = _attrCompSet.Next();
     }
     return true;
 }
 
-void CegoAttrCond::asConjunctionList(const ListT<CegoExpr*>& exprList, ListT<CegoPredDesc*>& conjunctionList) const
+bool CegoAttrCond::setup(ListT<CegoField>** pParentJoinBuf, int parentOffset, ListT<CegoField>** pJoinBuf, int offset)
 {
     CegoAttrComp *pAC = _attrCompSet.First();
     while ( pAC )
     {
-	CegoPredDesc *pP;
+	pAC->reset();
+	if ( pAC->getCompMode() == CegoAttrComp::ATTR || pAC->getCompMode() == CegoAttrComp::BTWN )
+	{
+	    if ( pAC->isParentSetup() )
+	    {
+		if ( pAC->setup(pParentJoinBuf, parentOffset) == false )
+		{
+		    return false;
+		}
+	    }
+	    else
+	    {
+		if ( pAC->setup(pJoinBuf, offset) == false )
+		{
+		    if ( pParentJoinBuf )
+		    {
+			if ( pAC->setup(pParentJoinBuf, parentOffset) == false )
+			{
+			    return false;
+			}
+			else
+			{
+			    pAC->setParentSetup();	    
+			}
+		    }
+		    else
+		    {
+			return false;
+		    }
+		}
+	    }
+	}	
+	pAC = _attrCompSet.Next();
+    }
+    return true;
+}
 
+/*
+  asConjunctionList returns for the given instance a list of predicates, which are needed for 
+  CegoSelect to build up joins.
+  For this, each comparison is checked 
+
+ */
+bool CegoAttrCond::asConjunctionList(const ListT<CegoExpr*>& exprList, ListT<CegoPredicate*>& conjunctionList) const
+{
+    bool isComplete = true;
+    
+    CegoAttrComp *pAC = _attrCompSet.First();
+    while ( pAC )
+    {
+
+	// Get an expression clone for the view condition attribute alias 
 	CegoExpr *pExpr = getExpressionForAlias(exprList, pAC->getAttrName() );
 
 	if ( pExpr == 0 )
 	{
+	    // if there is no alias, we assume this a an outer attribute reference from the view caller
 	    pExpr = new CegoExpr(new CegoTerm(new CegoFactor( new CegoAttrDesc(pAC->getAttrName()))));
 	}
 
-	if ( pAC->getCompMode() == CegoAttrComp::ATTR && pAC->isSetup() == false )   
-	{	   
-	    pP = new CegoPredDesc(pExpr,
-				  new CegoExpr(new CegoTerm(new CegoFactor( pAC->getAttrDesc().clone()))),
-				  pAC->getComparison());
-	}
-	else if ( pAC->getCompMode() == CegoAttrComp::VAL || 
-		  ( pAC->getCompMode() == CegoAttrComp::ATTR && pAC->isSetup() ) )
+	// we have to check for an aggregation in expression
+	// if so we have to skip this compare and return complete = false
+	if ( pExpr->getAggregationList().isEmpty() == false )
 	{
-	    pP = new CegoPredDesc(pExpr,
-				  new CegoExpr(new CegoTerm(new CegoFactor( pAC->getFieldValue()))),
-				  pAC->getComparison());
+	    isComplete = false;	    
 	}
-	else if ( pAC->getCompMode() == CegoAttrComp::BTWN )
+	else
 	{
-	    pP = new CegoPredDesc(pExpr,
-				  new CegoExpr(new CegoTerm(new CegoFactor( pAC->getFieldValue()))),
-				  new CegoExpr(new CegoTerm(new CegoFactor( pAC->getFieldValue2()))));
+
+	    // Depending on the attribute comparison mode, an appropriate predicate is
+	    // build up
+	    CegoPredicate *pP;	    
+	    
+	    if ( pAC->getCompMode() == CegoAttrComp::ATTR && pAC->isSetup() == false )   
+	    {
+		CegoExpr* pExternExpr = new CegoExpr(new CegoTerm(new CegoFactor( pAC->getAttrDesc().clone())));
+		pP = new CegoPredicate(pExpr,
+				      pExternExpr,
+				      pAC->getComparison());		
+	    }
+	    else if ( pAC->getCompMode() == CegoAttrComp::VAL || 
+		      ( pAC->getCompMode() == CegoAttrComp::ATTR && pAC->isSetup() ) )
+	    {
+		pP = new CegoPredicate(pExpr,
+				      new CegoExpr(new CegoTerm(new CegoFactor( pAC->getFieldValue()))),
+				      pAC->getComparison());
+	    }
+	    else if ( pAC->getCompMode() == CegoAttrComp::BTWN )
+	    {
+		pP = new CegoPredicate(pExpr,
+				      new CegoExpr(new CegoTerm(new CegoFactor( pAC->getFieldValue()))),
+				      new CegoExpr(new CegoTerm(new CegoFactor( pAC->getFieldValue2()))));
+	    }
+	    else if ( pAC->getCompMode() == CegoAttrComp::ISLIKE )
+	    {
+		pP = new CegoPredicate(pExpr,
+				      pAC->getPattern(),
+				      false);
+	    }
+	    else if ( pAC->getCompMode() == CegoAttrComp::ISNOTLIKE )
+	    {
+		pP = new CegoPredicate(pExpr,
+				      pAC->getPattern(),
+				      true);
+	    }	    
+	    conjunctionList.Insert(pP);
 	}
-	else if ( pAC->getCompMode() == CegoAttrComp::ISLIKE )
-	{
-	    pP = new CegoPredDesc(pExpr,
-				  pAC->getPattern(),
-				  false);
-	}
-	else if ( pAC->getCompMode() == CegoAttrComp::ISNOTLIKE )
-	{
-	    pP = new CegoPredDesc(pExpr,
-				  pAC->getPattern(),
-				  true);
-	}
-  
-	conjunctionList.Insert(pP);
-	
 	pAC = _attrCompSet.Next();
-    }	
+    }
+    return isComplete;
 }
 
 CegoExpr* CegoAttrCond::getExpressionForAlias(const ListT<CegoExpr*>& exprList, const Chain& alias) const
@@ -240,7 +309,6 @@ CegoAttrCond::IndexMatch CegoAttrCond::checkIndex(const ListT<CegoField>& schema
 {
     int numFound=0;
 
-    bool isPrimary = true;
     bool hasLeak=false;
     CegoField *pF = schema.First();
 
@@ -249,12 +317,11 @@ CegoAttrCond::IndexMatch CegoAttrCond::checkIndex(const ListT<CegoField>& schema
 	bool isFound=false;	
 	CegoAttrComp *pComp = _attrCompSet.First();	
 	while ( pComp ) 
-	{	    
+	{
 	    if ( pComp->getAttrName() == pF->getAttrName() 
 		 && ( pComp->getCompMode() == CegoAttrComp::VAL 
 		      || pComp->getCompMode() == CegoAttrComp::ATTR
-		      || pComp->getCompMode() == CegoAttrComp::BTWN )
-		 && ( isPrimary || pComp->getComparison() == EQUAL  ) )
+		      || pComp->getCompMode() == CegoAttrComp::BTWN ) )
 	    {		
 		numFound++;
 		isFound=true;
@@ -264,8 +331,6 @@ CegoAttrCond::IndexMatch CegoAttrCond::checkIndex(const ListT<CegoField>& schema
 	    {		
 		pComp = _attrCompSet.Next();
 	    }
-
-	    isPrimary = false;
 	}
 	
 	if ( isFound == true )
@@ -275,17 +340,17 @@ CegoAttrCond::IndexMatch CegoAttrCond::checkIndex(const ListT<CegoField>& schema
 
 	pF = schema.Next();
     }
-    
+
     if ( numFound == _attrCompSet.Size() )
 	return FULL;
-    
+
     if ( numFound < _attrCompSet.Size() && numFound > 0 )
 	return PART;
     
     return INAPP;
 }
     
-void CegoAttrCond::setIdxSchema(ListT<CegoField>& schema)
+void CegoAttrCond::setIdxSchema(const ListT<CegoField>& schema)
 {
     _idxSchema = schema;
 }
@@ -337,7 +402,7 @@ bool CegoAttrCond::diff( const CegoAttrCond& ac) const
     CegoAttrComp* pAC = ac._attrCompSet.First();
     while ( pAC  )
     {
-	CegoAttrComp* pAC2 = _attrCompSet.Find(*pAC);
+	CegoAttrComp* pAC2 = findComp(pAC);
 	if ( pAC2  )
 	{
 	    if ( pAC->getFieldValue() != pAC2->getFieldValue() ) 
@@ -353,7 +418,7 @@ bool CegoAttrCond::diff( const CegoAttrCond& ac) const
     pAC = _attrCompSet.First();
     while ( pAC  )
     {
-	CegoAttrComp* pAC2 = ac._attrCompSet.Find(*pAC);
+	CegoAttrComp* pAC2 = ac.findComp(pAC);
 	if ( pAC2  )
 	{
 	    if ( pAC->getFieldValue() != pAC2->getFieldValue() ) 
@@ -368,6 +433,41 @@ bool CegoAttrCond::diff( const CegoAttrCond& ac) const
     return true;   
 }
 
+CegoAttrComp* CegoAttrCond::findComp(CegoAttrComp* pAttrComp) const
+{
+    // for searching attribute comparison, we use a dedicated loop, since for TreeT, just < and > operators are used.
+    // for comparison we have to use the = operator
+    
+    CegoAttrComp* pAC = _attrCompSet.First();
+    while ( pAC )
+    {
+	if ( *pAC == *pAttrComp )
+	    return pAC;
+	pAC = _attrCompSet.Next();
+    }
+    return 0;
+}
+
+void CegoAttrCond::update( const CegoAttrCond& ac )
+{    
+    CegoAttrComp* pAC = _attrCompSet.First();
+    while ( pAC  )
+    {
+	CegoAttrComp* pAC2 = ac.findComp(pAC);
+	if ( pAC2  )
+	{
+	    pAC->setFieldValue(pAC2->getFieldValue());
+	    if ( pAC->getCompMode() == CegoAttrComp::BTWN )
+		pAC->setFieldValue2(pAC2->getFieldValue2());
+	}
+	else
+	{
+	    throw Exception(EXLOC, Chain("Cannot set up diff for attribute condition"));
+	}
+	pAC = _attrCompSet.Next();
+    }
+}
+
 CegoAttrCond& CegoAttrCond::operator = ( const CegoAttrCond& ac)
 {
     _idxSchema = ac._idxSchema;
@@ -383,7 +483,7 @@ bool CegoAttrCond::operator == ( const CegoAttrCond& ac) const
     CegoAttrComp* pAC = ac._attrCompSet.First();
     while ( pAC  )
     {
-	if ( _attrCompSet.Find(*pAC) == false )
+	if ( findComp(pAC) == 0 )
 	    return false;
 	pAC = ac._attrCompSet.Next();
     }
@@ -391,7 +491,7 @@ bool CegoAttrCond::operator == ( const CegoAttrCond& ac) const
     pAC = _attrCompSet.First();
     while ( pAC  )
     {
-	if ( ac._attrCompSet.Find(*pAC) == false )
+	if ( ac.findComp(pAC) == 0 )
 	    return false;	
 	pAC = _attrCompSet.Next();
     }
