@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -22,12 +22,12 @@
 
 // LFC INCLUDES
 #include <lfcbase/ListT.h>
-#include <lfcbase/SetT.h>
+#include <lfcbase/TreeT.h>
 #include <lfcbase/Chain.h>
 
 // CEGO INCLUDES
 #include "CegoAttrComp.h"
-#include "CegoPredDesc.h"
+#include "CegoPredicate.h"
 
 class CegoAttrCond {
     
@@ -45,25 +45,30 @@ public:
     int numComp() const;
     int getStrength() const;
 
-    SetT<CegoAttrComp>& getAttrCompSet();
+    const TreeT<CegoAttrComp>& getAttrCompSet() const;
 
     bool setup(const ListT<CegoField>& fl);
     bool setup(ListT<CegoField>** pJoinBuf, int offset);
 
-    CegoAttrCond getFilterCond(const ListT<CegoField>& fl) const;    
+    bool setup(ListT<CegoField>** pParentJoinBuf, int parentOffset, ListT<CegoField>** pJoinBuf, int offset);
+    
+    CegoAttrCond getFilterCond(const ListT<CegoField>& fl, bool ignoreNullComp) const;    
     CegoAttrCond getIndexCond(const ListT<CegoField>& fl) const;
 
     IndexMatch checkIndex(const ListT<CegoField>& schema) const;
-    void setIdxSchema(ListT<CegoField>& schema);
+    void setIdxSchema(const ListT<CegoField>& schema);
 
     CegoAttrComp::CompMode getPrimaryCompMode() const;
 
     CegoComparison getPrimaryComparison() const;
     void setPrimaryComparison(CegoComparison comp);
 
-    void asConjunctionList(const ListT<CegoExpr*>& exprList, ListT<CegoPredDesc*>& conjunctionList) const;
+    bool asConjunctionList(const ListT<CegoExpr*>& exprList, ListT<CegoPredicate*>& conjunctionList) const;
 
+    CegoAttrComp* findComp(CegoAttrComp* pAttrComp) const;
+    
     bool diff( const CegoAttrCond& ac) const;
+    void update( const CegoAttrCond& ac);
 
     CegoAttrCond& operator = ( const CegoAttrCond& ac);
     bool operator == ( const CegoAttrCond& ac) const;
@@ -79,6 +84,6 @@ private:
     CegoExpr* getExpressionForAlias(const ListT<CegoExpr*>& exprList, const Chain& alias) const;
 
     ListT<CegoField> _idxSchema;
-    SetT<CegoAttrComp> _attrCompSet;
+    TreeT<CegoAttrComp> _attrCompSet;
 };
 #endif
