@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -25,6 +25,7 @@
 #include <lfcbase/Chain.h>
 
 // CEGO INCLUDES
+#include "CegoDefs.h"
 #include "CegoField.h"
 #include "CegoContentObject.h"
 
@@ -34,28 +35,21 @@ public:
 
     CegoBTreeObject();
     CegoBTreeObject(const CegoBTreeObject& oe);
-    CegoBTreeObject(Element *pTO ); 
     CegoBTreeObject(int tabSetId, ObjectType type, const Chain& objName);
     CegoBTreeObject(int tabSetId, ObjectType type, const Chain& objName, const ListT<CegoField>& schema, const Chain& tabName);
-	
+    
     ~CegoBTreeObject();
     
-    void setDataFileId(int fileId);
-    int getDataFileId() const;
-
-    void setDataPageId(int pageId);
-    int getDataPageId() const;
+    void setDataPageId(PageIdType pageId);
+    PageIdType getDataPageId() const;
     
     void increaseRelevance();
     void decreaseRelevance();
     int getRelevance() const;
-
-    void setMaxFid(int maxFid);
-    int getMaxFid() const;
     
     int getEntrySize() const;
 
-    void encode(char *buf);
+    void encode(char *buf) const;
     void decode(char *buf);
 
     bool isValid(); 
@@ -65,20 +59,17 @@ public:
     CegoBTreeObject& operator = ( const CegoBTreeObject& oe);
     bool operator == ( const CegoBTreeObject& oe);
 
-    Chain getId() const;
-    Chain toChain() const;
+    Chain getId(CegoProcBlock *pBlock) const;
+    Chain toChain(int defTabSetId) const;
     CegoContentObject* clone(bool isAttrRef = false);
-
-    Element* getElement() const;
-    void putElement(Element* pElement);
+    
     Chain getFormatted() const;
     
 private:
 
     Chain fill(const Chain& s, int num) const;
 
-    int _fileId;
-    int _pageId;
+    PageIdType _pageId;
     int _relevance; 
 };
 #endif
