@@ -6,7 +6,7 @@
 //      
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -20,7 +20,6 @@
 
 // CEGO INCLUDES
 #include "CegoBlob.h"
-#include "CegoDefs.h"
 
 // LFC INCLUDES
 #include <lfcbase/Exception.h>
@@ -34,23 +33,20 @@ CegoBlob::CegoBlob()
 {
     _buf = 0;
     _size = 0;
-    _fileId = 0;
     _pageId = 0;
 }
 
-CegoBlob::CegoBlob(int fileId, int pageId)
+CegoBlob::CegoBlob(PageIdType pageId)
 {
     _buf = 0;
     _size = 0;
-    _fileId = fileId;
     _pageId = pageId;
 }
 
-CegoBlob::CegoBlob(int fileId, int pageId, unsigned char *blobBuf, unsigned long long blobSize)
+CegoBlob::CegoBlob(PageIdType pageId, unsigned char *blobBuf, unsigned long long blobSize)
 {
     _buf = blobBuf;
     _size = blobSize;
-    _fileId = fileId;
     _pageId = pageId;
 }
 
@@ -124,6 +120,15 @@ void CegoBlob::allocate(unsigned long long size)
     _buf = (unsigned char*)malloc(_size);
 }
 
+void CegoBlob::release()
+{
+    if ( _buf )
+    {
+	free ( _buf);
+	_buf = 0;
+    }
+}
+
 void CegoBlob::reset()
 {
     _chunkPtr = _buf;
@@ -164,22 +169,12 @@ unsigned long long CegoBlob::getChunkSize()
     return _chunkSize;
 }
 
-void CegoBlob::setFileId(int fileId)
+void CegoBlob::setPageId(PageIdType pageId)
 {
-    _fileId = fileId;
+    _pageId = pageId;
 }
 
-int CegoBlob::getFileId() const
-{
-    return _fileId;
-}
-
-void CegoBlob::setPageId(int pageId)
-{
-    _pageId = pageId;;
-}
-
-int CegoBlob::getPageId() const
+PageIdType CegoBlob::getPageId() const
 {
     return _pageId;
 }
@@ -187,7 +182,6 @@ int CegoBlob::getPageId() const
 CegoBlob& CegoBlob::operator = ( const CegoBlob& b)
 {
     _size = b._size;
-    _fileId = b._fileId;
     _pageId = b._pageId;
     _buf = b._buf;
     _chunkSize = b._chunkSize;
@@ -197,7 +191,7 @@ CegoBlob& CegoBlob::operator = ( const CegoBlob& b)
 
 bool CegoBlob::operator == ( const CegoBlob& b)
 {
-    if ( _fileId == b._fileId && _pageId == b._pageId )
+    if ( _pageId == b._pageId )
 	return true;
     return false;
 }
