@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -20,6 +20,8 @@
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+#include "CegoDefs.h"
+
 class CegoBufferPage {
     
 public:
@@ -28,8 +30,7 @@ public:
     
     typedef struct PageHead {
 	PageType pageType;
-	int nextFileId;
-	int nextPageId;
+	PageIdType nextPageId;
     } PageHead;
     
     CegoBufferPage();
@@ -42,17 +43,11 @@ public:
     void setFixed(bool isFixed);
     bool isFixed() const;
 
-    void setFileId(int fileId);
-    int getFileId() const;
-
-    void setPageId(int pageId);
-    int getPageId() const;
-    
-    void setNextFileId(int fileId);
-    int getNextFileId() const;
-    
-    void setNextPageId(int pageId);
-    int getNextPageId() const;
+    void setPageId(PageIdType pageId);
+    PageIdType& getPageId();
+        
+    void setNextPageId(PageIdType pageId);
+    PageIdType& getNextPageId() const;
     
     void* newEntry(int size);
     void freeEntry(void *);
@@ -70,7 +65,7 @@ public:
     int getEntryPos() const;
     
     void setType(PageType t);
-    PageType getType() const;
+    PageType& getType() const;
 
     void setPagePtr(char* ptr);
     void setPageHead(PageHead* ptr);
@@ -86,8 +81,7 @@ public:
 
  private:
 
-    int _pageId;
-    int _fileId;
+    PageIdType _pageId;
     
     int _pageSize;
     char *_pagePtr;
