@@ -8,7 +8,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -47,16 +47,19 @@ public:
     void attachPool();
     void detachPool();
     
-    void bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int pageId, FixMode m, CegoLockHandler *pLockHandle, int numTry = 0);
+    void bufferFix(CegoBufferPage &bp, int tabSetId, PageIdType pageId, FixMode m, CegoLockHandler *pLockHandle, int numTry = 0);
     void emptyFix(CegoBufferPage &bp, int tabSetId, FixMode m, CegoFileHandler::FileType ft, CegoLockHandler *pLockHandle, bool doAppend = false);
 
     void bufferUnfix(CegoBufferPage &bp, bool isDirty, CegoLockHandler *pLockHandle);
     void bufferRelease(CegoBufferPage &bp, CegoLockHandler *pLockHandle);
 
-    int writeCheckPoint(int tabSetId, bool switchLog, const Chain& escCmd, int timeout, CegoLockHandler *pLockHandle);
+    unsigned long long writeCheckPoint(const Chain& tableSet, bool switchLog,  bool archComplete, CegoLockHandler *pLockHandle, const Chain& escCmd = Chain(""), int escTimeout = ESCCMDTIMEOUT, int archTimeout = LOGARCHTIMEOUT);
+    unsigned long long writeCheckPoint(int tabSetId, bool switchLog, const Chain& escCmd, int timeout, CegoLockHandler *pLockHandle);
     void writeAndRemoveTabSet(int tabSetId, CegoLockHandler *pLockHandle);
 
-    int uptime() const;
+    unsigned long long getCPCount() const;
+    
+    unsigned long long uptime() const;
     void poolInfo(int& pageSize,
 		  unsigned long long& numTotal,
 		  unsigned long long& numUsed,
@@ -73,23 +76,41 @@ public:
 		  unsigned long long& avgWriteDelay,
 		  unsigned long long& curFixCount,
 		  unsigned long long& maxFixCount,
-		  int& statStart,
-	          int& uptime) const;
+		  unsigned long long& avgFixTry,
+		  unsigned long long& statStart,
+	          unsigned long long& uptime) const;
 
     void getPoolEntryList(ListT<CegoBufferPoolEntry>& entryList);
 
     void resetStats();
 
+    void logIt(int tabSetId, CegoLogRecord& lr, CegoLockHandler* pLockHandle, bool flushLog = true);
+	
     void printPool();
 
+    void getLMLockStat(int tabSetId, Chain& lockName, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay);
+
+    void getAggLMLockStat(Chain& lockGroup, int& numLock, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay);
+    
+    void restoreCheckpointDump(int tabSetId, CegoLockHandler *pLockHandle);
+	
     Chain getDBName();
 
+    void lockPool(int tabSetId, bool doWrite);
+    void unlockPool(int tabSetId);
+    
 private:
 
-    // int _hashkey;
-    int calcSegment(int fileId, int pageId);
-    int calcHash(int fileId, int pageId);
+    int calcHash(PageIdType pageId);
+    int calcSegment(PageIdType pageId);
+    void logBM(int tabSetId, int fileId, unsigned* fbm, int fbmSize, CegoLockHandler* pLockHandle);	
+    bool archiveComplete(const Chain& tableSet);
+    void optimizePool(int tabSetId);
 
+    // dump methods
+    void dumpCheckpoint(int tabSetId);
+    void commitCheckpoint(int tabSetId, CegoLockHandler *pLockHandle);
+    
     typedef struct BufferPoolHead {
 	int numPages;
     } BufferPoolHead;
@@ -101,13 +122,10 @@ private:
 	char isDirty;
 	int numFixes;
 	int tabSetId;
-	int fileId;
-	int pageId;
-	int fixStat;
+	PageIdType pageId;
+	unsigned fixStat;
 	unsigned long long numUsage;
     } BufferHead;
-
-    void logBM(int tabSetId, int fileId, unsigned* fbm, int fbmSize);
     
     Chain _dbName;
     unsigned long long _numSegment;
@@ -117,18 +135,24 @@ private:
     void **_pBufPool;
 
     unsigned long long _fixCount;
+    unsigned long long _fixTry;
     unsigned long long _numDiskRead;    
     unsigned long long _numDiskWrite;
     unsigned long long _avgReadDelay;
     unsigned long long _avgWriteDelay;
 
+    unsigned long long _cpCount;
+
     NanoTimer _diskReadTimer;
     NanoTimer _diskWriteTimer;
 
-    int _poolStart;
-    int _statStart;
+    unsigned long long _poolStart;
+    unsigned long long _statStart;
     int _maxFixTries;
+    int _maxPageDelete;
 
-    unsigned long _modId;
+    ListT<PageIdType> _deletedPageList;
+    
+    unsigned long _modId;    
 };
 #endif
