@@ -6,7 +6,7 @@
 //      
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -20,7 +20,6 @@
 
 // CEGO INCLUDES
 #include "CegoBufferPoolEntry.h"
-#include "CegoDefs.h"
 
 // LFC INCLUDES
 #include <lfcbase/Exception.h>
@@ -34,15 +33,14 @@ CegoBufferPoolEntry::CegoBufferPoolEntry()
 {
 }
 
-CegoBufferPoolEntry::CegoBufferPoolEntry(int seg, int pos, const Chain& occState, bool isDirty, int numFixes, int tabSetId, int fileId, int pageId, int fixStat, unsigned long long numUsage)
+CegoBufferPoolEntry::CegoBufferPoolEntry(int seg, int pos, const Chain& occState, bool isDirty, int numFixes, int tabSetId, PageIdType pageId, unsigned fixStat, unsigned long long numUsage)
 {
     _seg = seg;
     _pos = pos;
     _occState = occState;
     _isDirty = isDirty;
     _numFixes = numFixes;
-    _tabSetId = tabSetId;   
-    _fileId = fileId;
+    _tabSetId = tabSetId;
     _pageId = pageId;
     _fixStat = fixStat;
     _numUsage = numUsage;
@@ -82,17 +80,12 @@ int CegoBufferPoolEntry::getTabSetId() const
     return _tabSetId;
 }
 
-int CegoBufferPoolEntry::getFileId() const
-{
-    return _fileId;
-}
-
-int CegoBufferPoolEntry::getPageId() const
+PageIdType CegoBufferPoolEntry::getPageId() const
 {
     return _pageId;
 }
 
-int CegoBufferPoolEntry::getFixStat() const
+unsigned CegoBufferPoolEntry::getFixStat() const
 {
     return _fixStat;
 }
@@ -109,8 +102,7 @@ CegoBufferPoolEntry& CegoBufferPoolEntry::operator = ( const CegoBufferPoolEntry
     _occState = b._occState;
     _isDirty = b._isDirty;
     _numFixes = b._numFixes;
-    _tabSetId = b._tabSetId;   
-    _fileId = b._fileId;
+    _tabSetId = b._tabSetId;
     _pageId = b._pageId;
     _fixStat = b._fixStat;
     _numUsage = b._numUsage;
