@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -23,6 +23,7 @@
 #include "CegoQueryHelper.h"
 #include "CegoXMLdef.h"
 #include "CegoDatabaseFormater.h"
+#include "CegoTypeConverter.h"
 
 // POSIX INCLUDES
 #include <string.h>
@@ -33,22 +34,15 @@ CegoCaseCond::CegoCaseCond()
     _elseExpr = 0;
 }
 
-CegoCaseCond::CegoCaseCond(char* buf, CegoDistManager *pGTM, int tabSetId)
+CegoCaseCond::CegoCaseCond(char* buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId)
 {
     _elseExpr = 0;
-    decode(buf, pGTM, tabSetId);
-}
-
-CegoCaseCond::CegoCaseCond(Element* pCaseElement, CegoDistManager *pGTM)
-{
-    _elseExpr = 0;
-    fromElement(pCaseElement, pGTM);
+    decode(buf, pGTM, pBlock, tabSetId);
 }
 
 CegoCaseCond::~CegoCaseCond()
 {
-
-    CegoPredDesc **pPredDesc = _predList.First();
+    CegoPredicate **pPredDesc = _predList.First();
     while ( pPredDesc )
     {
 	delete *pPredDesc;
@@ -65,7 +59,44 @@ CegoCaseCond::~CegoCaseCond()
 	delete _elseExpr;
 }
 
-void CegoCaseCond::addPred(CegoPredDesc *pPredDesc, CegoExpr* pExpr)
+void CegoCaseCond::cleanUp()
+{
+    CegoPredicate **pPredDesc = _predList.First();
+    while ( pPredDesc )
+    {
+	(*pPredDesc)->cleanUp();
+	pPredDesc = _predList.Next();
+    }
+    CegoExpr **pExpr = _exprList.First();
+    while ( pExpr) 
+    {
+	(*pExpr)->cleanUp();
+	pExpr = _exprList.Next();
+    }
+    
+    if ( _elseExpr )
+	_elseExpr->cleanUp();
+}
+
+void CegoCaseCond::setTabSetId(int tabSetId)
+{
+    CegoPredicate **pPredDesc = _predList.First();
+    while ( pPredDesc )
+    {
+	(*pPredDesc)->setTabSetId(tabSetId);
+	pPredDesc = _predList.Next();
+    }
+    CegoExpr **pExpr = _exprList.First();
+    while ( pExpr) 
+    {
+	(*pExpr)->setTabSetId(tabSetId);
+	pExpr = _exprList.Next();
+    }
+    if ( _elseExpr )
+	_elseExpr->setTabSetId(tabSetId);
+}
+
+void CegoCaseCond::addPred(CegoPredicate *pPredDesc, CegoExpr* pExpr)
 {
     _predList.Insert(pPredDesc);
     _exprList.Insert(pExpr);
@@ -76,7 +107,7 @@ void CegoCaseCond::setElseExpr(CegoExpr *pExpr)
     _elseExpr = pExpr;
 }
 
-ListT<CegoPredDesc*>& CegoCaseCond::getPredList()
+ListT<CegoPredicate*>& CegoCaseCond::getPredList()
 {
     return _predList;
 }
@@ -91,36 +122,11 @@ CegoExpr* CegoCaseCond::getElseExpr()
     return _elseExpr;
 }
 
-void CegoCaseCond::setBlock(CegoProcBlock *pBlock)
-{    
-    CegoPredDesc **pPredDesc = _predList.First();
-    while ( pPredDesc )
-    {
-	(*pPredDesc)->setBlock(pBlock);
-	pPredDesc = _predList.Next();
-    }
-    CegoExpr **pExpr = _exprList.First();
-    while ( pExpr) 
-    {
-	(*pExpr)->setBlock(pBlock);
-	pExpr = _exprList.Next();
-    }
-    if ( _elseExpr )
-	_elseExpr->setBlock(pBlock);
-
-    _pBlock = pBlock;
-}
-
-void CegoCaseCond::setFieldListArray(ListT<CegoField> **pFLA)
-{
-    _pFLA = pFLA;
-}
-
 ListT<CegoAttrDesc*> CegoCaseCond::getAttrRefList() const
 {
     ListT<CegoAttrDesc*> attrList;
 
-    CegoPredDesc **pPredDesc = _predList.First();
+    CegoPredicate **pPredDesc = _predList.First();
     while ( pPredDesc )
     {
 	attrList += (*pPredDesc)->getAttrRefList();
@@ -143,7 +149,7 @@ int CegoCaseCond::evalReferences(CegoContentObject *pCO, const ListT<CegoField>&
 {
     int refCount = 0;
 
-    CegoPredDesc **pPredDesc = _predList.First();
+    CegoPredicate **pPredDesc = _predList.First();
     while ( pPredDesc )
     {
 	refCount += (*pPredDesc)->evalReferences(pCO, fl);
@@ -162,9 +168,34 @@ int CegoCaseCond::evalReferences(CegoContentObject *pCO, const ListT<CegoField>&
     return refCount;
 }
 
+ListT<CegoAggregation*> CegoCaseCond::getAggregationList()
+{
+    ListT<CegoAggregation*> aggList;	
+
+    CegoPredicate **pPredDesc = _predList.First();
+    while ( pPredDesc )
+    {
+	aggList = aggList + (*pPredDesc)->getAggregationList();
+	pPredDesc = _predList.Next();
+    }
+
+    CegoExpr **pExpr = _exprList.First();
+    while ( pExpr ) 
+    {
+	aggList = aggList + (*pExpr)->getAggregationList();		    
+	pExpr = _exprList.Next();
+    }
+    
+    if ( _elseExpr )
+	aggList = aggList + _elseExpr->getAggregationList();
+
+    return aggList;
+}
+
+
 void CegoCaseCond::clearAttrCache()
 {
-    CegoPredDesc **pPredDesc = _predList.First();
+    CegoPredicate **pPredDesc = _predList.First();
     while ( pPredDesc )
     {
 	(*pPredDesc)->clearAttrCache();
@@ -181,63 +212,92 @@ void CegoCaseCond::clearAttrCache()
 	_elseExpr->clearAttrCache();
 }
 
-ListT<CegoField> CegoCaseCond::getFieldList() const
+void CegoCaseCond::getFieldList(ListT<CegoField>& fl, ListT<CegoField>** pFLA) const
 {
-    ListT<CegoField> fl;
-
-    CegoPredDesc **pPredDesc = _predList.First();
+    CegoPredicate **pPredDesc = _predList.First();
     while ( pPredDesc )
     {
-	fl += (*pPredDesc)->getFieldList();
+	(*pPredDesc)->getFieldList(fl, pFLA);
 	pPredDesc = _predList.Next();
     }
 
     CegoExpr **pExpr = _exprList.First();
     while ( pExpr) 
     {
-	fl += (*pExpr)->getFieldList();
+	(*pExpr)->getFieldList(fl, pFLA);
 	pExpr = _exprList.Next();
     }
     if ( _elseExpr )
-	fl += _elseExpr->getFieldList();
-
-    return fl;
+	_elseExpr->getFieldList(fl, pFLA);
 }
 
-CegoField CegoCaseCond::evalField() const
+CegoField CegoCaseCond::evalField(const ListT<CegoField>& fl, bool graceful) const
 {
-    CegoField cf = _elseExpr->evalField();
-    CegoField f = CegoField( Chain("CASE"), Chain("CASE"), Chain("case when ..."), cf.getType(), cf.getLength());
+    ListT<CegoField> cfList;
+    CegoExpr **pExpr = _exprList.First();
+    
+    CegoField elsecf = _elseExpr->evalField(fl);
+
+    CegoField refcf = elsecf;
+    while ( pExpr)
+    {
+	CegoField cf = (*pExpr)->evalField(fl);
+	cfList.Insert(cf);
+	
+	if ( refcf.getType() == NULL_TYPE && cf.getType() != NULL_TYPE )
+	    refcf = cf;
+	pExpr = _exprList.Next();
+    }
+
+    // type match is required for all cases
+
+    if ( graceful == false )
+    {
+	CegoField* pF = cfList.First();
+	while ( pF )
+	{
+	    if ( pF->getType() != NULL_TYPE && elsecf.getType() != NULL_TYPE
+		 && ( pF->getType() != elsecf.getType() || pF->getDim() != elsecf.getDim() ) )
+	    {
+		Chain msg = Chain("Case condition type mismatch for type ")
+		    + CegoTypeConverter::getTypeString( pF->getType()) + Chain("(") + Chain(pF->getDim()) + Chain(")")
+		    + Chain(" != ")
+		    + CegoTypeConverter::getTypeString( elsecf.getType()) + Chain("(") + Chain(elsecf.getDim()) + Chain(")");
+		    
+		throw Exception(EXLOC, msg);
+	    }
+	    pF = cfList.Next();
+	}
+    }
+
+    CegoField f = CegoField( Chain("CASE"), Chain("CASE"), Chain("case when ..."), refcf.getType(), refcf.getLength(), refcf.getDim());
     return f;
 }
 
-CegoFieldValue CegoCaseCond::evalFieldValue() const
-{
-
-    CegoPredDesc **pPredDesc = _predList.First();
+CegoFieldValue CegoCaseCond::evalFieldValue(ListT<CegoField> **pFLA, CegoProcBlock *pBlock) const
+{    
+    CegoPredicate **pPredDesc = _predList.First();
     CegoExpr **pExpr = _exprList.First();
 
     while ( pPredDesc && pExpr) 
     {
-	if ( CegoQueryHelper::evalPredicate(0, 0, _pFLA, 0, *pPredDesc, _pBlock) )
+	if ( (*pPredDesc)->eval(0, 0, pFLA, 0, pBlock) )
 	{
-	    (*pExpr)->setFieldListArray(_pFLA);
-	    return (*pExpr)->evalFieldValue();
+	    return (*pExpr)->evalFieldValue(pFLA, pBlock);
 	}
 	pPredDesc = _predList.Next();
 	pExpr = _exprList.Next();
     }
     CegoFieldValue fv;
-    
-    _elseExpr->setFieldListArray(_pFLA);
-    return _elseExpr->evalFieldValue();   
+   
+    return _elseExpr->evalFieldValue(pFLA, pBlock);   
 }
 
 CegoCaseCond* CegoCaseCond::clone(bool isAttrRef)
 {
     CegoCaseCond *pCaseCond = new CegoCaseCond();
 
-    CegoPredDesc **pPredDesc = _predList.First();
+    CegoPredicate **pPredDesc = _predList.First();
     CegoExpr **pExpr = _exprList.First();
     while ( pPredDesc && pExpr)
     {
@@ -255,43 +315,41 @@ CegoCaseCond& CegoCaseCond::operator = ( const CegoCaseCond& cc)
     _predList = cc._predList;
     _exprList = cc._exprList;
     _elseExpr = cc._elseExpr;
-    _pFLA = cc._pFLA;
-    _pBlock = cc._pBlock;
     return (*this);
 }
 
-Chain CegoCaseCond::getId() const
+Chain CegoCaseCond::getId(CegoProcBlock *pBlock) const
 {
     Chain s;
     s = Chain("case");
 
-    CegoPredDesc **pPredDesc = _predList.First();
+    CegoPredicate **pPredDesc = _predList.First();
     CegoExpr **pExpr = _exprList.First();
     while ( pPredDesc && pExpr)
     {
-	s += Chain("w") + (*pPredDesc)->getId() + Chain("t") + (*pExpr)->getId();
+	s += Chain("w") + (*pPredDesc)->getId(pBlock) + Chain("t") + (*pExpr)->getId(pBlock);
 	pPredDesc = _predList.Next();
 	pExpr = _exprList.Next();
     }
-    s += Chain("e") + _elseExpr->getId();
+    s += Chain("e") + _elseExpr->getId(pBlock);
 
     return s;
 }
 
-Chain CegoCaseCond::toChain() const
+Chain CegoCaseCond::toChain(int defTabSetId) const
 {
     Chain s;
     s = Chain("case");
 
-    CegoPredDesc **pPredDesc = _predList.First();
+    CegoPredicate **pPredDesc = _predList.First();
     CegoExpr **pExpr = _exprList.First();
     while ( pPredDesc && pExpr)
     {
-	s += Chain(" when ") + (*pPredDesc)->toChain() + Chain(" then ") + (*pExpr)->toChain();
+	s += Chain(" when ") + (*pPredDesc)->toChain(defTabSetId) + Chain(" then ") + (*pExpr)->toChain(defTabSetId);
 	pPredDesc = _predList.Next();
 	pExpr = _exprList.Next();
     }
-    s += Chain(" else ") + _elseExpr->toChain();
+    s += Chain(" else ") + _elseExpr->toChain(defTabSetId);
     s += Chain(" end ");
 
     return s;
@@ -302,80 +360,7 @@ Chain CegoCaseCond::dbFormat(CegoDatabaseFormater *pForm) const
     return pForm->formatCaseCond(_predList, _exprList, _elseExpr);
 }
 
-Element* CegoCaseCond::toElement() const
-{
-    Element* pCaseClauseElement = new Element(XML_CASECLAUSE_ELEMENT);
-
-    CegoPredDesc **pPredDesc = _predList.First();
-    CegoExpr **pExpr = _exprList.First();
-    while ( pPredDesc && pExpr)
-    {
-
-	Element* pCaseElement = new Element(XML_CASE_ELEMENT);
-	
-	pCaseElement->addContent((*pPredDesc)->toElement());
-	pCaseElement->addContent((*pExpr)->toElement());
-	
-	pCaseClauseElement->addContent(pCaseElement);
-
-	pPredDesc = _predList.Next();
-	pExpr = _exprList.Next();
-    }
-
-    pCaseClauseElement->addContent(_elseExpr->toElement());
-    
-    return pCaseClauseElement;
-}
-
-void CegoCaseCond::fromElement(Element *pCaseClauseElement, CegoDistManager *pGTM)
-{
-
-    CegoPredDesc **pPredDesc = _predList.First();
-    while ( pPredDesc )
-    {
-	delete *pPredDesc;
-	pPredDesc = _predList.Next();
-    }
-    _predList.Empty();
-
-    CegoExpr **pExpr = _exprList.First();
-    while ( pExpr) 
-    {
-	delete *pExpr;
-	pExpr = _exprList.Next();
-    }
-    _exprList.Empty();
-    
-    if ( _elseExpr )
-	delete _elseExpr;
-        
-    ListT<Element*> cl = pCaseClauseElement->getChildren(XML_CASE_ELEMENT);
-    Element **pCE = cl.First();
-    while ( pCE )
-    {	
-	ListT<Element*> pl = (*pCE)->getChildren(XML_PRED_ELEMENT);
-	ListT<Element*> el = (*pCE)->getChildren(XML_EXPR_ELEMENT);
-
-	Element **pPE = pl.First();
-	Element **pEE = el.First();
-	if ( pPE && pEE )
-	{
-	    addPred( new CegoPredDesc(*pPE, pGTM), new CegoExpr(*pEE, pGTM));
-	}
-
-	pCE = cl.Next();
-    }
-
-    ListT<Element*> el = pCaseClauseElement->getChildren(XML_EXPR_ELEMENT);
-    
-    Element **pEE = el.First();
-    if ( pEE )
-    {
-	_elseExpr = new CegoExpr(*pEE, pGTM);
-    }
-}
-
-void CegoCaseCond::encode(char *buf)
+void CegoCaseCond::encode(char *buf, CegoProcBlock *pBlock)
 {
     char* pE = (char*)buf;
 
@@ -383,23 +368,23 @@ void CegoCaseCond::encode(char *buf)
     memcpy( pE, &numPred, sizeof(int));
     pE = pE + sizeof(int);
 
-    CegoPredDesc **pPredDesc = _predList.First();
+    CegoPredicate **pPredDesc = _predList.First();
     CegoExpr **pExpr = _exprList.First();
     while ( pPredDesc && pExpr)
     {
-	(*pPredDesc)->encode(pE);
-	pE = pE + (*pPredDesc)->getEncodingLength();
+	(*pPredDesc)->encode(pE, pBlock);
+	pE = pE + (*pPredDesc)->getEncodingLength(pBlock);
 	
-	(*pExpr)->encode(pE);
-	pE = pE + (*pExpr)->getEncodingLength();
+	(*pExpr)->encode(pE, pBlock);
+	pE = pE + (*pExpr)->getEncodingLength(pBlock);
 
 	pPredDesc = _predList.Next();
 	pExpr = _exprList.Next();
     }
-    _elseExpr->encode(pE);    
+    _elseExpr->encode(pE, pBlock);    
 }
 
-void CegoCaseCond::decode(char *buf, CegoDistManager* pGTM, int tabSetId)
+void CegoCaseCond::decode(char *buf, CegoDistManager* pGTM, CegoProcBlock *pBlock, int tabSetId)
 {
     char* pE = (char*)buf;
 
@@ -410,42 +395,50 @@ void CegoCaseCond::decode(char *buf, CegoDistManager* pGTM, int tabSetId)
     int i=0;
     while ( i < numPred )
     {
-	CegoPredDesc *pPred = new CegoPredDesc(pE, pGTM, tabSetId);
-	pE = pE + pPred->getEncodingLength();
+	CegoPredicate *pPred = new CegoPredicate(pE, pGTM, pBlock, tabSetId);
+	pE = pE + pPred->getEncodingLength(pBlock);
 	_predList.Insert(pPred);
 	
-	CegoExpr *pExpr = new CegoExpr(pE, pGTM, tabSetId);
-	pE = pE + pExpr->getEncodingLength();
+	CegoExpr *pExpr = new CegoExpr(pE, pGTM, pBlock, tabSetId);
+	pE = pE + pExpr->getEncodingLength(pBlock);
 	_exprList.Insert(pExpr);
 	i++;
     }
-
-    // else
-    _elseExpr = new CegoExpr(pE, pGTM, tabSetId);
-
+    
+    _elseExpr = new CegoExpr(pE, pGTM, pBlock, tabSetId);
 }
 
-int CegoCaseCond::getEncodingLength() const
+int CegoCaseCond::getEncodingLength(CegoProcBlock *pBlock) const
 {
-    // numPred
     int len = sizeof(int);
     
-    CegoPredDesc **pPredDesc = _predList.First();
+    CegoPredicate **pPredDesc = _predList.First();
     CegoExpr **pExpr = _exprList.First();
     while ( pPredDesc && pExpr)
     {
-	len += (*pPredDesc)->getEncodingLength();	
-	len += (*pExpr)->getEncodingLength();
+	len += (*pPredDesc)->getEncodingLength(pBlock);
+	len += (*pExpr)->getEncodingLength(pBlock);
 
 	pPredDesc = _predList.Next();
 	pExpr = _exprList.Next();
     }
-    len += _elseExpr->getEncodingLength();
+    len += _elseExpr->getEncodingLength(pBlock);
     return len;
 }
 
+void CegoCaseCond::getPlanList(ListT<Element*>& planList)
+{
+    CegoExpr **pExpr = _exprList.First();
+    while ( pExpr)
+    {
+	(*pExpr)->getPlanList(planList);
+	pExpr = _exprList.Next();
+    }
+    _elseExpr->getPlanList(planList);
+}
+
 ostream& operator << (ostream& s, const CegoCaseCond& cc)
 {
-    s << cc.toChain();
+    s << cc.toChain(0);
     return s;
 }
