@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -25,7 +25,7 @@
 #include <lfcbase/ListT.h>
 
 // CEGO INCLUDES
-#include "CegoPredDesc.h"
+#include "CegoPredicate.h"
 #include "CegoExpr.h"
 
 class CegoDatabaseFormater;
@@ -35,54 +35,52 @@ class CegoCaseCond {
 public:
 
     CegoCaseCond();
-    CegoCaseCond(char* buf, CegoDistManager *pGTM, int tabSetId);
-    CegoCaseCond(Element* pCaseElement, CegoDistManager *pGTM);
+    CegoCaseCond(char* buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId);
     ~CegoCaseCond();
 
-    void addPred(CegoPredDesc *CegoPredDesc, CegoExpr* pExpr);
+    void cleanUp();    
+
+    void setTabSetId(int tabSetId);
+    
+    void addPred(CegoPredicate *CegoPredDesc, CegoExpr* pExpr);
     void setElseExpr(CegoExpr *pExpr);
 
-    ListT<CegoPredDesc*>& getPredList();
+    ListT<CegoPredicate*>& getPredList();
     ListT<CegoExpr*>& getExprList();
     CegoExpr* getElseExpr();
-    
-    void setBlock(CegoProcBlock *pBlock);
-    void setFieldListArray(ListT<CegoField> **pFLA);
 
     ListT<CegoAttrDesc*> getAttrRefList() const;
     int evalReferences(CegoContentObject *pCO, const ListT<CegoField>& fl);
-
+    ListT<CegoAggregation*> getAggregationList();
+    
     void clearAttrCache();
 
-    ListT<CegoField> getFieldList() const;
-    CegoField evalField() const;
-    CegoFieldValue evalFieldValue() const;
+    void getFieldList(ListT<CegoField>& fl, ListT<CegoField>** pFLA) const;
+    CegoField evalField(const ListT<CegoField>& fl, bool graceful = false) const;
+    
+    CegoFieldValue evalFieldValue(ListT<CegoField> **pFLA, CegoProcBlock *pBlock) const;
 
     CegoCaseCond* clone(bool isAttrRef = false);
     
     CegoCaseCond& operator = ( const CegoCaseCond& cc);
 
-    Chain getId() const;
-    Chain toChain() const;
+    Chain getId(CegoProcBlock *pBlock) const;
+    Chain toChain(int defTabSetId) const;
     Chain dbFormat(CegoDatabaseFormater *pForm) const;
-		   
-    Element* toElement() const;
-    void fromElement(Element *pCaseClauseElement, CegoDistManager *pGTM);
-
-    void encode(char *buf);
-    void decode(char *buf, CegoDistManager* pGTM, int tabSetId);
-    int getEncodingLength() const;
+    
+    void encode(char *buf, CegoProcBlock *pBlock);
+    void decode(char *buf, CegoDistManager* pGTM, CegoProcBlock *pBlock, int tabSetId);
+    int getEncodingLength(CegoProcBlock *pBlock) const;
 
+    void getPlanList(ListT<Element*>& planList);
+	
     friend ostream& operator << (ostream& s, const CegoCaseCond& cc);
       
 private:
 
-    ListT<CegoPredDesc*> _predList;
+    ListT<CegoPredicate*> _predList;
     ListT<CegoExpr*> _exprList;
     CegoExpr* _elseExpr;
 
-    ListT<CegoField> **_pFLA;
-
-    CegoProcBlock* _pBlock;
 };
 #endif
