@@ -6,7 +6,7 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke                        
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -35,6 +35,7 @@
 #include <lfcbase/Net.h>
 #include <lfcbase/SigHandler.h>
 #include <lfcbase/Tokenizer.h>
+#include <lfcbase/Pager.h>
 #include <lfcxml/Element.h>
 #include <lfcxml/Document.h>
 #include <lfcxml/XMLSuite.h>
@@ -44,6 +45,7 @@
 #include "CegoModule.h"
 #include "CegoNet.h"
 #include "CegoXMLdef.h"
+#include "CegoQueryHelper.h"
 
 // #include "CegoDefs.h"
 
@@ -56,16 +58,15 @@
 #define USAGE "Usage: cgclt --user=<user>/<password>\n\
           [ --server=<host> ]\n\
           [ --port=<port> ]\n\
-          [ --protocol={serial|xml} ]\n\
+          [ --protocol={xml|serial|fastserial} ]\n\
 	  [ --tableset=<tableset> ]\n\
           [ --batchfile=<batchfile> ]\n\
           [ --dumpfile=<dumpfile> ]\n\
-          [ --qescmode ]\n\
           [ --structure ]\n\
-          [ --cmd=<command> ]\n\
           [ --logfile=<logfile> ]\n\
-          [ --raw ]\n\
-          [ --flow ]\n\
+          [ --raw=<sep> ]\n\
+          [ --nohead ]\n\
+          [ --pager ]\n\
           [ --maxline=<maxline> ]\n\
           [ --maxresult=<maxresult> ]\n\
           [ --ignore ]\n\
@@ -79,19 +80,17 @@
 #define CGPROFILE ".cgprofile"
 #define LANGUAGE_ENV "LANG"
 #define LOCALE_CATEGORY LC_ALL
+#define PIPE_TIMEOUT 30 /* checking input pipe data timeout in ms */
+#define HOMEVAR "HOME"
 
-enum RunMode { INTERACTIVE, BATCH, COMMAND, DUMP };
+enum RunMode { INTERACTIVE, BATCH, DUMP };
 
 extern char __lfcVersionString[];
 extern char __lfcxmlVersionString[];
-extern char __quoteEscapeFlag;
 extern Chain __dateTimeFormat;
 extern char __currencySymbol;
 extern char __decimalPoint;
 
-bool flowCtl = false;
-int flowCount = 10;
-
 class AbortHandler : public SigHandler
 {
   
@@ -160,18 +159,20 @@ private:
 };
 
 int execute();
-bool dispatchCmd(bool outputBuffered, CegoNet* pCegoNet, AbortHandler* pAH, Chain cmd, bool rawMode, int& exitCode, Chain& msg);
+void treatEscape(Chain& cmd);
+bool dispatchCmd(bool outputBuffered, CegoNet* pCegoNet, AbortHandler* pAH, Chain cmd, bool rawMode, bool noHead, int& exitCode, Chain& msg);
 void loadProfile(const Chain& rcFileName, ListT<ProfileEntry>& pfeList);
+void dumpProlog(File *pDF);
 void dumpTableSet(CegoNet *pCegoNet, const Chain& dumpFileName, bool isStructure);
 void dumpCounter(CegoNet *pCegoNet, File *pDF);
 void dumpTable(CegoNet *pCegoNet, File *pDF, bool isStructure);
-void dumpIndex(CegoNet *pCegoNet, File *pDF);
+void dumpAVLtree(CegoNet *pCegoNet, File *pDF);
 void dumpBtree(CegoNet *pCegoNet, File *pDF);
 void dumpKey(CegoNet *pCegoNet, File *pDF);
 void dumpCheck(CegoNet *pCegoNet, File *pDF);
 void dumpView(CegoNet *pCegoNet, File *pDF);
 void dumpProcedure(CegoNet *pCegoNet, File *pDF);
-
+void dumpAlias(CegoNet *pCegoNet, File *pDF);
 
 Chain user;
 Chain password;
@@ -185,6 +186,7 @@ Chain protocol;
 unsigned long long maxLine;
 unsigned long long maxResult;
 bool rawMode = false;
+bool noHead = false;
 bool isBuffered = false;
 Chain rawSep;
 bool ignoreError = false;
@@ -201,8 +203,7 @@ CegoDbHandler::ProtocolType protType;
 Chain histPath;
 
 int main(int argc, char **argv)
-{
-    
+{    
     GetLongOpt longOpt(argc, argv);
     
     longOpt.addOpt("version");
@@ -210,17 +211,16 @@ int main(int argc, char **argv)
     longOpt.addOpt("logfile");
     longOpt.addOpt("tableset");
     longOpt.addOpt("user");
-    longOpt.addOpt("cmd");
     longOpt.addOpt("batchfile");
     longOpt.addOpt("dumpfile");
     longOpt.addOpt("structure");
-    longOpt.addOpt("qescmode");
     longOpt.addOpt("maxline", Chain(DEFAULTMAXLINE));
     longOpt.addOpt("maxresult", Chain(DEFAULTMAXRESULT));
     
     longOpt.addOpt("debug");
     longOpt.addOpt("raw", Chain(DEFAULTRAWSEP));
-    longOpt.addOpt("flow");
+    longOpt.addOpt("nohead");
+    longOpt.addOpt("pager");
     longOpt.addOpt("ignore");
     longOpt.addOpt("server", DEFAULTSERVER);
     longOpt.addOpt("protocol", DEFAULTPROTOCOL);
@@ -239,36 +239,33 @@ int main(int argc, char **argv)
 	cerr << USAGE << endl;
 	exit(1);	
     }
+
+    Chain homeDir(getenv(HOMEVAR));
     
 #ifdef HAVE_MINGW
-    Chain homeDir = getenv("HOMEPATH");
     histPath = homeDir + "\\" + Chain(HISTFILE);
 #else
-    Chain homeDir(getenv("HOME"));
     histPath = homeDir + "/" + Chain(HISTFILE);
 #endif
     
     if ( longOpt.isSet("debug") )
 	debug=true;
     
-    if ( longOpt.isSet("qescmode") )
-	__quoteEscapeFlag=1;
-    else
-	__quoteEscapeFlag=0;
-
     if ( longOpt.isSet("raw") )
     {
 	rawMode=true;
 	rawSep = longOpt.getOptValue("raw");
     }
-    
-    if ( longOpt.isSet("flow") )
+
+    if ( longOpt.isSet("nohead") )
+	noHead=true;
+
+    if ( longOpt.isSet("pager") )
 	isBuffered=true;
     
     if ( longOpt.isSet("ignore") )
 	ignoreError=true;
 
-    
     if ( longOpt.isSet("structure") )
 	isStructure=true;
 
@@ -313,11 +310,6 @@ int main(int argc, char **argv)
 	runMode = DUMP;
 	dumpFileName = longOpt.getOptValue("dumpfile");
     }
-    else if ( longOpt.isSet("cmd") )
-    {
-	runMode = COMMAND;
-	command = longOpt.getOptValue("cmd");
-    }
 
     if ( longOpt.isSet("profile") )
     {
@@ -374,13 +366,17 @@ int main(int argc, char **argv)
     {
 	protType = CegoDbHandler::SERIAL;
     }
+    else if ( protocol == Chain("fastserial") )
+    {
+	protType = CegoDbHandler::FASTSERIAL;
+    }
     else if ( protocol == Chain("xml") )
     {
 	protType = CegoDbHandler::XML;
     }
     else
     {
-	cerr << "Invalid protocol " << protocol;
+	cerr << "Invalid protocol " << protocol << endl;
 	exit (1);
     }
 
@@ -405,7 +401,7 @@ int main(int argc, char **argv)
     maxResult = longOpt.getOptValue("maxresult").asUnsignedLongLong();
 
     if ( logFile.length() > 0 )
-	pModule = new CegoModule(logFile, Chain(""));
+	pModule = new CegoModule(logFile, Chain());
     else
 	pModule = new CegoModule;
     
@@ -436,7 +432,7 @@ int main(int argc, char **argv)
 	cerr << msg << endl;
 	exitCode = 1;
     }
-
+    
     delete pModule;
     exit(exitCode);
 }
@@ -448,199 +444,146 @@ int execute()
     Chain logMode("notice");
     if ( debug )
 	logMode=Chain("debug");
-        
-    switch ( runMode )
+
+    File *pInput = 0;
+    pInput = new File(File::IN);
+    if ( pInput->hasData(PIPE_TIMEOUT) )
     {
-	
-    case COMMAND:
-    {
-		
-	CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(""), logMode );	
-	
-	try
-	{
-	    pCegoNet->connect(serverName, portNo, tableSet, user, password);
-	}
-	catch ( Exception e )
-	{
-	    cerr << "Cannot connect to database" << endl;
-	    delete pCegoNet;
-	    return 1;
-	}
-	
-	Timer t(6,3);
-	t.start();
-
-	Chain msg;
-	dispatchCmd(false, pCegoNet, 0, command, rawMode, exitCode, msg);
-	t.stop();
-
-	pCegoNet->disconnect();
-	
-	delete pCegoNet;
-
-	if ( exitCode != 0 )
-	{
-	    cerr << "Command error : " << msg << endl;
-	}
-	else
-	{
-	    cout << msg << endl;
-	    cout << "ok ( " << t << " s )" << endl;
-	}
-
-	break;
-
+	runMode = BATCH;	
     }
+    else
+    {
+	delete pInput;
+	pInput = 0;
+    }
+    
+    switch ( runMode )
+    {	
     case BATCH:
     {
 	int lineNo=0;
 
-	CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(""), logMode );	
+	CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(), logMode );	
 	
 	try
-	{
-
-	    File cmdFile(batchFileName);
-	    if ( cmdFile.exists() == false )
+	{	    
+	    if ( pInput == 0 )
 	    {
-		Chain msg = Chain("File ") + batchFileName + Chain(" does not exist");
-		throw Exception(EXLOC, msg);
+		pInput = new File(batchFileName);
+		if ( pInput->exists() == false )
+		{
+		    Chain msg = Chain("File ") + batchFileName + Chain(" does not exist");		    
+		    throw Exception(EXLOC, msg);
+		}
+		pInput->open(File::READ);
 	    }
 
 	    pCegoNet->connect(serverName, portNo, tableSet, user, password);
 		    
-	    cmdFile.open(File::READ);	    
-
 	    Chain cmd;
 	    Chain line;
 	    bool disableDelimiter=false;
 	    bool goOn = true;
 	    
-	    while (goOn && cmdFile.readLine(line, maxLine))
+	    while (goOn && pInput->readLine(line, maxLine))
 	    {
 		lineNo++;
-		
-		bool isCommentLine = false;
-		
-		line = line.truncRight(" \t");
-	    
-		if ( line.truncLeft(" \t").length() >= 2 )
+		if ( line.truncRight(" \t") == Chain("@") )
 		{
-		    if (line.truncLeft(" \t").subChain(1, 2) == Chain("--"))
-		    {
-			isCommentLine = true;
-		    }
+		    if ( disableDelimiter == false )
+			disableDelimiter=true;
+		    else
+			disableDelimiter=false;
 		}
-		
-		if ( ! isCommentLine )
+		else
 		{
+		    cmd = cmd + Chain(" ") + line;			
+		}
 
-		    if ( line == Chain("@") )
+		if ( cmd.length() > 0 )
+		{
+		    if ( CegoQueryHelper::hasOpenString(cmd) == false )
 		    {
-			if ( disableDelimiter == false )
-			    disableDelimiter=true;
-			else
-			    disableDelimiter=false;
+			cmd = CegoQueryHelper::skipComment(cmd);
+
+			if ( cmd != Chain() )
+			{
+			    if (cmd.subChain(cmd.length()-1, cmd.length()) == Chain(";") 
+				&& disableDelimiter==false)
+			    {		
+				// send command to server here				
+				// replace escape characters
+				
+				treatEscape(cmd);
+				
+				Timer t(6,3);
+				t.start();
+				
+				Chain msg;
+				goOn = dispatchCmd(false, pCegoNet, 0, cmd, rawMode, noHead, exitCode, msg);
+				
+				t.stop();
+				
+				if ( exitCode != 0 )
+				{
+				    cerr << "Line Number " + Chain(lineNo) + " : " << msg << endl;
+				}
+				else
+				{
+				    if ( rawMode == false )
+				    {
+					cout << msg << endl;				
+					cout << "ok ( " << t << " s )" << endl;
+				    }
+				}
+			
+				if ( exitCode != 0 && ignoreError )
+				    goOn = true;
+			
+				cmd = Chain();
+			    }
+			}
 		    }
 		    else
 		    {
-			// cmd = line;
-			cmd = cmd + Chain(" ") + line;			
-			// cmd = cmd + line;
-		    }
-
-		    // cmd = cmd.cutTrailing(" \t");
-
-		    if ( cmd.length() > 0 )
-		    {		       
-			if (cmd.subChain(cmd.length()-1, cmd.length()) == Chain(";") 
-			    && disableDelimiter==false)
-			{
-
-			    // send command to server here
-
-			    Chain bval;
-			    if ( cmd.replaceAll(Chain("\\n"), Chain("\n"), bval) > 0 )
-				cmd = bval;
-			    			    
-			    // escape quotes
-			    Chain qval;
-			    if ( __quoteEscapeFlag )
-			    {
-				if ( cmd.replaceAll(Chain("''"), Chain("'"), qval) > 0 )
-				    cmd = qval;
-			    }
-			    else
-			    {
-				if ( cmd.replaceAll(Chain("\\'"), Chain("'"), qval) > 0 )
-				    cmd = qval;
-			    }			  			    
-			   			    
-			    Timer t(6,3);
-			    t.start();
-			    			   
-			    Chain msg;
-			    goOn = dispatchCmd(false, pCegoNet, 0, cmd, rawMode, exitCode, msg);
-			    
-			    t.stop();
-
-			    if ( exitCode != 0 )
-			    {
-				cerr << "Line Number " + Chain(lineNo) + " : " << msg << endl;
-			    }
-			    else
-			    {
-				if ( rawMode == false )
-				{
-				    cout << msg << endl;				
-				    cout << "ok ( " << t << " s )" << endl;
-				}
-			    }
-
-			    if ( exitCode != 0 && ignoreError )
-				goOn = true;
-			    
-			    cmd = Chain("");
-			    
-			}
-			else
-			{
-			    // collect further data
-			}
+			// collect further data
 		    }
 		}
 	    }
+	
 	    cmd = cmd.cutTrailing(" \t");
 	    if ( cmd.length() > 1 )
 	    {
 		cout << "Incomplete command ..<" << cmd << ">" << endl;
 	    }
-	    
-	    cmdFile.close();
 
+	    if ( runMode == BATCH )
+	    {	    
+		pInput->close();
+	    }
+	    delete pInput;
+	    
 	    pCegoNet->disconnect();
 
 	    delete pCegoNet;
-
 	}
 	catch ( Exception e )
 	{
 	    Chain msg;
 	    e.pop(msg);
+
+	    if ( pInput )
+		delete pInput;
 	    
 	    cerr << msg << endl;
 	    delete pCegoNet;
 	    return 1;
-	}
-		    	
+	}	    	
 	break;
-
     }
     case DUMP:
     {
-     
-	CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(""), logMode );	
+	CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(), logMode );	
 	
 	try
 	{
@@ -661,16 +604,13 @@ int execute()
 	
 	exitCode = 0;
 	break;
-
     }
     case INTERACTIVE:
     {
-
-	CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(""), logMode );	
+	CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(), logMode );	
 
 	try
-	{
-	    
+	{	    
 	    pCegoNet->connect(serverName, portNo, tableSet, user, password);
 	    
 	    unsigned long long tid = pCegoNet->getTid();
@@ -681,16 +621,20 @@ int execute()
 	    pModule->log(modId, Logger::DEBUG, Chain("Got session ..."));
 #endif
 
+	    Chain dbName = pCegoNet->getDbName();
+	    Chain dbVersion = pCegoNet->getDbVersion();
+
+	    cout << "Connected to " << dbName << " ( Version " << dbVersion << " )" << endl;
+	    
 	    read_history((char*)histPath);
  
 	    bool goOn = true;
 	    while ( goOn )
-	    {
-		
+	    {	
 		char *pC = 0;
 		
 		Chain cs;
-		
+
 		pC = readline(promptString);
 		
 		if (pC)
@@ -724,34 +668,44 @@ int execute()
 				pC = readline(promptString);
 			    }
 			    if (pC)
-				cs = cs + Chain(" ") + Chain(pC);	   
+				cs = cs + Chain(" ") + Chain(pC);
+			    else
+			    {
+				stmtComplete=true;
+				goOn=false;
+				cout << "Bye" << endl;				
+			    }
 			}
-			
 		    }
+
+		    if ( goOn )
+		    {
+			treatEscape(cs);
 		    
-		    Timer t(6,3);
-		    t.start();
-
-		    Chain msg;
-		    goOn = dispatchCmd(isBuffered, pCegoNet, pAH, cs, rawMode, exitCode, msg);
-
-		    t.stop();
-
-		    if ( exitCode != 0 )
-		    {
-			cerr << "Error : " <<  msg << endl;
-			cerr << "Query failed" << endl;
-			goOn = true;
-		    }
-		    else
-		    {
-			if ( rawMode == false )
-			{			   
-			    cout << msg << endl;
-			    cout << "ok ( " << t << " s )" << endl;
+			Timer t(6,3);
+			t.start();
+			
+			Chain msg;
+			goOn = dispatchCmd(isBuffered, pCegoNet, pAH, cs, rawMode, noHead, exitCode, msg);
+			
+			t.stop();
+			
+			if ( exitCode != 0 )
+			{
+			    cerr << "Error : " <<  msg << endl;
+			    cerr << "Query failed" << endl;
+			    goOn = true;
 			}
+			else
+			{
+			    if ( rawMode == false )
+			    {			   
+				cout << msg << endl;
+				cout << "ok ( " << t << " s )" << endl;
+			    }
+			}
+			write_history((char*)histPath);
 		    }
-		    write_history((char*)histPath);
 		}
 		else
 		{
@@ -762,7 +716,6 @@ int execute()
 	    
 	    pCegoNet->disconnect();
 	    delete pCegoNet;
-	    
 	}
 	catch ( Exception e)
 	{
@@ -772,17 +725,27 @@ int execute()
 	    cerr << msg << endl;
 	    return 1;
 	}
-
 	break;
-
     }    
     }
     return exitCode;
 }
 
-bool dispatchCmd(bool outputBuffered, CegoNet *pCegoNet, AbortHandler *pAH, Chain cmd, bool rawMode, int& exitCode, Chain& msg)
+void treatEscape(Chain& cmd)
 {
+    Chain bval;
+    if ( cmd.replaceAll(Chain("\\n"), Chain("\n"), bval) > 0 )
+	cmd = bval;
     
+    if ( cmd.replaceAll(Chain("\\r"), Chain("\r"), bval) > 0 )
+	cmd = bval;
+    
+    if ( cmd.replaceAll(Chain("\\t"), Chain("\t"), bval) > 0 )
+	cmd = bval;
+}
+
+bool dispatchCmd(bool outputBuffered, CegoNet *pCegoNet, AbortHandler *pAH, Chain cmd, bool rawMode, bool noHead, int& exitCode, Chain& msg)
+{
     exitCode = 0;
 
     if ( cmd.cutTrailing(" ;") == Chain("quit") )
@@ -790,28 +753,26 @@ bool dispatchCmd(bool outputBuffered, CegoNet *pCegoNet, AbortHandler *pAH, Chai
 	msg = Chain("Goodbye");
 	return false;
     }
-    
-    CegoDbHandler::ResultType res;
-    
+        
     if ( pAH )
 	pAH->setQueryAbort(true);
 
     try
     {
-	unsigned long long affCount = pCegoNet->doQuery(cmd);
+	// cout << "Query= <<<" << cmd << ">>>" << endl;
+	// doQuery return number of affected, which we do not use here
+	pCegoNet->doQuery(cmd);
     }
     catch ( Exception e )
     {
+	e.pop(msg);
+	
 	if ( pAH )
 	    pAH->setQueryAbort(false);
 	
-	msg = pCegoNet->getMsg();
 	exitCode = 1;
 	return false;
     }
-
-    if ( pAH )
-	pAH->setQueryAbort(false);
     
     if ( pCegoNet->isFetchable() )
     {		
@@ -827,79 +788,134 @@ bool dispatchCmd(bool outputBuffered, CegoNet *pCegoNet, AbortHandler *pAH, Chai
 	if ( rawMode )
 	    output.setRawSeparator(rawSep);
 
-	ListT< ListT<CegoFieldValue> > outData;
+	ListT< ListT<Chain> > outData;
 
 	if ( outputBuffered == false )
-	    output.headOut();
-		
-	ListT<CegoFieldValue> fvl;
-	
-	unsigned long long count=0;
-
-	while ( pCegoNet->fetchData(schema, fvl)  )
 	{
-
-	    count++;
-
-	    if ( count > maxResult )
+	    if ( noHead == true )
 	    {
-		pCegoNet->abortQuery();
-		exitCode = 1;
-		msg=Chain("Result is too large");
-		return false;
-	    }
-	    
-	    if ( outputBuffered )
-	    {       	
-		outData.Insert(fvl);
-		if ( count % 100 )
+		if ( rawMode == false )
 		{
-		    cout << "Collecting data ... " << count << " rows\r";
-		    cout.flush();
+		    output.tailOut();
 		}
 	    }
 	    else
 	    {
-		output.rowOut(fvl);
+		output.headOut();
 	    }
-
-	    fvl.Empty();	    
 	}
 
-	if ( outputBuffered )
+	ListT<CegoFieldValue> fvl;
+	
+	unsigned long long count=0;
+
+	try
 	{
-	    output.manageOutput(cmd, outData);
+	    while ( pCegoNet->fetchData(schema, fvl)  )
+	    {
+		count++;
+		
+		if ( count > maxResult )
+		{
+		    pCegoNet->abortQuery();
+		    exitCode = 1;
+		    msg=Chain("Result is too large");
+		    return false;
+		}
+		
+		if ( outputBuffered )
+		{
+		    CegoFieldValue *pFV = fvl.First();
+		    ListT<Chain> outRow;
+		    while ( pFV )
+		    {
+			outRow.Insert(pFV->valAsChain());
+			pFV = fvl.Next();		    
+		    }
+		    outData.Insert(outRow);
+		    if ( count % 100 )
+		    {
+			cout << "Collecting data ... " << count << " rows\r";
+			cout.flush();
+		    }
+		}
+		else
+		{
+		    output.rowOut(fvl);
+		}
+		
+		fvl.Empty();	    
+	    }
+	    
+	    if ( outputBuffered )
+	    {
+		ListT<Chain> pagerSchema;
+		CegoField *pF = schema.First();
+		while ( pF )
+		{
+		    Chain attrDesc = pF->getAttrName() + Chain(":") + Chain(CegoQueryHelper::maxFieldSize(pF));
+		    pagerSchema.Insert(attrDesc);		
+		    pF = schema.Next();		
+		}
+		Pager pag;
+		pag.managePage(cmd, outData, pagerSchema);
+	    }
+	    else
+	    {	    
+		output.tailOut();
+	    }
+	    msg=Chain(count) + Chain(" tuples");
 	}
-	else
-	{	    
-	    output.tailOut();
+	catch ( Exception e )
+	{
+	    pCegoNet->abortQuery();
+	    e.pop(msg);
+	    exitCode=1;
 	}
-	msg=Chain(count) + Chain(" tuples");
     }
     else
     {	      
 	msg = pCegoNet->getMsg();
     }
-    
     return true;
 }
-
+    
 void dumpTableSet(CegoNet *pCegoNet, const Chain& dumpFileName, bool isStructure)
 {
- 
     File* pDF = new File(dumpFileName);
     pDF->open(File::WRITE);
 
     try
     {
-	dumpCounter(pCegoNet, pDF);	
+	cout << "Dumping prolog ..." << endl;
+	dumpProlog(pDF);
+
+	cout << "Dumping counters ..." << endl;
+	dumpCounter(pCegoNet, pDF);
+
+	cout << "Dumping tables ..." << endl;
 	dumpTable(pCegoNet, pDF, isStructure);
-	dumpIndex(pCegoNet, pDF);
+	
+	cout << "Dumping avl trees ..." << endl;
+	dumpAVLtree(pCegoNet, pDF);
+
+	cout << "Dumping btrees ..." << endl;
 	dumpBtree(pCegoNet, pDF);
+       
+	cout << "Dumping aliases ..." << endl;
+	dumpAlias(pCegoNet, pDF);
+
+	cout << "Dumping foreign keys ..." << endl;
 	dumpKey(pCegoNet, pDF);
+
+	cout << "Dumping checks ..." << endl;
 	dumpCheck(pCegoNet, pDF);
-	dumpView(pCegoNet, pDF);
+
+	cout << "Dumping procedures ..." << endl;
 	dumpProcedure(pCegoNet, pDF);
+
+	cout << "Dumping views ..." << endl;
+	dumpView(pCegoNet, pDF);
     }
     catch ( Exception e)
     {
@@ -910,12 +926,16 @@ void dumpTableSet(CegoNet *pCegoNet, const Chain& dumpFileName, bool isStructure
 
     pDF->close();
     delete pDF;
+}
 
+void dumpProlog(File *pDF)
+{
+    pDF->writeChain(Chain("set append on;\n"));
+    pDF->writeChain(Chain("set grace on;\n"));
 }
 
 void dumpCounter(CegoNet *pCegoNet, File *pDF)
 {
-	
     unsigned long long res;
     res = pCegoNet->doQuery(Chain("list counter;"));
         
@@ -935,7 +955,6 @@ void dumpCounter(CegoNet *pCegoNet, File *pDF)
 	cmd=Chain("drop if exists counter ") + counterName + Chain(";\n");
 	pDF->writeChain(cmd);
 
-
 	cmd=Chain("create counter ") + counterName + Chain(";\n");
 	pDF->writeChain(cmd);
 	cmd=Chain("set counter ") + counterName + Chain(" to ") + counterValue + Chain(";\n");
@@ -945,8 +964,7 @@ void dumpCounter(CegoNet *pCegoNet, File *pDF)
 }
 
 void dumpTable(CegoNet *pCegoNet, File *pDF, bool isStructure)
-{
-	
+{	
     unsigned long long res;
     res = pCegoNet->doQuery(Chain("list table;"));
         
@@ -980,12 +998,18 @@ void dumpTable(CegoNet *pCegoNet, File *pDF, bool isStructure)
     
 	ListT<CegoFieldValue> fvl;
 	bool isFirst=true;
+	Chain selectAttr;
 	while ( pCegoNet->fetchData(schema, fvl)  )
 	{
 	    if ( isFirst == false )
+	    {
 		cmd += Chain(", ");
-	    isFirst=false;
-
+		selectAttr += Chain(", ");
+	    }
+	    else
+	    {
+		isFirst=false;
+	    }
 	    CegoFieldValue *pFV = fvl.First();
 	    Chain attrName = pFV->valAsChain();
 	    pFV = fvl.Next();
@@ -993,24 +1017,31 @@ void dumpTable(CegoNet *pCegoNet, File *pDF, bool isStructure)
 	    pFV = fvl.Next();
 	    Chain attrSize = pFV->valAsChain();
 	    pFV = fvl.Next();
-	    Chain attrDef = pFV->valAsChain();
-	    Chain attrDefValue = pFV->toChain();
-	    
+	    Chain attrDim = pFV->valAsChain();
+	    pFV = fvl.Next();
+	    Chain attrDefault = pFV->valAsChain();	   
 	    pFV = fvl.Next();
 	    Chain attrNull = pFV->valAsChain();
 
+	    if ( attrType == Chain(XML_CLOB_VALUE) )
+		selectAttr += Chain("clob2str(") + attrName + Chain(")");
+	    else
+		selectAttr += attrName;
+	    
 	    cmd += attrName + Chain(" ") + attrType;
 	    if ( attrType == Chain(XML_STRING_VALUE)
-		 || attrType == Chain(XML_BIGINT_VALUE)
-		 || attrType == Chain(XML_DECIMAL_VALUE)
-		 || attrType == Chain(XML_FIXED_VALUE))
+		 || attrType == Chain(XML_BIGINT_VALUE) )
 		cmd +=  Chain("(") + attrSize + Chain(")");
-	    if ( attrDef != Chain(XML_NULL_VALUE))
+	    else if ( attrType == Chain(XML_DECIMAL_VALUE) )
+		cmd +=  Chain("(") + attrSize + Chain(",") + attrDim + Chain(")");
+		 
+	    if ( attrDefault != Chain(XML_NULL_VALUE))
 	    {
-		if ( attrType == Chain(XML_STRING_VALUE) )
-		    cmd += Chain(" default ") + attrDefValue;
+		if ( attrType == Chain(XML_STRING_VALUE)
+		     || ( attrType == Chain(XML_DATETIME_VALUE) && attrDefault != Chain("sysdate")) )
+		    cmd += Chain(" default '") + attrDefault + Chain("'");
 		else
-		    cmd += Chain(" default ") + attrDef;
+		    cmd += Chain(" default ") + attrDefault;
 	    }
 	    if ( attrNull == Chain("no"))
 		cmd += Chain(" not null");
@@ -1020,11 +1051,10 @@ void dumpTable(CegoNet *pCegoNet, File *pDF, bool isStructure)
 	cmd += Chain(");\n");
 	pDF->writeChain(cmd);
 
-
 	if ( isStructure == false )
 	{
 	    // dump table content
-	    res = pCegoNet->doQuery(Chain("select * from ") + *pTable + Chain(";"));
+	    res = pCegoNet->doQuery(Chain("select ") + selectAttr + Chain(" from ") + *pTable + Chain(";"));
 	    
 	    schema.Empty();
 	    fvl.Empty();
@@ -1055,30 +1085,28 @@ void dumpTable(CegoNet *pCegoNet, File *pDF, bool isStructure)
     }
 }
 
-void dumpIndex(CegoNet *pCegoNet, File *pDF)
-{
-	
+void dumpAVLtree(CegoNet *pCegoNet, File *pDF)
+{	
     unsigned long long res;
-    res = pCegoNet->doQuery(Chain("list index;"));
+    res = pCegoNet->doQuery(Chain("list avltree;"));
         
     ListT<CegoField> schema;
     pCegoNet->getSchema(schema);
     
-    ListT<Chain> indexList;
+    ListT<Chain> avlList;
     ListT<CegoFieldValue> fvl;
     while ( pCegoNet->fetchData(schema, fvl)  )
     {
 	CegoFieldValue *pFV = fvl.First();
-	Chain indexName = pFV->valAsChain();
-	indexList.Insert(indexName);
+	Chain avlName = pFV->valAsChain();
+	avlList.Insert(avlName);
 	fvl.Empty();	
     }
     
-    Chain *pIndex = indexList.First();
-    while ( pIndex ) 
+    Chain *pAVL = avlList.First();
+    while ( pAVL ) 
     {
-	res = pCegoNet->doQuery(Chain("desc index ") + *pIndex + Chain(";"));
-
+	res = pCegoNet->doQuery(Chain("desc avltree ") + *pAVL + Chain(";"));
 
 	ListT<CegoField> schema;
 	pCegoNet->getSchema(schema);
@@ -1103,15 +1131,15 @@ void dumpIndex(CegoNet *pCegoNet, File *pDF)
 
 	Chain cmd;
 
-	cmd=Chain("drop if exists index ") + *pIndex + Chain(";\n");
+	cmd=Chain("drop if exists avltree ") + *pAVL + Chain(";\n");
 	pDF->writeChain(cmd);
 
 	if ( indexType == Chain(XML_PINDEX_VALUE) )
-	    cmd=Chain("create primary index ");
+	    cmd=Chain("create primary avltree ");
 	else if ( indexType == Chain(XML_UINDEX_VALUE) )
-	    cmd=Chain("create unique index ") + *pIndex; 
+	    cmd=Chain("create unique avltree ") + *pAVL; 
 	else
-	    cmd=Chain("create index ") + *pIndex;
+	    cmd=Chain("create avltree ") + *pAVL;
 
 	cmd += Chain(" on ") + tableName + Chain("(");
 	
@@ -1130,13 +1158,12 @@ void dumpIndex(CegoNet *pCegoNet, File *pDF)
 	cmd += Chain(");\n");
 	pDF->writeChain(cmd);
 
-	pIndex = indexList.Next();
+	pAVL = avlList.Next();
     }
 }
 
 void dumpBtree(CegoNet *pCegoNet, File *pDF)
-{
-	
+{	
     unsigned long long res;
     res = pCegoNet->doQuery(Chain("list btree;"));
         
@@ -1158,7 +1185,6 @@ void dumpBtree(CegoNet *pCegoNet, File *pDF)
     {
 	res = pCegoNet->doQuery(Chain("desc btree ") + *pIndex + Chain(";"));
 
-
 	ListT<CegoField> schema;
 	pCegoNet->getSchema(schema);
     
@@ -1214,8 +1240,7 @@ void dumpBtree(CegoNet *pCegoNet, File *pDF)
 }
 
 void dumpKey(CegoNet *pCegoNet, File *pDF)
-{
-	
+{	
     unsigned long long res;
     res = pCegoNet->doQuery(Chain("list key;"));
         
@@ -1266,7 +1291,6 @@ void dumpKey(CegoNet *pCegoNet, File *pDF)
 		refTable=tableName;
 		refAttrList.Insert(attrName);		
 	    }
-
 	    fvl.Empty();	
 	}
 
@@ -1275,7 +1299,6 @@ void dumpKey(CegoNet *pCegoNet, File *pDF)
 	cmd=Chain("drop if exists foreign key ") + *pKey + Chain(";\n");
 	pDF->writeChain(cmd);
 
-
 	cmd=Chain("alter table ") + keyTable + Chain(" add foreign key ") + *pKey + Chain("(");
 	Chain *pKeyAttr = keyAttrList.First();
 	while ( pKeyAttr )
@@ -1330,7 +1353,6 @@ void dumpCheck(CegoNet *pCegoNet, File *pDF)
 	pCegoNet->getSchema(schema);
     
 	ListT<CegoFieldValue> fvl;
-	bool isFirst=true;
 	while ( pCegoNet->fetchData(schema, fvl) );
 
 	CegoFieldValue *pFV = fvl.First();
@@ -1379,7 +1401,6 @@ void dumpView(CegoNet *pCegoNet, File *pDF)
 	pCegoNet->getSchema(schema);
     
 	ListT<CegoFieldValue> fvl;
-	bool isFirst=true;
 	Chain viewText;
 	while ( pCegoNet->fetchData(schema, fvl)  )
 	{
@@ -1424,7 +1445,6 @@ void dumpProcedure(CegoNet *pCegoNet, File *pDF)
     {
 	res = pCegoNet->doQuery(Chain("show procedure ") + *pProc + Chain(";"));
 
-
 	ListT<CegoField> schema;
 	pCegoNet->getSchema(schema);
     
@@ -1450,6 +1470,79 @@ void dumpProcedure(CegoNet *pCegoNet, File *pDF)
     }
 }
 
+void dumpAlias(CegoNet *pCegoNet, File *pDF)
+{	
+    unsigned long long res;
+    res = pCegoNet->doQuery(Chain("list alias;"));
+        
+    ListT<CegoField> schema;
+    pCegoNet->getSchema(schema);
+    
+    ListT<Chain> aliasList;
+    ListT<CegoFieldValue> fvl;
+    while ( pCegoNet->fetchData(schema, fvl)  )
+    {
+	CegoFieldValue *pFV = fvl.First();
+	Chain aliasName = pFV->valAsChain();
+	aliasList.Insert(aliasName);
+	fvl.Empty();	
+    }
+    
+    Chain *pAlias = aliasList.First();
+    while ( pAlias ) 
+    {
+	res = pCegoNet->doQuery(Chain("desc alias ") + *pAlias + Chain(";"));
+	
+	ListT<CegoField> schema;
+	pCegoNet->getSchema(schema);
+
+	Chain refTable;
+	ListT<Chain> refAttrList;
+	ListT<Chain> aliasAttrList;
+
+	ListT<CegoFieldValue> fvl;
+	while ( pCegoNet->fetchData(schema, fvl)  )
+	{
+	    CegoFieldValue *pFV = fvl.First();
+
+	    refTable = pFV->valAsChain();
+
+	    pFV = fvl.Next();
+	    Chain refAttr = pFV->valAsChain();
+	    pFV = fvl.Next();
+	    Chain aliasAttr = pFV->valAsChain();
+
+	    refAttrList.Insert(refAttr);
+	    aliasAttrList.Insert(aliasAttr);
+	    
+	    fvl.Empty();	
+	}
+
+	Chain cmd;	
+
+	cmd=Chain("drop if exists alias ") + *pAlias + Chain(";\n");
+	pDF->writeChain(cmd);
+
+	cmd=Chain("create alias ") + *pAlias + Chain(" on ") + refTable + Chain("(");
+	Chain *pRefAttr = refAttrList.First();
+	Chain *pAliasAttr = aliasAttrList.First();
+	
+	while ( pRefAttr && pAliasAttr )
+	{
+	    cmd += *pRefAttr + Chain(" as ") + *pAliasAttr;	    
+	    pRefAttr = refAttrList.Next();
+	    pAliasAttr = aliasAttrList.Next();
+	    if ( pRefAttr && pAliasAttr )
+		cmd += Chain(",");
+	}
+	           
+	cmd += Chain(");\n");
+	pDF->writeChain(cmd);
+
+	pAlias = aliasList.Next();
+    }
+}
+
 void loadProfile(const Chain& rcFileName, ListT<ProfileEntry>& pfeList)
 {
     File rcFile(rcFileName);
@@ -1506,8 +1599,7 @@ void loadProfile(const Chain& rcFileName, ListT<ProfileEntry>& pfeList)
 
 	    Chain auth = user + Chain("/") + pwd;
 	    ProfileEntry pfe(pid, serverName, portNo.asInteger(), protocol, tableSet, auth, prompt);
-	    pfeList.Insert(pfe);
-	    
+	    pfeList.Insert(pfe);	    
 	}		
     }
 }
@@ -1539,10 +1631,9 @@ AbortHandler::~AbortHandler()
 } 
 
 void AbortHandler::sigCatch(int sig) 
-{
-    
-    try {
-	
+{    
+    try
+    {	
 	install(SIGINT);
 #ifndef HAVE_MINGW
 	install(SIGPIPE);
@@ -1555,7 +1646,7 @@ void AbortHandler::sigCatch(int sig)
 	    if ( debug )
 		logMode=Chain("debug");
 	    
-	    CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(""), logMode );	
+	    CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(), logMode );	
 	    
 	    try
 	    {
@@ -1583,7 +1674,6 @@ void AbortHandler::sigCatch(int sig)
 	e.pop(msg);
 	cout << "Error : " << msg << endl;
     }
-    
 }
 
 void AbortHandler::setQueryAbort(bool doAbort)
@@ -1674,4 +1764,3 @@ bool ProfileEntry::operator == ( const ProfileEntry& pe)
 	return true;
     return false;
 }
-
