@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -31,6 +31,7 @@
 
 // CEGO INCLUDES
 #include "CegoDatabaseManager.h"
+#include "CegoTableManager.h"
 #include "CegoTypeConverter.h"
 #include "CegoDefs.h"
 #include "CegoXMLdef.h"
@@ -40,10 +41,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-#ifndef _REENTRANT
-#define _REENTRANT    /* basic 3-lines for threads */
-#endif
-
 static ThreadLock dbmLock("DBM");
 extern bool __lockStatOn;
 
@@ -133,7 +130,6 @@ CegoDatabaseManager::ObjectRecord& CegoDatabaseManager::ObjectRecord::operator =
 
 bool CegoDatabaseManager::ObjectRecord::operator == ( const CegoDatabaseManager::ObjectRecord& r)
 {
-
     bool typeMatch=false;
     if ( ( _type == CegoObject::PAVLTREE || _type == CegoObject::UAVLTREE || _type == CegoObject::AVLTREE )
 	 && ( r._type == CegoObject::PAVLTREE || r._type == CegoObject::UAVLTREE || r._type == CegoObject::AVLTREE ) )
@@ -229,11 +225,9 @@ CegoDatabaseManager::CopyRecord& CegoDatabaseManager::CopyRecord::operator = ( c
     _tableSet = cr._tableSet;
     _targetHost = cr._targetHost;
     _mediatorHost = cr._mediatorHost;
-
     _user = cr._user;
     _passwd = cr._passwd;
     _msg = cr._msg;
-
     return (*this);   
 }
 
@@ -264,7 +258,7 @@ CegoDatabaseManager::DbSessionRecord::DbSessionRecord(const Chain& hostName, con
     _tableSet = tableSet;
     _userName = userName;
     _pHandler = pHandler;
-    _lastUsed = Datetime().asInt();
+    _lastUsed = Datetime().asLong();
 }
 
 CegoDatabaseManager::DbSessionRecord::~DbSessionRecord()
@@ -300,26 +294,24 @@ void CegoDatabaseManager::DbSessionRecord::setUsed(bool isUsed)
     _isUsed = isUsed;
 }
 
-int CegoDatabaseManager::DbSessionRecord::getTSLastUsed()
+unsigned long long CegoDatabaseManager::DbSessionRecord::getTSLastUsed()
 {
     return _lastUsed;
 }
 
-void CegoDatabaseManager::DbSessionRecord::setTSLastUsed(int ts)
+void CegoDatabaseManager::DbSessionRecord::setTSLastUsed(unsigned long long ts)
 {
     _lastUsed = ts;
 }
 
 CegoDatabaseManager::DbSessionRecord& CegoDatabaseManager::DbSessionRecord::operator = ( const CegoDatabaseManager::DbSessionRecord& sr)
 {
-
     _hostName = sr._hostName;
     _tableSet = sr._tableSet;
     _userName = sr._userName;
     _pHandler = sr._pHandler;
     _lastUsed = sr._lastUsed;    
     return (*this);
-    
 }
 
 bool CegoDatabaseManager::DbSessionRecord::operator == ( const CegoDatabaseManager::DbSessionRecord& sr)
@@ -335,9 +327,7 @@ bool CegoDatabaseManager::DbSessionRecord::operator == ( const CegoDatabaseManag
 
 CegoDatabaseManager::CegoDatabaseManager(const Chain& xmlDef, const Chain& lckFileName, const Chain& logFile, const Chain& progName, CegoDbHandler::ProtocolType protType) :  CegoBufferPool(xmlDef, logFile, progName)
 {
-
     _protType = protType;
-
     _lckFileName = lckFileName;
 
     File lckFile(_lckFileName);
@@ -364,7 +354,6 @@ CegoDatabaseManager::CegoDatabaseManager(const Chain& xmlDef, const Chain& lckFi
     _nextCopyId=1;
 
     _logConfigured=false;
-
     
     _modId = getModId("CegoDatabaseManager");
 }
@@ -387,15 +376,24 @@ void CegoDatabaseManager::allocateQueryCache(const Chain& tableSet)
 {
     if ( getQueryCacheMode(tableSet) )
     {	
-	int numEntry = getMaxQueryCacheEntry(tableSet);
-	int cacheSize = getMaxQueryCacheSize(tableSet);
+	int maxEntry = getMaxQueryCacheEntry(tableSet);
+	int maxSize = getMaxQueryCacheSize(tableSet);
+	int threshold = getQueryCacheThreshold(tableSet);
+	int hashRange = getQueryCacheHashRange(tableSet);
 
-	int tabSetId = getTabSetId(tableSet);
-	
-	if ( _pQueryCache[tabSetId] )
-	    delete _pQueryCache[tabSetId];
-	
-	_pQueryCache[tabSetId] = new CegoQueryCache(numEntry,cacheSize);
+	if ( maxEntry > 0 && maxSize > 0 && hashRange > 0)
+	{
+	    int tabSetId = getTabSetId(tableSet);
+	    
+	    if ( _pQueryCache[tabSetId] )
+		delete _pQueryCache[tabSetId];
+	    
+	    _pQueryCache[tabSetId] = new CegoQueryCache(tabSetId, maxEntry, maxSize, threshold, hashRange);
+	}
+	else
+	{
+	    log(_modId, Logger::NOTICE, Chain("Query Cache Size/Entry not appropriate, skipping cache allocation"));
+	}
     }
 }
 
@@ -403,21 +401,28 @@ void CegoDatabaseManager::allocateTableCache(const Chain& tableSet)
 {
     if ( getTableCacheMode(tableSet) )
     {
-
-	int numEntry = getMaxTableCacheEntry(tableSet);
-	int cacheSize = getMaxTableCacheSize(tableSet);
-
-	int tabSetId = getTabSetId(tableSet);
-
-	if ( _pTableCache[tabSetId] )
-	    delete _pTableCache[tabSetId];
-	_pTableCache[tabSetId] = new CegoTableCache(numEntry,cacheSize, this);
+	Chain cacheFilter = getTableCacheFilter(tableSet);
+	int maxSize = getTableCacheMaxSize(tableSet);
+	int maxEntry = getTableCacheMaxEntry(tableSet);
+	int hashRange = getTableCacheHashRange(tableSet);
+	
+	if (  maxSize > 0 && maxEntry > 0 && hashRange > 0 )
+	{
+	    int tabSetId = getTabSetId(tableSet);
+	    
+	    if ( _pTableCache[tabSetId] )
+		delete _pTableCache[tabSetId];
+	    _pTableCache[tabSetId] = new CegoTableCache(tabSetId, cacheFilter, maxSize, maxEntry, hashRange, this);
+	}
+	else
+	{
+	    log(_modId, Logger::NOTICE, Chain("Table Cache Size/Entry not appropriate, skipping cache allocation"));
+	}
     }
 }
 
 void CegoDatabaseManager::releaseQueryCache(const Chain& tableSet)
 {
-
     int tabSetId = getTabSetId(tableSet);
     
     if ( _pQueryCache[tabSetId] )
@@ -447,7 +452,7 @@ void CegoDatabaseManager::cleanCache(int tabSetId, CegoObject::ObjectType objTyp
     
     if ( _pTableCache[tabSetId] && objType == CegoObject::TABLE )
     {
-	_pTableCache[tabSetId]->invalidate(tabSetId, objName);
+	_pTableCache[tabSetId]->invalidate(objName);
     }
 }
 
@@ -482,6 +487,22 @@ void CegoDatabaseManager::beat()
     lckFile.close();    
 }
 
+void CegoDatabaseManager::checkTableSetRunState(int tabSetId)
+{
+    Chain runState = getTableSetRunState(tabSetId);
+
+    if ( runState != Chain(XML_ONLINE_VALUE)
+	 && runState != Chain(XML_BACKUP_VALUE)
+	 && runState != Chain(XML_RECOVERY_VALUE)
+	 && runState != Chain(XML_CHECKPOINT_VALUE) )
+    {
+	Chain msg = Chain("Tableset ") + getTabSetName(tabSetId) + Chain(" not online ( run state is ") + runState + Chain(" )");
+
+	throw Exception(EXLOC, msg);
+    }
+}
+
+
 void CegoDatabaseManager::PR()
 {
     dbmLock.readLock(DBM_LOCKTIMEOUT);
@@ -602,8 +623,7 @@ Element* CegoDatabaseManager::getCopyInfo()
     PR();
     CopyRecord *pCR = _copyList.First();
     while ( pCR )
-    {
-	
+    {	
 	Element *pN = new Element(XML_COPY_ELEMENT);
 
 	pN->setAttribute(XML_CID_ATTR, pCR->getId());
@@ -621,14 +641,17 @@ Element* CegoDatabaseManager::getCopyInfo()
     return pCopyInfo;
 }
 
-void CegoDatabaseManager::useObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type, ObjectUseMode mode, unsigned long long tid)
+void CegoDatabaseManager::useObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type, ObjectUseMode mode, CegoTableManager *pTabMng)
 {
+    // In SHARED mode, an object can be use in parallel by several threads. Normally, this is done, if the object is accessed read only.
 
-    // In SHARED mode, an object could be use in parallel by several threads. Normally, this is done, if the object is accessed read only.
-
+    unsigned long long tid = pTabMng->getThreadId();    
+	
+    // we fist check, if corresponding tableset is really not offline 
+    checkTableSetRunState(tabSetId);
+    
     if ( mode == SHARED )
     {
-
         PW();
         ObjectRecord *pOR = _objList.Find(ObjectRecord(tabSetId, objName, type));
         if ( pOR == 0 )
@@ -637,7 +660,14 @@ void CegoDatabaseManager::useObject(int tabSetId, const Chain& objName, CegoObje
 	    Chain msg = Chain("Cannot access object <") + objName + Chain(">");
             throw Exception(EXLOC, msg); 
         }
-
+	
+	if ( pOR->numUsed() > MAX_OBJECT_USAGE )
+	{
+            V();
+	    Chain msg = Chain("Usage exceeded for <") + objName + Chain(">");
+            throw Exception(EXLOC, msg); 	    
+	}
+	
 	if ( pOR->getTid() != 0 && pOR->getTid() == tid ) 
 	{
 	    pOR->incUsed();
@@ -653,11 +683,22 @@ void CegoDatabaseManager::useObject(int tabSetId, const Chain& objName, CegoObje
 
 		if ( numTries > 0 )
 		{
-		    log(_modId, Logger::NOTICE, Chain("Shared lock delay on ") + objName + Chain(" ( ") + Chain(numTries) + Chain(" tries )"));
-		    Sleeper ns;
-		    ns.milliSleep(DBM_LOCKDELAY);
-		}
+		    if ( numTries > 1 )
+		    {
+			log(_modId, Logger::NOTICE, Chain("Repeated shared lock delay on ") + objName + Chain(" ( ") + Chain(numTries) + Chain(" tries )"));
+		    }
 
+		    checkTableSetRunState(tabSetId);
+		    		    
+		    Sleeper ns;
+		    int sec = DBM_LOCKDELAY / 1000;
+		    if ( sec > 0 )
+			ns.secSleep(sec);
+		    int msec = DBM_LOCKDELAY % 1000;
+		    if ( msec > 0 )
+			ns.milliSleep(msec);
+		}
+		
 		PW();
 
 		pOR = _objList.Find(ObjectRecord(tabSetId, objName, type));
@@ -696,62 +737,78 @@ void CegoDatabaseManager::useObject(int tabSetId, const Chain& objName, CegoObje
             throw Exception(EXLOC, msg);
         }
 	
-	if ( pOR->getMode() == SHARED && pOR->numUsed() == 0 )
-	{
-	    pOR->setMode(mode);
-	    pOR->setTid(tid);
+	if ( pOR->getTid() != 0 && pOR->getTid() == tid ) 
+	{	    
+	    pOR->incUsed();
+	    V();
 	}
 	else
-	{	    
-	    int numTries=0;
-	    while ( pOR && pOR->numUsed() != 0 && numTries < DBM_MAXLOCKTRIES )
+	{
+	    if ( pOR->getMode() == SHARED && pOR->numUsed() == 0 )
 	    {
-
-		V();
-
-		if ( numTries > 0 )
-		{
-		    log(_modId, Logger::NOTICE, Chain("Exclusive write lock delay on ") + objName + Chain(" ( ") + Chain(numTries) + Chain(" tries )"));
-		    Sleeper ns;
-		    ns.milliSleep(DBM_LOCKDELAY);
-		    		    
-		}
-		
-		PW();
-
-		pOR = _objList.Find(ObjectRecord(tabSetId, objName, type));
-		if ( pOR == 0 )
+		pOR->setMode(mode);
+		pOR->setTid(tid);
+	    }
+	    else
+	    {	    
+		int numTries=0;
+		while ( pOR && pOR->numUsed() != 0 && numTries < DBM_MAXLOCKTRIES )
 		{
 		    V();
-		    Chain msg = Chain("Cannot access object ") + objName;
-		    throw Exception(EXLOC, msg);
+		    
+		    if ( numTries > 0 )
+		    {
+			if ( numTries > 1 )
+			{
+			    log(_modId, Logger::NOTICE, Chain("Repeated exclusive write lock delay on ") + objName + Chain(" ( ") + Chain(numTries) + Chain(" tries )"));
+			}
+
+			checkTableSetRunState(tabSetId);
+			
+			Sleeper ns;			
+			int sec = DBM_LOCKDELAY / 1000;
+			if ( sec > 0 )
+			    ns.secSleep(sec);
+			int msec = DBM_LOCKDELAY % 1000;
+			if ( msec > 0 )
+			    ns.milliSleep(msec);
+		    }
+		    
+		    PW();
+
+		    pOR = _objList.Find(ObjectRecord(tabSetId, objName, type));
+		    if ( pOR == 0 )
+		    {
+			V();
+			Chain msg = Chain("Cannot access object ") + objName;
+			throw Exception(EXLOC, msg);
+		    }
+		    
+		    numTries++;		   
 		}
 
-		numTries++;
-
+		if ( numTries == DBM_MAXLOCKTRIES )
+		{
+		    V();
+		    Chain msg = Chain("Access timeout on object ") + objName;
+		    throw Exception(EXLOC, msg); 
+		}
+		pOR->setMode(mode);
+		pOR->setTid(tid);		
 	    }
-
-	    if ( numTries == DBM_MAXLOCKTRIES )
-	    {
-		V();
-		Chain msg = Chain("Access timeout on object ") + objName;
-		throw Exception(EXLOC, msg); 
-	    }	       		
-
-	    pOR->setMode(mode);
-	    pOR->setTid(tid);
-
+	    
+	    pOR->incUsed();
+	    V();
 	}
-
-        pOR->incUsed();
-        V();
     }
     return;
 }
 
-void CegoDatabaseManager::unuseObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type)
-{
-    
+void CegoDatabaseManager::unuseObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type, unsigned long long tid )
+{        
+    // we fist check, if corresponding tableset is really not offline 
+    checkTableSetRunState(tabSetId);
+
     PW();
     ObjectRecord *pOR = _objList.Find(ObjectRecord(tabSetId, objName, type));
 
@@ -762,12 +819,25 @@ void CegoDatabaseManager::unuseObject(int tabSetId, const Chain& objName, CegoOb
 	throw Exception(EXLOC, msg);
 	return;
     }
-    
-    pOR->decUsed();
-    
-    pOR->setMode(SHARED);
-    pOR->setTid(0);
-    
+
+    // we just unuse object, if tid not specified or lockmode of object is SHARED or if transaction owns object lockmode of obejct id EXCLUSIVE_WRITE 
+    if ( ( tid == pOR->getTid() && pOR->getMode() == EXCLUSIVE_WRITE ) || tid == 0 || pOR->getMode() == SHARED )
+    {
+	pOR->decUsed();
+	
+	if ( pOR->numUsed() == 0 )
+	{
+	    pOR->setMode(SHARED);
+	    pOR->setTid(0);
+	}
+    }
+    else
+    {
+        V();
+	Chain msg = Chain("Table ") + objName + Chain(" not owned by thread");
+	throw Exception(EXLOC, msg);
+	return;       
+    }    
     V();    
 }
 
@@ -785,8 +855,30 @@ void CegoDatabaseManager::removeObject(int tabSetId, const Chain& objName, CegoO
     V();
 }
 
+void CegoDatabaseManager::removeAllObjects(int tabSetId) 
+{ 
+    PW(); 
+    ObjectRecord *pOR = _objList.First(); 
+    while ( pOR ) 
+    { 
+	if ( pOR->getTabSetId() == tabSetId ) 
+	{ 
+	    _objList.Remove(*pOR); 
+	    pOR = _objList.First(); 
+	} 
+	else 
+	{ 
+	    pOR = _objList.Next(); 
+	} 
+    } 
+    V(); 
+} 
+
 bool CegoDatabaseManager::objectExists(int tabSetId, const Chain& objName, CegoObject::ObjectType type)
 {
+    // we fist check, if corresponding tableset is really not offline
+    checkTableSetRunState(tabSetId);
+
     PR();
     ObjectRecord *pTR = _objList.Find(ObjectRecord(tabSetId, objName, type));
     V();    
@@ -867,7 +959,6 @@ void CegoDatabaseManager::getDBMLockStat(Chain& lockName, int& lockCount, unsign
 {
     lockName = dbmLock.getId();
     lockCount= dbmLock.numLockTry();
-
     numRdLock = dbmLock.numReadLock();
     numWrLock = dbmLock.numWriteLock();
     sumRdDelay = 0;
@@ -898,7 +989,7 @@ CegoDistDbHandler* CegoDatabaseManager::allocateSession(const Chain& hostName, c
 	    {
 		pSR->setUsed(true);
 		Datetime dt;
-		pSR->setTSLastUsed( dt.asInt() );
+		pSR->setTSLastUsed( dt.asLong() );
 		
 		V();
 		
@@ -908,10 +999,8 @@ CegoDistDbHandler* CegoDatabaseManager::allocateSession(const Chain& hostName, c
 	}
 	
 	// create new session
-
 	pSession = createSession(hostName, tableSet, userName, password);
 	_dbSessionList.Insert( DbSessionRecord(hostName, tableSet, userName, pSession));
-
     }
     catch ( Exception e )
     {
@@ -956,7 +1045,7 @@ void CegoDatabaseManager::cleanSession(int lifetime)
     DbSessionRecord *pSR = _dbSessionList.First();
     while ( pSR )
     {
-	if ( pSR->getTSLastUsed() <  ( dt.asInt() - lifetime )  )
+	if ( pSR->getTSLastUsed() <  ( dt.asLong() - lifetime )  )
 	{
 	    if ( pSR->isUsed() == false )
 	    {
@@ -988,7 +1077,7 @@ Element* CegoDatabaseManager::getSessionInfo(int lifetime)
 	    pN->setAttribute(XML_ISUSED_ATTR, XML_FALSE_VALUE);
 
 	Datetime dt;
-	pN->setAttribute(XML_TTL_ATTR, Chain(pSR->getTSLastUsed() + lifetime - dt.asInt()));	
+	pN->setAttribute(XML_TTL_ATTR, Chain(pSR->getTSLastUsed() + (unsigned long long)lifetime - dt.asLong()));	
 
 	pSessionInfo->addContent(pN);
 
@@ -1003,7 +1092,7 @@ CegoDistDbHandler* CegoDatabaseManager::createSession(const Chain& hostName, con
     int portNo;
     getDataPort(portNo);
     
-    Net n( NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN );
+    Net n( NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN, getMaxSendLen() );
     NetHandler* pN = 0;
     CegoDistDbHandler *pSH = 0;
 
@@ -1021,11 +1110,9 @@ CegoDistDbHandler* CegoDatabaseManager::createSession(const Chain& hostName, con
 	log(_modId, Logger::DEBUG, Chain("Using user ") + userName + Chain("/") + password); 
 #endif
 	pSH->requestSession(tableSet, userName, password, false);
-
     }
     catch ( Exception e ) 
     {
-	
 	if ( pSH )
 	{
 	    delete pSH;
@@ -1100,7 +1187,6 @@ bool CegoDatabaseManager::verifyJDBC(const Chain& user)
 
 bool CegoDatabaseManager::verifyAccess(const int tabSetId, const Chain& objName, CegoObject::ObjectType type, CegoXMLSpace::AccessMode mode, const Chain& user)
 {
-
     SetT<Chain> roleSet;
 
     getRoleSet(user, roleSet);
@@ -1108,8 +1194,7 @@ bool CegoDatabaseManager::verifyAccess(const int tabSetId, const Chain& objName,
 
     Chain *pRole = roleSet.First();
     while ( pRole ) 
-    {
-	
+    {	
 	Chain objPattern = objName;
 
 	if ( matchRole( *pRole, tableSet, objPattern, mode ) )
@@ -1121,7 +1206,6 @@ bool CegoDatabaseManager::verifyAccess(const int tabSetId, const Chain& objName,
 
 void CegoDatabaseManager::initLogFiles(const Chain& tableSet, bool overwrite)
 {
-
     ListT<Chain> lfList;
     ListT<int> sizeList;
     ListT<Chain> statusList;
@@ -1134,8 +1218,7 @@ void CegoDatabaseManager::initLogFiles(const Chain& tableSet, bool overwrite)
 
     bool isFirst = true;
     while ( pLog ) 
-    {
-	
+    {	
 	if ( isFirst )
 	    setLogFileStatus(tableSet, *pLog, XML_ACTIVE_VALUE);		
 	else
@@ -1159,8 +1242,7 @@ void CegoDatabaseManager::initLogFiles(const Chain& tableSet, bool overwrite)
 	initLog(tabSetId, *pSize);
 	
 	pLog = lfList.Next();
-	pSize = sizeList.Next();
-	
+	pSize = sizeList.Next();	
     }
 }
 
@@ -1173,33 +1255,31 @@ void CegoDatabaseManager::releaseLogFiles(const Chain& tableSet, bool waitForArc
     int tabSetId = getTabSetId(tableSet);
     getLogFileInfo(tableSet, lfList, sizeList, statusList);
 
-    Chain *pLog = lfList.First();
-    Chain *pStatus = statusList.First();
+    Chain *pLog = lfList.First(); 
+    Chain *pStatus = statusList.First(); 
 
     while ( pLog && pStatus ) 
-    {	
-	if ( *pStatus == Chain(XML_ACTIVE_VALUE) )
-	{
-	    setLogFile(tabSetId, *pLog, true);
-	    unsigned long long minlsn = getMinLSN(tabSetId);
-	    if ( minlsn > 0 )
-	    {
-		log(_modId, Logger::NOTICE, Chain("Releasing logfile ") + *pLog + Chain(" LSN=") + Chain(minlsn));
-		setLogFileStatus(tableSet, *pLog, XML_OCCUPIED_VALUE);
-	    }
-	}
-	pStatus = statusList.Next();
-	pLog = lfList.Next();	
+    { 
+        if ( *pStatus == Chain(XML_ACTIVE_VALUE) && File::exists(*pLog) ) 
+        { 
+            setLogFile(tabSetId, *pLog, true); 
+            unsigned long long minlsn = getMinLSN(tabSetId); 
+            if ( minlsn > 0 ) 
+            { 
+                log(_modId, Logger::NOTICE, Chain("Releasing logfile ") + *pLog + Chain(" LSN=") + Chain(minlsn)); 
+                setLogFileStatus(tableSet, *pLog, XML_OCCUPIED_VALUE); 
+            } 
+        } 
+        pStatus = statusList.Next(); 
+        pLog = lfList.Next(); 
     }
-
+ 
     if ( waitForArch )
     {
-
 	bool notArchived = true;
 
 	while ( notArchived )
 	{
-
 	    log(_modId, Logger::NOTICE, Chain("Waiting for archive ... "));	    
 
 	    ListT<Chain> lfList;
@@ -1210,14 +1290,16 @@ void CegoDatabaseManager::releaseLogFiles(const Chain& tableSet, bool waitForArc
 	    
 	    notArchived = false;
 
-	    Chain *pStatus = statusList.First();
-	    while ( pStatus ) 
-	    {	
-		if ( *pStatus != Chain(XML_FREE_VALUE) )
-		    notArchived = true;
-		pStatus = statusList.Next();
-	    }
-
+	    Chain *pStatus = statusList.First(); 
+            Chain *pLog = lfList.First(); 
+            while ( pLog && pStatus ) 
+            { 
+                if ( *pStatus != Chain(XML_FREE_VALUE) && File::exists(*pLog) ) 
+                    notArchived = true; 
+                pStatus = statusList.Next(); 
+                pLog = lfList.Next(); 
+            } 
+	    
 	    lfList.Empty();
 	    sizeList.Empty();
 	    statusList.Empty();
@@ -1227,3 +1309,98 @@ void CegoDatabaseManager::releaseLogFiles(const Chain& tableSet, bool waitForArc
 	}
     }
 }
+
+CegoDbHandler::ProtocolType CegoDatabaseManager::getProtType() const
+{
+    return _protType;
+}
+
+bool CegoDatabaseManager::getQCLockStat(int tabSetId, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay)
+{
+    if ( _pQueryCache[tabSetId] )
+    {
+	_pQueryCache[tabSetId]->getQCLockStat(lockCount, numRdLock, numWrLock, sumRdDelay, sumWrDelay);
+	return true;
+    }
+    return false;
+}
+
+void CegoDatabaseManager::getAggQCLockStat(Chain& lockGroup, int& numLock, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay)
+{
+    lockGroup = Chain("QCLCK");
+    
+    lockCount = 0;
+    numRdLock = 0;
+    numWrLock = 0;
+    sumRdDelay = 0;
+    sumWrDelay = 0;
+    numLock = 0;
+   
+    for ( int i=0; i<TABMNG_MAXTABSET; i++)
+    {
+	if ( _pQueryCache[i] )
+	{
+	    int qcLockCount;
+	    unsigned long long qcNumRdLock;
+	    unsigned long long qcNumWrLock;
+	    unsigned long long qcSumRdDelay;
+	    unsigned long long qcSumWrDelay;
+	    
+	    _pQueryCache[i]->getQCLockStat(qcLockCount, qcNumRdLock, qcNumWrLock, qcSumRdDelay, qcSumWrDelay);
+	    
+	    numLock++;
+	    
+	    lockCount += qcLockCount;
+	    numRdLock += qcNumRdLock;
+	    numWrLock += qcNumWrLock;
+	    
+	    sumRdDelay += qcSumRdDelay;
+	    sumWrDelay += qcSumWrDelay;
+	}
+    }
+}
+
+bool CegoDatabaseManager::getTCLockStat(int tabSetId, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay)
+{
+    if ( _pTableCache[tabSetId] )
+    {
+	_pTableCache[tabSetId]->getTCLockStat(lockCount, numRdLock, numWrLock, sumRdDelay, sumWrDelay);
+	return true;
+    }
+    return false;
+}
+
+void CegoDatabaseManager::getAggTCLockStat(Chain& lockGroup, int& numLock, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay)
+{
+    lockGroup = Chain("TCLCK");
+    
+    lockCount = 0;
+    numRdLock = 0;
+    numWrLock = 0;
+    sumRdDelay = 0;
+    sumWrDelay = 0;
+    numLock = 0;
+
+    for ( int i=0; i<TABMNG_MAXTABSET; i++)
+    {
+	if ( _pTableCache[i] )
+	{
+	    int qcLockCount;
+	    unsigned long long qcNumRdLock;
+	    unsigned long long qcNumWrLock;
+	    unsigned long long qcSumRdDelay;
+	    unsigned long long qcSumWrDelay;
+	    
+	    _pTableCache[i]->getTCLockStat(qcLockCount, qcNumRdLock, qcNumWrLock, qcSumRdDelay, qcSumWrDelay);
+	    
+	    numLock++;
+	    
+	    lockCount += qcLockCount;
+	    numRdLock += qcNumRdLock;
+	    numWrLock += qcNumWrLock;
+	    
+	    sumRdDelay += qcSumRdDelay;
+	    sumWrDelay += qcSumWrDelay;
+	}
+    }
+}
