@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -56,6 +56,8 @@ class CegoDatabaseManager : public CegoBufferPool {
     
     void beat();
 
+    void checkTableSetRunState(int tabSetId);
+    
     void startRecovery(const Chain& tableSet);
 
     void startCopy(const Chain& tableSet, 
@@ -75,10 +77,13 @@ class CegoDatabaseManager : public CegoBufferPool {
     void setCopyStatus(int id, const Chain& msg);
 
     void addObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type);
-    void useObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type, ObjectUseMode mode, unsigned long long tid = 0 );
-    void unuseObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type);
-    void removeObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type);
 
+    void useObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type, ObjectUseMode mode, CegoTableManager *pTabMng);
+
+    void unuseObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type, unsigned long long tid = 0 );
+    void removeObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type);
+    void removeAllObjects(int tabSetId);
+    
     bool objectExists(int tabSetId, const Chain& objName, CegoObject::ObjectType type);
     
     void printObjectList();
@@ -115,11 +120,19 @@ class CegoDatabaseManager : public CegoBufferPool {
 
     void initLogFiles(const Chain& tableSet, bool overwrite);
     void releaseLogFiles(const Chain& tableSet, bool waitForArchive);
+
+    CegoDbHandler::ProtocolType getProtType() const;
+
+    bool getQCLockStat(int tabSetId, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay);
+
+    void getAggQCLockStat(Chain& lockGroup, int& numLock, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay);
+
+    bool getTCLockStat(int tabSetId, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay);
+
+    void getAggTCLockStat(Chain& lockGroup, int& numLock, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay);
     
-
 private:
 
-
     CegoDistDbHandler* createSession(const Chain& hostName, const Chain& tableSet,
 				     const Chain& userName, const Chain& password);
 
@@ -229,12 +242,11 @@ private:
 
 	bool isUsed() const;
 	void setUsed(bool isUsed);
-	int getTSLastUsed();
-	void setTSLastUsed(int ts);
+	unsigned long long getTSLastUsed();
+	void setTSLastUsed(unsigned long long ts);
 
 	CegoDistDbHandler* getDbHandler() const;
 
-
 	DbSessionRecord& operator = ( const DbSessionRecord& sr);
         bool operator == ( const DbSessionRecord& sr);
 	
@@ -249,7 +261,7 @@ private:
 	Chain _hostName;
 	Chain _tableSet;
 	Chain _userName;
-	int _lastUsed;
+	unsigned long long _lastUsed;
 	bool _isUsed;
 	CegoDistDbHandler* _pHandler;
     };
@@ -283,9 +295,4 @@ private:
     
     unsigned long _modId;
 };
-
 #endif
-
-
-
-
