@@ -6,7 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke
 //               
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -27,6 +27,7 @@
 #include <lfcbase/Exception.h>
 #include <lfcbase/Sleeper.h>
 #include <lfcbase/ThreadLock.h>
+#include <lfcbase/Datetime.h>
 
 #include "CegoXMLdef.h"
 #include "CegoDefs.h"
@@ -36,7 +37,11 @@
 #include <string.h>
 #include <stdlib.h>
 
+static ThreadLock histLock("QUERYHIST");
+
 extern Chain __dateTimeFormat;
+extern char __quoteEscapeFlag;
+extern bool __lockStatOn;
 
 CegoDbThread::CegoDbThread() : Thread()
 {
@@ -47,20 +52,66 @@ CegoDbThread::CegoDbThread(CegoDbThreadPool *pPool, CegoDatabaseManager *pDBMng,
     _pPool = pPool;
     _pDBMng = pDBMng;
     _protType = protType;
+
+    histLock.init(LCKMNG_LOCKWAITDELAY, __lockStatOn);
+
+    _pTabMng = new CegoDistManager(_pDBMng);
+    _pPA = new CegoAction(_pTabMng, _pPool);
+
+    _pTim = new NanoTimer();
+    _pCost = new NanoTimer();
+    _qid = 0;
+    
     _modId = _pDBMng->getModId("CegoDbThread");
     _errorCode = 0;
-
 }
 
 CegoDbThread::~CegoDbThread()
 {
     delete _pPA;
     delete _pTabMng;
+    delete _pTim;
+    delete _pCost;
 }
 
-const Chain& CegoDbThread::lastAction() const
+ListT<CegoDbThread::QueryEntry> CegoDbThread::getLastQuery() const
 {
-    return _lastAction;
+    // we have to invert query list to indicate last query first
+    // since getHistory is not performance critical, we do it here
+    ListT<QueryEntry> history;
+
+    P();
+
+    for ( int i=_queryList.Size()-1; i>=0; i--)
+	history.Insert(_queryList[i]);
+    V();
+    
+    return history;
+}
+
+ListT<CegoDbThread::QueryEntry> CegoDbThread::getCostQuery() const
+{
+    // we have to invert query list to indicate last query first
+    // since getHistory is not performance critical, we do it here
+    ListT<QueryEntry> costList;
+    StackT<QueryEntry> costStack;
+    
+    P();
+
+    QueryEntry *pEntry = _costTree.First();
+    while ( pEntry )
+    {
+	costStack.Push(*pEntry);
+	pEntry = _costTree.Next();
+    }
+    V();
+    
+    QueryEntry stackEntry;
+
+    while ( costStack.Pop(stackEntry) )
+	costList.Insert(stackEntry);
+    
+    return costList;
 }
 
 unsigned long long CegoDbThread::allocatedSortArea() const
@@ -89,7 +140,6 @@ void CegoDbThread::addReloadRequest(int tabSetId)
 
 void CegoDbThread::checkReloadRequest()
 {   
-
     _pPool->P(_idx);
     int *pTS = _loadList.First();
     if ( pTS )
@@ -102,7 +152,6 @@ void CegoDbThread::checkReloadRequest()
 
 	try
 	{
-
 	    loadObjects(ts);
 	}
 	catch ( Exception e )
@@ -114,7 +163,6 @@ void CegoDbThread::checkReloadRequest()
 	}
 
 	_pTabMng->enableAuth();
-
     }
     else
     {
@@ -128,16 +176,16 @@ void* CegoDbThread::job(void* arg)
     {
 	_idx = *(unsigned long long*)arg;
 	
-	_pTabMng = new CegoDistManager(_pDBMng);
-	_pPA = new CegoAction(_pTabMng, _pPool);
+	// _pTabMng = new CegoDistManager(_pDBMng);
+	// _pPA = new CegoAction(_pTabMng, _pPool);
 	
-	_pTim = new NanoTimer();
-
 	_pTabMng->setPoolSyncInfo(_pPool, _idx);
 	_pTabMng->setThreadId(getTid());
 	_pPool->setTid(_idx, getTid());
 	_pPool->setThreadState(_idx, CegoDbThreadPool::READY);
-	
+
+	int queueDelay = _pDBMng->getQueueDelay();
+
 	while ( ! _pPool->isTerminated() )
 	{
 	    try
@@ -145,24 +193,22 @@ void* CegoDbThread::job(void* arg)
 		_pTim->reset();
 		_pTim->start();
 		
-		_pRequest = _pPool->nextRequest();
-		
-		if ( _pRequest ) 
+		NetHandler* pRequest = _pPool->nextRequest();
+
+		if ( pRequest ) 
 		{
-		    
 #ifdef CGDEBUG
 		    _pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Serving service request"));
 #endif
-
 		    _pPool->setState(_idx, CegoDbThreadPool::CONNECTED);
 		    _pPool->incNumRequest(_idx);
 		    
 		    _pDBMng->increaseActiveDbThread();
 		    
 		    CegoDistDbHandler *pSH = 0;
-		    try {
-			
-			pSH = new CegoDistDbHandler(_pRequest, _protType, _pDBMng); 
+		    try
+		    {	       	
+			pSH = new CegoDistDbHandler(pRequest, _protType, _pDBMng); 
 			serveSession(pSH);
 			delete pSH;
 		    }
@@ -182,18 +228,15 @@ void* CegoDbThread::job(void* arg)
 			}
 			
 			_pDBMng->log(_modId, Logger::LOGERR, Chain("Thread ") + Chain(_idx) + Chain(" : DB session aborted : ") + exep);
-			
 		    }
 		    
 		    // in any case, rollback any open transactions
 		    _pTabMng->rollbackDistTransaction(_pPA->getTableSet());
 		    
-		    _pDBMng->decreaseActiveDbThread();
-		    
+		    _pDBMng->decreaseActiveDbThread();		    
 #ifdef CGDEBUG
 		    _pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Service request finished"));
 #endif
-
 		    _pPool->setState(_idx, CegoDbThreadPool::READY);
 		    
 		    if ( _pTabMng->isAborted() )
@@ -206,26 +249,32 @@ void* CegoDbThread::job(void* arg)
 		    _pTabMng->setAppend(false);
 		    _pTabMng->setAutoCommit(true);
 		   
-		    delete _pRequest;
+		    delete pRequest;
 		}
 		else
 		{
 		    Sleeper s;
-		    s.nanoSleep(NETMNG_QUEUE_DELAY);
-
+		    s.microSleep(queueDelay);
 		}
 
-		checkReloadRequest();
-
 		_pTim->stop();
 		_pPool->addThreadIdle(_idx, _pTim->getSum());
-		
+
+		checkReloadRequest();
 	    }
 	    catch ( Exception e)
 	    {
 		Chain msg;
-		e.pop(msg);
-		_pDBMng->log(_modId, Logger::LOGERR, Chain("Thread ") + Chain(_idx) + Chain(" : ") + msg);
+		Chain module;
+		int line;	
+		Chain exep;
+		
+		while ( e.pop(module, line, msg) )
+		{
+		    exep += Chain("\n\t") + module + Chain("(") +  Chain(line) + Chain(") : ") + msg;
+		}
+		
+		_pDBMng->log(_modId, Logger::LOGERR, Chain("Thread ") + Chain(_idx) + Chain(" :") + exep);
 	    }
 	}
     }
@@ -241,7 +290,6 @@ void* CegoDbThread::job(void* arg)
 
 void CegoDbThread::loadObjects(int tabSetId)
 {
-
     Chain tableSet = _pDBMng->getTabSetName(tabSetId);
 
     _pPA->setTableSet(tableSet);
@@ -255,43 +303,12 @@ void CegoDbThread::loadObjects(int tabSetId)
 #ifdef CGDEBUG
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Compiling procedure ") + *pProcName);
 #endif
-	
-	CegoProcObject po;
-	_pTabMng->getObject(tabSetId, *pProcName, CegoObject::PROCEDURE, po);
-	
-	Chain loadString = Chain("load ") + po.getProcText();
-	
+	_pTabMng->reloadProcedure(tabSetId, *pProcName);
+
 #ifdef CGDEBUG
-	_pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Loading <<<  ") + loadString + Chain(">>>"));
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Procedure ") + *pProcName + Chain(" added"));	
 #endif
 
-	// cout << "Loading procedure " << loadString << endl;
-	
-	try
-	{
-	    _pPA->cleanUp();
-	    _pPA->setCommandChain(loadString);
-	    _pPA->parse();
-	    
-	    CegoProcedure* pProc = _pPA->getProcedure();
-	    _pTabMng->addCompProcedure(tabSetId, pProc);
-	    
-#ifdef CGDEBUG
-	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Procedure ") + *pProcName + Chain(" added"));
-#endif
-
-
-	}
-	catch ( Exception e ) 
-	{
-
-	    Chain msg;
-	    e.pop(msg);	    
-	    _pDBMng->log(_modId, Logger::LOGERR, Chain("Thread ") + Chain(_idx) + Chain(" procedure load error : ") + msg);
-	    _pPA->cleanUp();
-	}
-
-	
 	pProcName = procList.Next();
     }
     
@@ -304,49 +321,50 @@ void CegoDbThread::loadObjects(int tabSetId)
 #ifdef CGDEBUG
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Compiling view ") + *pViewName);
 #endif
-	CegoViewObject vo;
-	_pTabMng->getObject(tabSetId, *pViewName, CegoObject::VIEW, vo);
-	
-	Chain loadString = Chain("load ") + vo.getViewStmt();
-	
-#ifdef CGDEBUG
-	_pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Loading <<<  ") + loadString + Chain(">>>"));
-#endif
-	
-	
-	_pPA->cleanUp();
-	_pPA->setCommandChain(loadString);
-	_pPA->parse();
-	
-	CegoSelect* pSelect = _pPA->getSelect();
-	
-	CegoView *pView = new CegoView(*pViewName, pSelect);
-	
-	_pTabMng->addCompView(tabSetId, pView);
-	
+
+	_pTabMng->reloadView(tabSetId, *pViewName);
+		
 #ifdef CGDEBUG
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : View ") + *pViewName + Chain(" added"));	
 #endif
 	pViewName = viewList.Next();
     }
+
+    ListT<Chain> triggerList;
+    _pTabMng->getObjectList(tabSetId, CegoObject::TRIGGER, triggerList);
+    
+    Chain *pTriggerName = triggerList.First();
+    while ( pTriggerName )
+    {
+#ifdef CGDEBUG
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Compiling trigger ") + *pTriggerName);
+#endif
+
+	_pTabMng->reloadTrigger(tabSetId, *pTriggerName);
+
+#ifdef CGDEBUG
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Trigger ") + *pTriggerName + Chain(" added"));	
+#endif
+
+	pTriggerName = triggerList.Next();
+    }
 }
 
 void CegoDbThread::unloadObjects(int tabSetId)
 {
     try
-    {
+    {	
 	_pTabMng->getTransactionManager()->release(tabSetId);
 	_pTabMng->removeAllComp(tabSetId);
     }
     catch ( Exception e )
     {
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot unload objects"), e);
     }
 }
 
 void CegoDbThread::invalidateObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type)
 {
-    
     if ( type == CegoObject::VIEW )
     {
 	_pTabMng->removeCompView(tabSetId, objName);
@@ -363,7 +381,6 @@ void CegoDbThread::invalidateObject(int tabSetId, const Chain& objName, CegoObje
 
 void CegoDbThread::serveSession(CegoDistDbHandler *pSH)
 {       
-
 #ifdef CGDEBUG
     _pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Serving session"));
 #endif
@@ -385,20 +402,25 @@ void CegoDbThread::serveSession(CegoDistDbHandler *pSH)
 		pSH->sendError(msg);
 		return;	    
 	    }
-		     
+	    
 	    _pTabMng->setActiveUser(pSH->getTableSet(), pSH->getUser(), pSH->getPassword());
 
 	    msg = Chain("Access granted");
-	    pSH->sendSessionConfirm(msg, _idx, XML_DBPRODNAME_VALUE, XML_DBPRODVERSION_VALUE, __dateTimeFormat);
+	    pSH->sendSessionConfirm(msg, _idx, XML_DBPRODNAME_VALUE, XML_DBPRODVERSION_VALUE, __dateTimeFormat, __quoteEscapeFlag);
 	    
 #ifdef CGDEBUG
 	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Accepted session for tableset ") + pSH->getTableSet());
 #endif
-	    
-	}
-	
+	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Thread ") + Chain(_idx) + Chain(" : Accepted session for user=<") + pSH->getUser() + Chain("> and tableset <") + pSH->getTableSet() + Chain(">"));	    
+	}	
 	
 	_pPA->setTableSet(pSH->getTableSet());
+
+	_pTabMng->setAppend(_pDBMng->getAppendMode(pSH->getTableSet()));
+
+	// setup query history configuration for each session
+	_numLast = _pDBMng->getNumQueryLast();
+	_numCost = _pDBMng->getNumQueryCost();
 	
 	bool isTerminated = false;
 
@@ -414,38 +436,26 @@ void CegoDbThread::serveSession(CegoDistDbHandler *pSH)
 	    
 	    if ( reqType != CegoDbHandler::REQTIMEOUT )
 	    {
-		Chain runState = _pDBMng->getTableSetRunState(pSH->getTableSet());
-		if ( runState != Chain(XML_ONLINE_VALUE) && runState != Chain(XML_BACKUP_VALUE) && runState != Chain(XML_CHECKPOINT_VALUE) )
+		if ( isTraceOn )
 		{
-		    Chain msg = Chain("Tableset " + pSH->getTableSet() + " not online ( run state is " + runState + ")");
-		    pSH->sendError(msg);
+		    _pDBMng->incUserQuery(pSH->getUser());
 		}
-		else
-		{
 
-		    if ( isTraceOn )
-		    {
-			_pDBMng->incUserQuery(pSH->getUser());
-		    }
-
-		    _pPool->incNumQueryRequest(_idx);
-		    		    
-		    _pPool->setState(_idx, CegoDbThreadPool::BUSY);
-
-		    _pTim->stop();
-		    _pPool->addThreadIdle(_idx, _pTim->getSum());
-		    
-		    isTerminated = serveRequest(pSH, reqType);
-		    
-		    _pTim->reset();
-		    _pTim->start();
-
-		    _pPool->setState(_idx, CegoDbThreadPool::CONNECTED);	    
-		}
+		_pPool->incNumQueryRequest(_idx);	
+		_pPool->setState(_idx, CegoDbThreadPool::BUSY);
+		
+		_pTim->stop();
+		_pPool->addThreadIdle(_idx, _pTim->getSum());
+		
+		isTerminated = serveRequest(pSH, reqType);
+		
+		_pTim->reset();
+		_pTim->start();
+		
+		_pPool->setState(_idx, CegoDbThreadPool::CONNECTED);
 	    }
 	    else
-	    {
-
+	    {		
 #ifdef CGDEBUG
 		_pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Client request timeout occured, waiting ..."));
 #endif
@@ -462,11 +472,15 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
     // at the beginning of the request, in any case we reset abort status
     _pTabMng->proceed();
 
-    try {
-	
+    Chain lastAction;
+
+    _pCost->reset();
+    _pCost->start();
+    
+    try
+    {	
 	switch ( reqType ) 
-	{
-	    
+	{	    
 	case CegoDbHandler::DBPRODINFO:
 	{
 	    pSH->sendProdInfo();
@@ -482,24 +496,24 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    break;
 	}
 	case CegoDbHandler::QUERY:
-	{
-	    
-	    _lastAction = pSH->getQueryArg();
-	    
+	{  
+	    lastAction = pSH->getQueryArg(); 
+				
 #ifdef CGDEBUG
-	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Query Arg = <<<") + _lastAction + Chain(">>>"));
+	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Query Arg = <<<") + lastAction + Chain(">>>"));
 #endif
 	    
 	    _pPA->cleanUp();
-	    _pPA->setCommandChain( _lastAction );
+	    _pPA->setCommandChain( lastAction );
 	    _pPA->setDbHandle(pSH);
 	    
 	    _pPA->parse();
 
+	    _pPA->execute();
+
 #ifdef CGDEBUG
 	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Query request done"));
-#endif
-	    
+#endif   
 	    break;
 	}
 	case CegoDbHandler::INSERT:
@@ -514,15 +528,14 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    
 	    CegoTableObject oe;
 	    _pTabMng->getObject(tabSetId, tableName, CegoObject::TABLE, oe);
-	    _pTabMng->insertDataTableSynced(oe, fl);
-	    
-	    pSH->sendResponse(Chain("insert ok"));
+	    _pTabMng->insertLocalDataTable(oe, fl);
 	    
+	    pSH->sendResponse(Chain("insert ok"));	    
 	    break;
 	}
 	case CegoDbHandler::DELETETABLE:
 	{
-	    
+	    /*
 	    Chain tableSet;
 	    Chain tableName;
 	    CegoPredDesc *pPred = 0;
@@ -534,15 +547,16 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    CegoTableObject oe;
 	    _pTabMng->getObject(tabSetId, tableName, CegoObject::TABLE, oe);
 	    
-	    unsigned long long delCount = _pTabMng->deleteDataTableSynced(oe, pPred);
+	    unsigned long long delCount = _pTabMng->deleteLocalDataTable(oe, pPred);
 	    	    
 	    pSH->sendResponse(Chain("delete ok"), delCount);
+	    */
 	    
 	    break;
 	}
 	case CegoDbHandler::UPDATE:
 	{
-	    
+	    /*
 	    Chain tableSet;
 	    Chain tableName;
 	    CegoPredDesc *pPred = 0;
@@ -555,13 +569,15 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    
 	    CegoTableObject oe;
 	    _pTabMng->getObject(tabSetId, tableName, CegoObject::TABLE, oe);
-	    
-	    unsigned long long updCount = _pTabMng->updateDataTableSynced(oe, pPred, updSchema, exprList);
+
+	    bool returnOnFirst = false; // still not supported
+	    ListT<CegoField> returnList;
+	    unsigned long long updCount = _pTabMng->updateLocalDataTable(oe, pPred, updSchema, exprList, returnOnFirst, returnList);
 	    
 	    pSH->sendResponse(Chain("update ok"), updCount);
-	    
+	    */   
 	    break;
-	}	
+	}
 	case CegoDbHandler::CREATETABLE:
 	{	    
 	    Chain tableSet;
@@ -614,8 +630,14 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    pSH->sendResponse(Chain("create procedure ok"));	    
 	    break;	    
 	}
+	/*
+           CREATECHECK and ALTERTABLE actually no more supported, since 
+           the required XML emcoding for CegoPredDesc is deprecated.
+           For distributed mode, also fast serial protocol needs support
+	*/
 	case CegoDbHandler::CREATECHECK:
-	{	    
+	{
+	    /*
 	    Chain tableSet;
 	    Chain checkName;
 	    Chain tableName;
@@ -629,11 +651,14 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    _pDBMng->addObject(tabSetId, checkName, CegoObject::CHECK);
 
 	    pSH->sendResponse(Chain("create check ok"));	    
+	    */
+	    
 	    break;	    
 	}
 
 	case CegoDbHandler::ALTERTABLE:
-	{	    
+	{
+	    /*
 	    Chain tableSet;
 	    Chain tableName;
 	    ListT<CegoAlterDesc> alterList;
@@ -642,8 +667,10 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    int tabSetId = _pDBMng->getTabSetId(tableSet);	    
 	    _pTabMng->alterDataTable(tabSetId, tableName, CegoObject::TABLE, alterList);	    
 	    pSH->sendResponse(Chain("alter ok"));	    
+	    */
 	    break;
 	}
+       
 	case CegoDbHandler::DROPOBJECT:
 	{	    
 	    Chain tableSet;
@@ -665,7 +692,9 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    CegoObject::ObjectType type;	    
 	    pSH->getCreateIndexArg(tableSet, indexName, tableName, idxList, type);	    
 	    int tabSetId = _pDBMng->getTabSetId(tableSet);
-	    _pTabMng->createIndexTableSynced(tabSetId, indexName, tableName, type, idxList);
+
+	    bool isCached = false;
+	    _pTabMng->createIndexTableSynced(tabSetId, indexName, tableName, type, idxList, isCached);
 
 	    _pDBMng->addObject(tabSetId, indexName, type);
 
@@ -683,8 +712,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    if ( _pDBMng->objectExists(tabSetId, objName, objType) == false )
 		pSH->sendError(Chain("Object does not exist"));
 	    else
-	    {
-	    
+	    {	 
 		switch ( objType )
 		{
 		case CegoObject::SYSTEM:
@@ -707,7 +735,6 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 		    pSH->sendObjInfo(bto);
 		    break;
 		}
-	
 		case CegoObject::VIEW:
 		{
 		    CegoViewObject vo;
@@ -736,6 +763,20 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 		    pSH->sendObjInfo(co);
 		    break;
 		}
+		case CegoObject::TRIGGER:
+		{
+		    CegoTriggerObject to;
+		    _pTabMng->getLocalObject(tabSetId, objName, objType, to);
+		    pSH->sendObjInfo(to);
+		    break;
+		}
+		case CegoObject::ALIAS:
+		{
+		    CegoAliasObject ao;
+		    _pTabMng->getLocalObject(tabSetId, objName, objType, ao);
+		    pSH->sendObjInfo(ao);
+		    break;
+		}
 		case CegoObject::JOIN:
 		case CegoObject::RBSEG:
 		case CegoObject::UNDEFINED:
@@ -804,7 +845,6 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    pSH->sendObjList(objList);
 	    
 	    break;
-	    
 	}
 	case CegoDbHandler::GETOBJLISTBYTABLE:
 	{
@@ -820,15 +860,16 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    ListT<CegoBTreeObject> btreeList;
 	    ListT<CegoKeyObject> keyList;
 	    ListT<CegoCheckObject> checkList;
+	    ListT<CegoTriggerObject> triggerList;
+	    ListT<CegoAliasObject> aliasList;
 	    int numInvalid;
 
 	    _pTabMng->setIgnoreInvalid(false);
-	    _pTabMng->getObjectListByTable(tabSetId, tableName, idxList, btreeList, keyList, checkList, numInvalid);
+	    _pTabMng->getObjectListByTable(tabSetId, tableName, idxList, btreeList, keyList, checkList, triggerList, aliasList, numInvalid);
 	    
 	    pSH->sendObjByTableList(idxList, keyList, checkList);
 	    
 	    break;
-	    
 	}	
 	case CegoDbHandler::OBJRENAME:
 	{
@@ -846,7 +887,6 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    pSH->sendResponse(Chain("rename ok"));
 	    
 	    break;
-	    
 	}
 	case CegoDbHandler::REORGOBJ:
 	{
@@ -863,10 +903,8 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    
 	    pSH->sendResponse(Chain("reorg ok"));
 	    
-	    break;
-	    
+	    break;	   
 	}
-
 	case CegoDbHandler::SYNC:
 	{
 	    Chain tableSet;
@@ -877,12 +915,11 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    
 	    int tabSetId = _pDBMng->getTabSetId(tableSet);
 	    
-	    _pTabMng->writeCheckPoint(tabSetId, true, true, escCmd, timeout);
+	    _pDBMng->writeCheckPoint(tabSetId, true, true, _pTabMng->getLockHandle(), escCmd, timeout );
 	    
 	    pSH->sendResponse(Chain("sync ok"));
 	    
-	    break;
-	    
+	    break;	    
 	}
 	case CegoDbHandler::GETPAGECOUNT:
 	{
@@ -899,13 +936,11 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    
 	    pSH->sendPageCount(pageCount);
 	    
-	    break;
-	    
+	    break;	    
 	}
 	case CegoDbHandler::PUTBLOB:
 	{
-
-	    _lastAction = Chain("put blob");
+	    lastAction = Chain("put blob");
 
 	    Chain tableSet;
 	    unsigned long long blobSize;
@@ -916,8 +951,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 
 	    NetHandler *pN = pSH->getNetHandler();
 
-	    int fileId = 0;
-	    int pageId = 0;
+	    PageIdType pageId = 0;
 
 	    CegoBufferPage bp;
 
@@ -935,21 +969,23 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 
 	    bp.initPage(CegoBufferPage::BLOB);
 	    
-	    fileId = bp.getFileId();
 	    pageId = bp.getPageId();
 		
 	    try
 	    {
-
 #ifdef CGDEBUG
-		_pDBMng->log(_modId, Logger::DEBUG, Chain(" Thread ") + Chain(_idx) + Chain(" : Sending blob info [") + Chain(fileId) + Chain(",") + Chain(pageId) + Chain("]"));
+		_pDBMng->log(_modId, Logger::DEBUG, Chain(" Thread ") + Chain(_idx) + Chain(" : Sending blob info [") + Chain(pageId) + Chain("]"));
 #endif
-		pSH->sendBlobInfo(fileId, pageId);
-
+		pSH->sendBlobInfo(pageId);
 		
 		int freeInPage = bp.getChunkLen();
 		char* pagePtr = bp.getChunkEntry();
-		
+
+		unsigned long long blobRef = 0;
+		memcpy(pagePtr, &blobRef, sizeof(unsigned long long));
+		pagePtr += sizeof(unsigned long long);
+		freeInPage -= sizeof(unsigned long long);
+	       
 		memcpy(pagePtr, &blobSize, sizeof(unsigned long long));
 		pagePtr += sizeof(unsigned long long);
 		freeInPage -= sizeof(unsigned long long);
@@ -960,11 +996,9 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 		unsigned long long recvSize = 0;
 
 		while ( recvSize < blobSize )
-		{	
-		    
+		{		    
 		    try
 		    {
-
 			bool ackRequired = false;
 
 			if ( availFromBuf == 0 )		    
@@ -983,7 +1017,6 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 			    _pTabMng->getNewFilePage(nextPage, tabSetId, CegoObject::TABLE);
 			    nextPage.initPage(CegoBufferPage::BLOB);
 			    
-			    bp.setNextFileId(nextPage.getFileId());
 			    bp.setNextPageId(nextPage.getPageId());
 			    
 			    _pDBMng->bufferUnfix(bp, true, _pTabMng->getLockHandler());
@@ -998,13 +1031,12 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 			{
 			    pN->sendAck();
 			    ackRequired = false;
-			}
-		    
+			}		    
 		    }
 		    catch ( Exception e )
 		    {			
 			pN->sendNack();			    
-			throw e;
+			throw Exception(EXLOC, Chain("Cannot put blob"), e);
 		    }
 		    
 		    if ( freeInPage >= availFromBuf )
@@ -1030,45 +1062,40 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    {
 		if ( bp.isFixed() )
 		    _pDBMng->bufferUnfix(bp, true, _pTabMng->getLockHandler());
-		if ( fileId || pageId )
+		if ( pageId )
 		{
-		    _pTabMng->releaseBlob(tabSetId, fileId, pageId);
+		    _pTabMng->decreaseBlobRef(tabSetId, pageId);
 		}
 	    }
-
+	    
 #ifdef CGDEBUG
 	    _pDBMng->log(_modId, Logger::DEBUG, Chain(" Thread ") + Chain(_idx) + Chain(" : Blob transfer finished"));
 #endif
 
 	    break;
-
 	}
 	case CegoDbHandler::GETBLOB:
 	{
-
-	    _lastAction = Chain("get blob");
+	    lastAction = Chain("get blob");
 
 	    Chain tableSet;
-	    int fileId;
-	    int pageId;
+
+	    PageIdType pageId;
 	    
-	    pSH->getGetBlobArg(tableSet, fileId, pageId);
+	    pSH->getGetBlobArg(tableSet, pageId);
 
 	    int tabSetId = _pDBMng->getTabSetId(tableSet);
 
 	    NetHandler *pN = pSH->getNetHandler();	    	
 
-	    bool isFixed = false;
 	    CegoBufferPage bp;
 	    try
 	    {
-
-		_pDBMng->bufferFix(bp, tabSetId, fileId, pageId, CegoBufferPool::SYNC,
-						     _pTabMng->getLockHandler());
-
+		_pDBMng->bufferFix(bp, tabSetId, pageId, CegoBufferPool::SYNC,
+				   _pTabMng->getLockHandler());
 	    	
 		unsigned long long blobSize;
-		memcpy (&blobSize, bp.getChunkEntry(), sizeof(unsigned long long));
+		memcpy (&blobSize, bp.getChunkEntry() + sizeof(unsigned long long), sizeof(unsigned long long));
 		
 		pSH->sendBlobSize(blobSize);
 		
@@ -1079,20 +1106,36 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 		    pN->recvAck();
 		    
 		    int chunkSize;
-		    if (  bp.getChunkLen() < blobSize - sentByte )
-		    {		       
-			chunkSize = bp.getChunkLen();
-			if ( sentByte == 0 )
-			    chunkSize -= sizeof(unsigned long long);
-		    }
-		    else
-		    {
-			chunkSize = blobSize - sentByte;
-		    }
-		    
+
+		    // first page with written clob size, so we have to sub the size bytes
 		    if ( sentByte == 0 )
 		    {
-			pN->setMsg(bp.getChunkEntry() + sizeof(unsigned long long), chunkSize );
+			// take the full page chunk ?
+			if (  bp.getChunkLen() - 2 * sizeof(unsigned long long ) < blobSize  )
+			{
+			    chunkSize = bp.getChunkLen() - 2 * sizeof(unsigned long long);
+			}
+			else // just take the needed chunk
+			{
+			    chunkSize = blobSize;
+			}
+		    }
+		    else // just take the needed
+		    {
+			// take the full page chunk ?
+			if (  bp.getChunkLen() < blobSize - sentByte  )
+			{
+			    chunkSize = bp.getChunkLen();
+			}
+			else // just take the needed chunk
+			{
+			    chunkSize = blobSize - sentByte;
+			}
+		    }
+
+		    if ( sentByte == 0 )
+		    {
+			pN->setMsg(bp.getChunkEntry() + 2 * sizeof(unsigned long long), chunkSize );
 		    }
 		    else
 		    {
@@ -1102,14 +1145,13 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 
 		    pN->writeMsg();
 		    
-		    fileId = bp.getNextFileId();
 		    pageId = bp.getNextPageId();
 		    
 		    _pDBMng->bufferUnfix(bp, false, _pTabMng->getLockHandler());
 		    
-		    if ( fileId || pageId )			
+		    if ( pageId )			
 		    {
-			_pDBMng->bufferFix(bp, tabSetId, fileId, pageId, CegoBufferPool::SYNC,
+			_pDBMng->bufferFix(bp, tabSetId, pageId, CegoBufferPool::SYNC,
 					   _pTabMng->getLockHandler());
 		    }		    
 		}
@@ -1119,36 +1161,28 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 		if ( bp.isFixed() )
 		    _pDBMng->bufferUnfix(bp, true, _pTabMng->getLockHandler());
 	    }
-
-	    break;
-	    
+	    break;	    
 	}
 	case CegoDbHandler::DELBLOB:
 	{
-
-	    _lastAction = Chain("del blob");
+	    lastAction = Chain("del blob");
 
 	    Chain tableSet;
-	    int fileId;
-	    int pageId;
+
+	    PageIdType pageId;
 	    
-	    pSH->getGetBlobArg(tableSet, fileId, pageId);
+	    pSH->getGetBlobArg(tableSet, pageId);
 
 	    int tabSetId = _pDBMng->getTabSetId(tableSet);
 
-	    _pTabMng->releaseBlob(tabSetId, fileId, pageId);
+	    _pTabMng->decreaseBlobRef(tabSetId, pageId);
 
 	    pSH->sendResponse(Chain("Blob deleted"));
-	    
-	    break;
-	    
+	    break;	    
 	}
-
 	case CegoDbHandler::PUTCLOB:
 	{
-
-	    _lastAction = Chain("put clob");
-
+	    lastAction = Chain("put clob");
 
 	    Chain tableSet;
 	    unsigned long long clobSize;
@@ -1159,8 +1193,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 
 	    NetHandler *pN = pSH->getNetHandler();
 
-	    int fileId = 0;
-	    int pageId = 0;
+	    PageIdType pageId = 0;
 
 	    CegoBufferPage bp;
 
@@ -1178,20 +1211,22 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 
 	    bp.initPage(CegoBufferPage::CLOB);
 	    
-	    fileId = bp.getFileId();
 	    pageId = bp.getPageId();
 		
 	    try
 	    {
-
 #ifdef CGDEBUG
-		_pDBMng->log(_modId, Logger::DEBUG, Chain(" Thread ") + Chain(_idx) + Chain(" : Sending clob info [") + Chain(fileId) + Chain(",") + Chain(pageId) + Chain("]"));
+		_pDBMng->log(_modId, Logger::DEBUG, Chain(" Thread ") + Chain(_idx) + Chain(" : Sending clob info [") + Chain(pageId) + Chain("]"));
 #endif
-		pSH->sendClobInfo(fileId, pageId);
-
+		pSH->sendClobInfo(pageId);
 		
 		int freeInPage = bp.getChunkLen();
 		char* pagePtr = bp.getChunkEntry();
+
+		unsigned long long clobRef = 0;
+		memcpy(pagePtr, &clobRef, sizeof(unsigned long long));
+		pagePtr += sizeof(unsigned long long);
+		freeInPage -= sizeof(unsigned long long);
 		
 		memcpy(pagePtr, &clobSize, sizeof(unsigned long long));
 		pagePtr += sizeof(unsigned long long);
@@ -1204,10 +1239,8 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 
 		while ( recvSize < clobSize )
 		{	
-		    
 		    try
 		    {
-
 			bool ackRequired = false;
 
 			if ( availFromBuf == 0 )		    
@@ -1226,7 +1259,6 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 			    _pTabMng->getNewFilePage(nextPage, tabSetId, CegoObject::TABLE);
 			    nextPage.initPage(CegoBufferPage::CLOB);
 			    
-			    bp.setNextFileId(nextPage.getFileId());
 			    bp.setNextPageId(nextPage.getPageId());
 			    
 			    _pDBMng->bufferUnfix(bp, true, _pTabMng->getLockHandler());
@@ -1241,13 +1273,12 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 			{
 			    pN->sendAck();
 			    ackRequired = false;
-			}
-		    
+			}		    
 		    }
 		    catch ( Exception e )
 		    {			
-			pN->sendNack();			    
-			throw e;
+			pN->sendNack();
+			throw Exception(EXLOC, Chain("Cannot put clob"), e);
 		    }
 		    
 		    if ( freeInPage >= availFromBuf )
@@ -1273,45 +1304,38 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    {
 		if ( bp.isFixed() )
 		    _pDBMng->bufferUnfix(bp, true, _pTabMng->getLockHandler());
-		if ( fileId || pageId )
+		if ( pageId )
 		{
-		    _pTabMng->releaseClob(tabSetId, fileId, pageId);
+		    _pTabMng->decreaseClobRef(tabSetId, pageId);
 		}
 	    }
-
 #ifdef CGDEBUG
 	    _pDBMng->log(_modId, Logger::DEBUG, Chain(" Thread ") + Chain(_idx) + Chain(" : Clob transfer finished"));
 #endif
-
 	    break;
-
 	}
 	case CegoDbHandler::GETCLOB:
 	{
-
-	    _lastAction = Chain("get clob");
+	    lastAction = Chain("get clob");
 
 	    Chain tableSet;
-	    int fileId;
-	    int pageId;
 	    
-	    pSH->getGetClobArg(tableSet, fileId, pageId);
+	    PageIdType pageId;
+	    
+	    pSH->getGetClobArg(tableSet, pageId);
 
 	    int tabSetId = _pDBMng->getTabSetId(tableSet);
 
 	    NetHandler *pN = pSH->getNetHandler();	    	
 
-	    bool isFixed = false;
 	    CegoBufferPage bp;
 	    try
 	    {
-
-		_pDBMng->bufferFix(bp, tabSetId, fileId, pageId, CegoBufferPool::SYNC,
-						     _pTabMng->getLockHandler());
-
-	    	
+		_pDBMng->bufferFix(bp, tabSetId, pageId, CegoBufferPool::SYNC,
+				   _pTabMng->getLockHandler());
+	
 		unsigned long long clobSize;
-		memcpy (&clobSize, bp.getChunkEntry(), sizeof(unsigned long long));
+		memcpy (&clobSize, bp.getChunkEntry() + sizeof(unsigned long long), sizeof(unsigned long long));
 		
 		pSH->sendClobSize(clobSize);
 		
@@ -1322,20 +1346,36 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 		    pN->recvAck();
 		    
 		    int chunkSize;
-		    if (  bp.getChunkLen() < clobSize - sentByte )
-		    {		       
-			chunkSize = bp.getChunkLen();
-			if ( sentByte == 0 )
-			    chunkSize -= sizeof(unsigned long long);
-		    }
-		    else
+
+		    // first page with written clob size, so we have to sub the size bytes
+		    if ( sentByte == 0 )
 		    {
-			chunkSize = clobSize - sentByte;
+			// take the full page chunk ?
+			if (  bp.getChunkLen() - 2 * sizeof(unsigned long long ) < clobSize  )
+			{
+			    chunkSize = bp.getChunkLen() - 2 * sizeof(unsigned long long);
+			}
+			else // just take the needed chunk
+			{
+			    chunkSize = clobSize;
+			}
+		    }
+		    else // just take the needed
+		    {
+			// take the full page chunk ?
+			if (  bp.getChunkLen() < clobSize - sentByte  )
+			{
+			    chunkSize = bp.getChunkLen();
+			}
+			else // just take the needed chunk
+			{
+			    chunkSize = clobSize - sentByte;
+			}
 		    }
 		    
 		    if ( sentByte == 0 )
 		    {
-			pN->setMsg(bp.getChunkEntry() + sizeof(unsigned long long), chunkSize );
+			pN->setMsg(bp.getChunkEntry() + 2 * sizeof(unsigned long long), chunkSize );
 		    }
 		    else
 		    {
@@ -1345,14 +1385,13 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 
 		    pN->writeMsg();
 		    
-		    fileId = bp.getNextFileId();
 		    pageId = bp.getNextPageId();
 		    
 		    _pDBMng->bufferUnfix(bp, false, _pTabMng->getLockHandler());
 		    
-		    if ( fileId || pageId )			
+		    if ( pageId )			
 		    {
-			_pDBMng->bufferFix(bp, tabSetId, fileId, pageId, CegoBufferPool::SYNC,
+			_pDBMng->bufferFix(bp, tabSetId, pageId, CegoBufferPool::SYNC,
 					   _pTabMng->getLockHandler());
 		    }		    
 		}
@@ -1362,73 +1401,69 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 		if ( bp.isFixed() )
 		    _pDBMng->bufferUnfix(bp, true, _pTabMng->getLockHandler());
 	    }
-
-	    break;
-	    
+	    break;   
 	}
 	case CegoDbHandler::DELCLOB:
 	{
-
-	    _lastAction = Chain("del clob");
+	    lastAction = Chain("del clob");
 
 	    Chain tableSet;
-	    int fileId;
-	    int pageId;
-	    
-	    pSH->getGetClobArg(tableSet, fileId, pageId);
-
+	    PageIdType pageId;
+	    pSH->getGetClobArg(tableSet, pageId);
 	    int tabSetId = _pDBMng->getTabSetId(tableSet);
-
-	    _pTabMng->releaseClob(tabSetId, fileId, pageId);
-
+	    _pTabMng->decreaseClobRef(tabSetId, pageId);
 	    pSH->sendResponse(Chain("Clob deleted"));
 	    
-	    break;
-	    
+	    break;   
 	}
-
 	case CegoDbHandler::STARTTRANSACTION:
 	{
-	    _lastAction = Chain("start transaction");
-
+	    lastAction = Chain("start transaction");
+	    
 	    Chain tableSet;
 	    pSH->getArgValue(XML_TABLESET_ATTR, tableSet);
 	    int tabSetId = _pDBMng->getTabSetId(tableSet);
-	    _pTabMng->beginTransaction(tabSetId);
+	    _pTabMng->beginTransaction(tabSetId, true);
 	    pSH->sendResponse(Chain("Transaction started"));	    
 	    break;	    
 	}
 	case CegoDbHandler::COMMITTRANSACTION:
 	{
-	    _lastAction = Chain("commit transaction");
+	    lastAction = Chain("commit transaction");
+	    
 	    Chain tableSet;
 	    pSH->getArgValue(XML_TABLESET_ATTR, tableSet);
 	    int tabSetId = _pDBMng->getTabSetId(tableSet);
-	    _pTabMng->commitTransaction(tabSetId);
-	    pSH->sendResponse(Chain("Transaction committed"));	    
+	    _pTabMng->commitTransaction(tabSetId, true);
+	    pSH->sendResponse(Chain("Transaction committed"));
+			
 	    break;	    
 	}
 	case CegoDbHandler::ROLLBACKTRANSACTION:
 	{
-	    _lastAction = Chain("rollback transaction");
+	    lastAction = Chain("rollback transaction");
+	    
 	    Chain tableSet;
 	    pSH->getArgValue(XML_TABLESET_ATTR, tableSet);
 	    int tabSetId = _pDBMng->getTabSetId(tableSet);
-	    _pTabMng->rollbackTransaction(tabSetId);
-	    pSH->sendResponse(Chain("Transaction rollbacked"));	    
+	    _pTabMng->rollbackTransaction(tabSetId, true);
+	    pSH->sendResponse(Chain("Transaction rollbacked"));			
 	    break;	    
 	}
 	case CegoDbHandler::GETTID:
 	{
-	    _lastAction = Chain("get tid");
+	    lastAction = Chain("get tid");
+	    
 	    Chain tableSet;
 	    pSH->getArgValue(XML_TABLESET_ATTR, tableSet);
 	    int tabSetId = _pDBMng->getTabSetId(tableSet);	    
-	    pSH->sendTID(_pTabMng->getTID(tabSetId));
+	    pSH->sendTID(_pTabMng->getTID(tabSetId));	    
 	    break;	    
 	}
 	case CegoDbHandler::SESSION_CLOSE:
 	{
+	    lastAction = Chain("close session");
+	    
 	    isTerminated=true;
 	    pSH->sendResponse(Chain("Session Closed"));
 	    break;
@@ -1441,8 +1476,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	}
     }
     catch ( Exception e)
-    {
-	
+    {	
 	Chain msg;
 	Chain module;
 	int line;
@@ -1453,7 +1487,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    exep += Chain("\n\t") + module + Chain("(") +  Chain(line) + Chain(") : ") + msg;
 	}
 		
-	_pDBMng->log(_modId, Logger::LOGERR, Chain("Thread ") + Chain(_idx) + Chain(" : DB request failed : ") + exep);
+	_pDBMng->log(_modId, Logger::LOGERR, Chain("Thread ") + Chain(_idx) + Chain(" : DB request failed :") + exep);
 	
 	if ( _pTabMng->isAborted() )
 	{
@@ -1464,5 +1498,52 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	pSH->sendError(e.getBaseMsg());
     }
 
+    _pCost->stop();
+
+    Datetime ts;
+    
+    addHistory(ts.asLong(), lastAction, _pCost->getSum() / 1000000 );
+    
     return isTerminated;
 }
+
+void CegoDbThread::addHistory(unsigned long long ts, const Chain& queryString, unsigned long long cost)
+{
+    if ( _numLast > 0 || _numCost > 0 )
+    {
+	P();
+	if ( _numLast > 0 )
+	{
+	    QueryEntry *pFirst = _queryList.First();
+	    if ( pFirst && _queryList.Size() >= _numLast )
+		_queryList.Remove(*pFirst);
+	    _queryList.Insert( QueryEntry(_idx, _qid, ts, queryString, cost));
+	}
+	
+	if ( _numCost > 0 )
+	{
+	    QueryEntry costEntry(_idx, _qid, ts, queryString, cost);
+	    costEntry.setCostOrder();
+	    _costTree.Insert(costEntry);
+	    QueryEntry *pFirst = _costTree.First();
+	    if ( pFirst && _costTree.Size() >= _numCost )
+		_costTree.Remove(*pFirst);
+	}
+	
+	_qid++;
+    
+	V();
+    }
+}
+
+// private methods
+
+void CegoDbThread::P() const
+{    
+    histLock.writeLock(QH_LOCKTIMEOUT);
+}
+
+void CegoDbThread::V() const
+{
+    histLock.unlock();
+}
