@@ -6,7 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke
 //               
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -38,6 +38,8 @@
 #include <stdlib.h>
 
 static ThreadLock queueLock("DBQUEUE"); 
+
+// db pool thread array is used to synchronize compiled objects use ( procedures and views ) 
 static ThreadLock** thrLockArray; 
 extern bool __lockStatOn;
 
@@ -91,14 +93,17 @@ CegoDbThreadPool::~CegoDbThreadPool()
 	delete _threadList[i];
 	delete thrLockArray[i];
     }
+    
     delete _numRequest;
     delete _numQueryRequest;
     delete _threadId;
     delete _threadLoad;
+
     for ( int i = 0; i < THRMNG_NUMLOADSAMPLE; i++ )
     {
 	delete _threadIdle[i];
     }
+
     delete _threadState;
     delete thrLockArray;
 }
@@ -120,13 +125,15 @@ void  CegoDbThreadPool::syncToReady()
 
     for (int i = 0; i<_poolLimit; i++)
     {
-        thrLockArray[i] = new ThreadLock(Chain("THRLCK") + Chain(i));
+        thrLockArray[i] = new ThreadLock(Chain("DBTHRLCK") + Chain(i));
         thrLockArray[i]->init(LCKMNG_LOCKWAITDELAY, __lockStatOn);
     }
     
     _pDBMng->getDataPort(_dataPortNo);
     _pDBMng->getDBHost(_dataHostName);
 
+    _maxSendLen = _pDBMng->getMaxSendLen();
+    
     _threadId = (unsigned long long*)malloc(_poolLimit * sizeof(unsigned long long));
     _threadLoad = (unsigned long long*)malloc(_poolLimit * sizeof(unsigned long long));
     _numRequest = (unsigned long long*)malloc(_poolLimit * sizeof(unsigned long long));
@@ -169,12 +176,9 @@ void  CegoDbThreadPool::syncToReady()
 		numReady++;
 	    i++;
 	}
-	// _pDBMng->log(_modId, Logger::NOTICE, Chain(numReady) + Chain(" of " ) + Chain(_poolLimit) + Chain(" db threads ready"));
-	
 
 	Sleeper s;
 	s.milliSleep(1);
-
     }
     _pDBMng->log(_modId, Logger::NOTICE, Chain("All db threads ready"));
 }
@@ -222,14 +226,80 @@ int CegoDbThreadPool::getPoolLimit() const
     return _poolLimit;
 }
 
-void CegoDbThreadPool::getThreadInfo(int i, unsigned long long& numRequest, unsigned long long& numQueryRequest, unsigned long long& threadLoad, unsigned long long& numAllocated, CegoDbThreadPool::ThreadState& state, Chain& action)
+void CegoDbThreadPool::getThreadInfo(int i, unsigned long long& numRequest, unsigned long long& numQueryRequest, unsigned long long& threadLoad, unsigned long long& numAllocated, CegoDbThreadPool::ThreadState& state)
 {
     state = _threadState[i];
     numRequest = _numRequest[i];
     numQueryRequest = _numQueryRequest[i];
     threadLoad = _threadLoad[i];
     numAllocated = _threadList[i]->allocatedSortArea();
-    action = _threadList[i]->lastAction();
+}
+
+void CegoDbThreadPool::getThreadHistory(int i, ListT<CegoDbThread::QueryEntry>& queryList)
+{
+    if ( i < _poolLimit )
+	queryList = _threadList[i]->getLastQuery();
+}
+
+void CegoDbThreadPool::getQueryHistoryLast(int numLast, ListT<CegoDbThread::QueryEntry>& queryList)
+{    
+    TreeT<CegoDbThread::QueryEntry> queryTree;
+    
+    int i=0;
+    while ( i < _poolLimit )
+    {
+	ListT<CegoDbThread::QueryEntry> threadQueryList;
+	threadQueryList = _threadList[i]->getLastQuery();
+
+	CegoDbThread::QueryEntry *pEntry =  threadQueryList.First();
+	while ( pEntry )
+	{
+	    pEntry->setRevLastOrder();
+	    queryTree.Insert(*pEntry);	    
+	    pEntry =  threadQueryList.Next();
+	}
+	
+	i++;
+    }
+
+    i=0;
+    CegoDbThread::QueryEntry *pEntry =  queryTree.First();
+    while ( pEntry && i < numLast )
+    {
+	queryList.Insert(*pEntry);
+	pEntry =  queryTree.Next();
+	i++;
+    }
+}
+
+void CegoDbThreadPool::getQueryHistoryCost(int numLast, ListT<CegoDbThread::QueryEntry>& queryList)
+{
+    TreeT<CegoDbThread::QueryEntry> queryTree;
+    
+    int i=0;
+    while ( i < _poolLimit )
+    {
+	ListT<CegoDbThread::QueryEntry> threadQueryList;
+	threadQueryList = _threadList[i]->getCostQuery();
+
+	CegoDbThread::QueryEntry *pEntry =  threadQueryList.First();
+	while ( pEntry )
+	{
+	    pEntry->setRevCostOrder();
+	    queryTree.Insert(*pEntry);	    
+	    pEntry =  threadQueryList.Next();
+	}	
+	i++;
+    }
+
+    i=0;
+    CegoDbThread::QueryEntry *pEntry =  queryTree.First();
+    while ( pEntry && i < numLast )
+    {
+	queryList.Insert(*pEntry);
+	pEntry =  queryTree.Next();
+	i++;
+    }
 }
 
 void CegoDbThreadPool::setState(int i, CegoDbThreadPool::ThreadState state)
@@ -248,10 +318,7 @@ void CegoDbThreadPool::incNumQueryRequest(int i)
 }
 
 void CegoDbThreadPool::addThreadIdle(int i, unsigned long long usec)
-{
-    // Chain s = Chain("Thread ") + Chain(i) + Chain(" pos ") + Chain(_samplePos) + Chain(" reports ") + Chain(usec) + Chain(" idle time");
-    // cout << s << endl;
-
+{    
     _threadIdle[_samplePos][i] += usec;
 }
 
@@ -272,24 +339,28 @@ void  CegoDbThreadPool::abortThread(int i)
 
 void* CegoDbThreadPool::job(void* arg)
 {
-    try {
-
+    try
+    {
 	NanoTimer tim;
 	
-	Net net( NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN );
-	
-	// cout << "Serving portno " << _portNo << endl;
-	net.serve(_dataHostName, _dataPortNo);
+	Net net( NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN, _maxSendLen );
+
+	if ( _dataHostName != Chain() )
+	    net.serve(_dataHostName, _dataPortNo);
+	else
+	    net.serve6(_dataPortNo);
 
 	unsigned long long usedTime[THRMNG_NUMLOADSAMPLE];
 	for ( int i = 0; i < THRMNG_NUMLOADSAMPLE; i++ )
 	{
 	    usedTime[i] = 0;
 	}
+
+	int selectTimeout = _pDBMng->getSelectTimeout();
+	int queueDelay = _pDBMng->getQueueDelay();
 	
 	while ( ! _terminated ) 
 	{
-
 	    usedTime[_samplePos] = 0;
 	    
 	    tim.reset();
@@ -301,16 +372,21 @@ void* CegoDbThreadPool::job(void* arg)
 	    lockQueue();
 	    if ( _requestQueue.Size() == 0  )  noReq=true;
 	    unlockQueue();
-	    	    
-	    if ( noReq )
-		lockQueue();
+
+	    /*
+	      locking the queue to avoid cpu load for idle db threads
+	      This leads to fuzzy db thread load statistics.
+	      Instead we reduce cpu load with the QUEUEDELAY parameter
+
+	      if ( noReq )
+	      lockQueue();
+	    */
 	    
-	    NetHandler* pHandle;
-
-	    try {
-
-		pHandle = net.nextRequest( NETMNG_SELECT_TIMEOUT );
+	    NetHandler* pHandle = 0;
 
+	    try
+	    {
+		pHandle = net.nextRequest( selectTimeout );	
 	    }
 	    catch ( Exception e )
 	    {
@@ -321,24 +397,25 @@ void* CegoDbThreadPool::job(void* arg)
 	    
 	    if ( noReq )
 	    {
-		unlockQueue();
-		
-		// struct timespec delay;	
-		// delay.tv_sec = 0; 
-		// delay.tv_nsec = NETMNG_QUEUE_DELAY;
-		// nanosleep(&delay, NULL);
-
 		Sleeper s;
-		s.nanoSleep(NETMNG_QUEUE_DELAY);
+		s.microSleep(queueDelay);
 	    }
 	    
 	    if ( pHandle )
 	    {
-		// pHandle->setSendTimeout(NETMNG_SEND_TIMEOUT);
-		// pHandle->setRecvTimeout(NETMNG_RECV_TIMEOUT);
-
+		_pDBMng->log(_modId, Logger::NOTICE, Chain("Connection request from <") + pHandle->getSource() + Chain(">"));		
 		lockQueue();
-		_requestQueue.Insert(pHandle);
+
+		if ( _requestQueue.Size() < NETMNG_MAXQUEUELEN )
+		{
+		    _requestQueue.Insert(pHandle);
+		}
+		else
+		{
+		    delete pHandle;
+		    _pDBMng->log(_modId, Logger::NOTICE, Chain("Rejected incoming request since connection queue is full ( ") +  Chain(NETMNG_MAXQUEUELEN) + Chain(" max )"));
+		}
+		
 		unlockQueue();
 	    }
 
@@ -351,8 +428,6 @@ void* CegoDbThreadPool::job(void* arg)
 	   
 	    // calculate load
 	    int i=0;
-	    // cout << "___" << endl;
-	    // cout << "UsedTime=" << usedTime << endl;
 	    while ( i < _poolLimit )
 	    {
 		unsigned long long ut=0;
@@ -373,13 +448,11 @@ void* CegoDbThreadPool::job(void* arg)
 	    }
 
 	    _samplePos = ( _samplePos + 1 ) % THRMNG_NUMLOADSAMPLE;
-
-	    
+	   
 	    for ( int i = 0; i < _poolLimit; i++ )
 	    {
 		_threadIdle[_samplePos][i]=0;
-	    }
-	    
+	    }	    
 	}
 	int i=0;
 	while ( i < _poolLimit )
@@ -387,7 +460,6 @@ void* CegoDbThreadPool::job(void* arg)
 #ifdef CGDEBUG	
 	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Waiting for db thread with tid ") + Chain(_threadList[i]->getTid()) + Chain(" to terminate"));
 #endif
-
 	    join(_threadList[i]->getTid());
 	    i++;
 	}
@@ -405,6 +477,18 @@ void* CegoDbThreadPool::job(void* arg)
     return 0;
 }
 
+bool CegoDbThreadPool::isBusy()
+{
+    int i=0;
+    while ( i < _poolLimit )
+    {
+	if ( _threadState[i] == BUSY )
+	    return true;
+	i++;
+    }
+    return false;
+}
+
 bool CegoDbThreadPool::isTerminated()
 {
     return _terminated;
@@ -412,14 +496,12 @@ bool CegoDbThreadPool::isTerminated()
 
 NetHandler* CegoDbThreadPool::nextRequest()
 {
-
     lockQueue();
 
     NetHandler **pRequest = _requestQueue.First();
 
     if ( pRequest )
     {
-
 	NetHandler *pN = *pRequest;
 	_requestQueue.Remove(*pRequest);
 	unlockQueue();
@@ -459,3 +541,28 @@ void CegoDbThreadPool::getLockStat(Chain& lockName, int& lockCount, unsigned lon
     if ( queueLock.numWriteLock() > 0 )
 	sumWrDelay = queueLock.sumWriteDelay() / LCKMNG_DELRES;
 }
+
+void CegoDbThreadPool::getAggThreadLockStat(Chain& lockGroup, int& numLock, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay)
+{
+    lockGroup = Chain("DBTHRLCK");
+    
+    lockCount = 0;
+    numRdLock = 0;
+    numWrLock = 0;
+    sumRdDelay = 0;
+    sumWrDelay = 0;
+    numLock = 0;
+
+    for ( int i = 0; i<_poolLimit; i++)
+    {	
+	numLock++;
+	lockCount += thrLockArray[i]->numLockTry();
+	numRdLock += thrLockArray[i]->numReadLock();
+	numWrLock += thrLockArray[i]->numWriteLock();
+	    
+	if ( thrLockArray[i]->numReadLock() > 0 )
+	    sumRdDelay += thrLockArray[i]->sumReadDelay() / LCKMNG_DELRES;   
+	if ( thrLockArray[i]->numWriteLock() > 0 )
+	    sumWrDelay += thrLockArray[i]->sumWriteDelay() / LCKMNG_DELRES;
+    }
+}
