@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -38,11 +38,9 @@ public:
     virtual ~CegoDecodableObject();
 
     virtual int getEntrySize() const = 0;
-    virtual void encode(char *buf) = 0;
+    virtual void encode(char *buf) const = 0;
     virtual void decode(char *buf) = 0;
-
-    virtual Element* getElement() const = 0;
-    virtual void putElement(Element *pElement) = 0;
+    
     virtual Chain getFormatted() const = 0;
 
     Chain getTableSet() const { return _tableSet; } 
