@@ -8,7 +8,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -28,7 +28,7 @@
 
 #define CEGO_PRODUCT PACKAGE_NAME
 #define CEGO_VERSION PACKAGE_VERSION
-#define CEGO_COPYRIGHT "Copyright (C) 2000-2016 by Bjoern Lemke. All rights reserved"
+#define CEGO_COPYRIGHT "Copyright (C) 2000-2025 by Bjoern Lemke. All rights reserved"
 
 #define CGEXESHELLVARNAME "CGEXESHELL"
 #define CGSTDEXESHELL "/bin/sh"
@@ -41,6 +41,8 @@
 /* Please note, that id values from 1 to TABMNG_MAXTABSET are reserved for system files */
 #define FILMNG_MAXDATAFILE 5000
 
+#define PageIdType unsigned long long
+
 /*******************************/
 /* Table Management Parameters */
 /*******************************/
@@ -50,6 +52,8 @@
 
 /* maximum size of index entry values */
 #define TABMNG_MAXINDEXVALUE 1000
+/* maximum size of btree dimension */
+#define TABMNG_MAXBTREECOMP 10
 
 /* size of the hash array for parallel object access */
 #define TABMNG_HASHSIZE 15
@@ -77,8 +81,9 @@
 #define LOGMNG_RECBUFSIZE 65536
 #define LOGMNG_NONEPROG "NONE"
 #define LOGMNG_WBUFSIZE 1024
-#define LOGMNG_MAXNUMLOGFILE 30
-
+#define LOGMNG_MAXNUMLOGFILE 100
+#define LOGMNG_ARCHLOGSEP "-"
+#define LOGMNG_ARCHLOGSUFFIX ".dbf"
 
 /************************/
 /* Data Type Parameters */
@@ -87,8 +92,6 @@
 #define MAX_LONG_LEN 20
 #define MAX_FLOAT_LEN 40
 #define MAX_DOUBLE_LEN 45
-#define MAX_DECIMAL_LEN 30
-#define MAX_FIXED_LEN 30
 #define MAX_SMALLINT_LEN 6
 #define MAX_TINYINT_LEN 4
 #define MAX_DATETIME_LEN 30 /* Mon Dec 11 09:10:26 2006 CET*/
@@ -100,6 +103,12 @@
 #define BLOB_BLOCKSIZE 1024
 #define CLOB_BLOCKSIZE 1024
 #define STATICFIELDBUF 20 /* statically allocated buffer for field values, should be at least 8 byte */
+#define CLOB_FORMAT_LEN 30
+#define RETVAL_LEN 30
+
+/* Number of bytes which are reserved for btree objects for type fixed, decimal and bigint
+   This is the max dimension, the attributes can be defined, if btree objects should be created on it */
+#define RESERVED_BTREE_FLOATLEN 30
 
 #define MAX_CAST 20
 /******************************/
@@ -148,31 +157,43 @@
 #define LCKMNG_NUMLOCKTRIES 3
 
 /* FileHandler lock timeout in msec */
-#define FH_LOCKTIMEOUT 10000
+#define FH_LOCKTIMEOUT 30000
 
 /* BufferPool lock timeout in msec */
 
 /* this was increased becasue for large bufferpool configurations, timeouts may 
-   occur during checkpoint writes */
-#define BP_LOCKTIMEOUT 300000
+   occur during checkpoint writes, of caused by deadlocks */
+#define BP_LOCKTIMEOUT 60000
 
 /* Page lock timeout in msec */
-#define PG_LOCKTIMEOUT 10000
+#define PG_LOCKTIMEOUT 30000
 
 /* Record lock timeout in msec */
-#define RC_LOCKTIMEOUT 10000
+#define RC_LOCKTIMEOUT 30000
 
 /* XMLSpace lock timeout in msec */
-#define XS_LOCKTIMEOUT 10000
+#define XS_LOCKTIMEOUT 30000
 
-/* Transaction manager lock timeout in msec */
-#define TS_LOCKTIMEOUT 10000
+/* Db Thread Query History lock timeout in msec */
+#define QH_LOCKTIMEOUT 30000
+
+/* Date Format lock timeout in msec */
+#define DF_LOCKTIMEOUT 30000
+
+/* Log manager lock timeout in msec */
+#define LM_LOCKTIMEOUT 30000
 
 /* Database Manager lock timeout in msec */
-#define DBM_LOCKTIMEOUT 10000
+#define DBM_LOCKTIMEOUT 30000
+
+/* Database Manager number of lock tries */
 #define DBM_MAXLOCKTRIES 30
-/* Database manager lock delay timeout in msec, must be less 1000 ! */
-#define DBM_LOCKDELAY 800
+
+/* Database manager lock delay timeout in msec */
+#define DBM_LOCKDELAY 2500
+
+/* Tableset Manager stop delay to await save hanging query termination */
+#define TS_STOPDELAY 3
 
 #define RECLOCKGROUP "RECLOCK"
 #define RBRECLOCKGROUP "RBRECLOCK"
@@ -184,6 +205,11 @@
 #define DATAFILELOCKGROUP "DATAFILELOCK"
 #define POOLLOCKGROUP "POOLLOCK"
 
+/* max number of object usage for any object in parallel
+   this also limits the recursive depth for any user defined function/procedure call */
+
+#define MAX_OBJECT_USAGE 200
+
 /*************************************/
 /* Buffer Pool Management Parameters */
 /*************************************/
@@ -212,29 +238,41 @@
    The reset is triggered after BUPMNG_STATSPERIOD bufferFix operations have been performed */  
 
 #define BUPMNG_STATSPERIOD 100000
-#define BUPMNG_STATDTFORMAT "%T %Y-%m-%d %Z"
-#define BUPMNG_STATOUTFORMAT "%d.%m.%Y %H:%M:%S"
+#define BUPMNG_STATDTFORMAT "%d.%m.%Y %H:%M:%S"
 
+/* number of tries to fix a buffer by checkpointing */
 #define BUPMNG_FIXLOOP 3
 
+/* maximum number of released pages in the checkpoint queue */
+#define BUPMNG_MAXPAGEDELETE 500
+
+/* maximum number of released pages in the checkpoint queue */
+#define BUPMNG_RELOCATION_THRESHOLD 10
+
 /* timeout value for synchronous checkpoint escape command ( in seconds ) */
 #define ESCCMDTIMEOUT 60
 
 /* timeout value for logfile archiving ( in seconds ) */
 #define LOGARCHTIMEOUT 60
 
+#define CHECKPOINTINTERVAL 3600
+#define CHECKPOINTDUMPFILE "cpdump"
+#define DUMP_PROCESS_POSTFIX ".dip"
+#define DUMP_READY_POSTFIX ".rdy"
+
 /**********************/
 /* Network Parameters */
 /**********************/
 
 // size info buffer len
+// don't chane this value, since it must correlate with several DB driver implementation ( DBD, JDBC ) 
 #define NETMNG_SIZEBUFLEN 10
 
 // msg timeout for select system call in usec
-#define NETMNG_SELECT_TIMEOUT 5000000
+#define NETMNG_SELECT_TIMEOUT 3000000 // 3 sec
 
-// message queue delay to avoid CPU load (  in nano sec )
-#define NETMNG_QUEUE_DELAY 1000000
+// message queue delay to avoid CPU load in usec
+#define NETMNG_QUEUE_DELAY 200000 // 200 msec
 
 // msg timeout for recv system call in sec
 #define NETMNG_SEND_TIMEOUT 180
@@ -248,21 +286,32 @@
 // maximum net msg size
 #define NETMNG_MSG_BUFLEN 4096 
 
+// max send len of network msg
+// it has been observed, that on FreeBSD based systems, this size should be set to a higher value,
+// if large network messages occur. Otherwise, network performance is throttled down, since for a single message,
+// subsequent ::send socket calls have to be done. On Mac OSX this behaviour has not been observed
+// A value of 8k seems to be appropriate
+#define NETMNG_MAXSENDLEN 8192
+
 /* prefix sign for unique col identification */
 #define NETMNG_COLPREFIX "c"
 
-/* the number of tuples, which are transferd with one socket send call */
-#define NETMNG_MAXTUPLECOUNT 100
+/* the maximum number of tuples, which are transferd with one socket send call */
+#define NETMNG_MAXTUPLECOUNT 500
+#define NETMNG_MAXBYTECOUNT 10000000
 
 /* heart beat of managed nodes to send status request to mediator ( in seconds )  */
 #define NETMNG_BEATDELAY 15
 
-/* check interval to check for expired db connections */
+/* check interval to check for expired db connections in seconds */
 #define NETMNG_DBHANDLE_CHK 10
 
-/* time to life for a created db session hndler in seconds */
+/* time to life for a created db session handler in seconds */
 #define NETMNG_DBHANDLE_TTL 60
 
+// maximum number of requests in poolconnection queue
+#define NETMNG_MAXQUEUELEN 10
+
 /*******************/
 /* Pool Parameters */
 /*******************/
@@ -285,8 +334,6 @@
 
 /* name of variable, to store exception information message */
 #define EXCEPINFO "excep_info"
-#define INVALID_OBJECT_EXCEP_ID "invalid_obj"
-#define INVALID_ATTRIBUTE_EXCEP_ID "invalid_attr"
 #define COREOP_EXCEP_ID "core_op"
 #define OTHER_EXCEP_ID "other"
 #define ANY_EXCEP_ID "any"
@@ -309,6 +356,8 @@
 /* export/import predicate description buffer */
 #define XP_MAXPDBUF 10000
 #define XP_ROWINTERVAL 5000
+/* maximum length of a single row */
+#define XP_MAXROWBUF 20000
 
 /********************/
 /* field value defs */
@@ -316,11 +365,12 @@
 
 #define BLOBCHUNKSIZE 1024
 #define CLOBCHUNKSIZE 1024
-#define LOBSEP "[],"
-
-#define DEFAULTDATEFORMAT "%d.%m.%Y"
-#define DEFAULTDATETIMEFORMAT "%d.%m.%Y %H:%M:%S"
-
+#define LOBSEP "[]"
+#define DEFAULTDATETIMEFORMAT1 "%d.%m.%Y %H:%M:%S"
+#define DEFAULTDATETIMEFORMAT2 "%d.%m.%Y"
+#define DEFAULTDATETIMEFORMAT3 "%Y-%m-%d %H:%M:%S"
+#define DEFAULTDATETIMEFORMAT4 "%Y-%m-%d"
+#define DEFAULTINDENT "   "
 
 /*********************/
 /* system table defs */
@@ -336,8 +386,6 @@
 #define SYSTAB_NAME_ATTR "name"
 #define SYSTAB_SIZE_ATTR "size"
 #define SYSTAB_STATUS_ATTR "status"
-
-
 #define SYSTAB_BUSTAT_ID "bustat"
 #define SYSTAB_TS_ATTR "msgts"
 #define SYSTAB_BUINFO_ATTR "buinfo"
@@ -358,10 +406,12 @@
 /*****************/
 
 #define SELECTION_WILDCARD "*"
+#define MOD_QUERY_ID "MOD_QUERY"
 
 /*****************/
 /* info defs */
 /*****************/
+
 #define INFO_TABLESET "Tableset"
 #define INFO_RUNSTATE "RunState"
 #define INFO_SYNCSTATE "SyncState"
@@ -372,40 +422,43 @@
 #define INFO_TICKET "Ticket"
 #define INFO_TSINITFILE "InitFile"
 #define INFO_CHECKPOINT "Checkpoint"
-
+#define INFO_LOGUSER "LogUser"
 #define INFO_SYSTEMPAGETOTAL "SystemPageTotal"
 #define INFO_SYSTEMPAGEUSED "SystemPageUsed"
-
 #define INFO_TEMPPAGETOTAL "TempPageTotal"
 #define INFO_TEMPPAGEUSED "TempPageUsed"
-
 #define INFO_APPPAGETOTAL "AppPageTotal"
 #define INFO_APPPAGEUSED "AppPageUsed"
-
 #define INFO_SORTAREASIZE "SortAreaSize"
 #define INFO_TID "Tid"
 #define INFO_CLSN "CLSN"
-#define INFO_NLSN "NLSN"
+#define INFO_WLSN "WLSN"
 #define INFO_ARCHMODE "ArchMode"
 #define INFO_AUTOCORRECT "AutoCorrect"
+#define INFO_APPENDMODE "AppendMode"
+#define INFO_CPDUMP "CP Dump"
 #define INFO_TABLECACHE "TableCache"
 #define INFO_QUERYCACHE "QueryCache"
-
-#define INFO_TABLECACHEMAXENTRY "TC MaxEntryNum"
-#define INFO_TABLECACHEMAXSIZE "TC MaxEntrySize"
+#define INFO_TABLECACHEFILTER "TC Filter"
+#define INFO_TABLECACHEMAXSIZE "TC MaxSize"
+#define INFO_TABLECACHEMAXENTRY "TC MaxEntry"
+#define INFO_TABLECACHEHASHRANGE "TC HashRange"
 #define INFO_TABLECACHEUSEDSIZE "TC UsedSize"
-#define INFO_QUERYCACHEMAXENTRY "QC MaxEntryNum"
+#define INFO_QUERYCACHEMAXENTRY "QC MaxEntry"
 #define INFO_QUERYCACHEMAXSIZE "QC MaxEntrySize"
+#define INFO_QUERYCACHETHRESHOLD "QC Threshold"
+#define INFO_QUERYCACHEHASHRANGE "QC HashRange"
+#define INFO_QUERYCACHEUSEDNUM "QC UsedNum"
 #define INFO_QUERYCACHEUSEDSIZE "QC UsedSize"
-
+#define INFO_QUERYCACHENUMFAIL "QC NumFail"
 #define INFO_LOGFILE "LogFile"
 #define INFO_LOGSIZE "LogSize"
-
-#define INFO_MAXCACHEIDLEN 300
+#define INFO_MAXCACHEIDLEN 30
 
 /*****************/
 /* role defs */
 /*****************/
+
 #define ROLE_ADMIN "admin"
 #define ROLE_JDBC "jdbc"
 
@@ -422,5 +475,24 @@
 #define XML_VERSION_ATTR "version"
 #define XML_VERSION_VALUE "1.0"
 
-#endif
+#define BUBRANCHFORMAT "%Y%m%d%H%M%S"
+#define MAX_CLUSTERED_INSERT 300
 
+/*************************/
+/* procedure query cache */
+/*************************/
+
+#define PROCQUERYCACHE_MAXNUM 5
+
+/*************************/
+/* query and table cache */
+/*************************/
+
+#define QUERYCACHE_MAXTRY 7
+/* delay in msec */
+#define QUERYCACHE_TRYDELAYBASE 10
+#define TABLECACHE_MAXTRY 7
+/* delay in msec */
+#define TABLECACHE_TRYDELAYBASE 10
+
+#endif
