@@ -6,7 +6,7 @@
 //      
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -20,7 +20,6 @@
 
 // LFC INCLUDES
 #include <lfcbase/Net.h>
-// #include <lfcbase/Sleeper.h>
 
 // CEGO INCLUDES
 #include "CegoDistCursor.h"
@@ -36,7 +35,6 @@ CegoDistCursor::CegoDistCursor()
 
 CegoDistCursor::CegoDistCursor(CegoDistManager* pGTM, CegoContentObject *pCO)
 {
-    
     _isLocal = true;
     _moreTuple = false;
     
@@ -52,29 +50,42 @@ CegoDistCursor::CegoDistCursor(CegoDistManager* pGTM, CegoContentObject *pCO)
     
     _pDBMng = _pGTM->getDBMng(); 
 
+    _useCache = false;
     _pCache = _pDBMng->getTableCache(_tabSetId);
     _isCached = false;
     _pCacheArray = 0;
     _pCacheList = 0;
     _pCO = pCO;
-    
+
+    _pTCTab = 0;
+    _pTO = 0;
     _pTCLeft = 0;
     _pTCRight = 0;
     _pTC = 0;
     _pC = 0;
 
+    _pFLA = 0;
+    _pFLAmap = 0;
+    
     _idxMatch = CegoAttrCond::INAPP;
 
     checkType();
     
     _modId = _pDBMng->getModId("CegoDistCursor");
-
 }
 
 CegoDistCursor::~CegoDistCursor()
 {
-    
-    finishCaching();
+    try
+    {
+	finishCaching();
+    }
+    catch ( Exception e )
+    {
+	Chain msg;
+	e.pop(msg);
+	_pDBMng->log(_modId, Logger::NOTICE, Chain("Cannot finish caching ( ignored ) : " + msg ));
+    }
     
     if ( _pTC )
     {
@@ -86,43 +97,33 @@ CegoDistCursor::~CegoDistCursor()
     }
     if ( _pSelect )
     {
-
-	try
-	{
-#ifdef CGDEBUG
-	_pDBMng->log(_modId, Logger::DEBUG, Chain("Cleaning up select ..."));
-#endif
-	    _pSelect->cleanUp();
-	}
-	catch ( Exception e )
-	{
-	    Chain msg;
-	    e.pop(msg);
-	    _pDBMng->log(_modId, Logger::LOGERR, Chain("Ignoring error : ") + msg);
-	}
+	delete _pSelect;			
     }
     if ( _pCO->getType() == CegoObject::JOIN )
     {
-
-	CegoJoinObject *pJCO = (CegoJoinObject*)_pCO;
-
-	CegoContentObject *pCOLeft = pJCO->getLeftObject();
-	CegoContentObject *pCORight = pJCO->getRightObject();
-
 #ifdef CGDEBUG
+	CegoJoinObject *pJCO = (CegoJoinObject*)_pCO;
+	CegoContentObject *pCOLeft = pJCO->getLeftObject();
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Releasing global table cursor for ") + pCOLeft->getTabName());
-#endif
-	
+#endif	
 	delete _pTCLeft;
 	
 #ifdef CGDEBUG
+	CegoContentObject *pCORight = pJCO->getRightObject();
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Releasing global table cursor for ") + pCORight->getTabName());
-#endif
-	
-	delete _pTCRight;
-		
+#endif	
+	delete _pTCRight;		
     }
 
+    if ( _pCO->getType() == CegoObject::ALIAS )
+    {
+	if ( _pTCTab )
+	    delete _pTCTab;
+	if ( _pTO )
+	    delete _pTO;
+	if ( _pFLAmap )
+	    delete _pFLAmap;
+    }
     if ( _pC )
     {
 #ifdef CGDEBUG
@@ -131,32 +132,46 @@ CegoDistCursor::~CegoDistCursor()
 	delete _pC;
     }
 
-    unuseCursorObject();
-    
-    if ( _pSH )
+    try
     {
-	_pDBMng->releaseSession(_pSH);
+	unuseCursorObject();
+    }
+    catch ( Exception e )
+    {
+	Chain msg;
+	e.pop(msg);
+	_pDBMng->log(_modId, Logger::NOTICE, Chain("Cannot unuse cursor object  ( ignored ) : " + msg ));
+    }
+    
+    try
+    {
+	if ( _pSH )
+	{
+	    _pDBMng->releaseSession(_pSH);
+	}	
+    }
+    catch ( Exception e )
+    {
+	Chain msg;
+	e.pop(msg);
+	_pDBMng->log(_modId, Logger::NOTICE, Chain("Cannot release session  ( ignored ) : " + msg ));
     }
-#ifdef CGDEBUG
-    _pDBMng->log(_modId, Logger::DEBUG, Chain("Destructor for DistCursor finished"));
-#endif
 
     if ( _pCacheList )
     {
 	delete _pCacheList;
     }
-
+	
     if ( _pCacheArray && _pCache )
     {
-	// FOR DEBUGGING : sleep for a while
-	// cout << "Sleeping for 10 sec before destructing cursor  ..." << endl;
-	// Sleeper s;
-	// s.secSleep(10);
-
-	_pCache->releaseEntry(_tabSetId, _tableName);
+	_pCache->releaseEntry(_tableName);
 	_pCacheArray=0;
 	_isCached = true;
     }
+	
+#ifdef CGDEBUG
+    _pDBMng->log(_modId, Logger::DEBUG, Chain("Destructor for DistCursor finished"));
+#endif
 }
 
 void CegoDistCursor::useCursorObject()
@@ -165,11 +180,12 @@ void CegoDistCursor::useCursorObject()
     {
 	if ( _pCO->getType() == CegoObject::VIEW )
 	{
-	    _pDBMng->useObject(_tabSetId, _tableName, CegoObject::VIEW, CegoDatabaseManager::SHARED, _pGTM->getThreadId());	
+	    _pDBMng->useObject(_tabSetId, _tableName, CegoObject::VIEW, CegoDatabaseManager::SHARED, _pGTM);
 	}
-	else if ( _pCO->getType() == CegoObject::TABLE )
+	else if ( _pCO->getType() == CegoObject::TABLE
+		  || _pCO->getType() == CegoObject::ALIAS )
 	{
-	    _pDBMng->useObject(_tabSetId, _tableName, CegoObject::TABLE, CegoDatabaseManager::SHARED, _pGTM->getThreadId());	
+	    _pDBMng->useObject(_tabSetId, _tableName, CegoObject::TABLE, CegoDatabaseManager::SHARED, _pGTM);
 	}
 	else if ( _pCO->getType() == CegoObject::JOIN )
 	{
@@ -178,9 +194,9 @@ void CegoDistCursor::useCursorObject()
 	    CegoContentObject *pCORight = pJCO->getRightObject();
 
 	    if ( pCOLeft->getType() == CegoObject::VIEW || pCOLeft->getType() == CegoObject::TABLE )
-		_pDBMng->useObject(pCOLeft->getTabSetId(), pCOLeft->getTabName(), pCOLeft->getType(), CegoDatabaseManager::SHARED, _pGTM->getThreadId());	
+		_pDBMng->useObject(pCOLeft->getTabSetId(), pCOLeft->getTabName(), pCOLeft->getType(), CegoDatabaseManager::SHARED, _pGTM);
 	    if ( pCORight->getType() == CegoObject::VIEW || pCORight->getType() == CegoObject::TABLE )
-		_pDBMng->useObject(pCORight->getTabSetId(), pCORight->getTabName(), pCORight->getType(), CegoDatabaseManager::SHARED, _pGTM->getThreadId());	
+		_pDBMng->useObject(pCORight->getTabSetId(), pCORight->getTabName(), pCORight->getType(), CegoDatabaseManager::SHARED, _pGTM);
 	}
 
 	_cursorObjectUsed = true;
@@ -195,7 +211,8 @@ void CegoDistCursor::unuseCursorObject()
 	{
 	    _pDBMng->unuseObject(_tabSetId, _tableName, CegoObject::VIEW);	
 	}
-	else if ( _pCO->getType() == CegoObject::TABLE )
+	else if ( _pCO->getType() == CegoObject::TABLE
+		  || _pCO->getType() == CegoObject::ALIAS )
 	{
 	    _pDBMng->unuseObject(_tabSetId, _tableName, CegoObject::TABLE);	
 	}
@@ -210,7 +227,6 @@ void CegoDistCursor::unuseCursorObject()
 	    if ( pCORight->getType() == CegoObject::VIEW || pCORight->getType() == CegoObject::TABLE )
 		_pDBMng->unuseObject(pCORight->getTabSetId(), pCORight->getTabName(), pCORight->getType());	
 	}
-
 	_cursorObjectUsed = false;
     }
 }
@@ -221,7 +237,7 @@ void CegoDistCursor::checkType()
     
     if ( _pCO->getType() == CegoObject::VIEW )
     {
-
+	
 #ifdef CGDEBUG
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("View ") + _tableName + Chain(" detected as the cursor source"));
 #endif
@@ -235,19 +251,29 @@ void CegoDistCursor::checkType()
 	catch ( Exception e )
 	{
 	    _pDBMng->unuseObject(_tabSetId, _tableName, CegoObject::VIEW);
-	    throw e;
+	    throw Exception(EXLOC, Chain("Cannot get view object ") + _tableName, e);
 	}
-	_pSelect = pView->getSelect();
+
+	// since a view might be used several times inside a single query, we have to use a clone	
+	_pSelect = pView->getSelect()->clone(false);
 	_pSelect->cleanUp();
-	
+    }
+    else if ( _pCO->getType() == CegoObject::ALIAS )
+    {
+#ifdef CGDEBUG
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Alias ") + _tableAlias + Chain(" detected as the cursor source"));
+#endif	
+	Chain tableSet = _pDBMng->getTabSetName(_tabSetId);
+	_pTO = new CegoTableObject();
+	_pGTM->getDistObject(tableSet, _tableName, CegoObject::TABLE, *_pTO);
+
+	_pTCTab = new CegoDistCursor(_pGTM, _pTO);	
     }
     else if ( _pCO->getType() == CegoObject::TABLE )
     {
-
 #ifdef CGDEBUG
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Table ") + _tableName + Chain(" detected as the cursor source"));
 #endif
-
 	Chain tableSet = _pDBMng->getTabSetName(_tabSetId);
 
 	CegoTableObject oe;
@@ -255,11 +281,9 @@ void CegoDistCursor::checkType()
 	
 	if ( oe.isLocal() )
 	{
-
 #ifdef CGDEBUG
 	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Native Table ") + _tableName + Chain(" detected as the cursor source"));
 #endif
-
 	    _pTC = new CegoTableCursor(_pGTM, _tabSetId, _tableName, false);	
 
 	    _cacheSchema = oe.getSchema();
@@ -267,7 +291,6 @@ void CegoDistCursor::checkType()
 	}
 	else
 	{
-
 #ifdef CGDEBUG
 	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Remote Table ") + _tableName + Chain(" detected as the cursor source"));
 #endif
@@ -288,43 +311,43 @@ void CegoDistCursor::checkType()
 	    _pSH = _pDBMng->allocateSession(hostName, tableSet, user, password);
 	    	    
 	    _pSH->reqTableDataOp(tabSetId, _tableName,  CegoObject::TABLE);
-
 	}      
     }
     else if ( _pCO->getType() == CegoObject::JOIN )
     {	
-
 	CegoJoinObject *pJCO = (CegoJoinObject*)_pCO;
 	CegoContentObject *pCOLeft = pJCO->getLeftObject();
 	CegoContentObject *pCORight = pJCO->getRightObject();
 	
 	_pTCLeft = new CegoDistCursor(_pGTM, pCOLeft);
 	_pTCRight = new CegoDistCursor(_pGTM, pCORight);
-	
     }
 
     // we have to release object here to avoid dead locks
     unuseCursorObject();    
 }
 
-void CegoDistCursor::distSetup()
+void CegoDistCursor::distSetup( ListT<CegoField>** pFLA)
 {
-
     _isFirst=true;
     _doEval = false;
     _idxMatch = CegoAttrCond::INAPP;
     _isAttrCondValid = false;
-
+    _pFLA = pFLA;
+    
     if ( _pCO->getType() == CegoObject::VIEW )
     {
 	_pSelect->setTabSetId(_tabSetId);
 	CegoAttrCond noCond;
-	_pSelect->setViewCond(noCond);
-	_pSelect->prepare();
-	_pSelect->setParentJoinBuf();
 
+	_pSelect->setViewCond(noCond, pFLA);
+	_pSelect->prepare();
 	_pSelect->checkValidRef();
     }
+    else if ( _pCO->getType() == CegoObject::ALIAS )
+    {
+	_pTCTab->distSetup(_pFLA);
+    }
     else if ( _pCO->getType() == CegoObject::TABLE )
     {	
 	CegoAttrCond attrCond; // empty cond
@@ -332,22 +355,24 @@ void CegoDistCursor::distSetup()
 	
 	if ( _pCache )
 	{
-
 	    _useCache = true;
 	    
 	    finishCaching();
 
-	    _pCacheArray = _pCache->claimEntry( _tabSetId, _tableName, _cacheRows, _cacheCols);
-
-	    if ( _pCacheArray )
+	    if ( _pCache->matchEntry(_tableName) )
 	    {
-		_isCached = true;
-	    }
-	    else
-	    {
-		_isCached = false;
-		_cacheEntrySize = 0;
-		_pCacheList = new ListT< ListT< CegoFieldValue > >;
+		_pCacheArray = _pCache->claimEntry( _tableName, _cacheRows, _cacheCols);
+		
+		if ( _pCacheArray )
+		{
+		    _isCached = true;
+		}
+		else
+		{
+		    _isCached = false;
+		    _cacheEntrySize = 0;
+		    _pCacheList = new ListT< ListT< CegoFieldValue > >;
+		}
 	    }
 	}	
     }
@@ -357,43 +382,61 @@ void CegoDistCursor::distSetup()
     }
     else if ( _pCO->getType() == CegoObject::JOIN )
     {
-	CegoAttrCond attrCond;	
+	CegoAttrCond attrCond;
 	joinSetup(attrCond);
     }
 }
 
-void CegoDistCursor::distSetup( const CegoAttrCond& attrCond)
+void CegoDistCursor::distSetup( CegoAttrCond& attrCond, ListT<CegoField>** pFLA)
 {
+    
     if ( attrCond.numComp() == 0 )
-	return distSetup();
-
+    {
+	return distSetup(pFLA);
+    }
     _isAttrCondValid = false;
     _idxMatch = CegoAttrCond::INAPP;
     _isFirst=true;
     _doEval = false;
+    _pFLA = pFLA;
     
     if ( _pCO->getType() == CegoObject::VIEW )
     {	
 #ifdef CGDEBUG
-	_pDBMng->log(_modId, Logger::DEBUG, Chain("Setting view condition ") + attrCond.toChain());
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Setting condition for view ")
+		     + _tableName
+		     + Chain(" [") + _tableAlias + Chain("] to ") + attrCond.toChain());
 #endif
-
-	_pSelect->setViewCond(attrCond);
+	
+	// first setup tabSetId with anabling / disabling quey cache
 	_pSelect->setTabSetId(_tabSetId);
+	
+	// we setup view condition with the given attrcond
+	// if cannot be setup completely ( aggregation in expression ),
+	// we force a dedicated evaluation
+
+	if ( _pSelect->setViewCond(attrCond, pFLA) )
+	    _doEval=false;
+	else
+	    _doEval=true;
+
 	_pSelect->prepare();
-	_pSelect->setParentJoinBuf();
 
 	_pSelect->checkValidRef();
 
 	_cursorCond = attrCond;
-
-	_doEval=false;
+    }
+    else if ( _pCO->getType() == CegoObject::ALIAS )
+    {
+	CegoAliasObject *pAO = (CegoAliasObject*)_pCO;
+	CegoAttrCond mapAttrCond = CegoQueryHelper::mapAttrCond(attrCond, pAO);
+	
+	_pTCTab->distSetup(mapAttrCond, pFLA);
     }
     else if ( _pCO->getType() == CegoObject::TABLE )
     {
 	if ( _isLocal == true )
 	{
-
 	    if ( _pCache )
 	    {
 		// check if a previous cursor trace has to be finished to complete cache 
@@ -401,35 +444,36 @@ void CegoDistCursor::distSetup( const CegoAttrCond& attrCond)
 	    }
 	    
 	    _useCache = false;
-		
+
 	    _idxMatch = _pTC->setup(attrCond);
 	    
 	    if ( _idxMatch != CegoAttrCond::FULL )
-	    {	       
+	    {
 		_doEval=true;
 
 		// if index match was inappropriate, all tuples are retrieved from table cursor
 		// In this case, we can use cache
 		if ( _pCache && _idxMatch == CegoAttrCond::INAPP )
 		{
-		    _useCache = true;
-		    _pCacheArray = _pCache->claimEntry( _tabSetId, _tableName, _cacheRows, _cacheCols);
-		    
-		    if ( _pCacheArray )
+		    if ( _pCache->matchEntry(_tableName) )	   
 		    {
-			_isCached = true;
-		    }
-		    else
-		    {
-			_isCached = false;
-			_cacheEntrySize = 0;
-			_pCacheList = new ListT< ListT< CegoFieldValue > >;
+			_useCache = true;
+			_pCacheArray = _pCache->claimEntry( _tableName, _cacheRows, _cacheCols);
+			
+			if ( _pCacheArray )
+			{
+			    _isCached = true;
+			}
+			else
+			{
+			    _isCached = false;
+			    _cacheEntrySize = 0;
+			    _pCacheList = new ListT< ListT< CegoFieldValue > >;
+			}
 		    }
 		}
 	    }
-	    
-	    _cursorCond = attrCond;
-	   
+	    _cursorCond = attrCond;  
 	}
     }
     else if ( _pCO->getType() == CegoObject::SYSTEM )
@@ -440,22 +484,43 @@ void CegoDistCursor::distSetup( const CegoAttrCond& attrCond)
     }
     else if ( _pCO->getType() == CegoObject::JOIN )
     {
+
 #ifdef CGDEBUG
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Setting join condition ") + attrCond.toChain());	
 #endif
-
 	_cursorCond = attrCond;
-	_doEval = true;
+
+	// for inner joins, predicate is evaluated inside subjoin, so no need for evaluation	
 	joinSetup(attrCond);
     }
 }
 
 bool CegoDistCursor::nextTuple(ListT<CegoField>** pFLA, int pos, int size)
 {
+    if ( _pGTM->isAborted() )
+    {
+	throw Exception(EXLOC, "Query aborted");
+    }
+    
     useCursorObject();
     
     while ( getTuple(pFLA, pos, size) )
-    {	
+    {
+	/*
+	cout << "### TUPLE = ";
+	for ( int i=0; i<size; i++)
+	{
+	    
+	    CegoField *pF = pFLA[pos+i]->First();
+	    while ( pF )
+	    {
+		cout << pF->getTableAlias() << "." << pF->getAttrName() << "=" << pF->getValue() << " ";
+		pF = pFLA[pos+i]->Next();
+	    }		
+	}
+	cout << endl;
+	*/
+	
 	if ( _doEval == true )
 	{
 	    if ( evalCondition(pFLA, pos, size) )
@@ -463,7 +528,6 @@ bool CegoDistCursor::nextTuple(ListT<CegoField>** pFLA, int pos, int size)
 #ifdef CGDEBUG
 		_pDBMng->log(_modId, Logger::DEBUG, Chain("Returning tuple with eval condition"));
 #endif
-
 		return true;
 	    }
 	}
@@ -472,11 +536,9 @@ bool CegoDistCursor::nextTuple(ListT<CegoField>** pFLA, int pos, int size)
 #ifdef CGDEBUG
 	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Returning tuple without eval condition"));
 #endif
-
 	    return true;
 	}
     }
-
     return false;
 }
 
@@ -486,16 +548,18 @@ bool CegoDistCursor::evalCondition(ListT<CegoField>** pFLA, int pos, int size)
     
     while ( pAC )
     {
-
+	// cout << "Eval condition " << *pAC << endl;	
+	// cout << "Condition " << pAC->getTableName() << " . " <<  pAC->getAttrName() << endl;
+	
 	CegoField *pF = 0;
 	    
 	int i=0;
 	while ( pF == 0 && i < size)
-	{	   
+	{
 	    if ( _pCO->getType() == CegoObject::VIEW || _pCO->getType() == CegoObject::TABLE )
 		pF = pFLA[pos+i]->Find( CegoField(_tableAlias, pAC->getAttrName()));
 	    else
-		pF = pFLA[pos+i]->Find( CegoField(Chain(), pAC->getAttrName()) );
+		pF = pFLA[pos+i]->Find( CegoField(pAC->getTableName(), pAC->getAttrName()) );
 	    i++;
 	}
 	
@@ -520,11 +584,12 @@ bool CegoDistCursor::evalCondition(ListT<CegoField>** pFLA, int pos, int size)
 		    return false;		
 	    }
 	    else
-	    {
+	    {		
 		switch ( pAC->getComparison() )
 		{	    
 		case EQUAL:
 		{
+		    // cout << "Eval " << pF->getValue() << " = " << pAC->getFieldValue() << endl;
 		    if ( ( (CegoFieldValue)pF->getValue() == (CegoFieldValue)(pAC->getFieldValue()) ) == false )		    
 			return false;		    
 		    break;
@@ -580,16 +645,40 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
     if ( _pCO->getType() == CegoObject::VIEW )
     {
 	try 
-	{	    
-	    _moreTuple = _pSelect->nextTuple(*pFLA[offset]);
-	    
+	{
+	    ListT<CegoField> fvl;
+	    _moreTuple = _pSelect->nextTuple(fvl);
+
+	    /*
+	    cout << "FVL Size = " << fvl.Size() << endl;
+
+	    CegoField *pX = fvl.First();
+	    while ( pX ) 
+	    {
+		cout << "Alias = " << pX->getTableAlias() << " Attr =" << pX->getAttrName() << " " << pX->getValue() << endl;
+		
+		pX = fvl.Next();
+	    }
+	    */
+		    
 	    if ( _moreTuple )
 	    {
 		CegoField *pF = pFLA[offset]->First();
 		while ( pF ) 
 		{
-		    pF->setTableName(_tableName);
-		    pF->setTableAlias(_tableAlias);
+		    // cout << "Checking FLA .. Alias =" << pF->getTableAlias() << " Attr = " << pF->getAttrName() << endl;
+		    
+		    CegoField *pSF = fvl.First();
+		    bool notFound=true;
+		    while ( pSF && notFound)
+		    {
+			if ( pSF->getAttrName() == pF->getAttrName() )
+			{
+			    pF->setValue(pSF->getValue());
+			    notFound=false;
+			}
+			pSF = fvl.Next();
+		    }
 		    pF = pFLA[offset]->Next();
 		}
 	    }
@@ -598,9 +687,27 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 	catch ( Exception e )
 	{
 	    _pSelect->cleanUp();
-	    throw e;
+	    throw Exception(EXLOC, Chain("Cannot get tuple from view"), e);
 	}
     }
+    else if ( _pCO->getType() == CegoObject::ALIAS )
+    {
+	CegoAliasObject *pAO = (CegoAliasObject*)_pCO;
+	
+	if ( _isFirst ) 
+	{
+	    _pFLAmap = new ListT<CegoField>();
+	    CegoQueryHelper::mapFLA(_pFLAmap, pFLA, offset, size, pAO);
+	    _isFirst = false;
+	}
+	
+	if ( _pTCTab->nextTuple(&_pFLAmap, 0, 1) )
+	{
+	    CegoQueryHelper::propFLA(_pFLAmap, pFLA, offset, size, pAO);
+	    return true;
+	}
+	return false;	
+    }
     else if ( _pCO->getType() == CegoObject::TABLE )
     {
 	if ( _isLocal )
@@ -608,7 +715,7 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 	    if ( _useCache )
 	    {
 		if ( _isCached )
-		{		    
+		{
 		    CegoFieldValue** pCacheRow = 0;
 		    if ( _isFirst ) 
 		    {
@@ -623,7 +730,7 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 		    else
 		    {
 			if ( _cachePos < _cacheRows )
-			{			    
+			{
 			    pCacheRow = _pCacheArray[_cachePos];
 			    _cachePos++;
 			}
@@ -653,28 +760,20 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 			_moreTuple = true;
 		    }
 		    else
-		    {
-			
+		    {			
 			if ( _pCacheArray && _pCache )
 			{
-			    // FOR DEBUGGING : sleep for a while
-			    // cout << "Sleeping for 10 sec before end of cursor  ..." << endl;
-			    // Sleeper s;
-			    // s.secSleep(10);
-			    
-			    _pCache->releaseEntry( _tabSetId, _tableName);
+			    _pCache->releaseEntry( _tableName);
 			    _pCacheArray = 0;
 			    _isCached = true;
 			}
-
 			_moreTuple = false;
 		    }
 
 		    return _moreTuple;
 		}
 		else // still not cached, so we have to retrieve and cache it
-		{
-
+		{		    
 		    CegoDataPointer dp;
 		    if ( _isFirst )
 		    {
@@ -732,37 +831,30 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 		    }
 		    else
 		    {
-
 			if ( _pCacheList )
 			{
 			    // put table cache entry into cache
-			    // cout << "PUT TCE " << _tableName << " into cache" << endl;
-			    _pCache->addEntry( _tabSetId, _tableName, _pCacheList );
-			    // cout << "PUT DONE" << endl;
+			    _pCache->addEntry( _tableName, _pCacheList );
 			    _isCached = true;
 			    delete _pCacheList;
 			    _pCacheList = 0;
 			}
 		    }
-
 		    return _moreTuple;
-
 		}
 	    }
 	    else // retrieve tuple via native table cursor
 	    {
-		// cout << "Retrieve native .." << endl;
 		CegoDataPointer dp;
 		if ( _isFirst )
-		{
+		{		
 		    _moreTuple = _pTC->getFirst(*pFLA[offset], dp);
-		    _isFirst = false;
+		    _isFirst = false;		  
 	        }
 		else
 		{
-		    _moreTuple = _pTC->getNext(*pFLA[offset], dp);	    
+		    _moreTuple = _pTC->getNext(*pFLA[offset], dp);		    		   
 		}
-		
 		return _moreTuple;
 	    }
 	}
@@ -781,10 +873,8 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
     }
     else if ( _pCO->getType() == CegoObject::SYSTEM )
     {
-
 	if ( _pC )
 	{
-
 	    CegoDataPointer dp;
 
 	    int len;
@@ -801,7 +891,6 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 
 	    if ( pc && len > 0 )
 	    {
-
 		unsigned long long tid;
 		unsigned long long tastep;
 		CegoTupleState ts;
@@ -822,7 +911,6 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 	}
 	else
 	{
-	
 	    Chain *pName = 0;
 	    if ( _isFirst )
 	    {
@@ -845,8 +933,7 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 		if ( _tableName == Chain(SYSTAB_TABLE_ID) 
 		     || _tableName == Chain(SYSTAB_INDEX_ID)
 		     || _tableName == Chain(SYSTAB_BTREE_ID) )
-		{
-		    
+		{	    
 		    int pageCount;
 		    
 		    if ( _tableName == Chain(SYSTAB_TABLE_ID)  )
@@ -902,7 +989,6 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 		_moreTuple = false;
 	    }
 	}
-
 	return _moreTuple;
     }
     else if ( _pCO->getType() == CegoObject::JOIN )
@@ -916,15 +1002,14 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 	CegoJoinObject *pJO = (CegoJoinObject*)_pCO;
 	
 	if ( pJO->getJoinType() == CegoJoinObject::INNER )
-	{
-	    
+	{   
 	    bool isEvaluated = false;
 	    
 	    while ( isEvaluated == false && _moreLeft )
 	    {
 		if ( _isFirst )
 		{
-		    _pTCLeft->distSetup(_outerCond);
+		    _pTCLeft->distSetup(_outerCond, _pFLA);
 		    _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 		    if ( _moreLeft )
 		    {			
@@ -933,18 +1018,19 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 			_pTCRight->reset();
 			
 			if ( _isAttrCondValid )
-			    _pTCRight->distSetup(_innerCond);
+			{
+			    _pTCRight->distSetup(_innerCond, _pFLA);
+			}
 			else
-			    _pTCRight->distSetup();
-
+			{
+			    _pTCRight->distSetup(_pFLA);
+			}
 			_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		    }
-
 		    _isFirst = false;
 		}
 		else if ( _moreRight )
 		{
-
 		    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		    if ( _moreRight == false )
 		    {
@@ -952,14 +1038,16 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 			if ( _moreLeft )
 			{
 			    _innerCond.setup(pFLA, offset);
-
 			    _pTCRight->reset();
 
 			    if ( _isAttrCondValid )
-				_pTCRight->distSetup(_innerCond);
+			    {
+				_pTCRight->distSetup(_innerCond, _pFLA);
+			    }
 			    else
-				_pTCRight->distSetup();
-
+			    {
+				_pTCRight->distSetup(_pFLA);
+			    }
 			    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 			}
 		    }
@@ -970,14 +1058,16 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 		    if ( _moreLeft )
 		    {
 			_innerCond.setup(pFLA, offset);
-
 			_pTCRight->reset();
 
 			if ( _isAttrCondValid )
-			    _pTCRight->distSetup(_innerCond);
+			{
+			    _pTCRight->distSetup(_innerCond, _pFLA);
+			}
 			else
-			    _pTCRight->distSetup();
-
+			{
+			    _pTCRight->distSetup(_pFLA);
+			}
 			_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		    }
 		}
@@ -986,85 +1076,53 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 		{
 		    if ( _evalPredicate )
 		    {		       
+			// attr cache is left for performace
+			// pJO->getPredDesc()->clearAttrCache();
 			
-			pJO->getPredDesc()->clearAttrCache();
-			
-			isEvaluated = CegoQueryHelper::evalPredicate(0,
-								     0,
-								     pFLA,
-								     offset,
-								     pJO->getPredDesc(),
-								     0);
+			isEvaluated = pJO->getPredDesc()->eval(0,
+							       0,
+							       pFLA,
+							       offset,							       
+							       0); 
 		    }
 		    else
 		    {			
 			isEvaluated = true;
 		    }
 		}
-	    }	    
-	    
+	    }
 	    return _moreLeft && _moreRight;
 	}
 	else if ( pJO->getJoinType() == CegoJoinObject::LEFTOUTER )
 	{
-
-	    bool isEvaluated = false;
-	    
-	    while ( isEvaluated == false )
+	    if ( _isFirst )
 	    {
+		_pTCLeft->distSetup(_outerCond, _pFLA);	    
+		_moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 		
-		if ( _isFirst )
+		if ( _moreLeft )
 		{
-		    _pTCLeft->distSetup(_outerCond);
+		    _innerCond.setup(pFLA, offset);
+		    _pTCRight->reset();
 		    
-		    _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
-		    
-		    if ( _moreLeft )
+		    if ( _isAttrCondValid )
 		    {
-			_innerCond.setup(pFLA, offset);
-			
-			_pTCRight->reset();
-			
-			if ( _isAttrCondValid )
-			{
-			    _pTCRight->distSetup(_innerCond);
-			}
-			else
-			{
-			    _pTCRight->distSetup();
-			}
-			
-			_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
+			_pTCRight->distSetup(_innerCond, _pFLA);
 		    }
-		    _isFirst = false;
-		}
-		else if ( _moreRight )
-		{
-		    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
-		    
-		    if ( _moreRight == false )
+		    else
 		    {
-			_moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
-			if ( _moreLeft )
-			{
-			    _innerCond.setup(pFLA, offset);
-			    
-			    _pTCRight->reset();
-			    
-			    if ( _isAttrCondValid )
-			    {
-				_pTCRight->distSetup(_innerCond);
-			    }
-			    else
-			    {
-				_pTCRight->distSetup();
-			    }
-			    
-			    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
-			}
-		    }		
+			_pTCRight->distSetup(_pFLA);
+		    }
+		    
+		    nextRight(pFLA, offset, size);
 		}
-		else if ( _moreLeft )
+		_isFirst = false;
+	    }
+	    else if ( _moreRight )
+	    {
+		nextRight(pFLA, offset, size);
+		
+		if ( _moreRight == false )
 		{
 		    _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 		    if ( _moreLeft )
@@ -1075,36 +1133,42 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 			
 			if ( _isAttrCondValid )
 			{
-			    _pTCRight->distSetup(_innerCond);
+			    _pTCRight->distSetup(_innerCond, _pFLA);
 			}
 			else
 			{
-			    _pTCRight->distSetup();
+			    _pTCRight->distSetup(_pFLA);
 			}
-			
-			_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
+
+			nextRight(pFLA, offset, size);
+
 		    }
-		}
-		
-		if ( _evalPredicate && _moreLeft && _moreRight )
-		{		    
-		    pJO->getPredDesc()->clearAttrCache();
-		    isEvaluated = CegoQueryHelper::evalPredicate(0,
-								 0,
-								 pFLA,
-								 offset,
-								 pJO->getPredDesc(),
-								 0);		   
-		}
-		else
+		}		
+	    }
+	    else if ( _moreLeft )
+	    {
+		_moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
+		if ( _moreLeft )
 		{
-		    isEvaluated = true;
+		    _innerCond.setup(pFLA, offset);
+		    _pTCRight->reset();
+		    
+		    if ( _isAttrCondValid )
+		    {
+			_pTCRight->distSetup(_innerCond, _pFLA);
+		    }
+		    else
+		    {
+			_pTCRight->distSetup(_pFLA);
+		    }
+
+		    nextRight(pFLA, offset, size);
 		}
 	    }
-	    	    
+	    
 	    if ( _moreLeft )
-	    {
-		if ( _moreRight == false )
+	    {		
+		if ( _moreRight == false  )
 		{
 		    // make right fields null
 		    CegoField *pF = pFLA[offset+size-1]->First();
@@ -1120,58 +1184,32 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 	    return false;
 	}	
 	else if ( pJO->getJoinType() == CegoJoinObject::RIGHTOUTER )
-	{
+	{    
+	    if ( _isFirst )
+	    {		
+		_pTCRight->distSetup(_outerCond, _pFLA);
+		
+		_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
+		
+		if ( _moreRight )
+		{
+		    _innerCond.setup(pFLA, offset+size-1);	    
+		    _pTCLeft->reset();
+		    
+		    if ( _isAttrCondValid )
+			_pTCLeft->distSetup(_innerCond, _pFLA);
+		    else
+			_pTCLeft->distSetup(_pFLA);
 
-	    bool isEvaluated = false;
-	    
-	    while ( isEvaluated == false )
+		    nextLeft(pFLA, offset, size);
+		}
+		_isFirst = false;
+	    }
+	    else if ( _moreLeft )
 	    {
-
-		if ( _isFirst )
-		{		
-		    _pTCRight->distSetup(_cursorCond);
-		    
-		    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
-		    
-		    if ( _moreRight )
-		    {
-			_innerCond.setup(pFLA, offset+size-1);
-
-			_pTCLeft->reset();
-			
-			if ( _isAttrCondValid )
-			    _pTCLeft->distSetup(_innerCond);
-			else
-			    _pTCLeft->distSetup();
-
-			_moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
-
-		    }
-		    _isFirst = false;
-		}
-		else if ( _moreLeft )
-		{
-		    _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
-		    
-		    if ( _moreLeft == false )
-		    {
-			_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
-			if ( _moreRight )
-			{
-			    _innerCond.setup(pFLA, offset+size-1);
-			    
-			    _pTCLeft->reset();
-			    
-			    if ( _isAttrCondValid )
-				_pTCLeft->distSetup(_innerCond);
-			    else
-				_pTCLeft->distSetup();
-			    
-			    _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
-			}
-		    }		
-		}
-		else if ( _moreRight )
+		nextLeft(pFLA, offset, size);
+		
+		if ( _moreLeft == false )
 		{
 		    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		    if ( _moreRight )
@@ -1181,28 +1219,30 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 			_pTCLeft->reset();
 			
 			if ( _isAttrCondValid )
-			    _pTCLeft->distSetup(_innerCond);
+			    _pTCLeft->distSetup(_innerCond, _pFLA);
 			else
-			    _pTCLeft->distSetup();
-			
-			_moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
-		    }
-		}
+			    _pTCLeft->distSetup(_pFLA);
 
-		if ( _evalPredicate && _moreLeft && _moreRight )
-		{		    
-		    pJO->getPredDesc()->clearAttrCache();
-		    isEvaluated = CegoQueryHelper::evalPredicate(0,
-								 0,
-								 pFLA,
-								 offset,
-								 pJO->getPredDesc(),
-								 0);		   
-		}
-		else
+			nextLeft(pFLA, offset, size);
+		    }
+		}		
+	    }
+	    else if ( _moreRight )
+	    {
+		_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
+		if ( _moreRight )
 		{
-		    isEvaluated = true;
-		}
+		    _innerCond.setup(pFLA, offset+size-1);
+		    
+		    _pTCLeft->reset();
+		    
+		    if ( _isAttrCondValid )
+			_pTCLeft->distSetup(_innerCond, _pFLA);
+		    else
+			_pTCLeft->distSetup(_pFLA);
+
+		    nextLeft(pFLA, offset, size);
+		}	   	
 	    }
 
 	    if ( _moreRight )
@@ -1227,12 +1267,64 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 	}	    
     }
 
-    throw Exception(EXLOC, Chain("Cannot resolve object type"));
+    Chain msg = Chain("Invalid cursor object ") + _pCO->getTabAlias() + Chain(" (") + _pCO->getTabName() + Chain(")");
+    throw Exception(EXLOC, msg);
+}
+
+void CegoDistCursor::nextRight(ListT<CegoField>** pFLA, int offset, int size)
+{
+    CegoJoinObject *pJO = (CegoJoinObject*)_pCO;
+	
+    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
+    
+    bool notFound = true;
+    while ( _evalPredicate && _moreRight && notFound )
+    {
+	// attr cache is left for performace
+	// pJO->getPredDesc()->clearAttrCache();
+	
+	if ( pJO->getPredDesc()->eval(0,
+				      0,
+				      pFLA,
+				      offset,
+				      0) )
+	{
+	    notFound = false;
+	}
+	else
+	{
+	    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
+	}			      
+    }
+}
+
+void CegoDistCursor::nextLeft(ListT<CegoField>** pFLA, int offset, int size)
+{
+    CegoJoinObject *pJO = (CegoJoinObject*)_pCO;
+    
+    _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
+    
+    bool notFound = true;
+    while ( _evalPredicate && _moreLeft && notFound )
+    {
+	
+	if ( pJO->getPredDesc()->eval(0,
+				      0,
+				      pFLA,
+				      offset,
+				      0) )
+	{
+	    notFound = false;
+	}
+	else
+	{
+	    _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
+	}			      
+    }
 }
 
 void CegoDistCursor::finishCaching()
-{
-    
+{    
     if ( _useCache && _pCache && _pCacheList && _isCached == false && _pTC && _isFirst == false )
     {	
 	CegoDataPointer dp;
@@ -1263,7 +1355,7 @@ void CegoDistCursor::finishCaching()
 	if ( _pCacheList )
 	{
 	    // put table cache entry into cache
-	    _pCache->addEntry( _tabSetId, _tableName, _pCacheList );
+	    _pCache->addEntry( _tableName, _pCacheList );
 	    _isCached = true;
 	    delete _pCacheList;
 	    _pCacheList = 0;
@@ -1296,13 +1388,7 @@ void CegoDistCursor::reset()
 	    
 	    if ( _pCacheArray && _pCache )
 	    {
-
-		// FOR DEBUGGING : sleep for a while
-		// cout << "Sleeping for 10 sec before cursor reset  ..." << endl;
-		// Sleeper s;
-		// s.secSleep(10);
-
-		_pCache->releaseEntry( _tabSetId, _tableName);
+		_pCache->releaseEntry( _tableName);
 		_pCacheArray = 0;
 		_isCached = 0;
 	    }
@@ -1313,6 +1399,11 @@ void CegoDistCursor::reset()
 		_pDBMng->releaseSession(_pSH);
 	}
     }
+    else if ( _pCO->getType() == CegoObject::ALIAS )
+    {
+	if ( _pTCTab )
+	    _pTCTab->reset();
+    }
     else if ( _pCO->getType() == CegoObject::JOIN )
     {	
 	if ( _pTCLeft )
@@ -1330,8 +1421,7 @@ void CegoDistCursor::reset()
 }
 
 void CegoDistCursor::sysSetup()
-{
-    
+{    
     Chain tableSet = _pDBMng->getTabSetName(_tabSetId);	
     if ( _tableName == Chain(SYSTAB_TABLE_ID) )
     {
@@ -1365,49 +1455,78 @@ void CegoDistCursor::sysSetup()
 
 void CegoDistCursor::joinSetup(const CegoAttrCond& attrCond)
 {
-    
     CegoJoinObject *pJO = (CegoJoinObject*)_pCO;
         
     ListT<CegoField> outerSchema;
     ListT<CegoField> innerSchema;
-    
-    CegoAttrCond addInnerCond;
-
-    if ( pJO->getJoinType() == CegoJoinObject::INNER || pJO->getJoinType() == CegoJoinObject::LEFTOUTER )
+       
+    if ( pJO->getJoinType() == CegoJoinObject::INNER || pJO->getJoinType() == CegoJoinObject::LEFTOUTER ) 
     {
 	outerSchema = pJO->getLeftObject()->getSchema();
-	innerSchema = pJO->getRightObject()->getSchema(); 
+	innerSchema = pJO->getRightObject()->getSchema();
 
-	_outerCond = attrCond.getFilterCond(outerSchema);
-	addInnerCond = attrCond.getFilterCond(innerSchema);
+	if ( pJO->getJoinType() == CegoJoinObject::INNER )
+	{
+	    _outerCond = attrCond.getFilterCond(outerSchema, false);
+	    _innerCond = attrCond.getFilterCond(innerSchema, false);
 
+	    _doEval = false;
+	}
+	else
+	{
+	    _outerCond = attrCond.getFilterCond(outerSchema, true);
+	    _innerCond = attrCond.getFilterCond(innerSchema, true);
+
+	    _doEval = true;
+	}    
     }
     else if (  pJO->getJoinType() == CegoJoinObject::RIGHTOUTER )
     {
 	innerSchema = pJO->getLeftObject()->getSchema();
-	outerSchema = pJO->getRightObject()->getSchema();
+        outerSchema = pJO->getRightObject()->getSchema();
 
-	_outerCond = attrCond.getFilterCond(outerSchema);
-	addInnerCond = attrCond.getFilterCond(innerSchema);
-    }
-    
-    CegoAttrCond ac;
-    
-    CegoQueryHelper::AttrCondMatch m  = CegoQueryHelper::checkAttrCond(ac, pJO->getPredDesc(), innerSchema, &outerSchema, 1, 0);
-    
-    if ( m == CegoQueryHelper::COMPLETE )
-    {
-	_evalPredicate = false;
-	_innerCond = ac + addInnerCond;
-    }
-    else
-    {
-	_evalPredicate = true;
-	_innerCond = addInnerCond;
+	_outerCond = attrCond.getFilterCond(outerSchema, true);
+	_innerCond = attrCond.getFilterCond(innerSchema, true);
+
+	// for right outer join, we have to evaluate the where condition outside the join 
+	_doEval = true;	
     }
 
-    _isAttrCondValid = true;
-    
+    // cout << "Setting up join with " << pJO->getPredDesc()->toChain() << endl;
+
+    ListT<CegoPredicate*> conjunctionList;
+
+    CegoQueryHelper::createConjunctionList(pJO->getPredDesc(), &conjunctionList);
+
+    _evalPredicate = false;
+
+    CegoPredicate **pPred = conjunctionList.First();
+    while ( pPred )
+    {
+	CegoAttrCond ac;
+	// CegoQueryHelper::AttrCondMatch m  = CegoQueryHelper::checkAttrCond(ac, pJO->getPredDesc(), innerSchema, &outerSchema, 1, 0);
+
+	CegoQueryHelper::AttrCondMatch m  = CegoQueryHelper::checkAttrCond(ac, *pPred, innerSchema, &outerSchema, 1, 0);
+
+	if ( m == CegoQueryHelper::COMPLETE )
+	{
+	    _innerCond = _innerCond + ac;
+	}
+	else if ( m == CegoQueryHelper::PARTIAL )
+	{
+	    // for Partial conditions, we have to evaluate in any case
+	    _evalPredicate = true;
+	    _innerCond = _innerCond + ac;
+	}
+	else
+	{
+	    // for inapp conditions, we have to evaluate the whole condition
+	    _evalPredicate = true;
+	}
+	pPred = conjunctionList.Next();
+    }
+
+    _isAttrCondValid = true;    
 }
 
 Element* CegoDistCursor::getPlan()
@@ -1425,6 +1544,7 @@ Element* CegoDistCursor::getPlan()
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Getting view plan for ") + _tableAlias);
 #endif
 
+	
 	pCursorPlan->addContent( _pSelect->getPlan() );
 
 
@@ -1433,9 +1553,24 @@ Element* CegoDistCursor::getPlan()
 #endif
 
     }
+    else if ( _pCO->getType() == CegoObject::ALIAS )
+    {
+	if ( _pTCTab )
+	{
+	    pCursorPlan->setAttribute(XML_TABLETYPE_ATTR, XML_ALIAS_VALUE);
+	    pCursorPlan->setAttribute(XML_NAME_ATTR, _pCO->getName());
+	    // Chain tableName = _pTCTab->getPlan()->getAttributeValue(XML_TABLENAME_ATTR);
+	    // Chain aliasName = _pTCTab->getPlan()->getAttributeValue(XML_NAME_ATTR);
+	    // Chain tableType = _pTCTab->getPlan()->getAttributeValue(XML_TABLETYPE_ATTR);
+	    // cout << "TableName=" << tableName << endl;
+	    // cout << "AliasName=" << aliasName << endl;
+	    // cout << "TableType=" << tableType << endl;
+	    
+	    pCursorPlan->addContent( _pTCTab->getPlan() );
+	}
+    }
     else if ( _pCO->getType() == CegoObject::TABLE )
     {
-
 	pCursorPlan->setAttribute(XML_TABLETYPE_ATTR, XML_TABLE_VALUE);
 	
 	if ( _idxMatch == CegoAttrCond::FULL)
@@ -1455,8 +1590,7 @@ Element* CegoDistCursor::getPlan()
 	}	
     }
     else if ( _pCO->getType() == CegoObject::JOIN )
-    {	
-
+    {
 	CegoJoinObject *pJCO = (CegoJoinObject*)_pCO;
 	
 	if ( pJCO->getJoinType() == CegoJoinObject::INNER )	    
@@ -1465,24 +1599,27 @@ Element* CegoDistCursor::getPlan()
 	    pCursorPlan->setAttribute(XML_TABLETYPE_ATTR, XML_LEFTOUTERJOIN_VALUE);
 	else if ( pJCO->getJoinType() == CegoJoinObject::RIGHTOUTER)	    
 	    pCursorPlan->setAttribute(XML_TABLETYPE_ATTR, XML_RIGHTOUTERJOIN_VALUE);
-	
+
+	if ( _evalPredicate )	    
+	    pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, Chain("with condition evaluation"));
+	else
+	    pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, Chain("no condition evaluation"));
+
 	if ( _pTCLeft )
 	{
-	    _pTCLeft->distSetup(_outerCond);
+	    _pTCLeft->distSetup(_outerCond, _pFLA);
 	}
 	if ( _pTCRight )
-	{
-	    
+	{	    
 	    if ( _isAttrCondValid )
 	    {
-		_pTCRight->distSetup(_innerCond);
+		_pTCRight->distSetup(_innerCond, _pFLA);
 	    }
 	    else
 	    {
-		_pTCRight->distSetup();
+		_pTCRight->distSetup(_pFLA);
 	    }	    
 	}
-	
 	if ( _pTCLeft )
 	    pCursorPlan->addContent( _pTCLeft->getPlan() );
 	if ( _pTCRight )
