@@ -8,7 +8,7 @@
 //      
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -40,8 +40,8 @@ public:
     CegoDistCursor(CegoDistManager* pGTM, CegoContentObject *pCO); 
     ~CegoDistCursor();
 
-    void distSetup();
-    void distSetup(const CegoAttrCond& attrCond);
+    void distSetup(ListT<CegoField>** pFLA);
+    void distSetup(CegoAttrCond& attrCond, ListT<CegoField>** pFLA);
 
     // bool nextTuple(ListT<CegoField>& fl);
     bool nextTuple(ListT<CegoField>** pFLA, int pos, int size);
@@ -61,6 +61,9 @@ public:
     bool evalCondition(ListT<CegoField>** pFLA, int pos, int size);
     bool getTuple(ListT<CegoField>** pFLA, int offset, int size);
 
+    void nextRight(ListT<CegoField>** pFLA, int offset, int size);
+    void nextLeft(ListT<CegoField>** pFLA, int offset, int size);
+
     void finishCaching();
     
     CegoDistManager* _pGTM;
@@ -78,6 +81,10 @@ public:
     CegoDistCursor* _pTCLeft;
     CegoDistCursor* _pTCRight;
 
+    CegoTableObject* _pTO;
+    CegoDistCursor* _pTCTab;
+    ListT<CegoField>* _pFLAmap;
+    
     CegoContentObject *_pCO;
     CegoDatabaseManager *_pDBMng;
     CegoDistDbHandler* _pSH; 
@@ -116,5 +123,7 @@ public:
     CegoFieldValue*** _pCacheArray;
     ListT< ListT<CegoFieldValue> >* _pCacheList;
     bool _isCached;
+
+    ListT<CegoField>** _pFLA;
 };
 #endif
