@@ -6,7 +6,7 @@
 //                                                
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -18,7 +18,6 @@
 //
 ///////////////////////////////////////////////////////////////////////////////
 
-
 // CEGO INCLUDES
 #include "CegoExpr.h"
 #include "CegoXMLdef.h"
@@ -28,12 +27,13 @@
 #include <string.h>
 #include <stdlib.h>
 
-CegoExpr::CegoExpr(char* buf, CegoDistManager *pGTM, int tabSetId)
+CegoExpr::CegoExpr(char* buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId)
 {
     _pTerm=0;
     _pExpr=0;
     _isAgg = false;
-    decode(buf, pGTM, tabSetId);
+    _isExtern = false;
+    decode(buf, pGTM, pBlock, tabSetId);
 }
 
 CegoExpr::CegoExpr()
@@ -41,31 +41,27 @@ CegoExpr::CegoExpr()
     _pTerm=0;
     _pExpr=0;
     _isAgg = false;
+    _isExtern = false;
 }
 
-CegoExpr::CegoExpr(Element* pExpElement, CegoDistManager *pGTM)
-{
-    _pTerm=0;
-    _pExpr=0;
-    _isAgg = false;
-    fromElement(pExpElement, pGTM);
-}
-
-CegoExpr::CegoExpr(CegoExpr *pExpr, CegoTerm *pTerm, ExpType expType)
+CegoExpr::CegoExpr(CegoExpr *pExpr, CegoTerm *pTerm, ExpType expType, const Chain& alias)
 {
     _expType = expType;
     _pExpr=pExpr;
     _pTerm=pTerm;
     _isAgg = false;
-    _alias = Chain();
+    _alias = alias;
+    _isExtern = false;
 }
 
-CegoExpr::CegoExpr(CegoTerm *pTerm)
+CegoExpr::CegoExpr(CegoTerm *pTerm, const Chain& alias)
 {
     _expType = TERM;
     _pTerm=pTerm;
     _pExpr = 0;
     _isAgg = false;
+    _alias = alias;
+    _isExtern = false;
 }
 
 CegoExpr::~CegoExpr()
@@ -76,6 +72,22 @@ CegoExpr::~CegoExpr()
 	delete _pTerm;
 }
 
+void CegoExpr::cleanUp()
+{
+    if ( _pExpr )
+	_pExpr->cleanUp();
+    if ( _pTerm )
+	_pTerm->cleanUp();
+}
+
+void CegoExpr::setTabSetId(int tabSetId)
+{
+    if ( _pExpr )
+	_pExpr->setTabSetId(tabSetId);
+    if ( _pTerm )
+	_pTerm->setTabSetId(tabSetId);
+}
+
 void CegoExpr::setAlias(const Chain& alias)
 {
     _alias = alias;
@@ -86,6 +98,18 @@ const Chain& CegoExpr::getAlias() const
     return _alias;
 }
 
+bool CegoExpr::checkConst() const
+{
+    if ( _expType == TERM)
+    {
+	return _pTerm->checkConst();
+    }
+    else
+    {
+	return _pExpr->checkConst() && _pTerm->checkConst();
+    }
+}
+
 bool CegoExpr::checkVar(Chain& var) const
 {
     if ( _pTerm )
@@ -104,6 +128,15 @@ CegoAttrDesc* CegoExpr::checkAttr() const
     return 0;
 }
 
+bool CegoExpr::checkLob(CegoFieldValue& fv, CegoProcBlock *pBlock)
+{
+    if ( _pTerm && _expType == TERM)
+    {
+	return _pTerm->checkLob(fv, pBlock);
+    }
+    return false;
+}
+
 void CegoExpr::getFunctionList(SetT<CegoObject>& funcList)
 {
     if ( _pExpr )
@@ -112,37 +145,6 @@ void CegoExpr::getFunctionList(SetT<CegoObject>& funcList)
 	_pTerm->getFunctionList(funcList);
 }
 
-void CegoExpr::setFieldListArray(ListT<CegoField>* pFL)
-{
-    _fla[0]=pFL;
-    _fla[1]=0;
-    
-    if ( _pExpr )
-	_pExpr->setFieldListArray(_fla);
-    if ( _pTerm )
-	_pTerm->setFieldListArray(_fla);
-}
-
-void CegoExpr::setFieldListArray(ListT<CegoField> **pFLA)
-{
-    if ( _pExpr )
-	_pExpr->setFieldListArray(pFLA);
-    if ( _pTerm )
-	_pTerm->setFieldListArray(pFLA);
-}
-
-void CegoExpr::setBlock(CegoProcBlock *pBlock)
-{
-    if ( _pTerm )
-    {
-	_pTerm->setBlock(pBlock);
-    }
-    if ( _pExpr )
-    {
-	_pExpr->setBlock(pBlock);
-    }
-}
-
 void CegoExpr::getSelectQueryList(ListT<CegoSelect*>& queryList)
 {
     if ( _pTerm )
@@ -167,31 +169,28 @@ void CegoExpr::clearAttrCache()
     }
 }
 
-ListT<CegoField> CegoExpr::getFieldList() const
+void CegoExpr::getFieldList(ListT<CegoField>& fl, ListT<CegoField>** pFLA) const
 {
-    ListT<CegoField> fl;
-
     switch ( _expType )
     {
     case ADD:
     case SUB:
     case CONCAT:
     {
-	fl = _pExpr->getFieldList() + _pTerm->getFieldList();
+	_pExpr->getFieldList(fl, pFLA);
+	_pTerm->getFieldList(fl, pFLA);
 	break;
     }
     case TERM:
     {
-	fl = _pTerm->getFieldList();
+	_pTerm->getFieldList(fl, pFLA);
 	break;
     }
     }
-    return fl;
 }
 
 ListT<CegoAttrDesc*> CegoExpr::getAttrRefList() const
-{
-    
+{    
     ListT<CegoAttrDesc*> attrList;
     switch ( _expType )
     {
@@ -235,22 +234,91 @@ int CegoExpr::evalReferences(CegoContentObject *pCO, const ListT<CegoField>& fl)
     return refCount;
 }
 
-CegoField CegoExpr::evalField() const
+CegoField CegoExpr::evalField(const ListT<CegoField>& fl, bool graceful) const
 {
     CegoField f;
 
     switch ( _expType )
     {
     case ADD:
+    {
+	CegoField f1 =  _pExpr->evalField(fl, graceful);
+	CegoField f2 =  _pTerm->evalField(fl, graceful);
+
+	// the type conversion refers to method : CegoFieldValue operator + ( const CegoFieldValue& fv1, const CegoFieldValue& fv2 )
+	CegoDataType dt;
+	int len;
+	int dim;
+	
+	if ( f1.getType() == DECIMAL_TYPE )
+	{
+	    dt = f1.getType();
+	    len = f1.getLength();
+	    dim = f1.getDim();
+	}
+	else if ( f2.getType() == DECIMAL_TYPE )
+	{
+	    dt = f2.getType();
+	    len = f2.getLength();
+	    dim = f2.getDim();
+	}
+	else
+	{
+	    dt =  f1.getLength() > f2.getLength() ? f1.getType() : f2.getType();
+	    len = f1.getLength() > f2.getLength() ? f1.getLength() : f2.getLength();
+	    dim = f1.getLength() > f2.getLength() ? f1.getDim() : f2.getDim();
+	}
+	
+	f = CegoField( Chain("EXPR"),
+		       Chain("EXPR"),
+		       Chain("(") + f1.getAttrName() + Chain("+") + f2.getAttrName() + Chain(")"),
+		       dt, len, dim);
+	break;	
+
+    }
     case SUB:
+    {
+	CegoField f1 =  _pExpr->evalField(fl, graceful);
+	CegoField f2 =  _pTerm->evalField(fl, graceful);
+
+	CegoDataType dt;
+	int len;
+	int dim;
+	
+	if ( f1.getType() == DECIMAL_TYPE )
+	{
+	    dt = f1.getType();
+	    len = f1.getLength();
+	    dim = f1.getDim();
+	}
+	else if ( f2.getType() == DECIMAL_TYPE )
+	{
+	    dt = f2.getType();
+	    len = f2.getLength();
+	    dim = f2.getDim();
+	}
+	else
+	{
+	    dt =  f1.getLength() > f2.getLength() ? f1.getType() : f2.getType();
+	    len = f1.getLength() > f2.getLength() ? f1.getLength() : f2.getLength();
+	    dim = f1.getLength() > f2.getLength() ? f1.getDim() : f2.getDim();
+	}
+		
+	f = CegoField( Chain("EXPR"),
+		       Chain("EXPR"),
+		       Chain("(") + f1.getAttrName() + Chain("-") + f2.getAttrName() + Chain(")"),
+		       dt, len, dim);
+	break;	
+
+    }
     case CONCAT:
     {
-	f = CegoField( Chain("EXPR"), Chain("EXPR"), toChain(), VARCHAR_TYPE, 20 );
+	f = CegoField( Chain("EXPR"), Chain("EXPR"), toChain(0), VARCHAR_TYPE, 20 );
 	break;
     }
     case TERM:
     {
-	f = _pTerm->evalField();
+	f = _pTerm->evalField(fl, graceful);
 	break;
     }
     }
@@ -260,31 +328,32 @@ CegoField CegoExpr::evalField() const
 	f.setAttrName(_alias);
     }
     return f;
-	    
 }
 
-CegoFieldValue CegoExpr::evalFieldValue() const
+CegoFieldValue CegoExpr::evalFieldValue(ListT<CegoField>** pFLA, CegoProcBlock *pBlock) const
 {
     switch ( _expType )
     {
     case ADD:
     {
-	return ( _pExpr->evalFieldValue() + _pTerm->evalFieldValue() );	
+	return ( _pExpr->evalFieldValue(pFLA, pBlock) + _pTerm->evalFieldValue(pFLA, pBlock) );	
     }
     case SUB:
     {
-	return ( _pExpr->evalFieldValue() - _pTerm->evalFieldValue() );
+	return ( _pExpr->evalFieldValue(pFLA, pBlock) - _pTerm->evalFieldValue(pFLA, pBlock) );
     }
     case CONCAT:
     {	
-	Chain con = _pExpr->evalFieldValue().valAsChain() + _pTerm->evalFieldValue().valAsChain();
+	Chain con = _pExpr->evalFieldValue(pFLA, pBlock).valAsChain() + _pTerm->evalFieldValue(pFLA, pBlock).valAsChain();
 	CegoFieldValue  fv(VARCHAR_TYPE, con );
 	return fv;
     }
     case TERM:
     {
-	return ( _pTerm->evalFieldValue() );
+	return ( _pTerm->evalFieldValue(pFLA, pBlock) );
     }
+    default:
+        throw Exception(EXLOC, Chain("Unknown expression type"));
     }
 }
 
@@ -331,9 +400,11 @@ CegoExpr* CegoExpr::clone(bool isAttrRef)
     case ADD:
     case SUB:
     case CONCAT:
-	return ( new CegoExpr( _pExpr->clone(isAttrRef), _pTerm->clone(isAttrRef), _expType ) );	
+	return ( new CegoExpr( _pExpr->clone(isAttrRef), _pTerm->clone(isAttrRef), _expType, _alias ) );	
     case TERM:
-	return ( new CegoExpr( _pTerm->clone(isAttrRef) ) );
+	return ( new CegoExpr( _pTerm->clone(isAttrRef), _alias ) );
+    default:
+        throw Exception(EXLOC, Chain("Unknown expression type"));
     }    
 }    
 
@@ -343,46 +414,47 @@ CegoExpr& CegoExpr::operator = ( const CegoExpr& e)
     _expType = e._expType;
     _pExpr = e._pExpr;
     _pTerm = e._pTerm;
+    _isExtern = e._isExtern;
     return (*this);
 }
 
-Chain CegoExpr::getId() const
+Chain CegoExpr::getId(CegoProcBlock *pBlock) const
 {
     Chain s;
     switch ( _expType )
     {
     case CegoExpr::ADD:
-	s = _pExpr->getId() + "+" + _pTerm->getId();
+	s = _pExpr->getId(pBlock) + "+" + _pTerm->getId(pBlock);
 	break;
     case CegoExpr::SUB:
-	s = _pExpr->getId() + "-" + _pTerm->getId();
+	s = _pExpr->getId(pBlock) + "-" + _pTerm->getId(pBlock);
 	break;
     case CegoExpr::CONCAT:
-	s = _pExpr->getId() + "|" + _pTerm->getId();
+	s = _pExpr->getId(pBlock) + "|" + _pTerm->getId(pBlock);
 	break;
     case CegoExpr::TERM:
-	s = _pTerm->getId();
+	s = _pTerm->getId(pBlock);
 	break;
     }
     return s;
 }
 
-Chain CegoExpr::toChain(const Chain& indent) const
+Chain CegoExpr::toChain(int defTabSetId, const Chain& indent) const
 {
     Chain s;
     switch ( _expType )
     {
     case CegoExpr::ADD:
-	s = indent + _pExpr->toChain() + " + " + _pTerm->toChain();
+	s = _pExpr->toChain(defTabSetId, indent) + " + " + _pTerm->toChain(defTabSetId, indent);
 	break;
     case CegoExpr::SUB:
-	s = indent + _pExpr->toChain() + " - " + _pTerm->toChain();
+	s = _pExpr->toChain(defTabSetId, indent) + " - " + _pTerm->toChain(defTabSetId, indent);
 	break;
     case CegoExpr::CONCAT:
-	s = indent + _pExpr->toChain() + " | " + _pTerm->toChain();
+	s = _pExpr->toChain(defTabSetId, indent) + " | " + _pTerm->toChain(defTabSetId, indent);
 	break;
     case CegoExpr::TERM:
-	s = indent + _pTerm->toChain();
+	s = _pTerm->toChain(defTabSetId, indent);
 	break;
     }
     return s;
@@ -393,111 +465,8 @@ Chain CegoExpr::dbFormat(CegoDatabaseFormater *pForm) const
     return pForm->formatExpr(_expType, _pExpr, _pTerm);
 }
 
-Element* CegoExpr::toElement() const
-{
-    Element* pExprElement = new Element(XML_EXPR_ELEMENT);
-
-    switch ( _expType )
-    {
-    case CegoExpr::ADD:
-	pExprElement->setAttribute( XML_EXPR_ATTR, XML_PLUS_VALUE ); 
-	pExprElement->addContent( _pExpr->toElement() );
-	pExprElement->addContent( _pTerm->toElement() );
-	break;
-    case CegoExpr::SUB:
-	pExprElement->setAttribute( XML_EXPR_ATTR, XML_SUB_VALUE ); 
-	pExprElement->addContent( _pExpr->toElement() );
-	pExprElement->addContent( _pTerm->toElement() );
-	break;
-    case CegoExpr::CONCAT:
-	pExprElement->setAttribute( XML_EXPR_ATTR, XML_CONCAT_VALUE ); 
-	pExprElement->addContent( _pExpr->toElement() );
-	pExprElement->addContent( _pTerm->toElement() );
-	break;
-    case CegoExpr::TERM:
-	pExprElement->setAttribute( XML_EXPR_ATTR, XML_TERM_VALUE ); 
-	pExprElement->addContent( _pTerm->toElement() );
-	break;
-    }
-    return pExprElement;
-}
-
-void CegoExpr::fromElement(Element *pExpElement, CegoDistManager *pGTM)
-{
-
-    if ( _pTerm ) 
-	delete _pTerm;
-    if ( _pExpr )
-	delete _pExpr;
-
-    _pTerm=0;
-    _pExpr=0;
-
-    Chain expTypeString = pExpElement->getAttributeValue( XML_EXPR_ATTR );
-
-    if ( expTypeString == Chain(XML_PLUS_VALUE) )
-    {
-	_expType = CegoExpr::ADD;
-	ListT<Element*> el = pExpElement->getChildren(XML_EXPR_ELEMENT);
-	Element **pEE = el.First();
-	if ( pEE )
-	{
-	    _pExpr = new CegoExpr(*pEE, pGTM);
-	}
-	ListT<Element*> tl = pExpElement->getChildren(XML_TERM_ELEMENT);
-	Element **pTE = tl.First();
-	if ( pTE )
-	{
-	    _pTerm = new CegoTerm(*pTE, pGTM);
-	}
-    } 
-    else if ( expTypeString == Chain(XML_SUB_VALUE) )
-    {
-	_expType = CegoExpr::SUB;
-	ListT<Element*> el = pExpElement->getChildren(XML_EXPR_ELEMENT);
-	Element **pEE = el.First();
-	if ( pEE )
-	{
-	    _pExpr = new CegoExpr(*pEE, pGTM);
-	}
-	ListT<Element*> tl = pExpElement->getChildren(XML_TERM_ELEMENT);
-	Element **pTE = tl.First();
-	if ( pTE )
-	{
-	    _pTerm = new CegoTerm(*pTE, pGTM);
-	}
-    }
-    else if ( expTypeString == Chain(XML_CONCAT_VALUE) )
-    {
-	_expType = CegoExpr::CONCAT;
-	ListT<Element*> el = pExpElement->getChildren(XML_EXPR_ELEMENT);
-	Element **pEE = el.First();
-	if ( pEE )
-	{
-	    _pExpr = new CegoExpr(*pEE, pGTM);
-	}
-	ListT<Element*> tl = pExpElement->getChildren(XML_TERM_ELEMENT);
-	Element **pTE = tl.First();
-	if ( pTE )
-	{
-	    _pTerm = new CegoTerm(*pTE, pGTM);
-	}
-    }
-    else if ( expTypeString == Chain(XML_TERM_VALUE) )
-    {
-	_expType = CegoExpr::TERM;
-	ListT<Element*> tl = pExpElement->getChildren(XML_TERM_ELEMENT);
-	Element **pTE = tl.First();
-	if ( pTE )
-	{
-	    _pTerm = new CegoTerm(*pTE, pGTM);
-	}
-    }
-}
-
-void CegoExpr::encode(char *buf)
-{
-    
+void CegoExpr::encode(char *buf, CegoProcBlock *pBlock)
+{   
     char* pE = (char*)buf;
 
     memcpy( pE, &_expType, sizeof(CegoExpr::ExpType));
@@ -509,24 +478,23 @@ void CegoExpr::encode(char *buf)
     case CegoExpr::SUB:
     case CegoExpr::CONCAT:
     {
-	_pExpr->encode(pE);
-	pE = pE + _pExpr->getEncodingLength();
-	_pTerm->encode(pE);
-	pE = pE + _pTerm->getEncodingLength();
+	_pExpr->encode(pE, pBlock);
+	pE = pE + _pExpr->getEncodingLength(pBlock);
+	_pTerm->encode(pE, pBlock);
+	pE = pE + _pTerm->getEncodingLength(pBlock);
 	break;
     }
     case CegoExpr::TERM:
     {
-	_pTerm->encode(pE);
-	pE = pE + _pTerm->getEncodingLength();
+	_pTerm->encode(pE, pBlock);
+	pE = pE + _pTerm->getEncodingLength(pBlock);
 	break;
     }
     }
 }
 
-void CegoExpr::decode(char *buf, CegoDistManager *pGTM, int tabSetId)
+void CegoExpr::decode(char *buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId)
 {
-
     char* pE = (char*)buf;
 
     memcpy( &_expType, pE,  sizeof(CegoExpr::ExpType));
@@ -538,23 +506,23 @@ void CegoExpr::decode(char *buf, CegoDistManager *pGTM, int tabSetId)
     case CegoExpr::SUB:
     case CegoExpr::CONCAT:
     {
-	_pExpr = new CegoExpr(pE, pGTM, tabSetId);
-	pE = pE + _pExpr->getEncodingLength();
-	_pTerm = new CegoTerm(pE, pGTM, tabSetId);
-	pE = pE + _pTerm->getEncodingLength();
+	_pExpr = new CegoExpr(pE, pGTM, pBlock, tabSetId);
+	pE = pE + _pExpr->getEncodingLength(pBlock);
+	_pTerm = new CegoTerm(pE, pGTM, pBlock, tabSetId);
+	pE = pE + _pTerm->getEncodingLength(pBlock);
 	break;
     }
     case CegoExpr::TERM:
     {
 	_pExpr=0;
-	_pTerm = new CegoTerm(pE, pGTM, tabSetId);
-	pE = pE + _pTerm->getEncodingLength();
+	_pTerm = new CegoTerm(pE, pGTM, pBlock, tabSetId);
+	pE = pE + _pTerm->getEncodingLength(pBlock);
 	break;
     }
     }
 }
 
-int CegoExpr::getEncodingLength() const
+int CegoExpr::getEncodingLength(CegoProcBlock *pBlock) const
 {
     int len = 0;
     
@@ -566,13 +534,13 @@ int CegoExpr::getEncodingLength() const
     case CegoExpr::SUB:
     case CegoExpr::CONCAT:
     {
-	len +=_pExpr->getEncodingLength();
-	len +=_pTerm->getEncodingLength();
+	len +=_pExpr->getEncodingLength(pBlock);
+	len +=_pTerm->getEncodingLength(pBlock);
 	break;
     }
     case CegoExpr::TERM:
     {
-	len +=_pTerm->getEncodingLength();
+	len +=_pTerm->getEncodingLength(pBlock);
 	break;
     }
     }
@@ -582,9 +550,6 @@ int CegoExpr::getEncodingLength() const
 
 void CegoExpr::getPlanList(ListT<Element*>& planList)
 {
-
-    Element *pPlan = new Element(XML_PLAN_ELEMENT);
-
     if ( _pTerm )
     {
 	_pTerm->getPlanList(planList);
@@ -597,6 +562,6 @@ void CegoExpr::getPlanList(ListT<Element*>& planList)
 
 ostream& operator << (ostream& s, const CegoExpr& e)
 {
-    s << e.toChain();
+    s << e.toChain(0);
     return s;
 }
