@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -37,31 +37,35 @@ public:
     enum ExpType { ADD, SUB, TERM, CONCAT  };
     
     CegoExpr();
-    CegoExpr(char* buf, CegoDistManager *pGTM, int tabSetId);
-    CegoExpr(Element* pExpElement, CegoDistManager *pGTM);
-    CegoExpr(CegoExpr *pExpr, CegoTerm *pTerm, ExpType expType);
-    CegoExpr(CegoTerm *pTerm);
+    CegoExpr(char* buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId);
+    CegoExpr(CegoExpr *pExpr, CegoTerm *pTerm, ExpType expType, const Chain& alias = Chain());
+    CegoExpr(CegoTerm *pTerm, const Chain& alias = Chain());
 
     ~CegoExpr();
 
+    void cleanUp();
+
+    void setTabSetId(int tabSetId);
+    
     void setAlias(const Chain& alias);
     const Chain& getAlias() const;
 
+    bool checkConst() const;
     bool checkVar(Chain& var) const;
     CegoAttrDesc* checkAttr() const;
 
+    bool checkLob(CegoFieldValue& fv, CegoProcBlock *pBlock);
+    
     void getFunctionList(SetT<CegoObject>& funcList);
     
-    void setBlock(CegoProcBlock *pBlock);
-    void setFieldListArray(ListT<CegoField>* pFL);
-    void setFieldListArray(ListT<CegoField> **pFLA);
-
     ListT<CegoAttrDesc*> getAttrRefList() const;
     int evalReferences(CegoContentObject *pCO, const ListT<CegoField>& fl);
 
-    ListT<CegoField> getFieldList() const;
-    CegoField evalField() const;
-    CegoFieldValue evalFieldValue() const;
+    void getFieldList(ListT<CegoField>& fl, ListT<CegoField>** pFLA) const;
+    
+    CegoField evalField(const ListT<CegoField>& fl, bool graceful = false) const;
+
+    CegoFieldValue evalFieldValue(ListT<CegoField>** pFLA, CegoProcBlock *pBlock) const;
 
     void clearAttrCache();
 
@@ -77,18 +81,15 @@ public:
 
     CegoExpr& operator = ( const CegoExpr& e);
 
-    Chain getId() const;
-    Chain toChain(const Chain& indent = Chain("")) const;
+    Chain getId(CegoProcBlock *pBlock) const;
+    Chain toChain(int defTabSetId, const Chain& indent = Chain("")) const;
     Chain dbFormat(CegoDatabaseFormater *pForm) const;
-
-    Element* toElement() const;
-    void fromElement(Element *pExpElement, CegoDistManager *pGTM);
-
+    
     friend ostream& operator << (ostream& s, const CegoExpr& e);
 
-    void encode(char *buf);
-    void decode(char *buf, CegoDistManager *pGTM, int tabSetId);
-    int getEncodingLength() const;
+    void encode(char *buf, CegoProcBlock *pBlock);
+    void decode(char *buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId);
+    int getEncodingLength(CegoProcBlock *pBlock) const;
 
     void getPlanList(ListT<Element*>& planList);
 
@@ -103,7 +104,8 @@ private:
     CegoExpr *_pExpr;
     CegoTerm *_pTerm;
 
+    bool _isExtern;
+    
     ListT<CegoField>* _fla[2];
-
 };
 #endif
