@@ -6,7 +6,7 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke 
+// (C)opyright 2000-2025 Bjoern Lemke 
 //
 // IMPLEMENTATION MODULE
 //
@@ -33,38 +33,18 @@
 #include <string.h>
 #include <stdlib.h>
 
-CegoFactor::CegoFactor(char* buf, CegoDistManager *pGTM, int tabSetId)
+CegoFactor::CegoFactor(char* buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId)
 {
     _pAttrDesc = 0;
     _pExpr = 0;
     _pFetch = 0;
-    _pBlock = 0;
     _pAggr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _pFLA = 0;
     _pCaseCond = 0;
-
+    _pCond = 0;
     _flaCached = false;
-
-    decode(buf, pGTM, tabSetId);
-}
-
-CegoFactor::CegoFactor(Element* pFactorElement, CegoDistManager *pGTM)
-{
-    _pAttrDesc = 0;
-    _pExpr = 0;
-    _pFetch = 0;
-    _pBlock = 0;
-    _pAggr = 0;
-    _pFunction = 0;
-    _pSelect = 0;
-    _pFLA = 0;
-    _pCaseCond = 0;
-
-    _flaCached = false;
-
-    fromElement(pFactorElement, pGTM);
+    decode(buf, pGTM, pBlock, tabSetId);
 }
 
 CegoFactor::CegoFactor(const CegoFactor& pf)
@@ -79,13 +59,11 @@ CegoFactor::CegoFactor(const Chain& varName)
     _pAttrDesc = 0;
     _pExpr = 0;
     _pFetch = 0;
-    _pBlock = 0;
     _pAggr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _pFLA = 0;
     _pCaseCond = 0;
-
+    _pCond = 0;
     _flaCached = false;
 }
 
@@ -95,14 +73,12 @@ CegoFactor::CegoFactor(const CegoFieldValue& fv)
     _fv = fv;
     _pAttrDesc = 0;
     _pExpr = 0;
-    _pBlock = 0;
     _pFetch = 0;
     _pAggr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _pFLA = 0;
     _pCaseCond = 0;
-
+    _pCond = 0;
     _flaCached = false;
 }
 
@@ -112,13 +88,11 @@ CegoFactor::CegoFactor(CegoExpr *pExpr)
     _pAttrDesc = 0;
     _pExpr = pExpr;
     _pFetch = 0;
-    _pBlock = 0;
     _pAggr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _pFLA = 0;
     _pCaseCond = 0;
-
+    _pCond = 0;
     _flaCached = false;
 }
 
@@ -128,13 +102,11 @@ CegoFactor::CegoFactor(CegoProcFetch *pFetch)
     _pAttrDesc = 0;
     _pExpr = 0;
     _pFetch = pFetch;
-    _pBlock = 0;
     _pAggr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _pFLA = 0;
     _pCaseCond = 0;
-
+    _pCond = 0;
     _flaCached = false;
 }
 
@@ -144,14 +116,12 @@ CegoFactor::CegoFactor(CegoAttrDesc* pAttrDesc, bool isAttrRef)
     _isAttrRef = isAttrRef;
     _pAttrDesc = pAttrDesc;
     _pFetch = 0;
-    _pBlock = 0;
     _pExpr = 0;
     _pAggr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _pFLA = 0;
     _pCaseCond = 0;
-
+    _pCond = 0;
     _flaCached = false;
 }
 
@@ -159,14 +129,13 @@ CegoFactor::CegoFactor(CegoFunction* pFunction)
 {
     _type = FUNCTION;
     _pFunction = pFunction;
-    _pBlock = 0;
     _pAttrDesc = 0;
     _pExpr = 0;
     _pFetch = 0;
     _pAggr = 0;
     _pSelect = 0;
-    _pFLA = 0;
     _pCaseCond = 0;
+    _pCond = 0;
     _flaCached = false;
 }
 
@@ -174,44 +143,55 @@ CegoFactor::CegoFactor(CegoAggregation* pAggr)
 {
     _type = AGGREGATION;
     _pAggr = pAggr;
-    _pBlock = 0;
     _pFetch = 0;
     _pAttrDesc = 0;
     _pExpr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _pFLA = 0;
     _pCaseCond = 0;
+    _pCond = 0;
     _flaCached = false;
 }
 
 CegoFactor::CegoFactor(CegoSelect* pSelect)
 {
     _type = QUERY;
-    _pBlock = 0;
     _pAttrDesc = 0;
     _pExpr = 0;
     _pFetch = 0;
     _pAggr = 0;
-    _pFLA = 0;
     _pFunction = 0;
     _pSelect = pSelect;
     _pCaseCond = 0;
+    _pCond = 0;
     _flaCached = false;
 }
 
 CegoFactor::CegoFactor(CegoCaseCond* pCaseCond)
 {
     _type = CASECOND;
-    _pBlock = 0;
     _pAttrDesc = 0;
     _pExpr = 0;
     _pFetch = 0;
     _pAggr = 0;
-    _pFLA = 0;
     _pFunction = 0;
     _pSelect = 0;
     _pCaseCond = pCaseCond;
+    _pCond = 0;
+    _flaCached = false;
+}
+
+CegoFactor::CegoFactor(CegoCondition* pCond)
+{
+    _type = CONDITION;
+    _pAttrDesc = 0;
+    _pExpr = 0;
+    _pFetch = 0;
+    _pAggr = 0;
+    _pFunction = 0;
+    _pSelect = 0;
+    _pCaseCond = 0;
+    _pCond = pCond;
     _flaCached = false;
 }
 
@@ -231,64 +211,105 @@ CegoFactor::~CegoFactor()
 	delete _pSelect;
     if ( _pCaseCond )
 	delete _pCaseCond;
+    if ( _pCond )
+	delete _pCond;
 }    
-    
-void CegoFactor::setBlock(CegoProcBlock *pBlock)
+
+void CegoFactor::cleanUp()
 {
     switch ( _type ) 
     {
-    case VAR:
+    case ATTR:
     {
-	_pBlock = pBlock;
+	// _pFLA = 0;
 	break;
     }
     case EXPR:
     {
-	_pExpr->setBlock(pBlock);
-	break;
-    }
-    case FETCH:
-    {
-	_pFetch->setBlock(pBlock);
+	_pExpr->cleanUp();
 	break;
     }
     case FUNCTION:
     {
-	_pFunction->setBlock(pBlock);
+	_pFunction->cleanUp();
 	break;
     }
     case AGGREGATION:
     {
-	_pAggr->setBlock(pBlock);
+	_pAggr->cleanUp();
 	break;
     }
     case QUERY:
     {
-	_pSelect->setProcBlock(pBlock);
-	_pSelect->prepare();
+	_pSelect->cleanUp();
 	break;
     }
     case CASECOND:
     {
-	_pCaseCond->setBlock(pBlock);
+	_pCaseCond->cleanUp();
+	break;
+    }
+    case CONDITION:
+    {
+	if ( _pCond )
+	    _pCond->cleanUp();
 	break;
     }
     case CONSTVAL:
-    case ATTR:
-    {
-	// nothing to do
+    case VAR:
+    case FETCH:
+	// TODO : check if fetch cursor should be reset
 	break;
     }
+}
+
+void CegoFactor::setTabSetId(int tabSetId)
+{
+    switch ( _type ) 
+    {
+    case EXPR:
+    {
+	_pExpr->setTabSetId(tabSetId);
+	break;
+    }
+    case FUNCTION:
+    {
+	_pFunction->setTabSetId(tabSetId);
+	break;
+    }
+    case AGGREGATION:
+    {
+	_pAggr->setTabSetId(tabSetId);
+	break;
+    }
+    case QUERY:
+    {
+	_pSelect->setTabSetId(tabSetId);
+	break;
+    }
+    case CASECOND:
+    {
+	_pCaseCond->setTabSetId(tabSetId);
+	break;
+    }
+    case CONDITION:
+    {
+	if ( _pCond )
+	    _pCond->setTabSetId(tabSetId);
+	break;
+    }
+
+    default:
+	// nothing to do
+	break;
     }
 }
 
 void CegoFactor::clearAttrCache()
 {
-    // if ( _pAttrDesc )
-    // cout << "Clearing cache for " << _pAttrDesc->toChain() << endl;
-
+    // cout << "Clearing attr cache .." << endl;
     if ( _pAttrDesc )
-	_flaCached = false;
+	_flaCached = false;   
     if ( _pExpr )
 	_pExpr->clearAttrCache();
     if ( _pFunction )
@@ -297,6 +318,8 @@ void CegoFactor::clearAttrCache()
         _pAggr->clearAttrCache();
     if ( _pCaseCond )
 	_pCaseCond->clearAttrCache();
+    if ( _pCond )
+	_pCond->clearAttrCache();
 }
 
 void CegoFactor::getFunctionList(SetT<CegoObject>& funcList)
@@ -307,47 +330,6 @@ void CegoFactor::getFunctionList(SetT<CegoObject>& funcList)
     }
 }
 
-void CegoFactor::setFieldListArray(ListT<CegoField>** pFLA)
-{
-    // cout << "Setting field list of size " << size  << endl;
-    switch ( _type ) 
-    {
-    case ATTR:
-    {
-	_pFLA = pFLA;
-	break;
-    }
-    case EXPR:
-    {
-	_pExpr->setFieldListArray(pFLA);
-	break;
-    }
-    case FUNCTION:
-    {
-	_pFunction->setFieldListArray(pFLA);
-	break;
-    }
-    case AGGREGATION:
-    {
-	_pAggr->setFieldListArray(pFLA);
-	break;
-    }
-    case QUERY:
-    {
-	_pSelect->setParentJoinBuf(pFLA);
-	_pSelect->prepare(); // not necessary
-	break;
-    }
-    case CASECOND:
-    {
-	_pCaseCond->setFieldListArray(pFLA);
-	break;
-    }
-    default:
-	break;
-    }
-}
-
 CegoFactor& CegoFactor::operator = ( const CegoFactor& pf)
 {
     _fv = pf._fv;
@@ -360,13 +342,17 @@ CegoFactor& CegoFactor::operator = ( const CegoFactor& pf)
     _pAggr = pf._pAggr;
     _pSelect = pf._pSelect;
     _type = pf._type;
-    _pBlock = pf._pBlock;
     _pCaseCond = pf._pCaseCond;
+    _pCond = pf._pCond;
     _flaCached = pf._flaCached;
-    _pFLA = pf._pFLA;
     return (*this);
 }
 
+bool CegoFactor::checkConst() const
+{
+    return _type == CONSTVAL;
+}
+
 bool CegoFactor::checkVar(Chain& var) const
 {
     if ( _type == VAR )
@@ -386,6 +372,45 @@ CegoAttrDesc* CegoFactor::checkAttr() const
     return 0;
 }
 
+bool CegoFactor::checkLob(CegoFieldValue& fv, CegoProcBlock *pBlock)
+{
+    switch ( _type )
+    {
+    case CONSTVAL:
+    {
+	if ( _fv.getType() == BLOB_TYPE || _fv.getType() == CLOB_TYPE )
+	{
+	    fv = _fv;
+	    return true;
+	}
+	break;
+    }
+    case VAR:
+    {
+	CegoFieldValue fv;
+	if ( pBlock )
+	{
+	    fv = pBlock->getValue(_varName);
+	    if ( fv.getType() == BLOB_TYPE || fv.getType() == CLOB_TYPE )
+	    {
+		return true;		
+	    }
+	}
+	break;
+    }
+    case EXPR:
+    case FETCH:
+    case ATTR:
+    case FUNCTION:
+    case QUERY:
+    case AGGREGATION:
+    case CASECOND:
+    case CONDITION:
+	return false;
+    }
+    return false;
+}
+
 CegoAttrDesc* CegoFactor::getAttr() const
 {
     return _pAttrDesc;
@@ -411,17 +436,25 @@ CegoCaseCond* CegoFactor::getCaseCond() const
     return _pCaseCond;
 }
 
-ListT<CegoField> CegoFactor::getFieldList() const
+CegoCondition* CegoFactor::getCondition() const
 {
+    return _pCond;
+}
 
-    ListT<CegoField> fl;
+CegoCondition* CegoFactor::grapCondition()
+{
+    CegoCondition *pCond = _pCond;
+    _pCond = 0;
+    return pCond;
+}
 
+void CegoFactor::getFieldList(ListT<CegoField>& fl, ListT<CegoField>** pFLA) const
+{
     switch ( _type )
-    {
-	
+    {	
     case CegoFactor::ATTR:
     {
-	if ( _pFLA == 0)
+	if ( pFLA == 0)
 	{
 	    Chain msg = Chain("Cannot get value for attribute " + _pAttrDesc->toChain());
 	    throw Exception(EXLOC, msg);
@@ -429,9 +462,9 @@ ListT<CegoField> CegoFactor::getFieldList() const
 	
 	int i=0;
 	CegoField* pF = 0;
-	while ( _pFLA[i] && pF == 0 )
+	while ( pFLA[i] && pF == 0 )
 	{		    
-	    if ( ( pF = _pFLA[i]->Find(CegoField(_pAttrDesc->getTableName(), _pAttrDesc->getAttrName()))) != 0 )
+	    if ( ( pF = pFLA[i]->Find(CegoField(_pAttrDesc->getTableName(), _pAttrDesc->getAttrName()))) != 0 )
 	    {
 		fl.Insert(*pF);	       
 	    }
@@ -441,44 +474,48 @@ ListT<CegoField> CegoFactor::getFieldList() const
     }
     case CegoFactor::EXPR:
     {
-	fl = _pExpr->getFieldList();
+	_pExpr->getFieldList(fl, pFLA);
 	break;
     }
     case CegoFactor::FUNCTION:
     {
-	fl = _pFunction->getFieldList();
+	_pFunction->getFieldList(fl, pFLA);
 	break;
     }
     case CegoFactor::AGGREGATION:
     {
 	if ( _pAggr->getExpr() )
-	    fl = _pAggr->getExpr()->getFieldList();
+	    _pAggr->getExpr()->getFieldList(fl, pFLA);
 	break;
     }
     case CegoFactor::CASECOND:
     {
-	fl = _pCaseCond->getFieldList();
+	_pCaseCond->getFieldList(fl, pFLA);
+	break;
+    }
+    case CegoFactor::CONDITION:
+    {
+	_pCond->getFieldList(fl, pFLA);
 	break;
     }
     case CegoFactor::QUERY:
     {
-	fl = _pSelect->getFieldList();
+	_pSelect->getFieldList(fl, pFLA);
 	break;
     }
-    default:
+    case CONSTVAL:
+    case VAR:
+    case FETCH:
 	break;
     }
-    return fl;
 }
 
 ListT<CegoAttrDesc*> CegoFactor::getAttrRefList() const
-{
-	
+{	
     ListT<CegoAttrDesc*> attrList;
 
     switch ( _type )
-    {
-	
+    {	
     case CegoFactor::ATTR:
     {
 	attrList.Insert(_pAttrDesc);
@@ -510,9 +547,15 @@ ListT<CegoAttrDesc*> CegoFactor::getAttrRefList() const
 	attrList = _pCaseCond->getAttrRefList();
 	break;
     }
-    default:
+    case CegoFactor::CONDITION:
+    {
+	attrList = _pCond->getAttrRefList();
+	break;
+    }
+    case CONSTVAL:
+    case VAR:
+    case FETCH:
 	break;
-
     }
     return attrList;
 }
@@ -523,7 +566,6 @@ int CegoFactor::evalReferences(CegoContentObject *pCO, const ListT<CegoField>& f
 
     switch ( _type )
     {
-
     case CegoFactor::ATTR:
     {
 	refCount += _pAttrDesc->evalReferences(pCO, fl);
@@ -548,8 +590,10 @@ int CegoFactor::evalReferences(CegoContentObject *pCO, const ListT<CegoField>& f
 	break;
     }
     case CegoFactor::QUERY:
-    {	
-	_pSelect->prepare();	
+    {
+	// reset of parent join buf is done via cleanUp
+	// _pSelect->setParentJoinBuf();
+	_pSelect->prepare();
 	refCount += _pSelect->evalExtTableReferences(pCO, fl);
 	break;
     }
@@ -558,30 +602,33 @@ int CegoFactor::evalReferences(CegoContentObject *pCO, const ListT<CegoField>& f
 	refCount += _pCaseCond->evalReferences(pCO, fl);
 	break;
     }
-    default:
+    case CegoFactor::CONDITION:
+    {
+	refCount += _pCond->evalReferences(pCO, fl);
+	break;
+    }
+    case CONSTVAL:
+    case VAR:
+    case FETCH:
 	break;
-
     }
-
     return refCount;
-
 }
 
-CegoField CegoFactor::evalField() const
+CegoField CegoFactor::evalField(const ListT<CegoField>& fl, bool graceful) const
 {
-
     CegoField f;
 
     switch ( _type )
     {
     case CegoFactor::CONSTVAL:
     {
-	f = CegoField("CONST", "CONST", "CONST", _fv.getType(), _fv.getLength() );
+	f = CegoField("CONST", "CONST", "CONST", _fv.getType(), _fv.getLength(), _fv.getDim() );
 	break;
     }
     case CegoFactor::EXPR:
     {
-	return _pExpr->evalField();
+	return _pExpr->evalField(fl, graceful);
     }
     case CegoFactor::FETCH:
     {
@@ -595,97 +642,86 @@ CegoField CegoFactor::evalField() const
     }
     case CegoFactor::ATTR:
     {
-	if ( _pFLA == 0)
-	{
-       	
+	if ( fl.Size() == 0 || graceful == true )
+	{       	
 	    f = CegoField(_pAttrDesc->getTableName(), 
-			  _pAttrDesc->getAttrName());
-	    
-	    // Chain msg = Chain("Fieldlist not set for " + _attrDesc.toChain());
-	    // throw Exception(EXLOC, msg);
+			  _pAttrDesc->getAttrName());	    
 	}
 	else
 	{
-	    CegoField *pF = 0;
-	    
-	    int i=0;
-	    while ( _pFLA[i] && pF == 0)
+	    CegoField *pF = fl.Find(CegoField(_pAttrDesc->getTableName(), _pAttrDesc->getAttrName()));
+	    if ( pF )
 	    {
-
-		/*
-		CegoField *pXF = _fla[i].First();
-		while ( pXF )
-		{
-		    cout << "FLA : Table = " << pXF->getTableName() << "." << pXF->getAttrName() << endl;
-		    pXF = _fla[i].Next();
-		}
-		*/
-		
-		if ( ( pF = _pFLA[i]->Find(CegoField(_pAttrDesc->getTableName(), _pAttrDesc->getAttrName()))) != 0 )
-		{
-		    f = CegoField(_pAttrDesc->getTableName(), 
-				  _pAttrDesc->getTableName(),
-				  _pAttrDesc->getAttrName(),
-				  pF->getType(),
-				  pF->getLength());
-		}
-		i++;
+		f = CegoField(_pAttrDesc->getTableName(), 
+			      _pAttrDesc->getTableName(),
+			      _pAttrDesc->getAttrName(),
+			      pF->getType(),
+			      pF->getLength(),
+			      pF->getDim());
 	    }
-	    
-	    if ( pF == 0 )
+	    else
 	    {
 		Chain msg("Unknown attribute field <" + _pAttrDesc->getTableName() + "." + _pAttrDesc->getAttrName() + ">");
 		throw Exception(EXLOC, msg);
 	    }
 	}
-	
 	break;
     }
     case CegoFactor::FUNCTION:
     {
-	f = CegoField("FUNC", "FUNC", _pFunction->toChain(), _pFunction->getReturnType(), _pFunction->getReturnTypeLen());
+	f = CegoField("FUNC", "FUNC", _pFunction->toChain(0), _pFunction->getReturnType(), _pFunction->getReturnTypeLen(fl));
+	_pFunction->cleanUp();
 	break;
     }
     case CegoFactor::AGGREGATION:
     {
-	
 	if ( _pAggr->getExpr() )
 	{
-	    CegoField af = _pAggr->getExpr()->evalField();
+	    CegoField af = _pAggr->getExpr()->evalField(fl);
 	    if ( _pAggr->getType() == CegoAggregation::COUNT )
 	    {
-		f = CegoField("AGGR", "AGGR", _pAggr->toChain(), INT_TYPE,  4);
+		f = CegoField("AGGR", "AGGR", _pAggr->toChain(0), LONG_TYPE, sizeof(long long));
 	    }
 	    else
 	    {
-		f = CegoField("AGGR", "AGGR", _pAggr->toChain(), af.getType(), af.getLength() );
+		f = CegoField("AGGR", "AGGR", _pAggr->toChain(0), af.getType(), af.getLength(), af.getDim() );
 	    }
 	}
 	else
 	{
 	    // must be count(*)
-	    f = CegoField("AGGR", "AGGR", _pAggr->toChain(), INT_TYPE,  4);
+	    f = CegoField("AGGR", "AGGR", _pAggr->toChain(0), LONG_TYPE, sizeof(long long));
 	}
 	break;
     }
     case CegoFactor::QUERY:
     {
-	f = CegoField("QUERY", "QUERY", Chain("select(..)"), VARCHAR_TYPE, 20 );
+	ListT<CegoField> schema;
+	_pSelect->getSchema(schema);
+	CegoField *pF = schema.First();
+	if ( pF )
+	    f = *pF;
+	
+	// f = CegoField("QUERY", "QUERY", Chain("select(..)"), VARCHAR_TYPE, 20 );
 	break;
     }
     case CegoFactor::CASECOND:
     {
-	return _pCaseCond->evalField();
+	return _pCaseCond->evalField(fl, graceful);
     }
+    case CegoFactor::CONDITION:
+    {
+	return _pCond->evalField(fl, graceful);
+    }
+    
     default:
 	break;
-
     }
     return f;
 }
 
-CegoFieldValue CegoFactor::evalFieldValue()
-{
+CegoFieldValue CegoFactor::evalFieldValue(ListT<CegoField>** pFLA, CegoProcBlock *pBlock)
+{    
     switch ( _type )
     {
     case CONSTVAL:
@@ -694,78 +730,77 @@ CegoFieldValue CegoFactor::evalFieldValue()
     }
     case EXPR:
     {
-	return _pExpr->evalFieldValue();
+	return _pExpr->evalFieldValue(pFLA, pBlock);
     }
     case FETCH:
     {
-	
 	char *pC = new (char);
 	*pC = 1;
 	CegoFieldValue fv(BOOL_TYPE, pC, 1, true);
 
+	_pFetch->setBlock(pBlock);
+	
 	if ( _pFetch->fetch() )
 	    *pC = 1;
 	else
 	    *pC = 0;
 
 	return fv;
-
     }
     case VAR:
     {
 	CegoFieldValue fv;
-	if ( _pBlock )
+	if ( pBlock )
 	{
-	    fv = _pBlock->getValue(_varName);
+	    fv = pBlock->getValue(_varName);
 	}
 	return fv;
     }
     case CegoFactor::ATTR:
     {
-	if ( _pFLA == 0 )
+	if ( pFLA == 0 )
 	{
 	    Chain msg = Chain("Cannot get value for attribute " + _pAttrDesc->toChain());
 	    throw Exception(EXLOC, msg);
 	}
-
+	
 	if ( _flaCached )
 	{
-	    // cout << "Using cached field for " << _pAttrDesc->toChain() << " with pos " << _flaCacheArrayPos << " / " << _flaCacheFieldPos << endl;	    
-	    return (*_pFLA[_flaCacheArrayPos])[_flaCacheFieldPos].getValue();
+	    // cout << "Using cached position " << _flaCacheArrayPos << "/" << _flaCacheFieldPos << endl;
+	    return (*pFLA[_flaCacheArrayPos])[_flaCacheFieldPos].getValue();
 	}
 	
 	CegoField *pF = 0;
 	
 	_flaCacheArrayPos=0;
-	while ( _pFLA[_flaCacheArrayPos] && pF == 0)
+	while ( pFLA[_flaCacheArrayPos] && pF == 0)
 	{
-
-	    CegoField *pF = _pFLA[_flaCacheArrayPos]->First();
+	    CegoField *pF = pFLA[_flaCacheArrayPos]->First();
 
 	    _flaCacheFieldPos = 0;
 	    while ( pF )
 	    {
+		// cout << "Checking  " << pF->getTableAlias()  << "/" <<  pF->getTableName() << "/" << pF->getAttrName() << endl;
 		if ( *pF == CegoField(_pAttrDesc->getTableName(), _pAttrDesc->getAttrName()))
 		{
+		    // cout << "Found value at position " << _flaCacheArrayPos << "/" << _flaCacheFieldPos << endl;
 		    _flaCached = true;
 		    return pF->getValue();
 		}  
-		pF = _pFLA[_flaCacheArrayPos]->Next();
+		pF = pFLA[_flaCacheArrayPos]->Next();
 		_flaCacheFieldPos++;
 	    }
 	    _flaCacheArrayPos++;
 	}
-	
 	if ( pF == 0 )
 	{
 	    Chain msg("Unknown attribute field <" + _pAttrDesc->getTableName() + "." + _pAttrDesc->getAttrName() + ">");
 	    throw Exception(EXLOC, msg);
-	}       
-
+	}
     }
     case CegoFactor::FUNCTION:
     {
-	return _pFunction->evalFieldValue();	
+	return _pFunction->evalFieldValue(pFLA, pBlock);	
     }
     case CegoFactor::AGGREGATION:
     {
@@ -775,41 +810,64 @@ CegoFieldValue CegoFactor::evalFieldValue()
     {
 	try 
 	{
+	    _pSelect->setParentJoinBuf(pFLA);
+	    _pSelect->setProcBlock(pBlock);
 	    _pSelect->prepare();
+
 	    _pSelect->reset(false);
 
 	    ListT<CegoField> fl;
 
+	    CegoFieldValue val;
+
 	    if ( ! _pSelect->nextTuple(fl) )
 	    {
 		_pSelect->reset(true);
 		return CegoFieldValue();
 	    }
-	    
-	    CegoField *pSF = fl.First();
+	    else
+	    {
+		// if fetch result is just one, next call of nextTuple will complete fetch results
+		// and an entry can be added to query cache ( if enabled ) 
+
+		// before, we have to save value to local copy
+		CegoField *pSF = fl.First();
+		val = pSF->getValue().getLocalCopy();
+		
+		ListT<CegoField> nfl;		
+		_pSelect->nextTuple(nfl);
+	    }
+	    	    
 	    _pSelect->reset(true);
-	    CegoFieldValue val = pSF->getValue();
-	    return pSF->getValue();
+	    
+	    return val;
 	}
 	catch ( Exception e ) 
 	{
 	    _pSelect->cleanUp();
-	    throw e;
+
+	    e.print();
+	    
+	    Chain msg;
+	    e.pop(msg);
+	    throw Exception(EXLOC, Chain("Cannot eval select : ") + msg);
 	}
     }
     case CegoFactor::CASECOND:
     {
-	return _pCaseCond->evalFieldValue();
+	return _pCaseCond->evalFieldValue(pFLA, pBlock);
+    }
+    case CegoFactor::CONDITION:
+    {
+	return _pCond->evalFieldValue(pFLA, pBlock);
     }
     default:
-	break;
-
+        throw Exception(EXLOC, Chain("Unknown factor type"));
     }
 }
 
 ListT<CegoAggregation*> CegoFactor::getAggregationList()
 {
-
     ListT<CegoAggregation*> aggList;
 
     switch ( _type )
@@ -837,7 +895,13 @@ ListT<CegoAggregation*> CegoFactor::getAggregationList()
     }
     case CegoFactor::CASECOND:
     {
-	throw Exception(EXLOC, "Case condition not supported in aggregation");
+	aggList = _pCaseCond->getAggregationList();
+	break;
+    }
+    case CegoFactor::CONDITION:
+    {
+	aggList = _pCond->getAggregationList();
+	break;
     }
     case CegoFactor::FETCH:
     {
@@ -913,20 +977,24 @@ CegoFactor* CegoFactor::clone(bool isAttrRef)
     {
 	return ( new CegoFactor( _pCaseCond->clone(isAttrRef) ) );
     }
+    case CegoFactor::CONDITION:
+    {
+	return ( new CegoFactor( _pCond->clone(isAttrRef) ) );
+    }
     case CegoFactor::FETCH:
 	throw Exception(EXLOC, "Clone of fetch not implemented");
     }
     return 0;
 }
 
-Chain CegoFactor::getId() const
+Chain CegoFactor::getId(CegoProcBlock *pBlock) const
 {
     Chain s;
     switch (_type)
     {
     case CegoFactor::CONSTVAL:
     {
-	s = _fv.valAsChain(true);
+	s = _fv.getId();
 	break;
     }
     case CegoFactor::ATTR:
@@ -937,16 +1005,16 @@ Chain CegoFactor::getId() const
     case CegoFactor::VAR:
     {
 	CegoFieldValue fv;
-	if ( _pBlock )
+	if ( pBlock )
 	{
-	    fv = _pBlock->getValue(_varName);
+	    fv = pBlock->getValue(_varName);
 	}
 	s = fv.toChain();
 	break;
     }
     case CegoFactor::EXPR:
     {
-	s = Chain("(") + _pExpr->getId() + Chain(")");
+	s = Chain("(") + _pExpr->getId(pBlock) + Chain(")");
 	break;
     }
     case CegoFactor::FETCH:
@@ -956,12 +1024,12 @@ Chain CegoFactor::getId() const
     }
     case CegoFactor::AGGREGATION:
     {
-	s = _pAggr->getId();
+	s = _pAggr->getId(pBlock);
 	break;
     }
     case CegoFactor::FUNCTION:
     {
-	s = _pFunction->getId();
+	s = _pFunction->getId(pBlock);
 	break;
     }
     case CegoFactor::QUERY:
@@ -971,16 +1039,22 @@ Chain CegoFactor::getId() const
     }
     case CegoFactor::CASECOND:
     {
-	s = _pCaseCond->getId();
+	s = _pCaseCond->getId(pBlock);
+	break;
+    }
+    case CegoFactor::CONDITION:
+    {
+	s = _pCond->getId(pBlock);
 	break;
     }
     }
     return s;
 }
 
-Chain CegoFactor::toChain() const
+Chain CegoFactor::toChain(int defTabSetId, const Chain& indent) const
 {
     Chain s;
+    
     switch (_type)
     {
     case CegoFactor::CONSTVAL:
@@ -993,22 +1067,25 @@ Chain CegoFactor::toChain() const
 	s = Chain(":") + _varName;
 	break;
     case CegoFactor::EXPR:
-	s = Chain("(") + _pExpr->toChain() + Chain(")");
+	s = Chain("(") + _pExpr->toChain(defTabSetId, indent) + Chain(")");
 	break;
     case CegoFactor::FETCH:
 	s = _pFetch->toChain();
 	break;
     case CegoFactor::AGGREGATION:
-	s = _pAggr->toChain();
+	s = _pAggr->toChain(defTabSetId);
 	break;
     case CegoFactor::FUNCTION:
-	s = _pFunction->toChain();
+	s = _pFunction->toChain(defTabSetId);
 	break;
     case CegoFactor::QUERY:
-	s = Chain("(") + _pSelect->toChain() + Chain(")");
+	s = Chain("( ") + _pSelect->toChain(defTabSetId, indent + Chain(" ")) + Chain(" )");
 	break;
     case CegoFactor::CASECOND:
-	s = _pCaseCond->toChain();
+	s = _pCaseCond->toChain(defTabSetId);
+	break;
+    case CegoFactor::CONDITION:
+	s = _pCond->toChain(defTabSetId);
 	break;
     }
     return s;
@@ -1016,142 +1093,11 @@ Chain CegoFactor::toChain() const
 
 Chain CegoFactor::dbFormat(CegoDatabaseFormater *pForm) const
 {
-    return pForm->formatFactor(_type, _fv, _pAttrDesc, _varName, _pExpr, _pFetch, _pAggr, _pFunction, _pSelect, _pCaseCond);
+    return pForm->formatFactor(_type, _fv, _pAttrDesc, _varName, _pExpr, _pFetch, _pAggr, _pFunction, _pSelect, _pCaseCond, _pCond);
 }
 
-Element* CegoFactor::toElement() const
-{
-    Element* pFactorElement = new Element(XML_FACTOR_ELEMENT);
-
-    switch (_type)
-    {
-    case CegoFactor::CONSTVAL:
-    {
-	CegoTypeConverter tc;
-	pFactorElement->setAttribute( XML_FACTOR_ATTR, XML_CONST_VALUE );
-	pFactorElement->setAttribute( XML_TYPE_ATTR, tc.getTypeString(_fv.getType()));
-	pFactorElement->setAttribute( XML_VALUE_ATTR, _fv.valAsChain() );
-	break;
-    }
-    case CegoFactor::ATTR:
-    {
-	pFactorElement->setAttribute( XML_FACTOR_ATTR, XML_ATTR_VALUE );
-	if ( _pAttrDesc->getTableName().length() > 0 )
-	    pFactorElement->setAttribute( XML_TABLENAME_ATTR, _pAttrDesc->getTableName() );
-	pFactorElement->setAttribute( XML_ATTRNAME_ATTR, _pAttrDesc->getAttrName() );
-	break;
-    }
-    case CegoFactor::VAR:
-    {
-	pFactorElement->setAttribute( XML_FACTOR_ATTR, XML_VAR_VALUE );
-	pFactorElement->setAttribute( XML_VALUE_ATTR, _varName );
-	break;
-    }
-    case CegoFactor::EXPR:
-    {
-	pFactorElement->setAttribute( XML_FACTOR_ATTR, XML_EXPR_VALUE );
-	pFactorElement->addContent( _pExpr->toElement() );
-	break;
-    }
-    case CegoFactor::FETCH:
-    {
-	throw Exception(EXLOC, "Cursor fetch not supported in distributed query");
-    }
-    case CegoFactor::CASECOND:
-    {
-	throw Exception(EXLOC, "case condition not supported in distributed query");
-    }
-    case CegoFactor::AGGREGATION:
-    {
-	throw Exception(EXLOC, "Aggregation not supported in distributed query");
-    }
-    case CegoFactor::FUNCTION:
-    {
-	pFactorElement->setAttribute( XML_FACTOR_ATTR, XML_FUNCTION_VALUE );
-	pFactorElement->addContent( _pFunction->toElement() );
-	break;
-    }
-    case CegoFactor::QUERY:
-    {
-	pFactorElement->setAttribute( XML_FACTOR_ATTR, XML_SELECT_VALUE );
-	pFactorElement->addContent( _pSelect->toElement() );
-	break;
-    }
-    }
-    return pFactorElement;
-}
-
-void CegoFactor::fromElement(Element *pFactorElement, CegoDistManager* pGTM)
-{
-
-    Chain factorTypeString = pFactorElement->getAttributeValue( XML_FACTOR_ATTR );
-
-    if ( factorTypeString == Chain(XML_CONST_VALUE) )
-    {
-	_type = CegoFactor::CONSTVAL;
-	CegoTypeConverter tc;
-	_fv = CegoFieldValue( tc.getTypeId(pFactorElement->getAttributeValue( XML_TYPE_ATTR )),
-			      pFactorElement->getAttributeValue( XML_VALUE_ATTR ));
-	
-    }
-    else if ( factorTypeString == Chain(XML_ATTR_VALUE) )
-    {
-	_type = CegoFactor::ATTR;
-	Chain tableName = pFactorElement->getAttributeValue( XML_TABLENAME_ATTR );
-	if ( tableName.length() > 0 )
-	    _pAttrDesc = new CegoAttrDesc( pFactorElement->getAttributeValue( XML_TABLENAME_ATTR ),
-				      pFactorElement->getAttributeValue( XML_ATTRNAME_ATTR ));
-	else
-	    _pAttrDesc = new CegoAttrDesc( pFactorElement->getAttributeValue( XML_ATTRNAME_ATTR ));
-    }
-    else if ( factorTypeString == Chain(XML_VAR_VALUE) )
-    {
-	_type = CegoFactor::VAR;
-	_varName =  pFactorElement->getAttributeValue( XML_VALUE_ATTR );
-    }
-    else if ( factorTypeString == Chain(XML_EXPR_VALUE) )
-    {
-	_type = CegoFactor::EXPR;
-	ListT<Element*> el = pFactorElement->getChildren(XML_EXPR_ELEMENT);
-	Element **pEE = el.First();
-	if ( pEE )
-	{
-	    _pExpr = new CegoExpr(*pEE, pGTM);
-	}
-    }
-    else if ( factorTypeString == Chain(XML_FETCH_VALUE) )
-    {
-	throw Exception(EXLOC, "Cursor fetch not supported in distributed query");
-    }
-    else if ( factorTypeString == Chain(XML_AGGREGATION_VALUE) )
-    {
-	throw Exception(EXLOC, "Aggregation not supported in distributed query");
-    }
-    else if ( factorTypeString == Chain(XML_FUNCTION_VALUE) )
-    {
-	_type = CegoFactor::FUNCTION;
-	ListT<Element*> fl = pFactorElement->getChildren(XML_FUNCTION_ELEMENT);
-	Element **pFE = fl.First();
-	if ( pFE )
-	{
-	    _pFunction = new CegoFunction(*pFE, pGTM);
-	}
-    }
-    else if ( factorTypeString == Chain(XML_SELECT_VALUE) )
-    {
-	_type = CegoFactor::QUERY;
-	ListT<Element*> sl = pFactorElement->getChildren(XML_SELECT_ELEMENT);
-	Element **pSE = sl.First();
-	if ( pSE )
-	{
-	    _pSelect = new CegoSelect(*pSE, pGTM);
-	}
-    }
-}
-
-void CegoFactor::encode(char *buf)
-{
-    
+void CegoFactor::encode(char *buf, CegoProcBlock *pBlock)
+{    
     char* pE = (char*)buf;
 
     switch (_type )
@@ -1178,12 +1124,13 @@ void CegoFactor::encode(char *buf)
 	CegoFactor::FacType type = CegoFactor::CONSTVAL;
 	memcpy( pE, &type, sizeof(CegoFactor::FacType));
 	pE = pE + sizeof(CegoFactor::FacType);
-	
-	CegoFieldValue fv;
-	if ( _pBlock )
+
+	CegoFieldValue fv;	
+	if ( pBlock )
 	{
-	    fv = _pBlock->getValue(_varName);
-	}      
+	    fv = pBlock->getValue(_varName);
+	}
+	
 	fv.encode(pE);
 	pE = pE + fv.getEncodingLength();
 	break;
@@ -1192,8 +1139,8 @@ void CegoFactor::encode(char *buf)
     {
 	memcpy( pE, &_type, sizeof(CegoFactor::FacType));
 	pE = pE + sizeof(CegoFactor::FacType);
-	_pExpr->encode(pE);
-	pE = pE + _pExpr->getEncodingLength();
+	_pExpr->encode(pE, pBlock);
+	pE = pE + _pExpr->getEncodingLength(pBlock);
 	break;
     }
     case CegoFactor::FETCH:
@@ -1205,40 +1152,47 @@ void CegoFactor::encode(char *buf)
     {
 	memcpy( pE, &_type, sizeof(CegoFactor::FacType));
 	pE = pE + sizeof(CegoFactor::FacType);
-	_pCaseCond->encode(pE);
-	pE = pE + _pCaseCond->getEncodingLength();
+	_pCaseCond->encode(pE, pBlock);
+	pE = pE + _pCaseCond->getEncodingLength(pBlock);
+	break;
+    }
+    case CegoFactor::CONDITION:
+    {
+	memcpy( pE, &_type, sizeof(CegoFactor::FacType));
+	pE = pE + sizeof(CegoFactor::FacType);
+	_pCond->encode(pE, pBlock);
+	pE = pE + _pCond->getEncodingLength(pBlock);
 	break;
     }
     case CegoFactor::AGGREGATION:
     {
 	memcpy( pE, &_type, sizeof(CegoFactor::FacType));
 	pE = pE + sizeof(CegoFactor::FacType);
-	_pAggr->encode(pE);
-	pE = pE + _pAggr->getEncodingLength();
+	_pAggr->encode(pE, pBlock);
+	pE = pE + _pAggr->getEncodingLength(pBlock);
 	break;
     }
     case CegoFactor::FUNCTION:
     {
 	memcpy( pE, &_type, sizeof(CegoFactor::FacType));
 	pE = pE + sizeof(CegoFactor::FacType);
-	_pFunction->encode(pE);
-	pE = pE + _pFunction->getEncodingLength();
+	_pFunction->encode(pE, pBlock);
+	pE = pE + _pFunction->getEncodingLength(pBlock);
 	break;
     }
     case CegoFactor::QUERY:
     {
 	memcpy( pE, &_type, sizeof(CegoFactor::FacType));
 	pE = pE + sizeof(CegoFactor::FacType);
-	_pSelect->encode(pE);
-	pE = pE + _pSelect->getEncodingLength();
+	_pSelect->encode(pE, pBlock);
+	pE = pE + _pSelect->getEncodingLength(pBlock);
 	break;
     }
     }
 }
 
-void CegoFactor::decode(char *buf, CegoDistManager* pGTM, int tabSetId)
+void CegoFactor::decode(char *buf, CegoDistManager* pGTM, CegoProcBlock *pBlock, int tabSetId)
 {
-
     char* pE = (char*)buf;
 
     memcpy( &_type, pE, sizeof(CegoFactor::FacType));
@@ -1266,8 +1220,8 @@ void CegoFactor::decode(char *buf, CegoDistManager* pGTM, int tabSetId)
     }
     case CegoFactor::EXPR:
     {
-	_pExpr = new CegoExpr(pE, pGTM, tabSetId);
-	pE = pE + _pExpr->getEncodingLength();
+	_pExpr = new CegoExpr(pE, pGTM, pBlock, tabSetId);
+	pE = pE + _pExpr->getEncodingLength(pBlock);
 	break;
     }
     case CegoFactor::FETCH:
@@ -1277,34 +1231,39 @@ void CegoFactor::decode(char *buf, CegoDistManager* pGTM, int tabSetId)
     }
     case CegoFactor::CASECOND:
     {
-	_pCaseCond = new CegoCaseCond(pE, pGTM, tabSetId);
-	pE = pE + _pCaseCond->getEncodingLength();
+	_pCaseCond = new CegoCaseCond(pE, pGTM, pBlock, tabSetId);
+	pE = pE + _pCaseCond->getEncodingLength(pBlock);
+	break;
+    }
+    case CegoFactor::CONDITION:
+    {
+	_pCond = new CegoCondition(pE, pGTM, pBlock, tabSetId);
+	pE = pE + _pCond->getEncodingLength(pBlock);
 	break;
     }
     case CegoFactor::AGGREGATION:
     {
-	_pAggr = new CegoAggregation(pE, pGTM, tabSetId);
-	pE = pE + _pAggr->getEncodingLength();
+	_pAggr = new CegoAggregation(pE, pGTM, pBlock, tabSetId);
+	pE = pE + _pAggr->getEncodingLength(pBlock);
 	break;
     }
     case CegoFactor::FUNCTION:
     {
-	_pFunction = new CegoFunction(pE, pGTM, tabSetId);
-	pE = pE + _pFunction->getEncodingLength();
+	_pFunction = new CegoFunction(pE, pGTM, pBlock, tabSetId);
+	pE = pE + _pFunction->getEncodingLength(pBlock);
 	break;
     }
     case CegoFactor::QUERY:
     {
-	_pSelect = new CegoSelect(pE, pGTM, tabSetId);
-	pE = pE + _pSelect->getEncodingLength();
+	_pSelect = new CegoSelect(pE, pGTM, pBlock, tabSetId);
+	pE = pE + _pSelect->getEncodingLength(pBlock);
 	break;
     }
     }
 }
 
-int CegoFactor::getEncodingLength() const
-{
-    
+int CegoFactor::getEncodingLength(CegoProcBlock *pBlock) const
+{   
     int len = 0;
 
     len += sizeof(CegoFactor::FacType);
@@ -1323,18 +1282,19 @@ int CegoFactor::getEncodingLength() const
     }
     case CegoFactor::VAR:
     {
-
 	CegoFieldValue fv;
-	if ( _pBlock )
+
+	if ( pBlock )
 	{
-	    fv = _pBlock->getValue(_varName);
+	    fv = pBlock->getValue(_varName);
 	}
+	
 	len += fv.getEncodingLength();
 	break;
     }
     case CegoFactor::EXPR:
     {
-	len += _pExpr->getEncodingLength();
+	len += _pExpr->getEncodingLength(pBlock);
 	break;
     }
     case CegoFactor::FETCH:
@@ -1344,38 +1304,46 @@ int CegoFactor::getEncodingLength() const
     }
     case CegoFactor::CASECOND:
     {
-	len += _pCaseCond->getEncodingLength();
+	len += _pCaseCond->getEncodingLength(pBlock);
+	break;
+    }
+    case CegoFactor::CONDITION:
+    {
+	len += _pCond->getEncodingLength(pBlock);
 	break;
     }
     case CegoFactor::AGGREGATION:
     {
-	len += _pAggr->getEncodingLength();
+	len += _pAggr->getEncodingLength(pBlock);
 	break;
     }
     case CegoFactor::FUNCTION:
     {
-	len += _pFunction->getEncodingLength();
+	len += _pFunction->getEncodingLength(pBlock);
 	break;
     }
     case CegoFactor::QUERY:
     {
-	len +=_pSelect->getEncodingLength();
+	len +=_pSelect->getEncodingLength(pBlock);
 	break;
     }
     }
-
-    return len;
     
+    return len;    
 }
 
 void CegoFactor::getPlanList(ListT<Element*>& planList)
 {
     if ( _pSelect )
-	planList.Insert(_pSelect->getPlan());
+    planList.Insert(_pSelect->getPlan());
+    if ( _pCaseCond )
+    _pCaseCond->getPlanList(planList);
+    if ( _pCond )
+    _pCond->getPlanList(planList);
 }
 
 ostream& operator << (ostream& s, const CegoFactor& pf)
 {
-    s << pf.toChain();
+    s << pf.toChain(0, Chain());
     return s;
 }
