@@ -8,7 +8,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -37,16 +37,15 @@ class CegoDistManager;
 class CegoExpr;
 class CegoSelect;
 class CegoCaseCond;
+class CegoCondition;
 
 class CegoFactor {
     
  public:
     
-    enum FacType { CONSTVAL, VAR, EXPR, FETCH, ATTR, FUNCTION, QUERY, AGGREGATION, CASECOND };
-
-    CegoFactor(char* buf, CegoDistManager *pGTM, int tabSetId);
-    CegoFactor(Element* pFactorElement, CegoDistManager *pGTM);
+    enum FacType { CONSTVAL, VAR, EXPR, FETCH, ATTR, FUNCTION, QUERY, AGGREGATION, CASECOND, CONDITION };
 
+    CegoFactor(char* buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId);
     CegoFactor(const CegoFactor& pf);
     CegoFactor(const Chain& varName);
     CegoFactor(const CegoFieldValue& fv);
@@ -57,33 +56,41 @@ class CegoFactor {
     CegoFactor(CegoAggregation* pAggr);
     CegoFactor(CegoSelect* pSelect);
     CegoFactor(CegoCaseCond* pCaseCond);
+    CegoFactor(CegoCondition* pCond);
 
     ~CegoFactor();
 
-    void setFieldListArray(ListT<CegoField> **pFLA);
-
-    void setBlock(CegoProcBlock *pBlock);
+    void cleanUp();
 
+    void setTabSetId(int tabSetId);
+    
     void clearAttrCache();
 
     void getFunctionList(SetT<CegoObject>& funcList);
 
+    bool checkConst() const;
     bool checkVar(Chain& var) const;
     CegoAttrDesc* checkAttr() const;
+    
+    bool checkLob(CegoFieldValue& fv, CegoProcBlock *pBlock);
 
     CegoAttrDesc* getAttr() const;
     CegoFunction* getFunction() const;
     CegoAggregation* getAggregation() const;
     CegoSelect* getSelect() const;
     CegoCaseCond* getCaseCond() const;
+    CegoCondition* getCondition() const;
 
-    ListT<CegoField> getFieldList() const;
+    CegoCondition* grapCondition();
+
+    void getFieldList(ListT<CegoField>& fl, ListT<CegoField>** pFLA) const;
 
     ListT<CegoAttrDesc*> getAttrRefList() const;
     int evalReferences(CegoContentObject *pCO, const ListT<CegoField>& fl);
     
-    CegoField evalField() const;
-    CegoFieldValue evalFieldValue();
+    CegoField evalField(const ListT<CegoField>& fl, bool graceful = false) const;
+
+    CegoFieldValue evalFieldValue(ListT<CegoField>** pFLA, CegoProcBlock *pBlock);
 
     const Chain& getVarName() const;
     const CegoFieldValue& getFieldValue() const;
@@ -95,16 +102,13 @@ class CegoFactor {
 
     CegoFactor& operator = ( const CegoFactor& pf);
 
-    Chain getId() const;
-    Chain toChain() const;
+    Chain getId(CegoProcBlock *pBlock) const;
+    Chain toChain(int defTabSetId, const Chain& indent) const;
     Chain dbFormat(CegoDatabaseFormater *pForm) const;
-
-    Element* toElement() const;
-    void fromElement(Element *pFactorElement, CegoDistManager* pGTM);
-
-    void encode(char *buf);
-    void decode(char *buf, CegoDistManager* pGTM, int tabSetId);
-    int getEncodingLength() const;
+    
+    void encode(char *buf, CegoProcBlock *pBlock);
+    void decode(char *buf, CegoDistManager* pGTM, CegoProcBlock *pBlock, int tabSetId);
+    int getEncodingLength(CegoProcBlock *pBlock) const;
 
     ListT<CegoAggregation*> getAggregationList();
 
@@ -125,16 +129,9 @@ class CegoFactor {
     CegoAggregation* _pAggr;
     CegoSelect* _pSelect;
     CegoCaseCond* _pCaseCond;
-    
-    ListT<CegoField>** _pFLA;
-
+    CegoCondition* _pCond;    
     int _flaCacheArrayPos;
     int _flaCacheFieldPos;
     bool _flaCached;
-
-    CegoProcBlock* _pBlock;
 };
-
 #endif
-
-
