@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -26,18 +26,23 @@
 #include "CegoBufferPage.h"
 
 #define BIT_PER_BYTE 8
+#define FILEHEADSIZE sizeof(int) + sizeof(FileType) + sizeof(int) + sizeof(PageIdType)
 
-CegoFileHandler::CegoFileHandler(const Chain& logFile, const Chain& progName, int pageSize) : CegoModule(logFile, progName)
+extern bool __fsyncOn;
+
+CegoFileHandler::CegoFileHandler(const Chain& logFile, const Chain& progName) : CegoModule(logFile, progName)
 {
-    _currentPageOffset=0;
     for (int i = 0; i<FILMNG_MAXDATAFILE; i++)
     {
 	_isReg[i]=false;
 	_tabSetId[i]=0;
 	_fhList[i]=0;
 	_buMask[i]=0;
-	_regPageOffset[i]=0;
+	_commitMask[i]=0;
+	_pageOffset[i]=0;
     }
+    _appendFid = 0;
+    _isReadOnly = false;
     _modId = getModId("CegoFileHandler");
 }
 
@@ -48,26 +53,33 @@ CegoFileHandler::~CegoFileHandler()
 	if ( _fhList[i] != 0 )
 	{
 	    _fhList[i]->close();
+	    delete _fhList[i];
 	}
+	if ( _buMask[i] != 0 )
+	{
+	    delete _buMask[i];
+	}
+	if ( _commitMask[i] != 0 )
+	{
+	    delete _commitMask[i];
+	}	
     }    
 }
 
-bool CegoFileHandler::isBackup(int fileId)
+void CegoFileHandler::setReadOnly(bool isReadOnly)
 {
-    if ( _buMask[fileId] )
-	return true;
-    return false;
+    _isReadOnly = isReadOnly;
 }
 
 void CegoFileHandler::setBackup(int fileId, bool m)
 {
     if ( m == true )
     {
-	_buMask[fileId] = new unsigned[ (  getNumPages(fileId) / (BIT_PER_BYTE * sizeof(int))) + 1];
+	_buMask[fileId] = new unsigned[ (  getNumPages(fileId) / (BIT_PER_BYTE * sizeof(unsigned))) + 1];
 
 	int i;
 	
-	for (i = 0; i < ( getNumPages(fileId) / (BIT_PER_BYTE * sizeof(int))) + 1; i++)
+	for (i = 0; i < ( getNumPages(fileId) / (BIT_PER_BYTE * sizeof(unsigned))) + 1; i++)
 	{
 	    _buMask[fileId][i] = 0;
 	}
@@ -83,15 +95,34 @@ void CegoFileHandler::setBackup(int fileId, bool m)
     }
 }
 
-void CegoFileHandler::markPage(int fileId, int pageId)
+bool CegoFileHandler::needPageBackup(PageIdType pageId)
 {
+    int fileId = getFileIdForPageId(pageId);
+
+    // check if backup is activated for file
+    if ( _buMask[fileId] )
+    {
+	// is page already marked, no need to backup page
+	if ( isPageMarked(pageId, fileId) )
+	    return false;
+	return true;	
+    }
+
+    return false;
+}
+
+void CegoFileHandler::markPage(PageIdType pageId)
+{
+    int fileId = getFileIdForPageId(pageId);
+    
     if ( _buMask[fileId] == 0 )
     {
 	Chain msg = Chain("No backup mode for fileId ") + Chain(fileId);
 	throw Exception(EXLOC, msg);
     }
-    unsigned bmid = pageId / ( BIT_PER_BYTE * sizeof(int));
-    unsigned bmoffset = pageId % ( BIT_PER_BYTE * sizeof(int));
+
+    unsigned bmid = ( pageId - _pageOffset[fileId] ) / ( BIT_PER_BYTE * sizeof(unsigned));
+    unsigned bmoffset = ( pageId - _pageOffset[fileId] ) % ( BIT_PER_BYTE * sizeof(unsigned));
     
     unsigned bm = _buMask[fileId][bmid];
     
@@ -99,20 +130,18 @@ void CegoFileHandler::markPage(int fileId, int pageId)
     f = f << bmoffset;
     
     _buMask[fileId][bmid] = bm | f;
-
 }
 
-bool CegoFileHandler::isMarked(int fileId, int pageId)
+bool CegoFileHandler::isPageMarked(PageIdType pageId, int fileId)
 {
-
     if ( _buMask[fileId] == 0 )
     {
 	Chain msg = Chain("No backup mode for fileId ") + Chain(fileId);
 	throw Exception(EXLOC, msg);
-
     }
-    unsigned bmid = pageId / ( BIT_PER_BYTE * sizeof(int));
-    unsigned bmoffset = pageId % ( BIT_PER_BYTE * sizeof(int));
+   
+    unsigned bmid = ( pageId - _pageOffset[fileId] ) / ( BIT_PER_BYTE * sizeof(unsigned));
+    unsigned bmoffset = ( pageId - _pageOffset[fileId] ) % ( BIT_PER_BYTE * sizeof(unsigned));
     
     unsigned bm = _buMask[fileId][bmid];    
         
@@ -123,17 +152,16 @@ bool CegoFileHandler::isMarked(int fileId, int pageId)
 	return true;
     }
     return false;
-
 }
 
-unsigned long long CegoFileHandler::getRegPageOffset(int fileId)
+PageIdType CegoFileHandler::getPageOffset(int fileId)
 {
-    return _regPageOffset[fileId];
+    return _pageOffset[fileId];
 }
 
 int CegoFileHandler::getFBMSize(int fileId)
 {
-    return (_numPages[fileId] / (BIT_PER_BYTE * sizeof(int))) + 1;
+    return (_numPages[fileId] / (BIT_PER_BYTE * sizeof(unsigned))) + 1;
 }
 
 void CegoFileHandler::readFBM(int fileId, unsigned *fbm, CegoLockHandler *pLockHandle)
@@ -146,120 +174,71 @@ void CegoFileHandler::readFBM(int fileId, unsigned *fbm, CegoLockHandler *pLockH
     }
     catch ( Exception e )
     {
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot get file handle"), e);
     }
 
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
 
     try 
     {
-	
-	pF->seek(0);
+	pF->seek(FILEHEADSIZE);
 
-	int tabSetId;
-	pF->readByte((char*)&tabSetId, sizeof(int));
-
-	int numPages = (_numPages[fileId] / (BIT_PER_BYTE * sizeof(int))) + 1;
+	int numBitmap = (_numPages[fileId] / (BIT_PER_BYTE * sizeof(unsigned))) + 1;
 	int i;
-	for (i = 0; i < numPages; i++)
+	for (i = 0; i < numBitmap; i++)
 	{
-	    pF->readByte((char*)&fbm[i], sizeof(int));
+	    pF->readByte((char*)&fbm[i], sizeof(unsigned));
 	}
 		
 	pLockHandle->unlockDataFile(fileId);
-
     }
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot read fbm"), e);
     }
 }
 
-
 void CegoFileHandler::writeFBM(int fileId, unsigned *fbm, CegoLockHandler *pLockHandle)
 {
+    if ( _isReadOnly == true )
+	return;
+    
     File *pF;
-
+    
     try 
     {
 	pF = getHandle(fileId);
     }
     catch ( Exception e )
     {
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot get file handle"), e);
     }
 
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
 
     try 
-    {
-	
-	pF->seek(0);
+    {	
+	pF->seek(FILEHEADSIZE);
 
-	int tabSetId;
-	pF->readByte((char*)&tabSetId, sizeof(int));
-
-
-	int numPages = (_numPages[fileId] / (BIT_PER_BYTE * sizeof(int))) + 1;
+	int numBitmap = (_numPages[fileId] / (BIT_PER_BYTE * sizeof(unsigned))) + 1;
 	int i;
 
-	for (i = 0; i < numPages; i++)
+	for (i = 0; i < numBitmap; i++)
 	{
-	    pF->writeByte((char*)&fbm[i], sizeof(int));
+	    pF->writeByte((char*)&fbm[i], sizeof(unsigned));
 	}
-		
+
+	if ( __fsyncOn )
+	    pF->flush();
+	
 	pLockHandle->unlockDataFile(fileId);
 
     }
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw e;
-    }
-}
-
-
-
-/* the cleanDataFile method traces the corresponding datafile to check for
-   allocated but unused pages. This may happen, if the allocatePage method
-   was called, but for any reason ( e.g. a system crash ) the page could never be written
-   back to file. In this case, the fixStat attribute is still zero. so the page
-   can be released for other use */ 
- 
-void CegoFileHandler::cleanDataFile(int fileId, CegoLockHandler *pLockHandle)
-{
-
-    File* pF;
-
-    try 
-    {
-	pF = getHandle(fileId);
-    }
-    catch ( Exception e )
-    {
-	throw e;
-    }
-    
-    int offset = sizeof(FileType) +  3 * sizeof(int)
-	+  (_numPages[fileId] / ( BIT_PER_BYTE * sizeof(int) )) * sizeof(int) ;
-    
-    int pageId;
-    
-    for (pageId = 0; pageId < _numPages[fileId]; pageId++)
-    {
-	
-	pF->seek(offset + pageId * ( _pageSize + sizeof(int)));
-	int fixStat;
-	pF->readByte((char*)&fixStat, sizeof(int));    
-	
-	if ( fixStat == 0 && isClaimed(fileId, pageId, pLockHandle) == true )
-	{
-	    log(_modId, Logger::NOTICE, Chain("Releasing page ") + Chain(pageId));
-	    unsigned *fbm;
-	    int fbmSize;
-	    releasePage(fileId, pageId, pLockHandle, fbm, fbmSize);
-	}
+	throw Exception(EXLOC, Chain("Cannot write fbm"), e);
     }
 }
 
@@ -267,12 +246,13 @@ void CegoFileHandler::cleanDataFile(int fileId, CegoLockHandler *pLockHandle)
    the file is created and initialized. Note, that after creation the datafile is still NOT registered for further usage.
    This must be done using the regDataFile method */
 
-void CegoFileHandler::initDataFile(int tabSetId, const Chain& path, int fileId, int numPages, FileType ft)
+void CegoFileHandler::initDataFile(int tabSetId, const Chain& path, int fileId, int numPages, PageIdType pageOffset, FileType ft)
 {
+    if ( _isReadOnly == true )
+	return;
     
     File* pF = new File(path);
     
-
     if ( pF->exists() )
     {      
 	Chain msg = Chain("Datafile ") + path + Chain(" already exists");
@@ -280,23 +260,23 @@ void CegoFileHandler::initDataFile(int tabSetId, const Chain& path, int fileId,
 	throw Exception( EXLOC, msg);
     }
 
-
     try 
     {
-
 	pF->open(File::WRITE);
 	
 	pF->writeByte((char*)&tabSetId, sizeof(int));
 	pF->writeByte((char*)&ft, sizeof(FileType));
 	// cout << "Init datafile " << path << " with numPages " << numPages << endl;
 	pF->writeByte((char*)&numPages, sizeof(int));
+	
+	pF->writeByte((char*)&pageOffset, sizeof(PageIdType));
 		
 	int i;
 	
-	for (i = 0; i < (numPages / (BIT_PER_BYTE * sizeof(int))) + 1; i++)
+	for (i = 0; i < (numPages / (BIT_PER_BYTE * sizeof(unsigned))) + 1; i++)
 	{
 	    unsigned bm = 0;
-	    pF->writeByte((char*)&bm, sizeof(int));
+	    pF->writeByte((char*)&bm, sizeof(unsigned));
 	}
 	
 	char* initBuf = new char[_pageSize];
@@ -305,8 +285,8 @@ void CegoFileHandler::initDataFile(int tabSetId, const Chain& path, int fileId,
 	    
 	for (i = 0; i < numPages; i++)
 	{
-	    int fixStat = 0;
-	    pF->writeByte((char*)&fixStat, sizeof(int));
+	    unsigned fixStat = 0;
+	    pF->writeByte((char*)&fixStat, sizeof(unsigned));
 	    pF->writeByte(initBuf, _pageSize);
 	}
 	delete[] initBuf;
@@ -317,10 +297,40 @@ void CegoFileHandler::initDataFile(int tabSetId, const Chain& path, int fileId,
     catch ( Exception e )
     {
 	delete pF;
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot init datafile"), e);
     }
 }
 
+void CegoFileHandler::resetDataFile(int fileId)
+{
+    if ( _isReadOnly == true )
+	return;
+
+    File* pF;
+
+    try 
+    {
+	pF = getHandle(fileId);
+    }
+    catch ( Exception e )
+    {
+	throw Exception(EXLOC, Chain("Cannot get file handle"), e);
+    }
+    
+    pF->seek(FILEHEADSIZE);
+        
+    int i;
+    
+    for (i = 0; i < ( _numPages[fileId] / (BIT_PER_BYTE * sizeof(unsigned))) + 1; i++)
+    {
+	unsigned bm = 0;
+	pF->writeByte((char*)&bm, sizeof(unsigned));
+    }
+
+    if ( __fsyncOn )
+	pF->flush();   
+}
+
 /* the regDataFile method is used for datafile registration. The given file is checked
    for consistency and is registered */
 
@@ -330,7 +340,6 @@ void CegoFileHandler::regDataFile(int tabSetId, const Chain& path, int fileId, C
 
     if ( _isReg[fileId] )
     {
-
 	pLockHandle->unlockDataFile(fileId);
 
 	if ( _path[fileId] == path )
@@ -347,19 +356,21 @@ void CegoFileHandler::regDataFile(int tabSetId, const Chain& path, int fileId, C
 
     int checkTabSetId;
     int numPage;
+    PageIdType pageOffset;
+    
     FileType ft;
 	
-    try {
-
+    try
+    {
 	File checkFile(path);	    
 	checkFile.open(File::READ);
 	
 	checkFile.readByte((char*)&checkTabSetId, sizeof(int));	
 	checkFile.readByte((char*)&ft, sizeof(FileType));
 	checkFile.readByte((char*)&numPage, sizeof(int));
+	checkFile.readByte((char*)&pageOffset, sizeof(PageIdType));
 
-	checkFile.close();
-	
+	checkFile.close();	
     }
     catch ( Exception e )
     {
@@ -375,16 +386,14 @@ void CegoFileHandler::regDataFile(int tabSetId, const Chain& path, int fileId, C
 	pLockHandle->unlockDataFile(fileId);
 	throw Exception(EXLOC, "Table Set Id does not match");
     }
-    
-    _regPageOffset[fileId] = _currentPageOffset;
+
+    _pageOffset[fileId] = pageOffset;
     _isReg[fileId] = true;
     _path[fileId] = path;
     _tabSetId[fileId] = tabSetId;
     _fileType[fileId] = ft;
     _numPages[fileId] = numPage;
 
-    _currentPageOffset += numPage;
-
     pLockHandle->unlockDataFile(fileId);
 }
 
@@ -393,13 +402,16 @@ void CegoFileHandler::regDataFile(int tabSetId, const Chain& path, int fileId, C
 
 void CegoFileHandler::releaseFiles(int tabSetId)
 {
-
     for (int i = 0; i<FILMNG_MAXDATAFILE; i++)
     {
-	if ( _tabSetId[i] == tabSetId && _fhList[i] != 0 )
-	{    
-	    _fhList[i]->close();
-	    _fhList[i]=0;
+	if ( _tabSetId[i] == tabSetId )
+	{
+	    if ( _fhList[i] != 0 )
+	    {
+		_fhList[i]->close();
+		delete _fhList[i];
+		_fhList[i]=0;
+	    }
 	    _tabSetId[i]=0;
 	    _isReg[i]=false;    
 	}
@@ -411,14 +423,13 @@ void CegoFileHandler::releaseFiles(int tabSetId)
    the number of bufferFix operations on the page and is normally used by the BufferPool class for the page
    relocation strategy */ 
 
-void CegoFileHandler::writePage(int fileId, int pageId, int fixStat, char* pageData, CegoLockHandler *pLockHandle)
-{
-
-    // if ( isClaimed(fileId, pageId, pLockHandle) == false )
-    //	cout << "WRITE PAGE " << pageId << " IS NOT CLAIMED" << endl;
-
-    // cout << "Write page " << fileId << " " << pageId << endl;
-
+void CegoFileHandler::writePage(PageIdType pageId, unsigned fixStat, char* pageData, CegoLockHandler *pLockHandle)
+{   
+    if ( _isReadOnly == true )
+	throw Exception(EXLOC, Chain("Cannot write page in read only mode"));
+        
+    int fileId = getFileIdForPageId(pageId);
+    
     File* pF;
 
     try 
@@ -427,47 +438,42 @@ void CegoFileHandler::writePage(int fileId, int pageId, int fixStat, char* pageD
     }
     catch ( Exception e )
     {
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot get file handle"), e);
     }
 
-
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
 
     try 
     {
-
-	int offset = sizeof(int) + sizeof(FileType) + 2 * sizeof(int)
-	    +  (_numPages[fileId] / ( BIT_PER_BYTE * sizeof(int) )) * sizeof(int) ;
+	unsigned long offset = FILEHEADSIZE
+	    +  ((_numPages[fileId] / ( BIT_PER_BYTE * sizeof(unsigned) )) + 1) * sizeof(unsigned);
 		
-	pF->seek(offset + pageId * ( _pageSize + sizeof(int)));
-	pF->writeByte((char*)&fixStat, sizeof(int));    
+	pF->seek(offset + ( pageId - _pageOffset[fileId] ) * ( _pageSize + sizeof(unsigned)));
+	pF->writeByte((char*)&fixStat, sizeof(unsigned));    
 	pF->writeByte(pageData, _pageSize);
-
 	
 	if ( _buMask[fileId] )
-	    markPage(fileId, pageId);
+	    markPage(pageId);
+
+	if ( __fsyncOn )
+	    pF->flush();
 	
-
 	pLockHandle->unlockDataFile(fileId);
-
     }
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot write data page"), e);
     }
-
 }
 
 /* pages from datafiles are read by using the readPage method. The method seeks to the appropriate offset
    in the corresponding datafile and reads the page data into the given data pointer */
 
-void CegoFileHandler::readPage(int fileId, int pageId, int& tabSetId, int& fixStat, char* pageData, CegoLockHandler *pLockHandle)
+void CegoFileHandler::readPage(PageIdType pageId, int& tabSetId, unsigned& fixStat, char* pageData, CegoLockHandler *pLockHandle)
 {
-
-    // if ( isClaimed(fileId, pageId, pLockHandle) == false )
-    //	cout << "READ PAGE " << pageId << "IS NOT CLAIMED" << endl;
-
+    int fileId = getFileIdForPageId(pageId);
+    
     File *pF;
 
     try 
@@ -476,31 +482,28 @@ void CegoFileHandler::readPage(int fileId, int pageId, int& tabSetId, int& fixSt
     }
     catch ( Exception e )
     {
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot get file handle"), e);
     }
 
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
 
     try 
     {
-
 	pF->seek(0);
 	pF->readByte((char*)&tabSetId, sizeof(int));
 	
-	int offset = sizeof(int) + sizeof(FileType) + 2 * sizeof(int) 
-	    +  (_numPages[fileId] / ( BIT_PER_BYTE * sizeof(int) )) * sizeof(int) ;
+	unsigned long offset = FILEHEADSIZE 
+	    +  ((_numPages[fileId] / ( BIT_PER_BYTE * sizeof(unsigned) )) + 1) * sizeof(unsigned) ;
 
-	pF->seek(offset + pageId * ( _pageSize + sizeof(int)) );
+	pF->seek(offset + ( pageId - _pageOffset[fileId] ) * ( _pageSize + sizeof(unsigned)) );
 	
-	pF->readByte((char*)&fixStat, sizeof(int));
+	pF->readByte((char*)&fixStat, sizeof(unsigned));
 	
 	unsigned long rb = pF->readByte(pageData, _pageSize);
 	while ( rb < _pageSize )
-	{
-	  
+	{	  
 	    unsigned long rrb = pF->readByte((char*)((long long)pageData + rb), _pageSize - rb);
-	    
-	    
+	       
 	    if ( rrb == 0 )
 	      throw Exception(EXLOC, Chain("Incomplete file read"));
 	    
@@ -508,12 +511,11 @@ void CegoFileHandler::readPage(int fileId, int pageId, int& tabSetId, int& fixSt
 	}
 	
 	pLockHandle->unlockDataFile(fileId);
-
     }
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot read data page"), e);
     }
 }
 
@@ -521,29 +523,32 @@ void CegoFileHandler::readPage(int fileId, int pageId, int& tabSetId, int& fixSt
    Normally this method is used for "well known entry pages", e.g. the system catalog page
    which is used as an entry point. */
  
-void CegoFileHandler::claimPage(int fileId, int pageId, CegoLockHandler *pLockHandle)
+void CegoFileHandler::claimPage(PageIdType pageId, CegoLockHandler *pLockHandle)
 {
+    if ( _isReadOnly == true )
+	throw Exception(EXLOC, Chain("Cannot claim page in read only mode"));
+
+    int fileId = getFileIdForPageId(pageId);
 
     File *pF;
 
-    try {
-
+    try
+    {
 	pF = getHandle(fileId);
     }
     catch ( Exception e )
     {
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot get file handle"), e);
     }
 
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
 
     try 
-    {
+    {	
+	unsigned bmid = ( pageId - _pageOffset[fileId] ) / ( BIT_PER_BYTE * sizeof(unsigned));
+	unsigned bmoffset = ( pageId - _pageOffset[fileId] ) % ( BIT_PER_BYTE * sizeof(unsigned));
 
-	unsigned bmid = pageId / ( BIT_PER_BYTE * sizeof(int));
-	unsigned bmoffset = pageId % ( BIT_PER_BYTE * sizeof(int));
-	
-	pF->seek(sizeof(int) + sizeof(FileType) + sizeof(int) + bmid * sizeof(int));
+	pF->seek(FILEHEADSIZE + bmid * sizeof(unsigned));
 	
 	unsigned bm;
 	pF->readByte((char*)&bm, sizeof(unsigned));
@@ -553,19 +558,21 @@ void CegoFileHandler::claimPage(int fileId, int pageId, CegoLockHandler *pLockHa
 	
 	bm = bm | f;
 	
-	pF->seek(sizeof(int) + sizeof(FileType) + sizeof(int) + bmid * sizeof(int));
+	pF->seek(FILEHEADSIZE + bmid * sizeof(unsigned));
 	pF->writeByte((char*)&bm, sizeof(unsigned));
 
 	if ( _buMask[fileId] )
 	    _fbmMask[fileId] = true;
+
+	if ( __fsyncOn )
+	    pF->flush();
 	
 	pLockHandle->unlockDataFile(fileId);
-
     }
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot claim data page"), e);
     }
 
     // write page data to datafile to avoid, that cleanup procedure frees the page
@@ -574,24 +581,28 @@ void CegoFileHandler::claimPage(int fileId, int pageId, CegoLockHandler *pLockHa
     CegoBufferPage ip(initBuf, _pageSize);
     ip.initPage(CegoBufferPage::TABLE);
     
-    writePage(fileId, pageId, 1, initBuf, pLockHandle);
-    
+    writePage(pageId, 1, initBuf, pLockHandle);
+
+    delete[] initBuf;    
 }
 
 /* any new required page can be requested using the allocatePage method.
    the required filetype must be specified. 
    the method returns the assigned file and page information */
 
-void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int& pageId, CegoLockHandler *pLockHandle, unsigned* &fbm, int& fbmSize, bool doAppend)
-{
+void CegoFileHandler::allocatePage(int tabSetId, FileType ft, PageIdType& pageId, CegoLockHandler *pLockHandle, unsigned* &fbm, int& fbmSize, bool doAppend)
+{    
+    if ( _isReadOnly == true )
+	throw Exception(EXLOC, Chain("Cannot allocate page in read only mode"));
 
-    int fid = 0;
-    
-    while (fid < FILMNG_MAXDATAFILE)
+    int fid = 0;  
+    if ( doAppend )
+	fid = _appendFid;
+   
+    while ( fid < FILMNG_MAXDATAFILE )
     {
 	if ( _isReg[fid] && _tabSetId[fid] == tabSetId && _fileType[fid] == ft )
-	{		
-	    
+	{	    
 	    File* pF;
 	    pF = getHandle(fid);
 	    
@@ -599,28 +610,13 @@ void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int&
 	    
 	    try 
 	    {
-
 		unsigned ws = sizeof(unsigned)*BIT_PER_BYTE;
 
-		int k;
+		PageIdType k;
 
 		if ( doAppend == false || _appendPos[fid] == 0)
 		{
-
-		    /*
-		    int ts;
-		    pF->readByte((char*)&ts, sizeof(int));		
-		    
-		    FileType aft;
-		    pF->readByte((char*)&aft, sizeof(FileType));
-		    
-		    int numPage;			
-		    pF->readByte((char*)&numPage, sizeof(int));
-		    */
-		    
-		    // cout << "AppendPos = " << _appendPos[fid] << " ... Seek to " << 2*sizeof(int) + sizeof(FileType) << endl;
-
-		    pF->seek(2*sizeof(int) + sizeof(FileType));
+		    pF->seek(FILEHEADSIZE);
 		    k = 0;
 		    // we reset the append postion to zero, so next allocation with append = true will search from the beginning
 		    _appendPos[fid] = 0;
@@ -628,7 +624,7 @@ void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int&
 		else
 		{		    
 		    // cout << "Seek to " << 2*sizeof(int) + sizeof(FileType) + (_appendPos[fid] / ws ) * sizeof(unsigned) << endl;		    
-		    pF->seek(2*sizeof(int) + sizeof(FileType) + ( _appendPos[fid] / ws) * sizeof(unsigned));
+		    pF->seek(FILEHEADSIZE + ( _appendPos[fid] / ws) * sizeof(unsigned));
 		    k = _appendPos[fid];
 		}
 		
@@ -646,6 +642,9 @@ void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int&
 		       
 		    if ( (bm & ~0) != ~0) 
 		    {
+			// we start next append search with this fid
+			_appendFid = fid;
+
 			// free page available
 			int l = 0;
 			while (l < ws && ! isEop)
@@ -654,13 +653,18 @@ void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int&
 			    {
 				unsigned checkbit = bm >> l & ~(~(unsigned)0 << 1);
 				if (checkbit == 0)
-				{
-				    
-				    pageId = k + l;
-				    fileId = fid;
+				{			    
+				    pageId = k + l + _pageOffset[fid];
+
+				    /*
+				    cout << "Allocated page :" << endl;
+				    cout << "Fid = " <<  fid << endl;
+				    cout << "PageOffset=" << _pageOffset[fid] << endl;
+				    cout << "PageId = " << pageId << endl;
+				    */
 				    
 				    // cout << "FH: " << fid << "," <<  k << "," << l << " using pageId " << pageId << endl;
-				    
+				    				    
 				    unsigned f = ~(~(unsigned)0 << 1);
 				    f = f << l;
 				    // printBitMap(bm);
@@ -668,34 +672,34 @@ void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int&
 				    bm = bm | f;
 				    // printBitMap(bm);
 				    
-				    if ( _buMask[fileId] )
-				    {
-					
-					if ( _fbmMask[fileId] == false )
+				    if ( _buMask[fid] )
+				    {					
+					if ( _fbmMask[fid] == false )
 					{	       						      							
-					    fbmSize = getFBMSize(fileId);
+					    fbmSize = getFBMSize(fid);
 					    fbm = new unsigned[fbmSize]; 							    
-					    readFBM(fileId, fbm, pLockHandle);
+					    readFBM(fid, fbm, pLockHandle);
 					}
 					else
 					{
 					    fbmSize = 0;
 					}
 					
-					_fbmMask[fileId] = true;
+					_fbmMask[fid] = true;
 				    }
-				    // cout << "2 bm=" << bm << endl;
-				    pF->seek(sizeof(int) + sizeof(FileType) + sizeof(int) + k / BIT_PER_BYTE);
+
+				    // cout << "Writing bitmask to file .." << endl;
+				    pF->seek(FILEHEADSIZE + k / BIT_PER_BYTE);
 				    pF->writeByte((char*)&bm, sizeof(unsigned));
+				    if ( __fsyncOn )
+					pF->flush();
 				    
 				    pLockHandle->unlockDataFile(fid);
 
 				    if ( _appendPos[fid] < k )
 				    {
-					// cout << "Increasing append pos to " << k << endl;
 					_appendPos[fid] = k;
 				    }
-				    // cout << "Got page " << pageId << endl;
 				    return;
 				}
 				l++;
@@ -713,7 +717,7 @@ void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int&
 	    catch ( Exception e )
 	    {
 		pLockHandle->unlockDataFile(fid);
-		throw e;
+		throw Exception(EXLOC, Chain("Cannot allocate file page"), e);
 	    }		
 	} 
 	fid++;		    
@@ -721,8 +725,9 @@ void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int&
 
     if ( doAppend )
     {
+	_appendFid=0;
 	// if no more pages available with append mode, we make another try without append
-	allocatePage(tabSetId, ft, fileId, pageId, pLockHandle, fbm, fbmSize, false);
+	allocatePage(tabSetId, ft, pageId, pLockHandle, fbm, fbmSize, false);
     }
     else
     {    
@@ -737,28 +742,35 @@ void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int&
     }
 }
 
-
 /* if a page is no longer used, it can be released using the releasePage method.
    Normally this is used, in case of delete operations or in case of temporarily used pages */
 
-void CegoFileHandler::releasePage(int fileId, int pageId, CegoLockHandler *pLockHandle, unsigned* &fbm, int& fbmSize)
-{
-
-    // cout << "Releasing pageId " << pageId << endl;
+void CegoFileHandler::releasePage(PageIdType pageId, CegoLockHandler *pLockHandle, unsigned* &fbm, int& fbmSize)
+{    
+    if ( _isReadOnly == true )
+	throw Exception(EXLOC, Chain("Cannot releases page in read only mode"));
 
+    int fileId = getFileIdForPageId(pageId);
+    
     File* pF;
-    pF = getHandle(fileId);
-
+    
+    try 
+    {
+	pF = getHandle(fileId);
+    }
+    catch ( Exception e )
+    {
+	throw Exception(EXLOC, Chain("Cannot get file handle"), e);
+    }
+    
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
 
     try
     {
-
 	if ( _buMask[fileId] )
 	{	    
 	    if ( _fbmMask[fileId] == false )
-	    {
-		
+	    {		
 		fbmSize = getFBMSize(fileId);
 		fbm = new unsigned[fbmSize]; 
 		readFBM(fileId, fbm, pLockHandle);
@@ -768,15 +780,13 @@ void CegoFileHandler::releasePage(int fileId, int pageId, CegoLockHandler *pLock
 	{
 	    fbmSize = 0;
 	}
-	
-	// pF->seek(0);
-	
-	unsigned bmid = pageId / ( sizeof(int) * BIT_PER_BYTE );
-	unsigned bmoffset = pageId % ( sizeof(int) * BIT_PER_BYTE) ;
+	       
+	unsigned bmid = ( pageId - _pageOffset[fileId] ) / ( sizeof(unsigned) * BIT_PER_BYTE );
+	unsigned bmoffset = ( pageId - _pageOffset[fileId] ) % ( sizeof(unsigned) * BIT_PER_BYTE) ;
 	
 	// cout << "FH release : " << bmid << "," << bmoffset << " using pageId " << pageId << endl;
-	
-	pF->seek(sizeof(int) + sizeof(FileType) + sizeof(int) + ( bmid * sizeof(int)));
+
+	pF->seek(FILEHEADSIZE + ( bmid * sizeof(unsigned)));
 	
 	unsigned bm;
 	pF->readByte((char*)&bm, sizeof(unsigned));
@@ -784,10 +794,9 @@ void CegoFileHandler::releasePage(int fileId, int pageId, CegoLockHandler *pLock
 	unsigned f = ~(~(unsigned)0 << 1);
 	f = f << bmoffset;
 
-	// cout << "Bitmap before " << endl;
 	// printBitMap(bm);
-
-	// printBitMap(f);	    
+	// printBitMap(f);
+	
 	bm = bm & ~f;
 
 	// cout << "Bitmap after " << endl;
@@ -795,35 +804,112 @@ void CegoFileHandler::releasePage(int fileId, int pageId, CegoLockHandler *pLock
 
 	// cout << "Write to pos " << sizeof(int) + sizeof(FileType) + sizeof(int) + ( bmid * sizeof(int)) << endl;
 
-	pF->seek(sizeof(int) + sizeof(FileType) + sizeof(int) + ( bmid * sizeof(int)));
+	pF->seek(FILEHEADSIZE + ( bmid * sizeof(unsigned)));
 	pF->writeByte((char*)&bm, sizeof(unsigned));
+	if ( __fsyncOn )
+	    pF->flush();
 	
 	if ( _buMask[fileId] )
 	    _fbmMask[fileId] = true;
-
-	pLockHandle->unlockDataFile(fileId);
 	
+	pLockHandle->unlockDataFile(fileId);	
     }
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot release data page"), e);
     }
+}
 
+void CegoFileHandler::commitPageEntry(PageIdType pageId)
+{
+    int fileId = getFileIdForPageId(pageId);
+
+    if ( _commitMask[fileId] == 0 )
+    {
+    	_commitMask[fileId] = new unsigned[ (  getNumPages(fileId) / (BIT_PER_BYTE * sizeof(unsigned))) + 1];
+
+	int i;
+	
+	for (i = 0; i < ( getNumPages(fileId) / (BIT_PER_BYTE * sizeof(unsigned))) + 1; i++)
+	{
+	    _commitMask[fileId][i] = 0;
+	}
+    }
+    
+    unsigned bmid = ( pageId - _pageOffset[fileId] ) / ( BIT_PER_BYTE * sizeof(unsigned));
+    unsigned bmoffset = ( pageId - _pageOffset[fileId] ) % ( BIT_PER_BYTE * sizeof(unsigned));
+    
+    unsigned bm = _commitMask[fileId][bmid];
+	
+    unsigned f = ~(~(unsigned)0 << 1);
+    f = f << bmoffset;
+    
+    _commitMask[fileId][bmid] = bm | f;    
+}
+
+int CegoFileHandler::cleanPages(CegoLockHandler *pLockHandle)
+{
+    if ( _isReadOnly == true )
+	throw Exception(EXLOC, Chain("Cannot clean pages in read only mode"));
+
+    int numCleanup = 0;
+    
+    for (int i = 0; i<FILMNG_MAXDATAFILE; i++)
+    {
+	if ( _isReg[i] && _fileType[i] == DATAFILE && _commitMask[i])
+	{
+	    
+	    int fbmSize = getFBMSize(i);
+	    unsigned* fbm = new unsigned[fbmSize]; 							    
+	    readFBM(i, fbm, pLockHandle);
+
+	    bool cleanupNeeded = false;
+	    
+	    for ( int j=0; j<fbmSize; j++ )
+	    {
+		// cout << "Checking mask " << _commitMask[i][j] << endl;
+		
+		if ( fbm[j] != _commitMask[i][j] )
+		{
+		    // cleanup required
+
+
+		    cleanupNeeded = true;
+
+		    numCleanup += mapDiff(fbm[i], _commitMask[i][j] );
+
+		    /*
+		    cout << "Commited :" << endl;
+		    printBitMap(_commitMask[i][j]);
+
+		    cout << "Written :" << endl;
+		    printBitMap(fbm[j]);		    		    
+		    */
+		}
+	    }
+
+	    if ( cleanupNeeded )
+		writeFBM(i, _commitMask[i], pLockHandle);
+	    
+	    delete[] fbm;
+	}
+    }
+    return numCleanup;
 }
 
 /* returns the number of total pages for the given fileId */
 
 int CegoFileHandler::getNumPages(int fileId)
 {
-
     try 
     {
-	File* pF = getHandle(fileId);
+	// check valid fileId
+	getHandle(fileId);
     }
     catch ( Exception e )
     {
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot get file handle"), e);
     }
 
     return _numPages[fileId];
@@ -832,8 +918,7 @@ int CegoFileHandler::getNumPages(int fileId)
 /* returns the number of used pages for the given fileId */
 
 int CegoFileHandler::getNumUsedPages(int fileId, CegoLockHandler *pLockHandle)
-{
-
+{   
     File *pF;
 
     try 
@@ -842,47 +927,46 @@ int CegoFileHandler::getNumUsedPages(int fileId, CegoLockHandler *pLockHandle)
     }
     catch ( Exception e )
     {
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot get file handle"), e);
     }
 
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
 
     try
-    {
-	
+    {	
 	pF->seek(sizeof(int) + sizeof(FileType));
 	
-	int numPage;
-	
+	int numPage;	
 	pF->readByte((char*)&numPage, sizeof(int));
-	
-	bool isEop = false;
-	
+
+	PageIdType pageOffset;
+	pF->readByte((char*)&pageOffset, sizeof(PageIdType));
+		
 	int p;
 	
-	int usedPages = 0 ;
-	
-	for ( p = 0; p < (numPage / (BIT_PER_BYTE * sizeof(int))) + 1; p++ )
+	int usedPages = 0;
+
+	for ( p = 0; p < (numPage / (BIT_PER_BYTE * sizeof(unsigned))) + 1; p++ )
 	{
 	    unsigned bm = 0;
-	    pF->readByte((char*)&bm, sizeof(int));
+	    pF->readByte((char*)&bm, sizeof(unsigned));
 	    
 	    // printBitMap(bm);
 	    
 	    int i,j;
-	    for (i = 0; i< sizeof(int); i++)
+	    for (i = 0; i< sizeof(unsigned); i++)
 	    {
 		for (j=0; j < BIT_PER_BYTE; j++)
 		{
 		    int one = 1;
 		    
 		    if ( one & bm )
-			
+		    {
 			usedPages++;
-		    
-		    bm = bm >> 1;		
+		    }
+		    bm = bm >> 1;
 		}
-	    }	      
+	    }
 	}
 
 	pLockHandle->unlockDataFile(fileId);
@@ -892,7 +976,7 @@ int CegoFileHandler::getNumUsedPages(int fileId, CegoLockHandler *pLockHandle)
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot get number of pages for file"), e);
     }
 }
 
@@ -900,12 +984,13 @@ int CegoFileHandler::getNumUsedPages(int fileId, CegoLockHandler *pLockHandle)
 ////////////// private methods /////////////////
 ////////////////////////////////////////////////
 
-/* to proof, if a given fileId/pageId  is already in use,
+/* to proof, if a given pageId  is already in use,
    the isClaimed method is used */
  
-bool CegoFileHandler::isClaimed(int fileId, int pageId, CegoLockHandler *pLockHandle)
+bool CegoFileHandler::isClaimed(PageIdType pageId, CegoLockHandler *pLockHandle)
 {
-
+    int fileId = getFileIdForPageId(pageId);
+	
     File *pF;
 
     try 
@@ -914,10 +999,10 @@ bool CegoFileHandler::isClaimed(int fileId, int pageId, CegoLockHandler *pLockHa
     }
     catch ( Exception e )
     {
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot get file handle"), e);
     }
 
-    if ( pageId >= _numPages[fileId] )
+    if ( pageId >= _pageOffset[fileId] + _numPages[fileId] )
     {
 	throw Exception(EXLOC, Chain("Invalid page id"));
     }
@@ -925,12 +1010,11 @@ bool CegoFileHandler::isClaimed(int fileId, int pageId, CegoLockHandler *pLockHa
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
 
     try 
-    {
-    
-	unsigned bmid = pageId / ( BIT_PER_BYTE * sizeof(int));
-	unsigned bmoffset = pageId % ( BIT_PER_BYTE * sizeof(int));
+    {   
+	unsigned bmid = ( pageId - _pageOffset[fileId] ) / ( BIT_PER_BYTE * sizeof(unsigned));
+	unsigned bmoffset = ( pageId - _pageOffset[fileId] ) % ( BIT_PER_BYTE * sizeof(unsigned));
 	
-	pF->seek(sizeof(int) + sizeof(FileType) + sizeof(int) + bmid * sizeof(int));
+	pF->seek(FILEHEADSIZE + bmid * sizeof(unsigned));
 	
 	unsigned bm;
 	pF->readByte((char*)&bm, sizeof(unsigned));
@@ -948,10 +1032,26 @@ bool CegoFileHandler::isClaimed(int fileId, int pageId, CegoLockHandler *pLockHa
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot check data page for claimed"), e);
+       
     }
 }
 
+int CegoFileHandler::getFileIdForPageId(PageIdType pageId)
+{
+    int i=0;
+    
+    while ( i < FILMNG_MAXDATAFILE )
+    {
+	if ( _isReg[i] == true && pageId >= _pageOffset[i] && pageId < _pageOffset[i] + _numPages[i] )
+	{	    
+	    return i;
+	}
+	i++;
+    }
+    throw Exception(EXLOC, Chain("Invalid page id ") + Chain(pageId));
+}
+
 /* getHandle returns the file handle for the given file id. The
    file id must be valid and registered. If already open, the method
    returns the file handle. Otherwise, the corresponding file is opened
@@ -959,7 +1059,6 @@ bool CegoFileHandler::isClaimed(int fileId, int pageId, CegoLockHandler *pLockHa
 
 File* CegoFileHandler::getHandle(int fileId)
 {
-
     if ( fileId > FILMNG_MAXDATAFILE - 1 ) 
     {
 	Chain msg = "File Id " + Chain(fileId)  + " out of valid range";
@@ -971,39 +1070,20 @@ File* CegoFileHandler::getHandle(int fileId)
 	Chain msg = "File Id " + Chain(fileId)  + " not registered";
 	throw Exception(EXLOC, msg);
     }
- 
 
-    if (_fhList[fileId] == 0 )
+    if ( _fhList[fileId] == 0 )
     {
-
 	File* pF = new File(_path[fileId]);
 	
-	try {
-	    
-	    pF->open(File::READWRITE);
-
-	    /*
-	    pF->seek(0);
-
-	    
-	    int tabSetId;
-	    int numPage;
-	    FileType ft;
-
-	    pF->readByte((char*)&tabSetId, sizeof(int));
-
-	    pF->readByte((char*)&ft, sizeof(FileType));
-
-	    pF->readByte((char*)&numPage, sizeof(int));
-
-	    _tabSetId[fileId] = tabSetId;
-	    _fileType[fildId] = ft;
-	    _numPages[fileId] = numPage;
-	    */
+	try
+	{	    
+	    if ( _isReadOnly == true )
+		pF->open(File::READ);
+	    else		
+		pF->open(File::READWRITE);
 
 	    _fhList[fileId] = pF;
 	    _appendPos[fileId] = 0;
-
 	}
 	catch (Exception e)
 	{
@@ -1021,20 +1101,18 @@ File* CegoFileHandler::getHandle(int fileId)
 //////// debugging/testing methods /////////////
 ////////////////////////////////////////////////
 
-
 void CegoFileHandler::printBitMap(unsigned bm)
 {    
     printf("--- BM -----------\n");
     
     int i,j;
-    for (i = 0; i< sizeof(int); i++)
+    for (i = 0; i< sizeof(unsigned); i++)
     {
 	for (j=0; j < BIT_PER_BYTE; j++)
 	{
-	    int one = 1;
+	    unsigned one = 1;
 	    
 	    if ( one & bm )
-		
 		printf("1");
 	    else
 		printf("0");
@@ -1044,6 +1122,26 @@ void CegoFileHandler::printBitMap(unsigned bm)
     }
     printf("\n");
     
-    printf("--------------\n");
+    printf("--------------\n");    
+}
+
+int CegoFileHandler::mapDiff(unsigned bm1, unsigned bm2)
+{    
+    int numDiff = 0;
     
+    int i,j;
+    for (i = 0; i< sizeof(unsigned); i++)
+    {
+	for (j=0; j < BIT_PER_BYTE; j++)
+	{
+	    unsigned one = 1;
+	    
+	    if ( ( one & bm1 ) != (  one & bm2 ) )
+		numDiff++;
+
+	    bm1 = bm1 >> 1;
+	    bm2 = bm2 >> 1;
+	}
+    }
+    return numDiff;
 }
