@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -28,76 +28,80 @@
 #include "CegoFieldValue.h"
 #include "CegoProcBlock.h"
 
-
+class CegoProcedure;
 class CegoDistManager;
 class CegoTableManager;
 class CegoAggregation;
 class CegoExpr;
 class CegoDatabaseFormater;
 
-
 class CegoFunction {
     
 public:
     
-    enum FunctionType {  TRIM, 
-			 RTRIM,
-			 LTRIM,
-			 ROUND,
-			 DATE2STR, 
-			 DATE2INT, 
-			 INT2DATE, 
-			 LEFT,
-			 RIGHT,
-			 GETPOS,
-			 SUBSTR,
-			 REPLACE,
-			 LENGTH,
-			 LOWER,
-			 UPPER,
-			 TRUNC,
-			 STR2INT,
-			 STR2LONG,
-			 STR2DATE,
-			 RANDSTR,
-			 RANDINT,
-			 MOD,
-			 DIV,
-			 POWER,
-			 BITAND,
-			 BITOR,
-			 BITXOR,
-			 BLOBSIZE,
-			 CLOBSIZE,
-			 CLOB2STR,
-			 NEXTCOUNT,
-			 SETCOUNT,
-			 USERDEFINED }; 
-
-
-
+    enum FunctionType { INT2ASC,
+			ASC2INT,
+			TRIM, 
+			RTRIM,
+			LTRIM,
+			ROUND,
+			DATE2STR, 
+			DATE2LONG, 
+			LONG2DATE,
+			NEWDATE,
+			LEFT,
+			RIGHT,
+			GETCOUNT,
+			GETPOS,
+			SUBSTR,
+			REPLACE,
+			LENGTH,
+			LOWER,
+			UPPER,
+			TRUNC,
+			SCANDATE, 
+			STR2INT,
+			STR2LONG,
+			RANDSTR,
+			RANDINT,
+			MOD,
+			DIV,
+			LMOD,
+			LDIV,
+			POWER,
+			BITAND,
+			BITOR,
+			BITXOR,
+			BLOBSIZE,
+			BLOBREF,
+			CLOBSIZE,
+			CLOBREF,
+			CLOB2STR,
+			NEXTCOUNT,
+			SETCOUNT,
+			USERDEFINED };
+    
     CegoFunction(FunctionType funcType);
     CegoFunction(FunctionType funcType, ListT<CegoExpr*>& exprList);
-
-    CegoFunction(char* buf, CegoDistManager *pTabMng, int tabSetId);
+    CegoFunction(char* buf, CegoDistManager *pTabMng, CegoProcBlock *pBlock, int tabSetId);
     CegoFunction(CegoDistManager* pTabMng, int tabSetId, CegoFunction::FunctionType type);
     CegoFunction(CegoDistManager* pTabMng, int tabSetId, const Chain& funcName, ListT<CegoExpr*>& exprList);
     
-    CegoFunction(Element* pFunctionElement, CegoDistManager *pGTM);
-
     ~CegoFunction();
 
+    void cleanUp();
+
+    void setTabSetId(int tabSetId);
+    
     int getTabSetId() const;
     Chain getName() const;
 	
-    CegoDataType getReturnType() const;
-    int getReturnTypeLen() const;
+    CegoDataType getReturnType();
+    int getReturnTypeLen(const ListT<CegoField>& fl);
 
     ListT<CegoExpr*>& getExprList();
-    ListT<CegoField> getFieldList() const;
-    void setFieldListArray(ListT<CegoField>** pFLA);
-    void setBlock(CegoProcBlock *pBlock);
-
+    void getFieldList(ListT<CegoField>& fl, ListT<CegoField>** pFLA) const;
+    
     const FunctionType getType() const;
 
     void setExprList(ListT<CegoExpr*>& exprList);
@@ -112,17 +116,15 @@ public:
 
     ListT<CegoAggregation*> getAggregationList();
     
-    CegoFieldValue evalFieldValue() const;
-    Chain getId() const;
-    Chain toChain(const Chain& indent = Chain("")) const;
+    CegoFieldValue evalFieldValue(ListT<CegoField>** pFLA, CegoProcBlock *pBlock);
+    
+    Chain getId(CegoProcBlock *pBlock) const;
+    Chain toChain(int defTabSetId, const Chain& indent = Chain("")) const;
     Chain dbFormat(CegoDatabaseFormater *pForm) const;
     
-    Element* toElement() const;
-    void fromElement(Element *pFunctionElement, CegoDistManager *pGTM);
-
-    void encode(char *buf);
-    void decode(char *buf, CegoDistManager* pGTM, int tabSetId);
-    int getEncodingLength() const;
+    void encode(char *buf, CegoProcBlock *pBlock);
+    void decode(char *buf, CegoDistManager* pGTM, CegoProcBlock *pBlock, int tabSetId);
+    int getEncodingLength(CegoProcBlock *pBlock) const;
 
     CegoFunction* clone(bool isAttrRef = false);
 
@@ -132,13 +134,12 @@ public:
       
 private:
 
-
-    
+    CegoProcedure* loadProcedure(int tabSetId, const Chain& procName) const;
+  
     CegoDistManager* _pTabMng;
-    CegoProcBlock* _pBlock;
+    CegoProcedure* _pProc;	    
     Chain _funcName;
-    Chain _counterId;
-    
+    Chain _counterId;  
     FunctionType _type;
     ListT<CegoExpr*> _exprList;
     int _tabSetId;
