@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -31,6 +31,8 @@
 #include "CegoBTreeObject.h"
 #include "CegoKeyObject.h"
 #include "CegoCheckObject.h"
+#include "CegoTriggerObject.h"
+#include "CegoAliasObject.h"
 #include "CegoAdminHandler.h"
 
 class CegoDistManager;
@@ -39,13 +41,13 @@ class CegoImpInStream : public XMLInStream {
     
 public:
     
-    CegoImpInStream(const Chain& tableSet, CegoDistManager* pGTM, bool doLogging, CegoAdminHandler* pAH);
-    CegoImpInStream(const Chain& tableSet, const Chain& tableName, CegoDistManager* pGTM, bool doLogging, CegoAdminHandler* pAH);
-
+    CegoImpInStream(const Chain& tableSet, CegoDistManager* pGTM, CegoAdminHandler* pAH);
+    CegoImpInStream(const Chain& tableSet, const Chain& tableName, CegoDistManager* pGTM, CegoAdminHandler* pAH);
     ~CegoImpInStream();
-    
-    void putFirst(Element* pParent, const Chain& name, const ListT<Attribute>& attrList, ListT<char*> dataList);
-    void putNext(Element* pParent, const Chain& name, const ListT<Attribute>& attrList, ListT<char*> dataList);
+
+    unsigned long long numImported() const;
+    void putFirst(Element* pParent, const Chain& name, const ListT<Attribute>& attrList, ListT<char*>& dataList);
+    void putNext(Element* pParent, const Chain& name, const ListT<Attribute>& attrList, ListT<char*>& dataList);
     
 private:
 
@@ -67,12 +69,13 @@ private:
     ListT<CegoBTreeObject> _btreeList;
     ListT<CegoKeyObject> _keyList;
     ListT<CegoCheckObject> _checkList;
+    ListT<CegoTriggerObject> _triggerList;
+    ListT<CegoAliasObject> _aliasList;
     
     CegoBufferPage _bp;
     CegoDataPointer _sysEntry;
-    bool _doLogging;
 
-    long _rowCount;
+    unsigned long long _rowCount;
     unsigned long _modId;
 };
 #endif
