@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -23,9 +23,7 @@
 
 // CEGO INCLUDES
 #include "CegoJoinObject.h"
-#include "CegoXMLHelper.h"
 #include "CegoXMLdef.h"
-#include "CegoTypeConverter.h"
 #include "CegoTableObject.h"
 #include "CegoViewObject.h"
 
@@ -45,12 +43,7 @@ CegoJoinObject::CegoJoinObject(const CegoJoinObject& jo) : CegoContentObject(jo)
     _pPred = jo._pPred;
 }
 
-CegoJoinObject::CegoJoinObject(Element *pJO) : CegoContentObject( 0, CegoObject::JOIN, Chain("join"))
-{
-    putElement(pJO);
-}
-
-CegoJoinObject::CegoJoinObject( JoinType joinType, CegoContentObject *pLeftObject, CegoContentObject *pRightObject, CegoPredDesc *pPred ) : CegoContentObject( 0, CegoObject::JOIN, Chain("join"))
+CegoJoinObject::CegoJoinObject( JoinType joinType, CegoContentObject *pLeftObject, CegoContentObject *pRightObject, CegoPredicate *pPred ) : CegoContentObject( 0, CegoObject::JOIN, Chain("join"))
 {
     _joinType = joinType;
     _pLeftObject = pLeftObject;
@@ -61,9 +54,7 @@ CegoJoinObject::CegoJoinObject( JoinType joinType, CegoContentObject *pLeftObjec
     setSchema(mergeSchema);
 
     if ( pLeftObject->getType() == CegoObject::JOIN )
-    {
-	
-	// CegoJoinObject *pJCO = (CegoJoinObject*)pLeftObject;
+    {	
 	CegoContentObject **pCO = pLeftObject->getSubCOList().First();
 	while ( pCO )
 	{
@@ -113,12 +104,15 @@ int CegoJoinObject::getEntrySize() const
     entrySize += sizeof(JoinType);
     entrySize += _pLeftObject->getEntrySize();
     entrySize += _pRightObject->getEntrySize();
-    entrySize += _pPred->getEncodingLength();
+    if ( _pPred )
+	entrySize += _pPred->getEncodingLength(0);
+    else
+	entrySize += sizeof(char); // no pred indicator
     
     return entrySize;
 }
 
-void CegoJoinObject::encode(char *buf)
+void CegoJoinObject::encode(char *buf) const
 {
     char* bufPtr = buf;
     int entrySize = getEntrySize();
@@ -133,9 +127,14 @@ void CegoJoinObject::encode(char *buf)
     
     _pRightObject->encode(bufPtr);
     bufPtr += _pRightObject->getEntrySize();
-    
-    _pPred->encode(bufPtr);
-    bufPtr += _pPred->getEncodingLength();
+
+    if ( _pPred )
+    {
+	_pPred->encode(bufPtr, 0);
+	bufPtr += _pPred->getEncodingLength(0);
+    }
+    else
+	*bufPtr = 0;
 }
 
 void CegoJoinObject::decode(char *buf)
@@ -175,9 +174,11 @@ void CegoJoinObject::decode(char *buf)
     bufPtr += _pLeftObject->getEntrySize();
 
     // TODO
-    // no support for subselects in Join objects ( pGTM required as argument )
-    _pPred = new CegoPredDesc(bufPtr, 0, 0);
-
+    if ( *bufPtr != 0 )
+    {
+	// no support for subselects in Join objects ( pGTM required as argument )
+	_pPred = new CegoPredicate(bufPtr, 0, 0 ,0);
+    }
 }
 
 CegoJoinObject::JoinType CegoJoinObject::getJoinType()
@@ -195,14 +196,16 @@ CegoContentObject* CegoJoinObject::getRightObject()
     return _pRightObject;
 }
 
-CegoPredDesc* CegoJoinObject::getPredDesc()
+CegoPredicate* CegoJoinObject::getPredDesc()
 {
     return _pPred;
 }
 
-void CegoJoinObject::getPredList(ListT<CegoPredDesc*>& predList)
+void CegoJoinObject::getPredList(ListT<CegoPredicate*>& predList)
 {
-    predList.Insert(_pPred);
+    if ( _pPred )
+	predList.Insert(_pPred);
+    
     if ( _pLeftObject )
 	if ( _pLeftObject->getType() == CegoObject::JOIN )
 	    ((CegoJoinObject*)_pLeftObject)->getPredList(predList);
@@ -228,7 +231,7 @@ bool CegoJoinObject::operator == ( const CegoJoinObject& jo)
     return CegoContentObject::operator==(jo);
 }
 
-Chain CegoJoinObject::getId() const
+Chain CegoJoinObject::getId(CegoProcBlock *pBlock) const
 {
     Chain s;
 
@@ -236,25 +239,30 @@ Chain CegoJoinObject::getId() const
     {
     case INNER:
     {
-	s += _pLeftObject->getId() + Chain("ij") + _pRightObject->getId() + Chain("o") + _pPred->getId();
+	s += _pLeftObject->getId(pBlock) + Chain("ij") + _pRightObject->getId(pBlock);
+	if ( _pPred )
+	    s += Chain("o") + _pPred->getId(pBlock);
 	break;
     }
     case LEFTOUTER:
     {
-	s += _pLeftObject->getId() + Chain("loj") + _pRightObject->getId() + Chain("o") + _pPred->getId();
+	s += _pLeftObject->getId(pBlock) + Chain("loj") + _pRightObject->getId(pBlock);
+	if ( _pPred )
+	    s += Chain("o") + _pPred->getId(pBlock);
 	break;
     }
     case RIGHTOUTER:
     {
-	s += _pLeftObject->getId() + Chain("roj") + _pRightObject->getId() + Chain("o") + _pPred->getId();
+	s += _pLeftObject->getId(pBlock) + Chain("roj") + _pRightObject->getId(pBlock);
+	if ( _pPred )
+	    s += Chain("o") + _pPred->getId(pBlock);
 	break;
     }
     }
-
     return s;
 }
 
-Chain CegoJoinObject::toChain() const
+Chain CegoJoinObject::toChain(int defTabSetId) const
 {
     Chain s;
 
@@ -262,17 +270,23 @@ Chain CegoJoinObject::toChain() const
     {
     case INNER:
     {
-	s += _pLeftObject->toChain() + Chain(" inner join ") + _pRightObject->toChain() + Chain(" on ") + _pPred->toChain();
+	s += _pLeftObject->toChain(defTabSetId) + Chain(" inner join ") + _pRightObject->toChain(defTabSetId);
+	if ( _pPred )
+	    s += Chain(" on ") + _pPred->toChain(defTabSetId);
 	break;
     }
     case LEFTOUTER:
     {
-	s += _pLeftObject->toChain() + Chain(" left outer join ") + _pRightObject->toChain() + Chain(" on ") + _pPred->toChain();
+	s += _pLeftObject->toChain(defTabSetId) + Chain(" left outer join ") + _pRightObject->toChain(defTabSetId);
+	if ( _pPred )
+	    s += Chain(" on ") + _pPred->toChain(defTabSetId);
 	break;
     }
     case RIGHTOUTER:
     {
-	s += _pLeftObject->toChain() + Chain(" right outer join ") + _pRightObject->toChain() + Chain(" on ") + _pPred->toChain();
+	s += _pLeftObject->toChain(defTabSetId) + Chain(" right outer join ") + _pRightObject->toChain(defTabSetId);
+	if ( _pPred )
+	    s += Chain(" on ") + _pPred->toChain(defTabSetId);
 	break;
     }
     }
@@ -280,72 +294,8 @@ Chain CegoJoinObject::toChain() const
     return s;
 }
 
-Element* CegoJoinObject::getElement() const
-{
-    Element* pRoot = new Element(XML_OBJ_ELEMENT);
-
-    pRoot->setAttribute(XML_TSID_ATTR, Chain(getTabSetId()));
-    pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_JOINOBJ_VALUE);	
-    pRoot->setAttribute(XML_OBJNAME_ATTR, getName());
-
-    pRoot->addContent(_pLeftObject->getElement());
-    pRoot->addContent(_pRightObject->getElement());
-    pRoot->addContent(_pPred->toElement());
-    
-    return pRoot;
-}
-
-void CegoJoinObject::putElement(Element* pElement)
-{
-
-    Element *pRoot = pElement;
-    
-    if ( pRoot )
-    {
-	
-	Chain objName = pRoot->getAttributeValue(XML_OBJNAME_ATTR); 
-	int tabSetId = pRoot->getAttributeValue(XML_TSID_ATTR).asInteger();
-	
-	setName(objName);
-	setTabName(objName);
-	setTabSetId(tabSetId);
-
-	setType(CegoObject::JOIN);
-	
-	ListT<Element*> subList = pRoot->getChildren(XML_OBJ_ELEMENT);
-	Element **pLeft = subList.First();
-
-	Chain objType;
-
-	objType = (*pLeft)->getAttributeValue(XML_OBJTYPE_ATTR);
-	if ( objType == Chain(XML_JOINOBJ_VALUE))
-	    _pLeftObject = new CegoJoinObject(*pLeft);
-	else if ( objType == Chain(XML_TABOBJ_VALUE))
-	    _pLeftObject = new CegoTableObject(*pLeft);
-	else if ( objType == Chain(XML_VIEWOBJ_VALUE))
-	    _pLeftObject = new CegoViewObject(*pLeft);
-
-	Element **pRight = subList.Next();
-	objType = (*pRight)->getAttributeValue(XML_OBJTYPE_ATTR);
-	if ( objType == Chain(XML_JOINOBJ_VALUE))
-	    _pRightObject = new CegoJoinObject(*pRight);
-	else if ( objType == Chain(XML_TABOBJ_VALUE))
-	    _pRightObject = new CegoTableObject(*pRight);
-	else if ( objType == Chain(XML_VIEWOBJ_VALUE))
-	    _pRightObject = new CegoViewObject(*pRight);
-	
-	ListT<Element*> predList = pRoot->getChildren(XML_PRED_ELEMENT);
-	Element **pPred = predList.First();
-	// for predicates containing subqueries, the GTM should be set up to an appropriate value
-	// for now we set to zero
-	_pPred = new CegoPredDesc(*pPred, 0);
-
-    }    
-}
-
 Chain CegoJoinObject::getFormatted() const
 {
-
     Chain s;
 	
     int maxAttrLen = 12;
@@ -357,15 +307,12 @@ Chain CegoJoinObject::getFormatted() const
 	pF = _schema.Next();
     }
 
-    int maxFillLen = maxAttrLen + 28;
-
     return s;
 }
 
 Chain CegoJoinObject::fill(const Chain& s, int num) const
 {
-    
-    Chain fs = Chain("");
+    Chain fs = Chain();
     while (num > 0)
     {
 	fs = fs + s;
@@ -379,7 +326,7 @@ CegoContentObject* CegoJoinObject::clone(bool isAttrRef)
 {
     CegoJoinObject *pClone = new CegoJoinObject( _joinType, 
 						 _pLeftObject->clone(isAttrRef),
-						 _pRightObject->clone(isAttrRef), 
-						 _pPred->clone(isAttrRef) );
+						 _pRightObject->clone(isAttrRef),
+						 _pPred == 0 ? 0 : _pPred->clone(isAttrRef) );
     return pClone;
 }
