@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -27,7 +27,7 @@
 // cego includes
 #include "CegoField.h"
 #include "CegoContentObject.h"
-#include "CegoPredDesc.h"
+#include "CegoPredicate.h"
 
 class CegoJoinObject : public CegoContentObject {
 
@@ -37,37 +37,27 @@ class CegoJoinObject : public CegoContentObject {
 
     CegoJoinObject();
     CegoJoinObject(const CegoJoinObject& jo);
-    CegoJoinObject( Element *pJO );
-    CegoJoinObject( JoinType joinType, CegoContentObject *pLeftObject, CegoContentObject *pRightObject, CegoPredDesc *pPred );
-	
+    CegoJoinObject( JoinType joinType, CegoContentObject *pLeftObject, CegoContentObject *pRightObject, CegoPredicate *pPred = 0);
     ~CegoJoinObject();
     
     int getEntrySize() const;
-    void encode(char *buf);
+    void encode(char *buf) const;
     void decode(char *buf);
-
-    void setSchema(const ListT<CegoField>& schema);   
-
+    void setSchema(const ListT<CegoField>& schema);
     JoinType getJoinType();
-
     CegoContentObject* getLeftObject();
     CegoContentObject* getRightObject();
-    CegoPredDesc* getPredDesc();
-    void getPredList(ListT<CegoPredDesc*>& predList);
-
+    CegoPredicate* getPredDesc();
+    void getPredList(ListT<CegoPredicate*>& predList);
     void setEmpty();
-    
+    Chain getId(CegoProcBlock *pBlock) const;
+    Chain toChain(int defTabSetId) const;
+    CegoContentObject* clone(bool isAttrRef = false);    
+    Chain getFormatted() const;
+
     CegoJoinObject& operator = ( const CegoJoinObject& jo);
     bool operator == ( const CegoJoinObject& jo);
 
-    Chain getId() const;
-    Chain toChain() const;
-    CegoContentObject* clone(bool isAttrRef = false);
-
-    Element* getElement() const;
-    void putElement(Element* pElement);
-    Chain getFormatted() const;
-
  private:
 
     Chain fill(const Chain& s, int num) const;
@@ -75,7 +65,6 @@ class CegoJoinObject : public CegoContentObject {
     JoinType _joinType;
     CegoContentObject *_pLeftObject;
     CegoContentObject *_pRightObject;
-    CegoPredDesc *_pPred;
-    
+    CegoPredicate *_pPred;
 };
 #endif
