@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -19,7 +19,7 @@
 //              A number of available semaphores is distributed among all required lock operations.
 //              
 //             
-// Status: CLEAN
+// Status: CLEAN 
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -42,88 +42,72 @@ public:
 
     void initLocks();
     void deleteLocks();
-
     unsigned long long lockRecord(const CegoDataPointer& dp, enum LockMode lockMode);
     void unlockRecord(unsigned long long id);
-
     unsigned long long lockRBRecord(const CegoDataPointer& dp, enum LockMode lockMode);
     void unlockRBRecord(unsigned long long id);
-
     unsigned long long lockSysRecord(const CegoDataPointer& dp, enum LockMode lockMode);
     void unlockSysRecord(unsigned long long id);
-
-    unsigned long long lockSysPage(int fileId, int pageId, enum LockMode lockMode);
+    unsigned long long lockSysPage(PageIdType pageId, enum LockMode lockMode);
     void unlockSysPage(unsigned long long id);
-    
-    unsigned long long lockData(CegoObject::ObjectType type, int fileId, int pageId, enum LockMode lockMode);
+    unsigned long long lockData(CegoObject::ObjectType type, PageIdType pageId, enum LockMode lockMode);
     void unlockData(CegoObject::ObjectType type, unsigned long long id);
-
     void lockDataFile(int fileId, enum LockMode lockMode);
     void unlockDataFile(int fileId);
-
     void lockBufferPool(int hashId, enum LockMode lockMode);
     void unlockBufferPool(int hashId);
-
     void lockBufferPool();
     void unlockBufferPool();
-
     void getLockStat(int semId, Chain& lockName, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay);
     void getAggLockStat(const Chain& lockName, int& numLock, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay);
 
 private:
 
-    unsigned long long lockDataPage(int fileId, int pageId, enum LockMode lockMode);
+    unsigned long long lockDataPage(PageIdType pageId, enum LockMode lockMode);
     void unlockDataPage(unsigned long long id);
-
-    unsigned long long lockIndexPage(int fileId, int pageId, enum LockMode lockMode);
+    unsigned long long lockIndexPage(PageIdType pageId, enum LockMode lockMode);
     void unlockIndexPage(unsigned long long id);
-
-    unsigned long long lockRBPage(int fileId, int pageId, enum LockMode lockMode);
+    unsigned long long lockRBPage(PageIdType pageId, enum LockMode lockMode);
     void unlockRBPage(unsigned long long id);
-
     unsigned long long _nextLockId;
 
     int *_lockCount; 
 
-    struct RecordLock {
+    struct RecordLock
+    {
  	unsigned long long lockId;
 	int semId;
     } *_recordLock, *_rbRecordLock, *_sysRecordLock; 
     
-    struct PageLock {
+    struct PageLock
+    {
 	unsigned long long lockId;
-	int fileId;
-	int pageId;
+	PageIdType pageId;
 	int semId;
     } *_sysPageLock, 
 	  *_dataPageLock, 
 	  *_idxPageLock,
 	  *_rbPageLock;
-           
-    bool _isDisabled;
     
     int _numRecSema;
     int _numRBRecSema;
     int _numSysRecSema;
-
     int _numSysPageSema;
     int _numDataPageSema;
     int _numIdxPageSema;
     int _numRBPageSema;
     int _numDataFileSema;
     int _numBufferPoolSema;
-
     int _numSema;
-
     int _pageLockTimeout;
     int _recLockTimeout;
     int _fileLockTimeout;
     int _poolLockTimeout;
     int _numLockTries;
-
+    int _poolLockRange;
+    
     CegoDatabaseManager *_pDBMng;
 
     unsigned long _modId;
-
 };
 #endif
