@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -38,11 +38,9 @@ public:
     CegoLogHandler(CegoModule *pModule, NetHandler *pN = 0);
     ~CegoLogHandler();
 
-    void requestLogSession(const Chain& tableSet);
-    bool acceptLogSession(Chain& tableSet);
-
+    void requestLogSession(const Chain& tableSet, const Chain& logUser, const Chain& logPwd);
+    bool acceptLogSession(Chain& tableSet, Chain& logUser, Chain& logPwd);
     void closeSession();
-
     void setNetHandle(NetHandler *pN);
 
     //////////////////////////
@@ -56,10 +54,8 @@ public:
     /////////////////////////
 
     void getTableSet(Chain& tableSet);
-
     bool sendLogEntry(char* logBuf, int len);
     bool receiveLogEntry(char* &logBuf,  int &len);
-
     void sendAck();
     void sendNack();
 
