@@ -6,7 +6,7 @@
 // 
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -46,7 +46,6 @@ extern char __lfcxmlVersionString[];
 
 int main(int argc, char **argv)
 {
-
     try 
     {
 	GetLongOpt longOpt(argc, argv);
@@ -67,8 +66,7 @@ int main(int argc, char **argv)
 	    cerr << USAGE << endl;
 	    exit(1);	
 	}
-	
-	
+		
 	if ( longOpt.isSet( Chain("help") ) )
 	{	
 	    cerr << USAGE << endl;
@@ -77,7 +75,7 @@ int main(int argc, char **argv)
 	
 	if ( longOpt.isSet( Chain("version") ) )
 	{
-	    cout << CEGO_PRODUCT << " Log Reader (" << sizeof(long) * 8 << " bit), Version " << CEGO_VERSION 
+	    cout << CEGO_PRODUCT << " Log Reader (" << sizeof(void*) * 8 << " bit), Version " << CEGO_VERSION 
 		 << " [ lfc: " << __lfcVersionString  
 		 << ", lfcxml: " <<  __lfcxmlVersionString << " ]" << endl;
 	    cout << CEGO_COPYRIGHT << endl;
@@ -94,9 +92,7 @@ int main(int argc, char **argv)
 	File logFile(logFileName);
 	
 	logFile.open(File::READ);
-	
-	bool moreLog = true;
-	
+		
 	int offset;
 	int c = logFile.readByte((char*)&offset, sizeof(int));	
 	
@@ -125,6 +121,7 @@ int main(int argc, char **argv)
 	    cout << "LSN=" << lr.getLSN() << ":";
 	    cout << "TS=" << dt.asChain() << ":";
 	    cout << "TID=" << lr.getTID() << ":";
+	    cout << "TAStep=" << lr.getTAStep() << ":";
 
 	    if ( lr.getObjName().length() > 0 )
 	    { 
@@ -152,14 +149,21 @@ int main(int argc, char **argv)
 		{
 		    CegoTableObject to;
 		    to.decode(lr.getData());
-		    cout << to.toChain();
+		    cout << to.toChain(0);
+		    break;
+		}
+		case CegoObject::PBTREE:
+		case CegoObject::UBTREE:
+		case CegoObject::BTREE:
+		{
+		    // TODO
 		    break;
 		}
 		case CegoObject::VIEW:
 		{
 		    CegoViewObject vo;
 		    vo.decode(lr.getData());
-		    cout << vo.toChain();
+		    cout << vo.toChain(0);
 		    break;		    
 		}
 		case CegoObject::PROCEDURE:
@@ -176,13 +180,32 @@ int main(int argc, char **argv)
 		    cout << ko.toChain();
 		    break;		    
 		}
-		default:
+		case CegoObject::CHECK:
+		{
+		    // TODO
 		    break;
 		}
-
+		case CegoObject::TRIGGER:
+		{
+		    // TODO
+		    break;
+		}
+		case CegoObject::ALIAS:
+		{
+		    // TODO
+		    break;
+		}
+		case CegoObject::RBSEG:
+		case CegoObject::JOIN:
+		case CegoObject::UNDEFINED:
+		{
+		    // ignore
+		    break;
+		}
+		}
+		
 		cout << "--- Log Data End   ---" << endl;
-
-	       
+       
 		break;
 	    }
 	    case CegoLogRecord::LOGREC_ALTER:
@@ -201,14 +224,22 @@ int main(int argc, char **argv)
 		{
 		    CegoTableObject to;
 		    to.decode(lr.getData());
-		    cout << to.toChain();
+		    cout << to.toChain(0);
 		    break;
 		}
+		case CegoObject::PBTREE:
+		case CegoObject::UBTREE:
+		case CegoObject::BTREE:
+		{
+		    // TODO
+		    break;
+		}
+		
 		case CegoObject::VIEW:
 		{
 		    CegoViewObject vo;
 		    vo.decode(lr.getData());
-		    cout << vo.toChain();
+		    cout << vo.toChain(0);
 		    break;		    
 		}
 		case CegoObject::PROCEDURE:
@@ -225,8 +256,28 @@ int main(int argc, char **argv)
 		    cout << ko.toChain();
 		    break;		    
 		}
-		default:
+		case CegoObject::CHECK:
+		{
+		    // TODO
 		    break;
+		}
+		case CegoObject::TRIGGER:
+		{
+		    // TODO
+		    break;
+		}
+		case CegoObject::ALIAS:
+		{
+		    // TODO
+		    break;
+		}
+		case CegoObject::RBSEG:
+		case CegoObject::JOIN:
+		case CegoObject::UNDEFINED:
+		{
+		    // ignore
+		    break;
+		}
 		}		
 		cout << "--- Log Data End   ---" << endl;
 		break;
@@ -242,16 +293,19 @@ int main(int argc, char **argv)
 	    case CegoLogRecord::LOGREC_DROP:
 	    {
 		cout << "DROP:" << endl;
-		
+		cout << "--- No log data ---" << endl;
+		break;
+	    }
+	    case CegoLogRecord::LOGREC_TRUNCATE:
+	    {
+		cout << "TRUNCATE:" << endl;	       
 		cout << "--- No log data ---" << endl;
 		break;
 	    }
 	    case CegoLogRecord::LOGREC_INSERT:
 	    {
-		cout << "INSERT:" << endl;
-		
-		cout << "--- Log Data Start ---" << endl;	
-
+		cout << "INSERT:" << endl;	
+		cout << "--- Log Data Start ---" << endl;
 		
 		unsigned long long tid;
 		unsigned long long tastep;
@@ -285,10 +339,20 @@ int main(int argc, char **argv)
 		    CegoBlob *pBlob = blobList.First();
 		    while ( pBlob )
 		    {
-			cout << "BlobSize = " << pBlob->getSize() << endl;
+			cout << "Blob [ ... ] " << pBlob->getSize() << " bytes" << endl;
 			pBlob = blobList.Next();
 		    }
 		}
+		if ( clobList.Size() > 0 )
+		{
+		    CegoClob *pClob = clobList.First();
+		    while ( pClob )
+		    {
+			cout << "Clob [" << pClob->getHead() << "] " << pClob->getSize() << " bytes" << endl;
+			pClob = clobList.Next();
+		    }
+		}
+
 		cout << "--- Log Data End   ---" << endl;
 		
 		// cout << lr << endl;;
@@ -301,7 +365,7 @@ int main(int argc, char **argv)
 		
 		cout << "--- Log Data Start ---" << endl;	
 
-		CegoPredDesc *pPred = 0;
+		CegoPredicate *pPred = 0;
 		Chain tableAlias;
 
 		CegoQueryHelper::decodeDelRec(tableAlias, pPred, lr.getData(), lr.getDataLen(), 0, 0);
@@ -310,56 +374,45 @@ int main(int argc, char **argv)
 		
 		if ( pPred )
 		{
-		    cout << "where " << pPred->toChain() << endl;
+		    cout << "where " << pPred->toChain(0) << endl;
 		}
 		
 		cout << "--- Log Data End   ---" << endl;
 		
 		break;
 	    }
-	    case CegoLogRecord::LOGREC_DELETE_COMMIT:
-	    {
-		cout << "DELETE_COMMIT:" << endl;	       
-		cout << "--- No log data ---" << endl;
-		break;
-	    }
 	    case CegoLogRecord::LOGREC_UPDATE:
 	    {
 		cout << "UPDATE:" << endl;		
 		cout << "--- Log Data Start ---" << endl;	
 		Chain tableAlias;
-		CegoPredDesc *pPred = 0;
+		CegoPredicate *pPred = 0;
 		ListT<CegoField> updList;
 		ListT<CegoExpr*> exprList;
+		bool returnOnFirst;
 		
 		CegoQueryHelper::decodeUpdRec(tableAlias,
 					      pPred, 
 					      updList,
 					      exprList,
+					      returnOnFirst,
 					      lr.getData(), lr.getDataLen(), 0, 0);
-		
-		
+				
 		cout << "TableAlias=" << tableAlias << endl;
 		CegoField *pF = updList.First();
 		CegoExpr **pExpr = exprList.First();
 		while ( pF && pExpr )
 		{
-		    cout << "Set " << pF->getAttrName() << " = " << (*pExpr)->toChain() << endl;
+		    cout << "Set " << pF->getAttrName() << " = " << (*pExpr)->toChain(0) << endl;
 		    pF = updList.Next();
 		    pExpr = exprList.Next();
 		}
 		if ( pPred )
-		    cout << "where " << pPred->toChain() << endl;
+		    cout << "where " << pPred->toChain(0) << endl;
 		cout << "--- Log Data End   ---" << endl;
 		
 		break;
 	    }
-	    case CegoLogRecord::LOGREC_UPDATE_COMMIT:
-	    {
-		cout << "UPDATE_COMMIT:" << endl;
-		cout << "--- No log data ---" << endl;
-		break;
-	    }
 	    case CegoLogRecord::LOGREC_BEGIN:
 	    {
 		cout << "BEGIN:" << endl;
@@ -369,9 +422,15 @@ int main(int argc, char **argv)
 	    case CegoLogRecord::LOGREC_COMMIT:
 	    {
 		cout << "COMMIT:" << endl;
-		cout << "--- No log data ---" << endl;
+		cout << "TID=" << lr.getTID() << ":";
 		break;
 	    }
+	    case CegoLogRecord::LOGREC_ABORT:
+	    {
+		cout << "ABORT:" << endl;
+		cout << "TID=" << lr.getTID() << endl;
+		break;
+	    }	    
 	    case CegoLogRecord::LOGREC_SYNC:
 	    {
 		cout << "SYNC:" << endl;
@@ -411,6 +470,7 @@ int main(int argc, char **argv)
 		break;
 	    }
 	    default:
+		cout << " -- Unknown logaction " << lr.getAction() << endl;
 		break;   		             	
 	    }
 	}
