@@ -6,7 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -20,6 +20,7 @@
 
 // CEGO INCLUDES
 #include "CegoLogRecord.h"
+
 // POSIX INCLUDES
 #include <string.h>
 #include <stdlib.h>
@@ -31,6 +32,8 @@ CegoLogRecord::CegoLogRecord()
     _data = 0;
     _ts=0;
     _tid=0;
+    _tastep=0;
+    _lsn=0;
 }
 
 CegoLogRecord::~CegoLogRecord()
@@ -57,12 +60,22 @@ void CegoLogRecord::setTID(unsigned long long tid)
     _tid = tid;
 }
 
-int CegoLogRecord::getTS() const
+unsigned long long CegoLogRecord::getTAStep() const
+{
+    return _tastep;
+}
+
+void CegoLogRecord::setTAStep(unsigned long long tastep)
+{
+    _tastep = tastep;
+}
+
+long long CegoLogRecord::getTS() const
 {
     return _ts;
 }
 
-void CegoLogRecord::setTS(int ts)
+void CegoLogRecord::setTS(long long ts)
 {
     _ts = ts;
 }
@@ -124,22 +137,26 @@ void CegoLogRecord::setFileId(int fileId)
     _fileId = fileId;
 }
 
-int CegoLogRecord::getPageId() const
+PageIdType CegoLogRecord::getPageId() const
 {
     return _pageId;
 }
 
-void CegoLogRecord::setPageId(int pageId)
+void CegoLogRecord::setPageId(PageIdType pageId)
 {
     _pageId = pageId;
 }
  
 int CegoLogRecord::getEncodingLength() const
 {
-    int len = 2 * sizeof(unsigned long long) + sizeof(int) + sizeof(CegoLogRecord::LogAction);
+    // 4 * sizeof(unsigned long long) => lsn, tid, ts
+    int len = 3 * sizeof(unsigned long long) + sizeof(CegoLogRecord::LogAction);
 
+    if ( _tid > 0 )
+	len += sizeof(unsigned long long);
+    
     if ( _logAction == CegoLogRecord::LOGREC_BUPAGE )
-	len += 2 * sizeof(int) ;
+	len += sizeof(PageIdType) ;
     else if ( _logAction == CegoLogRecord::LOGREC_BUFBM )
 	len += sizeof(int);
     else
@@ -156,8 +173,7 @@ int CegoLogRecord::getEncodingLength() const
 }
 
 void CegoLogRecord::encode(void* p) const
-{
-    
+{    
     char* lrp = (char*)p;
     
     memcpy( lrp, &_lsn, sizeof(unsigned long long));
@@ -166,30 +182,30 @@ void CegoLogRecord::encode(void* p) const
     memcpy( lrp, &_tid, sizeof(unsigned long long));
     lrp = lrp + sizeof(unsigned long long);
 
-    memcpy( lrp, &_ts, sizeof(int));
-    lrp = lrp + sizeof(int);
+    if ( _tid > 0 )
+    {
+	memcpy( lrp, &_tastep, sizeof(unsigned long long));
+	lrp = lrp + sizeof(unsigned long long);
+    }
+    
+    memcpy( lrp, &_ts, sizeof(unsigned long long));
+    lrp = lrp + sizeof(unsigned long long);
 
     memcpy( lrp, &_logAction, sizeof(CegoLogRecord::LogAction));
     lrp = lrp + sizeof(CegoLogRecord::LogAction);
     
     if ( _logAction == CegoLogRecord::LOGREC_BUPAGE )
     {
-	memcpy( lrp, &_fileId, sizeof(int));
-	lrp = lrp + sizeof(int);
-	memcpy( lrp, &_pageId, sizeof(int));
-	lrp = lrp + sizeof(int);
-	// memcpy(lrp, _data, _dataLen);
+	memcpy( lrp, &_pageId, sizeof(PageIdType));
+	lrp = lrp + sizeof(PageIdType);
     }
     else if ( _logAction == CegoLogRecord::LOGREC_BUFBM )
     {
 	memcpy( lrp, &_fileId, sizeof(int));
 	lrp = lrp + sizeof(int);
-
-	// memcpy(lrp, _data, _dataLen);
     }
     else 
     {
-
 	if ( _encObj  )
 	{
 	    char c = 1;
@@ -200,8 +216,7 @@ void CegoLogRecord::encode(void* p) const
 	    lrp = lrp + _objName.length();
 	    
 	    memcpy( lrp, &_objType, sizeof(CegoObject::ObjectType));
-	    lrp = lrp + sizeof(CegoObject::ObjectType);	
-	    
+	    lrp = lrp + sizeof(CegoObject::ObjectType);		    
 	}
 	else
 	{
@@ -222,7 +237,6 @@ void CegoLogRecord::encode(void* p) const
 
 void CegoLogRecord::decode(void* p)
 {
-
     char* lrp = (char*)p;
 
     memcpy( &_lsn, lrp, sizeof(unsigned long long));
@@ -231,45 +245,36 @@ void CegoLogRecord::decode(void* p)
     memcpy( &_tid, lrp, sizeof(unsigned long long));
     lrp = lrp + sizeof(unsigned long long);
 
-    memcpy( &_ts, lrp, sizeof(int));
-    lrp = lrp + sizeof(int);
+    if ( _tid > 0 )
+    {
+	memcpy( &_tastep, lrp, sizeof(unsigned long long));
+	lrp = lrp + sizeof(unsigned long long);
+    }
+    
+    memcpy( &_ts, lrp, sizeof(unsigned long long));
+    lrp = lrp + sizeof(unsigned long long);
 
     memcpy( &_logAction, lrp, sizeof(CegoLogRecord::LogAction));
     lrp = lrp + sizeof(CegoLogRecord::LogAction);
     
     if ( _logAction == CegoLogRecord::LOGREC_BUPAGE )
-    {
-
-	memcpy( &_fileId, lrp, sizeof(int));
+    {	
+	memcpy( &_pageId, lrp, sizeof(PageIdType));
 	lrp = lrp + sizeof(int);
-	
-	memcpy( &_pageId, lrp, sizeof(int));
-	lrp = lrp + sizeof(int);
-
-	// _data = (char*)malloc(_dataLen);
-	// memcpy(_data, lrp, _dataLen);
-	
     }
     else if ( _logAction == CegoLogRecord::LOGREC_BUFBM )
     {
-
 	memcpy( &_fileId, lrp, sizeof(int));
 	lrp = lrp + sizeof(int);
-	
-	// _data = (char*)malloc(_dataLen);
-	// memcpy(_data, lrp, _dataLen);
-	
     }
     else
     {
-
 	char c;
 	memcpy(&c, lrp, 1);
 	lrp++;
 	
 	if ( c > 0 )
-	{
-	    
+	{	    
 	    _objName = Chain(lrp);
 	    lrp = lrp + _objName.length();
 	    
@@ -301,7 +306,7 @@ void CegoLogRecord::decodeLSN(void* p)
 void CegoLogRecord::decodeLogAction(void* p)
 {
     char* lrp = (char*)p;
-    lrp = lrp + 2 * sizeof(unsigned long long) + sizeof(int);
+    lrp = lrp + 3 * sizeof(unsigned long long);
     memcpy( &_logAction, lrp, sizeof(CegoLogRecord::LogAction));
 }
 
@@ -309,13 +314,13 @@ CegoLogRecord& CegoLogRecord::operator = ( const CegoLogRecord& l)
 {
     _lsn = l._lsn;
     _tid = l._tid;
+    _tastep = l._tastep;
     _ts = l._ts;
     _logAction = l._logAction;
     _objName = l._objName;
     _objType = l._objType;
     _data = l._data;
     _dataLen = l._dataLen;
-    _fileId = l._fileId;
     _pageId = l._pageId;
     
     return (*this);
@@ -329,11 +334,10 @@ ostream& operator << (ostream& s, const CegoLogRecord& logRec)
 
 Chain CegoLogRecord::toChain() const
 {
-    
     Chain s;
     
-    s = Chain("[") + getObjName() + Chain(",") + Chain(_tid) + Chain(",");
-							      							      
+    s = Chain("[") + getObjName() + Chain(",") + Chain(_tid) + Chain(",") + Chain(_tastep) + Chain(",");
+    
     switch (_logAction)
     {
     case (CegoLogRecord::LOGREC_CREATE):
@@ -342,6 +346,9 @@ Chain CegoLogRecord::toChain() const
     case (CegoLogRecord::LOGREC_DROP):
 	s += Chain("DROP");
 	break;
+    case (CegoLogRecord::LOGREC_TRUNCATE):
+	s += Chain("TRUNCATE");
+	break;
     case (CegoLogRecord::LOGREC_ALTER):
 	s += Chain("ALTER");
 	break;
@@ -384,13 +391,6 @@ Chain CegoLogRecord::toChain() const
     case (CegoLogRecord::LOGREC_DELCOUNTER):
 	s += Chain("DELCOUNTER");
 	break;
-    case (CegoLogRecord::LOGREC_DELETE_COMMIT):
-	s += Chain("DELETE_COMMIT");
-	break;
-    case (CegoLogRecord::LOGREC_UPDATE_COMMIT):
-	s += Chain("UPDATE_COMMIT");
-	break;
-
     }
     
     s += Chain(",") + Chain(_dataLen);
