@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -61,10 +61,13 @@ const char *CEGO_MODULE_MAP[] =
     "CegoTransactionManager",
     "CegoLockHandler",
     "CegoBTreeManager",
+    "CegoBTreeCursor",
+    "CegoXMLSpace",
     // just for client programs
     "CegoClient",
     "CegoAdmin",
     "CegoAdmAction",
+    "CegoOutput",
     "cgwrap"
 };
 
