@@ -8,7 +8,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -22,6 +22,7 @@
 
 // LFC INCLUDES
 #include <lfcbase/AVLTreeT.h>
+
 // CEGO INCLUDES
 #include "CegoTableManager.h"
 #include "CegoDataPointer.h"
@@ -31,17 +32,16 @@ class CegoOrderCursor {
     
 public:
     
-    CegoOrderCursor(AVLTreeT<CegoOrderNode>* pAVL);
+    CegoOrderCursor(AVLTreeT<CegoOrderNode>* pAVL, ListT<CegoField>& schema);
     ~CegoOrderCursor();
     
     bool getFirst(ListT<CegoField>& fl);
     bool getNext(ListT<CegoField>& fl);
-    
     void abort();
 
  private:
 
+    ListT<CegoField> _schema;
     AVLTreeT<CegoOrderNode>* _pAVL;
-    
 };
 #endif
