@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // Class: CegoOrderNode
 //
@@ -18,17 +18,15 @@
 
 // LFC INCLUDES
 #include <lfcbase/Exception.h>
+
 // CEGO INCLUDES
 #include "CegoOrderNode.h"
-// POSIX INCLUDES
-#include <string.h>
-#include <stdlib.h>
 
 CegoOrderNode::CegoOrderNode()
 {
 }
 
-CegoOrderNode::CegoOrderNode(ListT<CegoFieldValue>& key, ListT<CegoField>& tuple, ListT<Ordering>* pOrdering)
+CegoOrderNode::CegoOrderNode(ListT<CegoFieldValue>& key, ListT<CegoFieldValue>& tuple, ListT<Ordering>* pOrdering)
 {
     _key = key;
     _tuple = tuple;
@@ -44,7 +42,7 @@ ListT<CegoFieldValue>& CegoOrderNode::getKey()
     return _key;
 }
 
-ListT<CegoField>& CegoOrderNode::getTuple()
+ListT<CegoFieldValue>& CegoOrderNode::getTuple()
 {
     return _tuple;
 }
@@ -80,11 +78,10 @@ bool CegoOrderNode::operator == ( const CegoOrderNode& n)
     }
 
     return true;
-
 }
 
 bool CegoOrderNode::operator > ( const CegoOrderNode& n)
-{    
+{
     Ordering *pOrd = _pOrdering->First();
 
     CegoFieldValue* pFV1 = _key.First();
@@ -95,16 +92,24 @@ bool CegoOrderNode::operator > ( const CegoOrderNode& n)
 	if ( *pFV1 < *pFV2 )
 	{
 	    if ( *pOrd == ASC )
+	    {
 		return false;
+	    }
 	    else
+	    {
 		return true;
+	    }
 	}
 	else if ( *pFV1 > *pFV2 )
 	{
 	    if ( *pOrd == ASC )
+	    {
 		return true;
+	    }
 	    else
+	    {
 		return false;
+	    }
 	}
 
 	pOrd = _pOrdering->Next();
@@ -117,7 +122,6 @@ bool CegoOrderNode::operator > ( const CegoOrderNode& n)
 
 bool CegoOrderNode::operator < ( const CegoOrderNode& n)
 {
-
     Ordering *pOrd = _pOrdering->First();
     CegoFieldValue* pFV1 = _key.First();
     CegoFieldValue* pFV2 = n._key.First();
@@ -146,6 +150,3 @@ bool CegoOrderNode::operator < ( const CegoOrderNode& n)
 
     return false;
 }
-
-
-
