@@ -8,7 +8,7 @@
 //      
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -28,55 +28,50 @@
 
 // cego includes
 #include "CegoProcVar.h"
-#include "CegoContentObject.h"
 #include "CegoField.h"
 #include "CegoFieldValue.h"
 #include "CegoDbHandler.h"
+#include "CegoDatabaseManager.h"
 
 class CegoOutput {
-
- public:
-
+    
+public:
+    
     CegoOutput();
     CegoOutput(const ListT<CegoField>& schema, const Chain& format = Chain(""));
     ~CegoOutput();
     
-    void setDbHandle(CegoDbHandler* pDbHandle, int rowsPerMsg = 0);
+    void setDbHandle(CegoDbHandler* pDbHandle, int maxRowPerMsg = 0, int maxBytePerMsg = 0);
+    void setDBMng(CegoDatabaseManager *pDBMng);
     void setRawMode(bool isOn);
     void setRawSeparator(const Chain& rawSep);
     void headOut();
-
     void rowOut(const ListT<CegoField>& fl);    
     void rowOut(const ListT<CegoFieldValue>& fl);
     void tailOut();
-    
     void tabOut(const ListT< ListT<CegoFieldValue> >& fa);
     void abort(const Chain& msg);
     void chainOut(const Chain& msg, long affCount = 0);
     void procResultOut(const Chain& msg, const ListT<CegoProcVar>& outParamList, CegoFieldValue *pRetVal);
-
     void manageOutput(const Chain& cmd, ListT< ListT<CegoFieldValue> > & outData);
 	
- private:
+private:
 
-    int maxFieldSize(CegoField *pF);
-    int max(int i1, int i2, int i3, int i4);
     Chain fill(const Chain& s, int num);
     Chain formatCell(int i, const Chain& s, int maxLen, bool printSep = true);
 
     // flow control methods
-
     void writeHeader(int cols);
     void writeFooter(int rows, int cols, char* filename, int start, int end, int totalRow);
 
     CegoDbHandler* _pDbHandle;
-
+    CegoDatabaseManager* _pDBMng;
+    unsigned long _modId;
     ListT<CegoField> _schema;
-
     int _rowCount;
-    int _rowsPerMsg;
+    int _maxRowPerMsg;
+    int _maxBytePerMsg;
     int _preFill;
-
     bool _rawMode;
     Chain _rawSep;
     Chain _format;
