@@ -6,7 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -21,13 +21,13 @@
 // LFC INCLUDES
 #include <lfcbase/Exception.h>
 #include <lfcbase/Tokenizer.h>
+
 // CEGO INCLUDES
 #include "CegoProcCond.h"
 #include "CegoDatabaseFormater.h"
 
 CegoProcCond::CegoProcCond()
 {
-
 }
 
 CegoProcCond::CegoProcCond(CegoProcCond *pCond, CegoProcPred *pPred, CegoProcCond::ConditionMode mode)
@@ -62,23 +62,31 @@ void CegoProcCond::setBlock(CegoProcBlock *pBlock)
 
 bool CegoProcCond::eval() const
 {
-
     switch ( _mode )
     {
     case AND:
     {
 	return _pCond->eval() && _pPred->eval();
-	break;
     }
     case OR:
     {
 	return _pCond->eval() || _pPred->eval();
-	break;
     }
     case PRED:
+    {
 	return _pPred->eval();
-	break;
     }
+    default:
+        throw Exception(EXLOC, Chain("Uknown condition mode"));
+    }
+}
+
+void CegoProcCond::cleanUp()
+{
+    if ( _pCond )
+	_pCond->cleanUp();
+    if ( _pPred )
+	_pPred->cleanUp();
 }
 
 CegoProcCond& CegoProcCond::operator = (const CegoProcCond& c)
@@ -89,7 +97,7 @@ CegoProcCond& CegoProcCond::operator = (const CegoProcCond& c)
     return (*this);
 }
 
-Chain CegoProcCond::toChain() const
+Chain CegoProcCond::toChain(int defTabSetId) const
 {
     Chain s;
 
@@ -97,17 +105,17 @@ Chain CegoProcCond::toChain() const
     {
     case CegoProcCond::AND:
     {
-	s = _pCond->toChain() + " and " + _pPred->toChain();
+	s = _pCond->toChain(defTabSetId) + " and " + _pPred->toChain(defTabSetId);
 	break;
     }
     case CegoProcCond::OR:
     {
-	s = _pCond->toChain() + " or " + _pPred->toChain();
+	s = _pCond->toChain(defTabSetId) + " or " + _pPred->toChain(defTabSetId);
 	break;
     }
     case CegoProcCond::PRED:
     {
-	s = _pPred->toChain();
+	s = _pPred->toChain(defTabSetId);
     }
     }
     
