@@ -8,7 +8,7 @@
 //                                                        
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -30,30 +30,26 @@ class CegoProcCond  {
     
 public:
     
-    enum ConditionMode { AND , OR, PRED };
+    enum ConditionMode { AND, OR, PRED };
     
     CegoProcCond();
-
     CegoProcCond(CegoProcCond* pCond, CegoProcPred *pPred, ConditionMode mode);
     CegoProcCond(CegoProcPred *pPred);
-
     ~CegoProcCond();
 
     void setBlock(CegoProcBlock *pBlock);
-
     bool eval() const;
+    void cleanUp();
+    Chain toChain(int defTabSetId) const;
+    Chain dbFormat(CegoDatabaseFormater *pForm) const;
 
     CegoProcCond& operator = (const CegoProcCond& c);
-
-    Chain toChain() const;
-    Chain dbFormat(CegoDatabaseFormater *pForm) const;
     friend ostream& operator << (ostream& s, const CegoProcCond& c);
     
 private:
 
     ConditionMode _mode;
     CegoProcCond* _pCond;
-    CegoProcPred* _pPred;
-    
+    CegoProcPred* _pPred;    
 };
 #endif
