@@ -1,12 +1,12 @@
 ///////////////////////////////////////////////////////////////////////////////
 //                                                         
 // CegoProcCursor.cc
-// ----------------
+// -----------------
 // Cego stored procedure cursor 
 //      
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -76,7 +76,11 @@ CegoSelect* CegoProcCursor::getSelect() const
 
 void CegoProcCursor::open()
 {
-
+    if ( _status == OPEN )
+    {
+	throw Exception(EXLOC, Chain("Cursor already open"));
+    }
+    
     if ( _pSelect )
     {
 	_pSelect->prepare();	
@@ -90,7 +94,6 @@ void CegoProcCursor::open()
 
 void CegoProcCursor::close()
 {
-
     if ( _pSelect )
     {
 	_pSelect->cleanUp();
@@ -104,7 +107,6 @@ void CegoProcCursor::close()
 
 void CegoProcCursor::reset()
 {
-
     if ( _pSelect )
     {
 	_pSelect->reset(true);
@@ -118,7 +120,6 @@ void CegoProcCursor::reset()
 
 bool CegoProcCursor::nextTuple(ListT<CegoField>& fl)
 {
-
     if ( _status == CLOSED  )
 	open();
 
