@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -36,9 +36,10 @@ public:
     ~CegoProcNoopStmt();
 
     CegoException execute();
-
-    Chain toChain(const Chain& indent = Chain("")) const;
+    void cleanUp();
+    Chain toChain(int defTabSetId, const Chain& indent = Chain("")) const;
     Chain dbFormat(CegoDatabaseFormater *pForm) const;
+    bool isStatic() const;
     
 private:
 
