@@ -8,7 +8,7 @@
 //                                                        
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -33,25 +33,24 @@ class CegoProcBlock;
 
 class CegoProcStmt  {
     
- public:
+public:
   
-
     CegoProcStmt(CegoProcBlock* pBlock);
     virtual ~CegoProcStmt();
 
     virtual CegoException execute() = 0;
-    virtual Chain toChain(const Chain& indent = Chain("")) const = 0;
+    virtual void cleanUp() = 0;
+    virtual Chain toChain(int defTabSetId, const Chain& indent = Chain("")) const = 0;
     virtual Chain dbFormat(CegoDatabaseFormater *pForm) const = 0;
-    
+    virtual bool isStatic() const = 0;
+
     CegoFieldValue getValue(const Chain& param) const;
     void setValue(const Chain& param, const CegoFieldValue& fv);
-
     CegoProcBlock* getParentBlock();
 
- private:
+private:
 
     CegoProcBlock* _pBlock;
     bool _isDebug;
 };
-
 #endif
