@@ -3,12 +3,12 @@
 ///////////////////////////////////////////////////////////////////////////////
 //                                                         
 // CegoProcThrowStmt.h  
-// --------------------
+// -------------------
 // Cego procedure throw statement implementation
 //
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -34,13 +34,13 @@ public:
     
     CegoProcThrowStmt(CegoExpr *pExpr,
 		    CegoProcBlock *pBlock);
-    
     ~CegoProcThrowStmt();
 
     CegoException execute();
-
-    Chain toChain(const Chain& indent) const;
+    void cleanUp();
+    Chain toChain(int defTabSetId, const Chain& indent) const;
     Chain dbFormat(CegoDatabaseFormater *pForm) const;
+    bool isStatic() const;
     
 private:
 
