@@ -6,7 +6,7 @@
 //               
 // Design and Implementation by Bjoern Lemke
 //                                   
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -26,21 +26,23 @@
 #include "CegoProcWhileStmt.h"
 #include "CegoDatabaseFormater.h"
 
-CegoProcWhileStmt::CegoProcWhileStmt(CegoProcCond *pCond,
-		      CegoProcBlock *pBlock,
-		      CegoProcBlock *pParentBlock) : CegoProcStmt(pParentBlock)
+CegoProcWhileStmt::CegoProcWhileStmt(CegoTableManager *pTabMng,
+				     CegoPredicate *pPred,
+				     CegoProcBlock *pBlock,
+				     CegoProcBlock *pParentBlock) : CegoProcStmt(pParentBlock)
 {
-    _pCond = pCond;
+    _pTabMng = pTabMng;
+    _pPred = pPred;
     _pBlock = pBlock;
-    _pCond->setBlock(_pBlock);
+    _pParentBlock = pParentBlock;
 }
 
 CegoProcWhileStmt::~CegoProcWhileStmt()  
 {
     if ( _pBlock )
 	delete _pBlock;
-    if ( _pCond )
-	delete _pCond;
+    if ( _pPred )
+	delete _pPred;
 }
 
 void CegoProcWhileStmt::setBlock(CegoProcBlock * pBlock)
@@ -50,26 +52,53 @@ void CegoProcWhileStmt::setBlock(CegoProcBlock * pBlock)
 
 CegoException CegoProcWhileStmt::execute()
 {
-    while ( _pCond->eval() )
+    try
     {
-	CegoException result;
-	result =  _pBlock->execute();
-	if ( result != NONE_EXCEP )
-	    return result;
+	while ( _pPred->eval(0,0,0,0,_pBlock) )
+	{
+	    CegoException result;
+	    result =  _pBlock->execute();
+	    if ( result != NONE_EXCEP )
+		return result;
+	    if ( _pTabMng->isAborted() ) 
+	    { 
+		_pParentBlock->setExceptionMsg(Chain("Procedure aborted")); 
+		return ABORT_EXCEP; 
+	    } 
+	}
+    }
+    catch ( Exception e)
+    {
+	Chain excepMsg = e.getBaseMsg();
+	_pParentBlock->setExceptionMsg(excepMsg);
+	return COREOP_EXCEP;	    
     }
     return NONE_EXCEP;
 }
 
-Chain CegoProcWhileStmt::toChain(const Chain& indent) const
+void CegoProcWhileStmt::cleanUp()
+{
+    if ( _pBlock )
+	_pBlock->cleanUp();
+    if ( _pPred )
+	_pPred->cleanUp();
+}
+    
+Chain CegoProcWhileStmt::toChain(int defTabSetId, const Chain& indent) const
 {
     Chain s;
-    s = indent + Chain("while ") + _pCond->toChain() + Chain("\n") + indent + Chain("begin\n");
-    s += _pBlock->toChain(indent + indent);
+    s = indent + Chain("while ") + _pPred->toChain(defTabSetId) + Chain("\n") + indent + Chain("begin\n");
+    s += _pBlock->toChain(defTabSetId, indent + Chain(DEFAULTINDENT));
     s += indent + Chain("end");
     return s;
 }
 
 Chain CegoProcWhileStmt::dbFormat(CegoDatabaseFormater *pForm) const
 {
-    return pForm->formatProcWhileStmt(_pCond, _pBlock);
+    return pForm->formatProcWhileStmt(_pPred, _pBlock);
+}
+
+bool CegoProcWhileStmt::isStatic() const
+{
+    return _pBlock->isStatic();
 }
