@@ -6,7 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -27,9 +27,7 @@
 // CEGO INCLUDES
 #include "CegoXMLdef.h"
 #include "CegoRecoveryManager.h"
-
-#include <string.h>
-#include <stdlib.h>
+#include "CegoPredicate.h"
 
 CegoRecoveryManager::CegoRecoveryManager(CegoDistManager *pGTM, CegoRecoveryManager::RecoveryMode mode)
 {
@@ -37,16 +35,15 @@ CegoRecoveryManager::CegoRecoveryManager(CegoDistManager *pGTM, CegoRecoveryMana
     _pLogger = _pGTM->getDBMng();
     _pDBMng = _pGTM->getDBMng();
     _recoveryMode = mode;
-    _modId = _pDBMng->getModId("CegoRecoveryManager"); 	    
+    _modId = _pDBMng->getModId("CegoRecoveryManager");
 }
 
 CegoRecoveryManager::~CegoRecoveryManager()
 {
 }
 
-unsigned long long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, int pit)
+unsigned long long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, long long pit, CegoAdminHandler* pAH)
 {   
-
     _pLogger->log(_modId, Logger::NOTICE, Chain("Recovering tableset ") + tableSet + Chain(" ..."));
 
     char *pShell = getenv(CGEXESHELLVARNAME);
@@ -62,12 +59,10 @@ unsigned long long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, i
     int tabSetId = _pDBMng->getTabSetId(tableSet);
 
     _pDBMng->setTableSetRunState(tableSet, XML_RECOVERY_VALUE);
-
     _pDBMng->setRecoveryMode(tabSetId, CegoDatabaseManager::ON ); 
 
     try
-    {
-	
+    {	
 	while ( _pDBMng->getTableSetSyncState(tableSet) != Chain(XML_SYNCHED_VALUE) &&  _pDBMng->getRecoveryMode(tabSetId) == CegoDatabaseManager::ON)
 	{
 #ifdef CGDEBUG	
@@ -85,17 +80,28 @@ unsigned long long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, i
 	    _pDBMng->log(_modId, Logger::LOGERR, msg);
 	    throw Exception(EXLOC, msg); 
 	}
-	
-	_pGTM->initLock(tabSetId);
-	_pGTM->regDataFiles(tableSet);
-	
-	// release logfiles to force archiving of current online log
+		
+	// release online logfiles to force archiving
 	if ( _recoveryMode == LOCAL )
+	{
+	    if ( pAH ) 
+		pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Releasing log files ...\n")); 
 	    _pDBMng->releaseLogFiles(tableSet, true);
+	}
 
 	// during recovery, we do not write to online log files
+	if ( pAH ) 
+            pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Stopping log ...\n")); 
 	_pDBMng->stopLog(tabSetId);	
-		
+
+	if ( pAH ) 
+            pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Registering datafiles ...\n")); 
+	_pGTM->regDataFiles(tableSet);
+
+	if ( pAH ) 
+            pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Registering objects ...\n")); 
+	_pGTM->registerObjects(tableSet);
+	
 	unsigned long long cplsn;
 	
 	Chain tsTicketName = _pDBMng->getTSTicket(tableSet);
@@ -119,96 +125,139 @@ unsigned long long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, i
 	    Element *pRoot = pDoc->getRootElement();
 	    pDoc->setRootElement(0);
 	    delete pDoc;
+
+	    Chain buId = pRoot->getAttributeValue(XML_BUBRANCH_ATTR);
+	    Chain currentId = _pDBMng->getTableSetBackupBranch(tableSet);
+
+	    if ( buId == currentId )
+		_pDBMng->log(_modId, Logger::NOTICE, Chain("Backup branch for ticket file verified"));
+	    else
+	    {
+		delete pRoot;
+
+		Chain msg = Chain("Backup branch mismatch : <") + buId + Chain("> != <") + currentId + Chain(">");
+		
+		_pDBMng->setRecoveryMode(tabSetId, CegoDatabaseManager::OFF );
+		_pDBMng->log(_modId, Logger::LOGERR, msg);
+
+		throw Exception(EXLOC, msg);
+	    }	
 	    
 	    _pDBMng->setTableSetInfo(tableSet, pRoot);
 	    
 	    cplsn = _pDBMng->getCommittedLSN(tableSet);
+
+	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Committed lsn = ") + Chain(cplsn));
+	    
+	    if ( pAH ) 
+		pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Committed lsn is ") + Chain(cplsn) + Chain("\n"));
 	    
 	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Starting datafile recovery for tableset ") + tableSet + Chain(" ..."));
-	    unsigned long long lsn = dataFileRecovery(tableSet, tabSetId, cplsn);
-	    
-
-	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Datafile recovery for tableset ") + tableSet + Chain(" finished with lsn ") + Chain(lsn));
+	    unsigned long long lsn = dataFileRecovery(tableSet, tabSetId, cplsn, pAH);
 	    
+	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Datafile recovery for tableset ") + tableSet + Chain(" finished with lsn ") + Chain(lsn));    
 	    tsTicket.close();
 	    
 #ifdef CGDEBUG	
 	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Removing backup ticket ") + tsTicket.getFileName() + Chain(" ..."));
 #endif
 	    
-	    tsTicket.remove();
-	    
+	    tsTicket.remove();	    
 	}
 	else
 	{	
 	    cplsn = _pDBMng->getCommittedLSN(tableSet);
+
+	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Committed lsn = ") + Chain(cplsn));
+
+	    if ( pAH ) 
+		pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Committed lsn is ") + Chain(cplsn) + Chain("\n"));
 	}
 	
 	if ( pit == 0 )
+	{
 	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Starting up-to-crash transaction recovery for tableset ") + tableSet + Chain(" ..."));
+	    if ( pAH ) 
+		pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Starting up-to-crash recovery ...\n"));
+	}
 	else
 	{
 	    Datetime d(pit);
 	    _pDBMng->log(_modId, Logger::NOTICE, 
-			 Chain("Starting point-in-time transaction recovery to ") + d.asChain() + Chain(" for tableset ") + tableSet + Chain(" ..."));	    
+			 Chain("Starting point-in-time transaction recovery to ") + d.asChain() + Chain(" for tableset ") + tableSet + Chain(" ..."));
+	    if ( pAH ) 
+		pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Starting point-in-time recovery ...\n"));
 	}
+
+	// during transaction recovery phase, we must ignore invalid btrees/index objects
+	// since these objects could be invalidated but Log data integrity and consistency is ensured 
+	_pGTM->setIgnoreInvalid(true);
+	unsigned long long lsn = transactionRecovery(tableSet, tabSetId, cplsn, pit, pAH);
+	_pGTM->setIgnoreInvalid(false);
 	
-	_pDBMng->log(_modId, Logger::NOTICE, Chain("Commited lsn =  ") + Chain(cplsn));
-	
-	unsigned long long lsn = transactionRecovery(tableSet, tabSetId, cplsn, pit);
+	// correct invalidated index an btrees
+	_pGTM->correctTableSet(tabSetId, false);
 	
+	if ( pAH ) 
+	    pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Init logfiles ...\n"));
 	_pDBMng->initLogFiles(tableSet, true);
 	
-	_pDBMng->setActiveLogFile(tableSet);    
-
+	_pDBMng->setActiveLogFile(tableSet);
+	
+	if ( pAH ) 
+	    pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Starting log ...\n"));
 	_pDBMng->startLog(tabSetId);
         
 	CegoLogRecord logRec;    
 	CegoLogRecord lr;
-	lr.setAction(CegoLogRecord::LOGREC_SYNC);    
-	if ( _pDBMng->logAction(tabSetId, lr) == CegoLogManager::LOG_ERROR )
-	{
-	    throw Exception(EXLOC, "Cannot write sync point after tableset recovery");
-	}
+	lr.setAction(CegoLogRecord::LOGREC_SYNC);
+	
+	_pDBMng->logIt(tabSetId, lr, _pGTM->getLockHandle());
 	
 	_pDBMng->setTableSetRunState(tableSet, XML_ONLINE_VALUE);    
 	
 	_pDBMng->setRecoveryMode(tabSetId, CegoDatabaseManager::OFF );
 
-	unsigned long long newcplsn = _pDBMng->getLSN(tabSetId);
+	unsigned long long newcplsn = _pDBMng->getCurrentLSN(tabSetId);
 	
 	_pDBMng->setCommittedLSN(tabSetId, newcplsn);
 		
 	_pGTM->registerObjects(tableSet);
+
+	if ( pit != 0 )
+	{
+	    // with a point-in-time recovery, we need new online backup
+	    Datetime d;
+	    _pDBMng->setTableSetBackupBranch(tableSet, d.asChain(BUBRANCHFORMAT));
+	}
 	
-	_pGTM->writeCheckPoint(tableSet, true, false);
+	if ( pAH ) 
+	    pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Writing final checkpoint ...\n"));
+	_pDBMng->writeCheckPoint(tableSet, true, false, _pGTM->getLockHandle());
 
 	_pDBMng->log(_modId, Logger::NOTICE, Chain("Recovery for tableset ") + tableSet + Chain(" finished"));
 	
-	return lsn;
-	
+	return lsn;	
     }
     catch ( Exception e )
     {
 	Chain msg = e.getBaseMsg();
 
-	_pDBMng->setTableSetRunState(tableSet, XML_OFFLINE_VALUE);    
-	_pDBMng->setRecoveryMode(tabSetId, CegoDatabaseManager::OFF );	
-
+	_pDBMng->setTableSetRunState(tableSet, XML_OFFLINE_VALUE);
+	_pDBMng->setRecoveryMode(tabSetId, CegoDatabaseManager::OFF );
+	_pDBMng->releaseFiles(tabSetId);
+	
 	throw Exception(EXLOC, Chain("Recovery failed : ") + msg);
     }
 }
 
-unsigned long long CegoRecoveryManager::dataFileRecovery(const Chain& tableSet, int tabSetId, unsigned long long cplsn)
+unsigned long long CegoRecoveryManager::dataFileRecovery(const Chain& tableSet, int tabSetId, unsigned long long cplsn, CegoAdminHandler* pAH)
 {
-
-    bool endOfBackup = false;       
-   
+    bool endOfBackup = false;   
     unsigned long long lsn = cplsn;
 
     while ( endOfBackup == false &&  _pDBMng->getRecoveryMode(tabSetId) == CegoDatabaseManager::ON )
-    {
-	
+    {	
 	Chain archLogFileName = _pDBMng->getArchiveLogName(tableSet, lsn);
 	
 	ListT<Chain> archIdList;
@@ -217,16 +266,11 @@ unsigned long long CegoRecoveryManager::dataFileRecovery(const Chain& tableSet,
 	_pDBMng->getArchLogInfo(tabSetId, archIdList, archPathList);
 	
 	Chain *pArchLogPath = archPathList.First();
-
-	Chain sourceFileName;
-	
+	Chain sourceFileName;	
 	bool fileFound = false;
 
 	while ( pArchLogPath  && fileFound == false )
-	{
-			
-	    // check if exists
-	    
+	{			
 	    sourceFileName = *pArchLogPath + Chain("/") + archLogFileName;
 	    
 	    File checkFile(sourceFileName);
@@ -247,58 +291,19 @@ unsigned long long CegoRecoveryManager::dataFileRecovery(const Chain& tableSet,
 	{
 	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Recovering offline logfile ") + sourceFileName + Chain(" ..."));
 	    _pDBMng->setLogFile(tabSetId, sourceFileName, true);
-	    _pDBMng->setLSN(tabSetId, lsn + 1);
-	    
+	    _pDBMng->setCurrentLSN(tabSetId, lsn);
+
+	    if ( pAH ) 
+		pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Datafile recovery with ") + sourceFileName + Chain("\n")); 
 	    // recover offline logs
-	    lsn = recoverCurrentDataFileLog(tabSetId, endOfBackup);	   
+	    lsn = recoverCurrentDataFileLog(tabSetId, endOfBackup);
+
+	    _pDBMng->setCurrentLSN(tabSetId, lsn);	    
 	}
 	else if ( _recoveryMode == LOCAL )
 	{
-	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Waiting for logfile ") + sourceFileName + Chain(" ..."));
-	
-	    Chain archRestoreProg = _pDBMng->getArchRestoreProg();
-    
-	    if ( archRestoreProg != Chain("NONE") )
-	    {
-
-		
-		Chain *pArchLogPath = archPathList.First();
-		Chain pathString;
-		while ( pArchLogPath )
-		{
-		    pathString += *pArchLogPath;
-		    pArchLogPath = archPathList.Next();
-		    if ( pArchLogPath )
-			pathString += Chain(":");
-		}
-				
-		CommandExecuter cmdExe(_shellCmd);
-		
-		int archRestoreTimeout = _pDBMng->getArchRestoreTimeout();
-		
-		Chain restoreCmd = archRestoreProg + Chain(" ") + tableSet + Chain(" ") + archLogFileName + Chain(" ") + pathString; 		
-		_pDBMng->log(_modId, Logger::NOTICE, Chain("Triggering external log manager with <") + Chain(restoreCmd) + Chain(">"));
-		int retCode = cmdExe.execute(restoreCmd, archRestoreTimeout);
-		_pDBMng->log(_modId, Logger::NOTICE, Chain("External log manager returned : <") + Chain(retCode) + Chain(">"));
-		if ( retCode == 0 )
-		{
-		    // file could be provided
-		}
-		else if ( retCode == 1 )
-		{
-		    endOfBackup = true;
-		}
-		else // ( retCode > 1 )
-		{
-		    Chain msg("External log maager failed");
-		    throw Exception(EXLOC, msg);			    
-		}
-	    }
-	    else
-	    {
-		Sleeper s;
-		s.secSleep(LOGMNG_RECOVERY_DELAY);
-	    }
+	    if  ( restoreLogFile(tableSet, lsn) == false)
+		endOfBackup = true;
 	}
 	else
 	{
@@ -310,26 +315,22 @@ unsigned long long CegoRecoveryManager::dataFileRecovery(const Chain& tableSet,
     return lsn;
 }
 
-unsigned long long CegoRecoveryManager::transactionRecovery(const Chain& tableSet, int tabSetId, unsigned long long cplsn, int pit)
+unsigned long long CegoRecoveryManager::transactionRecovery(const Chain& tableSet, int tabSetId, unsigned long long cplsn, long long pit, CegoAdminHandler* pAH)
 {
     bool endOfRecovery = false;    
     Chain sourceFileName;
-    int ts = 0;
-
+    long long ts = 0;
     unsigned long long lsn = cplsn;
-
     RecoveryState rstate = CegoRecoveryManager::RECOK;
-	    
+     
     while ( _pDBMng->getRecoveryMode(tabSetId) == CegoDatabaseManager::ON 
 	    && rstate != CegoRecoveryManager::RECPITREACHED
 	    && endOfRecovery == false)
-    {
-	
+    {	
 	bool fileFound = false;   
 
 	while ( fileFound == false && endOfRecovery == false &&  _pDBMng->getRecoveryMode(tabSetId) == CegoDatabaseManager::ON )
-	{
-	    
+	{	    
 	    Chain archLogFileName = _pDBMng->getArchiveLogName(tableSet, lsn);
 	    
 	    ListT<Chain> archIdList;
@@ -341,9 +342,6 @@ unsigned long long CegoRecoveryManager::transactionRecovery(const Chain& tableSe
 	    
 	    while ( pArchLogPath  && fileFound == false)
 	    {
-		
-		// check if exists
-		
 		sourceFileName = *pArchLogPath + Chain("/") + archLogFileName;
 		
 		File checkFile(sourceFileName);
@@ -362,51 +360,8 @@ unsigned long long CegoRecoveryManager::transactionRecovery(const Chain& tableSe
 	    
 	    if ( fileFound == false && _recoveryMode == LOCAL)
 	    {
-		
-		Chain archRestoreProg = _pDBMng->getArchRestoreProg();
-
-		if ( archRestoreProg != Chain("NONE") )
-		{	
-		    
-		    _pDBMng->log(_modId, Logger::NOTICE, Chain("Waiting for logfile ") + archLogFileName + Chain(" ..."));
-		    
-		    Chain *pArchLogPath = archPathList.First();
-		    Chain pathString;
-		    while ( pArchLogPath )
-		    {
-			pathString += *pArchLogPath;
-			pArchLogPath = archPathList.Next();
-			if ( pArchLogPath )
-			    pathString += Chain(":");
-		    }
-		    		    
-		    CommandExecuter cmdExe(_shellCmd);
-		    
-		    int archRestoreTimeout = _pDBMng->getArchRestoreTimeout();
-		    
-		    Chain restoreCmd = archRestoreProg + Chain(" ") + tableSet + Chain(" ") + archLogFileName + Chain(" ") + pathString; 
-		    
-		    _pDBMng->log(_modId, Logger::NOTICE, Chain("Triggering external log manager with <") + Chain(restoreCmd) + Chain(">"));
-		    int retCode = cmdExe.execute(restoreCmd, archRestoreTimeout); 
-		    _pDBMng->log(_modId, Logger::NOTICE, Chain("External log manager returned : <") + Chain(retCode) + Chain(">"));
-		    if ( retCode == 0 )
-		    {
-			// file could be provided
-		    }
-		    else if ( retCode == 1 )
-		    {
-			endOfRecovery = true;
-		    }
-		    else // ( retCode > 1 )
-		    {
-			Chain msg("External log manager failed");
-			throw Exception(EXLOC, msg);			    
-		    }		     
-		}	      
-		else
-		{
-		    endOfRecovery = true;
-		}
+		if  ( restoreLogFile(tableSet, lsn) == false)
+		    endOfRecovery = true;		
 	    }
 	    if ( fileFound == false )
 	    {
@@ -418,17 +373,19 @@ unsigned long long CegoRecoveryManager::transactionRecovery(const Chain& tableSe
 	// if we found the appropriate file, we try to recover it
 	if ( fileFound )
 	{
-
 	    bool isRecovered = false;
 	    while ( ! isRecovered  && _pDBMng->getRecoveryMode(tabSetId) == CegoDatabaseManager::ON )
-	    {
-		
+	    {		
 		_pDBMng->log(_modId, Logger::NOTICE, Chain("Recovering offline logfile ") + sourceFileName + Chain(" ..."));
-		_pDBMng->setLogFile(tabSetId, sourceFileName, true);
-		
-		_pDBMng->setLSN(tabSetId, lsn + 1);
+		_pDBMng->setLogFile(tabSetId, sourceFileName, true);		
+		_pDBMng->setCurrentLSN(tabSetId, lsn);
+
+		if ( pAH ) 
+		    pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Transaction recovery with ") + sourceFileName + Chain("\n"));
 		
 		rstate =  recoverCurrentTransactionLog(tabSetId, pit, lsn, ts);
+
+		_pDBMng->setCurrentLSN(tabSetId, lsn);
 		
 		if ( rstate == CegoRecoveryManager::RECOK )
 		{
@@ -475,7 +432,7 @@ unsigned long long CegoRecoveryManager::transactionRecovery(const Chain& tableSe
 	}		
     }
 
-    _pGTM->writeCheckPoint(tableSet, false, false);
+    _pDBMng->writeCheckPoint(tableSet, false, false, _pGTM->getLockHandle());
 
     _pDBMng->log(_modId, Logger::NOTICE, Chain("Transaction recovery finished with lsn ") + Chain(lsn));
     return lsn;
@@ -483,7 +440,6 @@ unsigned long long CegoRecoveryManager::transactionRecovery(const Chain& tableSe
 
 int CegoRecoveryManager::recoverCurrentDataFileLog(int tabSetId, bool& endOfBackup)
 {
-    
     unsigned long long lsn=0;
 
     _pDBMng->seekToStart(tabSetId);
@@ -497,31 +453,27 @@ int CegoRecoveryManager::recoverCurrentDataFileLog(int tabSetId, bool& endOfBack
     endOfBackup = false;
     CegoLogRecord logRec;
     while ( _pDBMng->logRead(tabSetId, logRec) && ! endOfBackup )
-    {
-	
-	if ( logRec.getLSN() == _pDBMng->getLSN(tabSetId)  )
-	{
-	    
+    {	
+	if ( logRec.getLSN() > _pDBMng->getCurrentLSN(tabSetId)  )
+	{	    
 	    lsn=logRec.getLSN();
 	    	    
 	    switch ( logRec.getAction() ) 
-	    {
-		
+	    {		
 	    case CegoLogRecord::LOGREC_BUPAGE:
-		
+	    {
 #ifdef CGDEBUG	
 		_pDBMng->log(_modId, Logger::DEBUG, Chain("Writing page (") + Chain(logRec.getFileId()) + Chain(",") + Chain(logRec.getPageId()) + Chain(")"));
 #endif
 
-		_pDBMng->writePage(logRec.getFileId(),logRec.getPageId(), 0, logRec.getData(), pLockHandle ); 
+		_pDBMng->writePage(logRec.getPageId(), 0, logRec.getData(), pLockHandle ); 
 		break;
-		
+	    }	
 	    case CegoLogRecord::LOGREC_BUFBM:
 	    {
 #ifdef CGDEBUG	
 		_pDBMng->log(_modId, Logger::DEBUG, Chain("Writing fbm for fileId ") + Chain(logRec.getFileId()));
 #endif
-
 		_pDBMng->writeFBM(logRec.getFileId(), (unsigned*)logRec.getData(), pLockHandle ); 
 		break;
 	    }
@@ -531,40 +483,58 @@ int CegoRecoveryManager::recoverCurrentDataFileLog(int tabSetId, bool& endOfBack
 		endOfBackup=true;
 		break;
 	    }
-	    default:
+	    case CegoLogRecord::LOGREC_CREATE:
+	    case CegoLogRecord::LOGREC_DROP:
+	    case CegoLogRecord::LOGREC_ALTER:
+	    case CegoLogRecord::LOGREC_RENAME:
+	    case CegoLogRecord::LOGREC_INSERT:
+	    case CegoLogRecord::LOGREC_DELETE:
+	    case CegoLogRecord::LOGREC_UPDATE: 
+	    case CegoLogRecord::LOGREC_BEGIN: 
+	    case CegoLogRecord::LOGREC_COMMIT: 
+	    case CegoLogRecord::LOGREC_ABORT: 
+	    case CegoLogRecord::LOGREC_SYNC: 		     
+	    case CegoLogRecord::LOGREC_ADDCOUNTER:
+	    case CegoLogRecord::LOGREC_DELCOUNTER:
+	    case CegoLogRecord::LOGREC_TRUNCATE:		
 		break;
 	    }
-
-	    _pDBMng->setLSN(tabSetId, lsn + 1);
 	}
-	else if ( logRec.getLSN() < _pDBMng->getLSN(tabSetId)  )
+	else // logRec.getLSN() <= _pDBMng->getLSN(tabSetId)
 	{
-	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Ignoring lsn ") + Chain(logRec.getLSN()) + Chain(" ( expected ") + Chain(_pDBMng->getLSN(tabSetId)) + Chain(")"));	    
-	}
-	else
-	{
-	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Skipping logfile lsn ") + Chain(logRec.getLSN()) + Chain(" ( expected ") + Chain(_pDBMng->getLSN(tabSetId)) + Chain(")"));	    
-	    endOfBackup=true;
+	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Ignoring lsn ") + Chain(logRec.getLSN()) + Chain(" ( expected greater than ") + Chain(_pDBMng->getCurrentLSN(tabSetId)) + Chain(")"));	 
 	}
+	
 	if ( logRec.getData() )
 	    free ( logRec.getData() ) ;	
     }
     return lsn;
 }
 
-CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactionLog(int tabSetId, int pit, unsigned long  long& lsn, int& ts)
+CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactionLog(int tabSetId, long long pit, unsigned long long& lsn, long long& ts)
 {
+    _invalidatedList.Empty();
+
+    _pDBMng->log(_modId, Logger::NOTICE, Chain("Invalidating transaction affected index objects ..."));
+    _pGTM->getTransactionManager()->getCrashAffectedTables(tabSetId, _invalidatedList);
+    
+    Chain *pTableName = _invalidatedList.First();
+    while ( pTableName )
+    {
+	_pDBMng->log(_modId, Logger::NOTICE, Chain("Invalidating index objects for table  ") + *pTableName + Chain(" ..."));
+	_pGTM->invalidateIndexForTable(tabSetId, *pTableName);
+	pTableName = _invalidatedList.Next();
+    }
+    _pDBMng->log(_modId, Logger::NOTICE, Chain("Index invalidation done"));
     
     // initially, we set the logfile to incomplete
     RecoveryState rstate = CegoRecoveryManager::RECINCOMPLETE;
 
     _pDBMng->seekToStart(tabSetId);
-
     _pDBMng->log(_modId, Logger::NOTICE, Chain("Recovering logfile ..."));
 
     // now recover to last lsn in log
-
-    SetT<Chain> invalidateSet;
+		    
     CegoLogRecord logRec;
     while ( _pDBMng->logRead(tabSetId, logRec) && rstate != CegoRecoveryManager::RECPITREACHED )
     {
@@ -574,42 +544,37 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 	    Datetime d(logRec.getTS());
 	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Required point in time reached, Ignoring logentry  ") + Chain(logRec.getLSN()) 
 			 + Chain(" ( ") + d.asChain() + Chain(" ) and higher"));
-	    
 
-	    rstate = CegoRecoveryManager::RECPITREACHED;
-	    
+	    rstate = CegoRecoveryManager::RECPITREACHED;	    
 	}
 	else if ( logRec.getTS() <  ts )
 	{
 	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Old log detected, ignoring logentry ") + Chain(logRec.getLSN()) + Chain(" and higher"));
 
 	    rstate = CegoRecoveryManager::RECPITREACHED;
-	    
 	}
 	else
 	{
-	    if ( logRec.getLSN() == _pDBMng->getLSN(tabSetId)  )
+	    if ( logRec.getLSN() > _pDBMng->getCurrentLSN(tabSetId)  )
 	    {
-
 		// we got a valid lsn, so the log file seems to be complete and we can switch from incomplete to ok
 		rstate = CegoRecoveryManager::RECOK;
 
 		lsn=logRec.getLSN();
 		ts=logRec.getTS();
-		int tid=logRec.getTID();
+		unsigned long long tid=logRec.getTID();
+		unsigned long long tastep=logRec.getTAStep();
 
 		_pDBMng->log(_modId, Logger::NOTICE, Chain("Recovering lsn ") + Chain(logRec.getLSN()) + Chain(" ..."));
 
 		// here comes the recovery
 		
 		switch ( logRec.getAction() ) 
-		{
-		    
+		{		    
 		case CegoLogRecord::LOGREC_CREATE:
 		{
 		    switch ( logRec.getObjType() )
-		    {
-			
+		    {	
 		    case CegoObject::TABLE:
 		    case CegoObject::SYSTEM:
 		    {
@@ -625,7 +590,15 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 		    {
 			CegoTableObject to;
 			to.decode(logRec.getData());
-			_pGTM->createIndexTable(tabSetId, logRec.getObjName(), to.getTabName(), to.getSchema(), logRec.getObjType());
+			bool isCached=false;
+
+			if ( _pGTM->objectExists(tabSetId, logRec.getObjName(), logRec.getObjType() ) )
+			{
+			    _pDBMng->log(_modId, Logger::NOTICE, Chain("Dropping suspect avl index ") + logRec.getObjName());
+			    _pGTM->dropIndex(tabSetId, logRec.getObjName());
+			}
+
+			_pGTM->createIndexTable(tabSetId, logRec.getObjName(), to.getTabName(), to.getSchema(), logRec.getObjType(), isCached);
 			_pDBMng->addObject(tabSetId, logRec.getObjName(), logRec.getObjType());
 			break;
 		    }
@@ -635,7 +608,17 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 		    {
 			CegoBTreeObject bo;
 			bo.decode(logRec.getData());
-			_pGTM->createIndexTable(tabSetId, logRec.getObjName(), bo.getTabName(), bo.getSchema(), logRec.getObjType());
+			bool isCached=false;
+
+			// it may happen that the the btree objects already exists
+			// in this case, we drop any rebuild
+			
+			if ( _pGTM->objectExists(tabSetId, logRec.getObjName(), logRec.getObjType() ) )
+			{
+			    _pDBMng->log(_modId, Logger::NOTICE, Chain("Dropping suspect btree ") + logRec.getObjName());
+			    _pGTM->dropBTree(tabSetId, logRec.getObjName());
+			}
+			_pGTM->createIndexTable(tabSetId, logRec.getObjName(), bo.getTabName(), bo.getSchema(), logRec.getObjType(), isCached);
 			_pDBMng->addObject(tabSetId, logRec.getObjName(), logRec.getObjType());
 			break;
 		    }
@@ -671,15 +654,72 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 			_pDBMng->addObject(tabSetId, logRec.getObjName(), CegoObject::CHECK);
 			break;
 		    }
-		    default:		    
+		    case CegoObject::TRIGGER:
+		    {
+			// TODO
+			break;
+		    }
+		    case CegoObject::ALIAS:
+		    {
+			// TODO
+			break;
+		    }
+		    case CegoObject::RBSEG:
+		    case CegoObject::JOIN:
+		    case CegoObject::UNDEFINED:
+			// N/A
 			break;
 		    }
 		    break;
 		}
 		case CegoLogRecord::LOGREC_DROP:
+		{		  
+		    switch ( logRec.getObjType() )
+		    {
+		    case CegoObject::TABLE:
+			_pGTM->dropTable(tabSetId, logRec.getObjName());
+			break;
+		    case CegoObject::AVLTREE:
+		    case CegoObject::UAVLTREE:
+		    case CegoObject::PAVLTREE:
+			_pGTM->dropIndex(tabSetId, logRec.getObjName());
+			break;
+		    case CegoObject::BTREE:
+		    case CegoObject::UBTREE:
+		    case CegoObject::PBTREE:
+			_pGTM->dropBTree(tabSetId, logRec.getObjName());
+			break;
+		    case CegoObject::VIEW:
+			_pGTM->dropView(tabSetId, logRec.getObjName());
+			break;
+		    case CegoObject::PROCEDURE:
+			_pGTM->dropProcedure(tabSetId, logRec.getObjName());
+			break;
+		    case CegoObject::FKEY:
+			_pGTM->dropFKey(tabSetId, logRec.getObjName());
+			break;
+		    case CegoObject::CHECK:
+			_pGTM->dropCheck(tabSetId, logRec.getObjName());
+			break;
+		    case CegoObject::TRIGGER:
+			_pGTM->dropTrigger(tabSetId, logRec.getObjName());
+			break;
+		    case CegoObject::ALIAS:
+			_pGTM->dropAlias(tabSetId, logRec.getObjName());
+			break;
+		    case CegoObject::SYSTEM:
+		    case CegoObject::RBSEG:
+		    case CegoObject::JOIN:
+		    case CegoObject::UNDEFINED:
+			break;
+			// ignore
+			// throw Exception(EXLOC, "Invalid object type for drop");
+		    }		    		   
+		    break;
+		}
+		case CegoLogRecord::LOGREC_TRUNCATE:
 		{
-		    _pGTM->removeObject(tabSetId, logRec.getObjName(), logRec.getObjType());
-		    _pDBMng->removeObject(tabSetId, logRec.getObjName(), logRec.getObjType());
+		    _pGTM->truncateTable(tabSetId, logRec.getObjName());
 		    break;
 		}
 		case CegoLogRecord::LOGREC_ALTER:
@@ -699,39 +739,38 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 		}
 		case CegoLogRecord::LOGREC_BEGIN:
 		{
+		    // tid is already setup by log record information
 		    // _pGTM->beginTransaction(tabSetId);
 		    break;
 		}
 		case CegoLogRecord::LOGREC_COMMIT:
-		{		    
-		    _pGTM->setTID(tabSetId, tid);		    
-		    _pGTM->commitTransaction(tabSetId);
+		{
+		    _pGTM->setTID(tabSetId, tid);	    
+		    _pGTM->commitTransaction(tabSetId, false);
 		    break;
 		}
 		case CegoLogRecord::LOGREC_ABORT:
 		{
 		    _pGTM->setTID(tabSetId, tid);
-		    _pGTM->rollbackTransaction(tabSetId);
+		    _pGTM->rollbackTransaction(tabSetId, false);
 		    break;
 		}
 		case CegoLogRecord::LOGREC_INSERT:
 		{
-		    
 		    CegoTableObject oe;
 		    _pGTM->getObject(tabSetId, logRec.getObjName(), CegoObject::TABLE, oe);
 
-		    if ( invalidateSet.Find(logRec.getObjName()) == 0 )
+		    if ( _invalidatedList.Find(logRec.getObjName()) == 0 )
 		    {
-			_pDBMng->log(_modId, Logger::NOTICE, Chain("Invalidating index for table ") + logRec.getObjName());
+			_pDBMng->log(_modId, Logger::NOTICE, Chain("Invalidating index objects for table ") + logRec.getObjName());
 			_pGTM->invalidateIndexForTable(tabSetId, logRec.getObjName());
-			invalidateSet.Insert(logRec.getObjName());
+			_invalidatedList.Insert(logRec.getObjName());
 		    }
-		    ListT<CegoField> fvl = oe.getSchema();
-		    
+		    		    
+		    ListT<CegoField> fvl = oe.getSchema();		    
 		    ListT<CegoBlob> blobList;
 		    ListT<CegoClob> clobList;
 
-
 #ifdef CGDEBUG	
 		    _pDBMng->log(_modId, Logger::DEBUG, Chain("Decoding log record of len ") + Chain(logRec.getDataLen()));
 #endif
@@ -754,19 +793,17 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 		    CegoBlob *pBlob = blobList.First();
 		    CegoField *pF = fvl.First();
 		    while ( pBlob && pF )
-		    {
-	    	    
-			int fileId;
-			int pageId;
+		    {	    	    
+			PageIdType pageId;
 			
-			_pGTM->putBlobData(tabSetId, pBlob->getBufPtr(), pBlob->getSize(), fileId, pageId);
+			_pGTM->putBlobData(tabSetId, pBlob->getBufPtr(), pBlob->getSize(), pageId);
 			
 			while ( pF->getValue().getType() != BLOB_TYPE && pF)
 			    pF = fvl.Next();
 			
 			if ( pF )
 			{
-			    Chain blobRef = Chain("[") + Chain(fileId) + Chain(",") + Chain(pageId) + Chain("]");
+			    Chain blobRef = Chain("[") + Chain(pageId) + Chain("]");
 			    CegoFieldValue fv(BLOB_TYPE, blobRef);
 			    pF->setValue(fv);
 			}
@@ -775,27 +812,25 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 			    throw Exception(EXLOC, "Cannot get blob reference");
 			}
 			
-			free ( pBlob->getBufPtr() );
+			pBlob->release();
 			pBlob = blobList.Next();
-
+			pF = fvl.Next();
 		    }
 
 		    CegoClob *pClob = clobList.First();
 		    pF = fvl.First();
 		    while ( pClob && pF )
-		    {
-	    	    
-			int fileId;
-			int pageId;
+		    {	    	   
+			PageIdType pageId;
 			
-			_pGTM->putClobData(tabSetId, pClob->getBufPtr(), pClob->getSize(), fileId, pageId);
+			_pGTM->putClobData(tabSetId, pClob->getBufPtr(), pClob->getSize(), pageId);
 			
 			while ( pF->getValue().getType() != CLOB_TYPE && pF)
 			    pF = fvl.Next();
 			
 			if ( pF )
 			{
-			    Chain clobRef = Chain("[") + Chain(fileId) + Chain(",") + Chain(pageId) + Chain("]");
+			    Chain clobRef = Chain("[") + Chain(pageId) + Chain("]");
 			    CegoFieldValue fv(CLOB_TYPE, clobRef);
 			    pF->setValue(fv);
 			}
@@ -804,156 +839,183 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 			    throw Exception(EXLOC, "Cannot get clob reference");
 			}
 			
-			free ( pClob->getBufPtr() );
+			pClob->release();
 			pClob = clobList.Next();
-
+			pF = fvl.Next();
 		    }
 		    
-		    bool lastPage = true;
+		    bool doLogging = true;
+		    bool flushLog = true;
+		    
 		    CegoDataPointer dp;
 
 		    _pGTM->setTID(tabSetId, tid);
-		    _pGTM->insertDataTable(oe, fvl, dp, lastPage);
+		    _pGTM->setTAStep(tabSetId, tastep);
+		    
+		    _pGTM->insertDataTable(oe, fvl, dp, doLogging, flushLog);
+		    
 		    break;
 		}
 		case CegoLogRecord::LOGREC_DELETE:
-		{
-		  
-		    CegoPredDesc *pPred = 0;
+		{		  
+		    CegoPredicate *pPred = 0;
 		    Chain tableAlias;
 
 		    CegoQueryHelper::decodeDelRec(tableAlias, pPred, logRec.getData(), logRec.getDataLen(), _pGTM, tabSetId);
 
-		    _delRecList.Insert(DeleteRecord(tid, tableAlias, pPred));
-		    
-		    break;
-		}
-		case CegoLogRecord::LOGREC_DELETE_COMMIT:
-		{
- 
-		    DeleteRecord* delRecord = _delRecList.Find(DeleteRecord(tid));
-		    
-		    if ( delRecord )
+		    CegoTableObject oe;
+		    _pGTM->getObject(tabSetId, logRec.getObjName(), CegoObject::TABLE, oe);
+
+		    if ( _invalidatedList.Find(logRec.getObjName()) == 0 )
 		    {
-
-			CegoTableObject oe;
-			_pGTM->getObject(tabSetId, logRec.getObjName(), CegoObject::TABLE, oe);
-			
-			if ( invalidateSet.Find(logRec.getObjName()) == 0 )
-			{
-			    _pDBMng->log(_modId, Logger::NOTICE, Chain("Invalidating index for table ") + logRec.getObjName());
-			    _pGTM->invalidateIndexForTable(tabSetId, logRec.getObjName());
-			    invalidateSet.Insert(logRec.getObjName());
-			}
-			
-			_pGTM->setTID(tabSetId, tid);
-			_pGTM->deleteDataTable(oe, delRecord->getPred(), 0);
-
-			delRecord->clean();
-			_delRecList.Remove(DeleteRecord(tid));
-
-			
-		    }
-		    else
-		    {
-			_pDBMng->log(_modId, Logger::NOTICE, Chain("Cannot find delete record for transaction ") + Chain(tid));
-			
+			_pDBMng->log(_modId, Logger::NOTICE, Chain("Invalidating index objects for table ") + logRec.getObjName());
+			_pGTM->invalidateIndexForTable(tabSetId, logRec.getObjName());
+			_invalidatedList.Insert(logRec.getObjName());
 		    }
+		    
+		    _pGTM->setTID(tabSetId, tid);
+		    _pGTM->setTAStep(tabSetId, tastep);		    
+		    _pGTM->deleteDataTable(oe, pPred, 0, false);
+		    
 		    break;
 		}
 		case CegoLogRecord::LOGREC_UPDATE:
 		{
-		    
 		    ListT<CegoField> updList;
 		    ListT<CegoExpr*> exprList;
 
-		    CegoPredDesc *pPred = 0;		    
+		    CegoPredicate *pPred = 0;		    
 		    Chain tableAlias;
+		    bool returnOnFirst;
 		    
-		    CegoQueryHelper::decodeUpdRec(tableAlias, pPred, updList, exprList, logRec.getData(), logRec.getDataLen(), _pGTM, tabSetId);
-
-		    _updRecList.Insert(UpdateRecord(tid, tableAlias, pPred, updList, exprList));
-
-		    break;
-		}
-		case CegoLogRecord::LOGREC_UPDATE_COMMIT:
-		{
-
-		    UpdateRecord* updRecord = _updRecList.Find(UpdateRecord(tid));
+		    CegoQueryHelper::decodeUpdRec(tableAlias, pPred, updList, exprList, returnOnFirst,
+						  logRec.getData(), logRec.getDataLen(), _pGTM, tabSetId);
 		    
-		    if ( updRecord )
+		    if ( _invalidatedList.Find(logRec.getObjName()) == 0 )
 		    {
+			_pDBMng->log(_modId, Logger::NOTICE, Chain("Invalidating index objects for table ") + logRec.getObjName());
+			_pGTM->invalidateIndexForTable(tabSetId, logRec.getObjName());
+			_invalidatedList.Insert(logRec.getObjName());
+		    }
 		    
-			if ( invalidateSet.Find(logRec.getObjName()) == 0 )
-			{
-			    _pDBMng->log(_modId, Logger::NOTICE, Chain("Invalidating index for table ") + logRec.getObjName());		    
-			    _pGTM->invalidateIndexForTable(tabSetId, logRec.getObjName());
-			    invalidateSet.Insert(logRec.getObjName());
-			}
-
-			_pGTM->setTID(tabSetId,tid);
-			_pGTM->updateDataTable(tabSetId, logRec.getObjName(),
-					       updRecord->getTable(), 
-					       updRecord->getPred(), 
-					       updRecord->getUpdateList(), 
-					       updRecord->getExprList(), 0);
-
-			updRecord->clean();
-			_updRecList.Remove(UpdateRecord(tid));
-			
-		    }
-		    else
-		    {
-			_pDBMng->log(_modId, Logger::NOTICE, Chain("Cannot find update record for transaction ") + Chain(tid));			
-		    }
+		    _pGTM->setTID(tabSetId, tid);
+		    _pGTM->setTAStep(tabSetId, tastep);
+		    
+		    ListT<CegoField> returnList;
+		    _pGTM->updateDataTable(tabSetId, logRec.getObjName(),
+					   tableAlias, 
+					   pPred, 
+					   updList, 
+					   exprList,
+					   returnOnFirst,
+					   returnList);		
 		    break;
 		}
 		case CegoLogRecord::LOGREC_ADDCOUNTER:
 		{
+		    // counters are not completely transaction safe since during some checkpoint of any tableset
+		    // counters could be already written down to XML configuration file
+		    // For recovery, we set counters with force = true
+		    
 		    Chain counterName ( logRec.getData() );
 		    long initValue = 0;
-		    _pDBMng->addCounter(tabSetId, counterName, initValue);
+		    _pDBMng->addCounter(tabSetId, counterName, initValue, true);
 		    break;
 		}
 		case CegoLogRecord::LOGREC_DELCOUNTER:
 		{
 		    Chain counterName ( logRec.getData() );
-		    long initValue = 0;
 		    _pDBMng->removeCounter(tabSetId, counterName);
 		    break;
 		}
-		default:
+		case CegoLogRecord::LOGREC_SYNC:		    
+		case CegoLogRecord::LOGREC_BUPAGE:
+		case CegoLogRecord::LOGREC_BUFBM:
+		case CegoLogRecord::LOGREC_BUFIN:
+		{
+		    // BUPAGE, BUFMB and BUFIN already handled in data file recovery
+		    // for SYNC nothing to do
 		    break;
 		}
-		_pDBMng->setLSN(tabSetId, lsn + 1);
-	    
+		}
 	    }
-	    else if ( logRec.getLSN() < _pDBMng->getLSN(tabSetId)  )
+	    else // logRec.getLSN() < _pDBMng->getLSN(tabSetId)
 	    {
 		_pDBMng->log(_modId, Logger::NOTICE, Chain("Ignoring lsn ") + Chain(logRec.getLSN()) 
-			     + Chain(" ( expected ") + Chain(_pDBMng->getLSN(tabSetId)) + Chain(")"));
+			     + Chain(" ( expected greater than ") + Chain(_pDBMng->getCurrentLSN(tabSetId)) + Chain(")"));
 
 		rstate = CegoRecoveryManager::RECOK;
-
-	    }
-	    else
-	    {
-		_pDBMng->log(_modId, Logger::NOTICE, Chain("Skipping logfile lsn ") + Chain(logRec.getLSN()) 
-			     + Chain(" ( expected ") + Chain(_pDBMng->getLSN(tabSetId)) + Chain(")"));
-
-
-		rstate = CegoRecoveryManager::RECPITREACHED;
-		
 	    }
 	}
 		    
 	if ( logRec.getData() )
 	    free ( logRec.getData() ) ;
-
     }
     
     _pDBMng->log(_modId, Logger::NOTICE, Chain("Current logfile recovery finished with lsn ") + Chain(lsn));
 
     return rstate;
+}
+
+bool CegoRecoveryManager::restoreLogFile(const Chain& tableSet, unsigned long lsn)
+{    
+    bool fileProvided=false;
+    Chain archRestoreProg = _pDBMng->getArchRestoreProg();
     
+    if ( archRestoreProg != Chain("NONE") )
+    {
+	Chain archLogFileName = _pDBMng->getArchiveLogName(tableSet, lsn);
+		
+	ListT<Chain> archIdList;
+	ListT<Chain> archPathList;
+	
+	int tabSetId = _pDBMng->getTabSetId(tableSet);    
+	_pDBMng->getArchLogInfo(tabSetId, archIdList, archPathList);
+
+	_pDBMng->log(_modId, Logger::NOTICE, Chain("Waiting for logfile ") + archLogFileName + Chain(" ..."));
+	
+	Chain *pArchLogPath = archPathList.First();
+	Chain pathString;
+	while ( pArchLogPath )
+	{
+	    pathString += *pArchLogPath;
+	    pArchLogPath = archPathList.Next();
+	    if ( pArchLogPath )
+		pathString += Chain(":");
+	}
+	
+	CommandExecuter cmdExe(_shellCmd);
+	
+	int archRestoreTimeout = _pDBMng->getArchRestoreTimeout();
+	
+	Chain restoreCmd = archRestoreProg + Chain(" -m recover -t ") + tableSet + Chain(" -f ") + archLogFileName + Chain(" -p ") + pathString;
+	
+	_pDBMng->log(_modId, Logger::NOTICE, Chain("Triggering external log manager with <") + Chain(restoreCmd) + Chain(">"));
+	int retCode = cmdExe.execute(restoreCmd, archRestoreTimeout); 
+	_pDBMng->log(_modId, Logger::NOTICE, Chain("External log manager returned : <") + Chain(retCode) + Chain(">"));
+	if ( retCode == 0 )
+	{
+	    // file could be provided
+	    fileProvided=true;
+	}
+	else if ( retCode == 1 )
+	{
+	    fileProvided = false;
+	}
+	else // ( retCode > 1 )
+	{
+	    Chain msg("External log manager failed");
+	    throw Exception(EXLOC, msg);			    
+	}		     
+    }	      
+    else
+    {
+	fileProvided = false;
+    }
+
+    // we don't have to check for online redo logs
+    // if recovery starts, the current online redo logs are copied to archive location
+    // so they are already available
+    
+    return fileProvided;
 }
