@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke  
+// (C)opyright 2000-2025 Bjoern Lemke  
 //
 // INTERFACE MODULE
 //
@@ -34,25 +34,20 @@ class CegoReturnVar  {
     
 public:
    
-
     CegoReturnVar(const Chain& varName,
 		  CegoExpr *pExpr);
-
     ~CegoReturnVar();
-
+    
     const Chain& getVarName() const;
     CegoExpr* getExpr() const;
+    Chain toChain(int defTabSetId) const;    
+    Chain dbFormat(CegoDatabaseFormater *pForm) const;
 
     CegoReturnVar& operator = ( const CegoReturnVar& rv);
     
-    Chain toChain() const;    
-    Chain dbFormat(CegoDatabaseFormater *pForm) const;
-    
 private:
 
     Chain _varName;
     CegoExpr *_pExpr;
-
 };
-
 #endif
