@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -23,6 +23,7 @@
 // base includes
 #include <lfcbase/ListT.h>
 #include <lfcbase/SetT.h>
+#include <lfcbase/NanoTimer.h>
 
 // xml includes
 #include <lfcxml/Element.h>
@@ -34,12 +35,11 @@
 #include "CegoDistManager.h"
 #include "CegoDistCursor.h"
 #include "CegoField.h"
-#include "CegoPredDesc.h"
-#include "CegoCondDesc.h"
+#include "CegoPredicate.h"
+#include "CegoCondition.h"
 #include "CegoContentObject.h"
 #include "CegoOrderSpace.h"
 #include "CegoGroupSpace.h"
-#include "CegoHavingDesc.h"
 #include "CegoTerm.h"
 #include "CegoExpr.h"
 #include "CegoContentObject.h"
@@ -51,175 +51,129 @@ class CegoSelect {
     
 public:
 
-    enum SelectMode { PLAIN, AGGREGATION, GROUPING };
+    enum SelectMode { SIMPLE, PLAIN, AGGREGATION, GROUPING };
 	
-    CegoSelect(char* buf, CegoDistManager* pGTM, int tabSetId);
-    CegoSelect(Element* pSelectElement, CegoDistManager* pGTM);
+    CegoSelect(char* buf, CegoDistManager* pGTM, CegoProcBlock *pBlock, int tabSetId);
     CegoSelect(ListT<CegoContentObject*>& coList,
 	       ListT<CegoExpr*>& exprList, 
-	       CegoPredDesc* pPred,
+	       CegoPredicate* pPred,
 	       ListT<CegoAttrDesc*> *pGroupList,
-	       CegoHavingDesc* pHaving,
+	       CegoPredicate* pHavingPred,
 	       ListT<CegoExpr*> *pOrderList,
 	       ListT<CegoOrderNode::Ordering>* pOrderOptList,
 	       bool isDistinct,
 	       int rowLimit,
 	       CegoDistManager* pGTM);
-
     ~CegoSelect();
 
     void setTableManager(CegoDistManager* pGTM);
-
     void setUnionSelect(CegoSelect* pUnion);
-
     void cleanUp();
-    void prepare();
+    void prepare(bool graceful = false);
     void reset(bool doRelease);
-
     Element* getPlan();
-
     bool nextTuple(ListT<CegoField>& jfl);
-
+    bool isCacheEnabled() const;
     bool isCached() const;
     Chain getQueryId() const;
-    
     void getObjectList( SetT<CegoObject>& objList ) const;
-    
     void checkValidRef();
     void setParentJoinBuf(ListT<CegoField>** pParentJoinBuf = 0);
     void setProcBlock(CegoProcBlock *pBlock);
-
     int evalExtTableReferences(CegoContentObject* pCO, const ListT<CegoField>& fl);
-
-    CegoPredDesc* getPredicate();
+    CegoPredicate* getPredicate();
+    bool hasAliasReference() const;
     void getSchema(ListT<CegoField>& schema);
-    Chain toChain(const Chain& indent = Chain("")) const;
+    Chain toChain(int defTabSetId, const Chain& indent = Chain("")) const;
     Chain dbFormat(CegoDatabaseFormater *pForm) const;
-    
-    Element* toElement() const;
-    void fromElement(Element *pSelectElement, CegoDistManager* pGTM);
-    
     ListT<CegoAttrDesc*> getAttrRefList() const;
-    ListT<CegoField> getFieldList() const;
+    void getFieldList(ListT<CegoField>& fl, ListT<CegoField>** pFLA) const;
 
-    void encode(char *buf);
-    void decode(char *buf, CegoDistManager* pGTM, int tabSetId);
-    int getEncodingLength() const;
-    
-    void setViewCond(const CegoAttrCond& attrCond);
-    
+    void encode(char *buf, CegoProcBlock *pBlock);
+    void decode(char *buf, CegoDistManager* pGTM, CegoProcBlock *pBlock, int tabSetId);
+    int getEncodingLength(CegoProcBlock *pBlock) const;
+
+    bool setViewCond( CegoAttrCond& attrCond, ListT<CegoField> **pFLA);
     void setTabSetId(int tabSetId);
-
     int getExtRefCount() const;
-
     CegoSelect* clone( bool isAttrRef = false);
 
 private:
 
+    void evalSchema(bool graceful);	
     int evalTableReferences(CegoContentObject* pCO, const ListT<CegoField>& fl);
-
     bool isPrepared() const;
     void analyzeJoin();
-
     bool nextAggTuple(ListT<CegoField>& jfl);
     bool nextOrderedTuple(ListT<CegoField>& jfl);
     bool nextResultTuple(ListT<CegoField>& jfl, ListT<CegoField>& ofl);
-
     bool nextGroupedTuple(ListT<CegoField>& jfl);
     bool nextJoinTuple(ListT<CegoField>& jfl, ListT<CegoField>& ofl);
-    void createConjunctionList(CegoPredDesc* pPred);
+    void createConjunctionList(CegoPredicate* pPred);
     void consolidateConjunctionList();
-
     void evalReferences();
-
-    void filterRefs(ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);   
-    void filterFacRef(CegoFactor *pFac, ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);
-    void filterExprRef(CegoExpr *pExpr, ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);
-    void filterTermRef(CegoTerm *pTerm, ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);
-    void filterPredRef(CegoPredDesc *pP, ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);
-    void filterCondRef(CegoCondDesc* pC, ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);
-    void filterAttrRef(CegoAttrDesc* pAttrDesc, ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);
-
-    // the evalSelection finalizes the selection procedure by evaluating the expressions defined in 
-    // the selection part. Either these are plain attributes, expressions or aggregation functions 
-    // the result is given in the outList parameter
-    void evalSelection(ListT<CegoExpr*>& exprList, ListT<CegoField>** joinBuf, bool isAgg, ListT<CegoField>& ol);
-
+    void filterRefs(const ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);   
+    void filterFacRef(CegoFactor *pFac, const ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);
+    void filterExprRef(CegoExpr *pExpr, const ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);
+    void filterTermRef(CegoTerm *pTerm, const ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);
+    void filterPredRef(CegoPredicate *pP, const ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);
+    void filterCondRef(CegoCondition* pC, const ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);
+    void filterAttrRef(CegoAttrDesc* pAttrDesc, const ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);    
+    void evalSelection(ListT<CegoExpr*>& exprList, ListT<CegoField>** joinBuf, ListT<CegoField>& ol);
     void buildJoinTables();
     void buildJoinRefs();
     void buildJoinConditions();
-
-    void getPredObjectList(CegoPredDesc *pPred, ListT<CegoContentObject*>& objectList);
-    void getCondObjectList(CegoCondDesc* pC, ListT<CegoContentObject*>& objectList);
-
+    void getPredObjectList(CegoPredicate *pPred, ListT<CegoContentObject*>& objectList);
+    void getCondObjectList(CegoCondition * pC, ListT<CegoContentObject*>& objectList);
     void initAggregation();
     void aggregateTuple(ListT<CegoField>& aggTuple);
     void evalAggregation(ListT<CegoField>& resultList);
-
-    void makeOrder(); 
+    void prepareOrder();    
     bool matchFieldValue(const CegoFieldValue& fv1, const CegoFieldValue& fv2, CegoComparison comp);    
     void setAggregationValue(CegoExpr *pExpr, ListT<CegoField>& jfl);
-
     void addObject4CO(CegoContentObject *pCO, SetT<CegoObject>& objList) const;
-    
-    ListT<CegoPredDesc*> _viewConjunctionList;
-    ListT<CegoPredDesc*> _conjunctionList;
-
+    void setExtRefCount(int refCount);
+	
+    ListT<CegoPredicate*> _viewConjunctionList;
+    ListT<CegoPredicate*> _conjunctionList;
+    ListT<CegoPredicate*> _consolidationList;
     ListT<CegoExpr*> _exprList;
-    CegoPredDesc* _pPred;
-    ListT<CegoPredDesc*> _joinPredList;
-
-
+    CegoPredicate* _pPred;
+    ListT<CegoPredicate*> _joinPredList;
     CegoDistManager* _pGTM;
-
-    // ListT<CegoGroupAggregation> _groupList;
-    
     ListT<CegoContentObject*> _coList;
     ListT<CegoContentObject*> _joinList;
-
     ListT<CegoField> _joinFields;
-
     int _joinLevel;
     int _joinSize;
-
     int _dcount;
-
     ListT<CegoField>* _joinBuf[TABMNG_MAXJOINLEVEL];
     ListT<CegoField>**  _pParentJoinBuf;
-
     CegoDistCursor*_pTC[TABMNG_MAXJOINLEVEL];
-
     CegoAttrCond _attrCond[TABMNG_MAXJOINLEVEL];
     bool _attrCondFlag[TABMNG_MAXJOINLEVEL];
-    CegoPredDesc* _attrPred[TABMNG_MAXJOINLEVEL];
-    int _joinStep[TABMNG_MAXJOINLEVEL];
-
+    CegoPredicate* _attrPred[TABMNG_MAXJOINLEVEL];
     bool _firstTuple[TABMNG_MAXJOINLEVEL];
-
     CegoDatabaseManager *_pDBMng;
-
     int _aggregationCount;
     int _nextAid;
     int _tabSetId;
-
     ListT<CegoAttrDesc*>* _pGroupList;
+    CegoPredicate* _pHavingPred;
     ListT<CegoExpr*>* _pNativeOrderList;
     ListT<CegoExpr*>* _pOrderList;
     ListT<CegoOrderNode::Ordering>* _pNativeOrderOptList;
     ListT<CegoOrderNode::Ordering>* _pOrderOptList;
-    
     CegoProcBlock *_pBlock;
-
     enum SelectMode _selectMode;
     bool _aggDone;
     bool _orderingDone;
     bool _groupingDone;
-
+    bool _simpleDone;
     bool _isDistinct;
-
     ListT<CegoField> _orderedSchema;
     ListT<CegoField> _dfl;
-
+    ListT<CegoField> _evalSchema;
     bool _cacheEnabled;
     CegoQueryCache *_pCache;
     ListT<CegoField> _cacheSchema;
@@ -228,27 +182,21 @@ private:
     int _cacheRows;
     unsigned long _cacheEntrySize;
     bool _isCached;
+    Chain _viewCondId;
     Chain _queryId;
-        
     CegoOrderCursor *_pOrderCursor;
     CegoOrderSpace* _pOrderSpace;
-
     CegoGroupCursor *_pGroupCursor;
     CegoGroupSpace* _pGroupSpace;
-
-    CegoHavingDesc* _pHaving;
-
     bool _isPrepared;
     int _rowCount;
     int _rowLimit;
-
     bool _checkUnion;
     CegoSelect* _pUnionSelect;
-
     int _extRefCount;
-
+    NanoTimer _execTime;
+ 
     unsigned long _modId;
-
 };
 #endif
 
