@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -35,16 +35,14 @@ CegoSystemObject::~CegoSystemObject()
 
 void CegoSystemObject::createSystemSpace(int tabSetId)
 {
-
     ListT<CegoField> fl;
 
-    fl.Insert(CegoField(Chain(), Chain(), Chain(SYSTAB_TS_ATTR), DATETIME_TYPE, sizeof(int), CegoFieldValue(), false, 1));
-    fl.Insert(CegoField(Chain(), Chain(), Chain(SYSTAB_BUINFO_ATTR), VARCHAR_TYPE, 20, CegoFieldValue(), false, 2));
-    fl.Insert(CegoField(Chain(), Chain(), Chain(SYSTAB_BUMSG_ATTR), VARCHAR_TYPE, 50, CegoFieldValue(), true, 3));
+    fl.Insert(CegoField(Chain(), Chain(), Chain(SYSTAB_TS_ATTR), DATETIME_TYPE, sizeof(unsigned long long), 0, CegoFieldValue(), false, 1));
+    fl.Insert(CegoField(Chain(), Chain(), Chain(SYSTAB_BUINFO_ATTR), VARCHAR_TYPE, 20, 0, CegoFieldValue(), false, 2));
+    fl.Insert(CegoField(Chain(), Chain(), Chain(SYSTAB_BUMSG_ATTR), VARCHAR_TYPE, 50, 0, CegoFieldValue(), true, 3));
 
     CegoTableObject bustatTable(tabSetId, CegoObject::SYSTEM, Chain(SYSTAB_BUSTAT_ID), fl, Chain(SYSTAB_BUSTAT_ID));
     createTableObject(bustatTable);
-
 }
 
 bool CegoSystemObject::objectExists(int tabSetId, const Chain& objectName, CegoObject::ObjectType type)
@@ -83,34 +81,34 @@ void CegoSystemObject::getObject(int tabSetId, const Chain& objectName, CegoObje
 	if ( objectName == (Chain(SYSTAB_TABLE_ID)))
 	{	  	
 	    CegoFieldValue defVal;
-	    schema.Insert(CegoField(Chain(SYSTAB_TABLE_ID), Chain(SYSTAB_TABLE_ID), Chain(SYSTAB_NAME_ATTR),VARCHAR_TYPE, MAX_OBJNAME_LEN, defVal, false, 1)); 
-	    schema.Insert(CegoField(Chain(SYSTAB_TABLE_ID), Chain(SYSTAB_TABLE_ID), Chain(SYSTAB_SIZE_ATTR),INT_TYPE, sizeof(int), defVal, false, 2)); 
+	    schema.Insert(CegoField(Chain(SYSTAB_TABLE_ID), Chain(SYSTAB_TABLE_ID), Chain(SYSTAB_NAME_ATTR),VARCHAR_TYPE, MAX_OBJNAME_LEN, 0, defVal, false, 1)); 
+	    schema.Insert(CegoField(Chain(SYSTAB_TABLE_ID), Chain(SYSTAB_TABLE_ID), Chain(SYSTAB_SIZE_ATTR),INT_TYPE, sizeof(int), 0, defVal, false, 2)); 
 	}
 	else if ( objectName == (Chain(SYSTAB_PROC_ID)))
 	{
 	    CegoFieldValue defVal;
-	    schema.Insert(CegoField(Chain(SYSTAB_PROC_ID), Chain(SYSTAB_PROC_ID), Chain(SYSTAB_NAME_ATTR),VARCHAR_TYPE, MAX_OBJNAME_LEN, defVal, false, 1));
-	    schema.Insert(CegoField(Chain(SYSTAB_PROC_ID), Chain(SYSTAB_PROC_ID), Chain(SYSTAB_STATUS_ATTR), VARCHAR_TYPE, 15, defVal, false, 2));
+	    schema.Insert(CegoField(Chain(SYSTAB_PROC_ID), Chain(SYSTAB_PROC_ID), Chain(SYSTAB_NAME_ATTR),VARCHAR_TYPE, MAX_OBJNAME_LEN, 0, defVal, false, 1));
+	    schema.Insert(CegoField(Chain(SYSTAB_PROC_ID), Chain(SYSTAB_PROC_ID), Chain(SYSTAB_STATUS_ATTR), VARCHAR_TYPE, 15, 0, defVal, false, 2));
 	}
 	else if ( objectName == (Chain(SYSTAB_VIEW_ID)))
 	{
 	    CegoFieldValue defVal;
-	    schema.Insert(CegoField(Chain(SYSTAB_VIEW_ID), Chain(SYSTAB_VIEW_ID), Chain(SYSTAB_NAME_ATTR),VARCHAR_TYPE, MAX_OBJNAME_LEN, defVal, false, 1));
-	    schema.Insert(CegoField(Chain(SYSTAB_VIEW_ID), Chain(SYSTAB_VIEW_ID), Chain(SYSTAB_STATUS_ATTR), VARCHAR_TYPE, 15, defVal, false, 2));
+	    schema.Insert(CegoField(Chain(SYSTAB_VIEW_ID), Chain(SYSTAB_VIEW_ID), Chain(SYSTAB_NAME_ATTR),VARCHAR_TYPE, MAX_OBJNAME_LEN, 0, defVal, false, 1));
+	    schema.Insert(CegoField(Chain(SYSTAB_VIEW_ID), Chain(SYSTAB_VIEW_ID), Chain(SYSTAB_STATUS_ATTR), VARCHAR_TYPE, 15, 0, defVal, false, 2));
 	}
 	else if ( objectName == (Chain(SYSTAB_INDEX_ID)))
 	{
 	    CegoFieldValue defVal;
-	    schema.Insert(CegoField(Chain(SYSTAB_INDEX_ID), Chain(SYSTAB_INDEX_ID), Chain(SYSTAB_NAME_ATTR),VARCHAR_TYPE, MAX_OBJNAME_LEN, defVal, false, 1));
-	    schema.Insert(CegoField(Chain(SYSTAB_INDEX_ID), Chain(SYSTAB_INDEX_ID), Chain(SYSTAB_SIZE_ATTR),INT_TYPE, sizeof(int), defVal, false, 2));
-	    schema.Insert(CegoField(Chain(SYSTAB_INDEX_ID), Chain(SYSTAB_INDEX_ID), Chain(SYSTAB_STATUS_ATTR), VARCHAR_TYPE, 15, defVal, false, 3));
+	    schema.Insert(CegoField(Chain(SYSTAB_INDEX_ID), Chain(SYSTAB_INDEX_ID), Chain(SYSTAB_NAME_ATTR),VARCHAR_TYPE, MAX_OBJNAME_LEN, 0, defVal, false, 1));
+	    schema.Insert(CegoField(Chain(SYSTAB_INDEX_ID), Chain(SYSTAB_INDEX_ID), Chain(SYSTAB_SIZE_ATTR),INT_TYPE, sizeof(int), 0, defVal, false, 2));
+	    schema.Insert(CegoField(Chain(SYSTAB_INDEX_ID), Chain(SYSTAB_INDEX_ID), Chain(SYSTAB_STATUS_ATTR), VARCHAR_TYPE, 15, 0, defVal, false, 3));
 	}
 	else if ( objectName == (Chain(SYSTAB_BTREE_ID)))
 	{
 	    CegoFieldValue defVal;
-	    schema.Insert(CegoField(Chain(SYSTAB_BTREE_ID), Chain(SYSTAB_BTREE_ID), Chain(SYSTAB_NAME_ATTR),VARCHAR_TYPE, MAX_OBJNAME_LEN, defVal, false, 1));
-	    schema.Insert(CegoField(Chain(SYSTAB_BTREE_ID), Chain(SYSTAB_BTREE_ID), Chain(SYSTAB_SIZE_ATTR),INT_TYPE, sizeof(int), defVal, false, 2));
-	    schema.Insert(CegoField(Chain(SYSTAB_BTREE_ID), Chain(SYSTAB_BTREE_ID), Chain(SYSTAB_STATUS_ATTR), VARCHAR_TYPE, 15, defVal, false, 3));
+	    schema.Insert(CegoField(Chain(SYSTAB_BTREE_ID), Chain(SYSTAB_BTREE_ID), Chain(SYSTAB_NAME_ATTR),VARCHAR_TYPE, MAX_OBJNAME_LEN, 0, defVal, false, 1));
+	    schema.Insert(CegoField(Chain(SYSTAB_BTREE_ID), Chain(SYSTAB_BTREE_ID), Chain(SYSTAB_SIZE_ATTR),INT_TYPE, sizeof(int), 0, defVal, false, 2));
+	    schema.Insert(CegoField(Chain(SYSTAB_BTREE_ID), Chain(SYSTAB_BTREE_ID), Chain(SYSTAB_STATUS_ATTR), VARCHAR_TYPE, 15, 0, defVal, false, 3));
 	}
 	else if ( objectName == (Chain(SYSTAB_KEY_ID)))
 	{
@@ -128,7 +126,6 @@ void CegoSystemObject::getObject(int tabSetId, const Chain& objectName, CegoObje
 	pTO->encode(buf);
 	oe.decode(buf);
 	free(buf);
-
     }
     else
     {
@@ -152,7 +149,6 @@ void CegoSystemObject::getObjectList(int tabSetId, CegoObject::ObjectType type,
 	objList.Insert(Chain(SYSTAB_BTREE_ID));
 	objList.Insert(Chain(SYSTAB_KEY_ID));
 	CegoObjectManager::getObjectList(tabSetId, type, objList);
-
     }
     else
     {
