@@ -6,7 +6,7 @@
 //      
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -21,9 +21,6 @@
 // cego includes
 #include "CegoTableCursor.h"
 
-#include <string.h>
-#include <stdlib.h>
-
 CegoTableCursor::CegoTableCursor()
 {
     _pIC = 0;
@@ -69,31 +66,32 @@ CegoTableCursor::~CegoTableCursor()
 }
 
 CegoAttrCond::IndexMatch CegoTableCursor::setup( const CegoAttrCond& attrCond )
-{
-
-
+{    
     if ( attrCond.numComp() == 0 )
     {
 	_idxSetup = false;
-	return CegoAttrCond::INAPP;
+	_idxMatch = CegoAttrCond::INAPP;
+	return _idxMatch;
     }
-
-    if ( _idxSetup == false || attrCond.diff(_attrCond) == false )
+   
+    if ( _idxSetup == false ) 
     {
 #ifdef CGDEBUG
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Setting up tablecursor for " ) + _tableName + Chain (" with ") + attrCond.toChain());
 #endif
-
+	
 	_idxSetup = true;
 
 	if ( _idxCached == false )
 	{
-
 	    ListT<CegoKeyObject> keyList;
 	    ListT<CegoCheckObject> checkList;
+	    ListT<CegoTriggerObject> triggerList;
+	    ListT<CegoAliasObject> aliasList;
 	    int numInvalid;
-    
-	    _pTM->getObjectListByTable(_tabSetId, _tableName, _idxList, _btreeList, keyList, checkList, numInvalid);
+
+	    _pTM->getObjectListByTable(_tabSetId, _tableName, _idxList, _btreeList, keyList, checkList, triggerList, aliasList, numInvalid);
+
 	    _idxCached = true;
 	}
     
@@ -110,7 +108,6 @@ CegoAttrCond::IndexMatch CegoTableCursor::setup( const CegoAttrCond& attrCond )
 	{	    
 	    if ( pIOE->isValid())
 	    {
-
 #ifdef CGDEBUG
 		_pDBMng->log(_modId, Logger::DEBUG, Chain("Checking index ") + pIOE->getName());
 #endif
@@ -119,25 +116,21 @@ CegoAttrCond::IndexMatch CegoTableCursor::setup( const CegoAttrCond& attrCond )
 
 		if ( indexMatch == CegoAttrCond::FULL || indexMatch == CegoAttrCond::PART )
 		{
-
 		    CegoAttrCond checkCond = attrCond.getIndexCond(pIOE->getSchema());
 
 		    if ( checkCond.getStrength() > strength )
 		    {
-
 			strength = checkCond.getStrength();
 #ifdef CGDEBUG
 			_pDBMng->log(_modId, Logger::DEBUG, Chain("Strength ") + Chain(strength) + Chain(" tops for ") + checkCond.toChain());
-#endif
-
+#endif			
 			_attrCond = checkCond; 
 			_attrCond.setIdxSchema(pIOE->getSchema());
 			
 			_idxName = pIOE->getName();
 #ifdef CGDEBUG
 			_pDBMng->log(_modId, Logger::DEBUG, Chain("Using index ") + _idxName );
-#endif
-			
+#endif			
 			_idxSchema = pIOE->getSchema();
 			_type = pIOE->getType();
 			_idxMatch = indexMatch;
@@ -160,8 +153,7 @@ CegoAttrCond::IndexMatch CegoTableCursor::setup( const CegoAttrCond& attrCond )
 	    CegoBTreeObject *pBTO = _btreeList.First();
 		    	    
 	    while ( pBTO )
-	    {	    
-		
+	    {
 #ifdef CGDEBUG
 		_pDBMng->log(_modId, Logger::DEBUG, Chain("Checking btree ") + pBTO->getName());
 #endif
@@ -171,26 +163,22 @@ CegoAttrCond::IndexMatch CegoTableCursor::setup( const CegoAttrCond& attrCond )
 		    
 		    if ( indexMatch == CegoAttrCond::FULL || indexMatch == CegoAttrCond::PART )
 		    {
-			
 			CegoAttrCond checkCond = attrCond.getIndexCond(pBTO->getSchema());
 			
 			if ( checkCond.getStrength() > strength )
-			{
-			    
+			{			    
 			    strength = checkCond.getStrength();
 			    relevance = pBTO->getRelevance();
 #ifdef CGDEBUG
 			    _pDBMng->log(_modId, Logger::DEBUG, Chain("Strength ") + Chain(strength) + Chain(" tops for ") + checkCond.toChain());
 #endif
-			    
 			    _attrCond = checkCond; 
 			    _attrCond.setIdxSchema(pBTO->getSchema());
 			    
 			    _idxName = pBTO->getName();
 #ifdef CGDEBUG
 			    _pDBMng->log(_modId, Logger::DEBUG, Chain("Using btree ") + _idxName );
-#endif
-			    
+#endif			    
 			    _idxSchema = pBTO->getSchema();
 			    _type = pBTO->getType();
 			    _idxMatch = indexMatch;
@@ -206,7 +194,6 @@ CegoAttrCond::IndexMatch CegoTableCursor::setup( const CegoAttrCond& attrCond )
 #ifdef CGDEBUG
 				_pDBMng->log(_modId, Logger::DEBUG, Chain("Relevance ") + Chain(pBTO->getRelevance()) + Chain(" tops ") + Chain(relevance));
 #endif
-				
 				_attrCond = checkCond; 
 				_attrCond.setIdxSchema(pBTO->getSchema());
 				
@@ -217,8 +204,7 @@ CegoAttrCond::IndexMatch CegoTableCursor::setup( const CegoAttrCond& attrCond )
 				
 				_idxSchema = pBTO->getSchema();
 				_type = pBTO->getType();
-				_idxMatch = indexMatch;
-				
+				_idxMatch = indexMatch;				
 			    }
 			}
 			else
@@ -236,12 +222,12 @@ CegoAttrCond::IndexMatch CegoTableCursor::setup( const CegoAttrCond& attrCond )
     else
     {
 #ifdef CGDEBUG
-	_pDBMng->log(_modId, Logger::DEBUG, Chain("Tablecursor already set up "));
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Tablecursor already set up, just update ..."));
 #endif
+	_attrCond.update(attrCond);
     }
 
-#ifdef CGDEBUG    
-
+#ifdef CGDEBUG
     if ( _idxMatch == CegoAttrCond::FULL )
     {
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Index classified as full qualified"));
@@ -251,7 +237,9 @@ CegoAttrCond::IndexMatch CegoTableCursor::setup( const CegoAttrCond& attrCond )
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Index classified as part qualified"));
     }
     else
+    {
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Index classified as not qualified"));
+    }
 #endif
 
     return _idxMatch;
@@ -263,7 +251,7 @@ void CegoTableCursor::getIdxSchema(ListT<CegoField>& idxSchema)
 }
 
 bool CegoTableCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
-{ 
+{		 
     if ( _pTM->getIsolationLevel() == CegoTableManager::READ_UNCOMMITTED )
 	_readUncommitted = true;
     else
@@ -297,8 +285,9 @@ bool CegoTableCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 	    {
 		_pBTC = new CegoBTreeCursor(_pTM, _tabSetId, _idxName, _type, &_attrCond, _ignoreTouched, _readUncommitted);
 	    }
-	    
-	    return _pBTC->getFirst(fl, dp);
+
+	    return _pBTC->getFirst(fl, dp);	    
+
 	} 
 	else
 	{
@@ -307,7 +296,6 @@ bool CegoTableCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
     }
     else
     {
-
 	if ( _pC )
 	{
 	    _pC->reset();
@@ -321,10 +309,7 @@ bool CegoTableCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 	char* pc = (char*)_pC->getFirst(len, dp);
 
 	if ( pc && len > 0 )
-	{
-
-	    bool useEntry=true;
-	    
+	{	    
 	    unsigned long long tid;
 	    unsigned long long tastep;
 	    CegoTupleState ts;
@@ -338,10 +323,9 @@ bool CegoTableCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 	    {
 		if ( _ignoreTouched == true )
 		{
-		    if ( ts == INSERTED 
+		    if ( ts == CegoTupleState::INSERTED 
 			 && tid == _pTM->getTID(_tabSetId) 
-			 && tastep < _pTM->getTAStep(_tabSetId) ) 
-
+			 && tastep < _pTM->getTAStep(_tabSetId) )
 		    {
 			_qh.decodeFVL(fl, tp, tlen);
 			return true;
@@ -354,10 +338,10 @@ bool CegoTableCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 		else
 		{
 		    if ( ( _readUncommitted == true 
-			   && ts == INSERTED )
+			   && ts == CegoTupleState::INSERTED )
 			 || ( _readUncommitted == false 
-			      && (( ts == INSERTED && tid == _pTM->getTID(_tabSetId)) 
-				  || ( ts == DELETED && tid != _pTM->getTID(_tabSetId)))))
+			      && (( ts == CegoTupleState::INSERTED && tid == _pTM->getTID(_tabSetId)) 
+				  || ( ts == CegoTupleState::DELETED && tid != _pTM->getTID(_tabSetId)))))
 		    {
 			_qh.decodeFVL(fl, tp, tlen);
 			return true;
@@ -370,7 +354,6 @@ bool CegoTableCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 	    }
 
 	    _qh.decodeFVL(fl, tp, tlen);
-
 	    return true;
 	}
 	return false;
@@ -379,22 +362,27 @@ bool CegoTableCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 
 bool CegoTableCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 {
-
     if ( _idxMatch == CegoAttrCond::FULL || _idxMatch == CegoAttrCond::PART  )
     {
-
 	if ( _type == CegoObject::AVLTREE 
 	     || _type == CegoObject::UAVLTREE 
 	     || _type == CegoObject::PAVLTREE )
 	{
-
+	    if ( _pIC == 0 )
+	    {
+		throw Exception(EXLOC, "Invalid avl cursor");
+	    }
 	    return _pIC->getNext(fl, dp);
 	}
 	else if ( _type == CegoObject::BTREE 
 	     || _type == CegoObject::UBTREE
 	     || _type == CegoObject::PBTREE )
 	{
-	    return _pBTC->getNext(fl, dp);
+	    if ( _pBTC == 0 )
+	    {
+		throw Exception(EXLOC, "Invalid btree cursor");
+	    }
+	    return  _pBTC->getNext(fl, dp);
 	}
 	else
 	{
@@ -408,15 +396,14 @@ bool CegoTableCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 	    throw Exception(EXLOC, "Invalid ObjectCursor");
 	}
 	
-	do {
-	    
+	do
+	{	    
 	    int len;
 
 	    char* pc = (char*)_pC->getNext(len, dp);
 
 	    if ( pc && len > 0 )
-	    {
-	       	      
+	    {	       	      
 		bool useEntry=true;
 		
 		unsigned long long tid;
@@ -424,17 +411,15 @@ bool CegoTableCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 		CegoTupleState ts;
 
 		int toff = _qh.decodeTupleHeader(tid, tastep, ts, pc);
-
+		
 		char* tp = pc + toff;
 		int tlen = len - toff;
 
-		if (tid != 0 )
-		{
-		    
+		if ( tid != 0 )
+		{	    
 		    if ( _ignoreTouched == true )
-		    {
-			
-			if ( ts == INSERTED 
+		    {			
+			if ( ts == CegoTupleState::INSERTED 
 			     && tid == _pTM->getTID(_tabSetId) 
 			     && tastep < _pTM->getTAStep(_tabSetId) )
 			{
@@ -447,16 +432,14 @@ bool CegoTableCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 			}
 		    }
 		    else
-		    {
-			
+		    {	
 			if ( ( _readUncommitted == true 
-			       && ts == INSERTED )
+			       && ts == CegoTupleState::INSERTED )
 			     || ( _readUncommitted == false 
-				  && (( ts == INSERTED && tid == _pTM->getTID(_tabSetId)) 
-				      || ( ts == DELETED && tid != _pTM->getTID(_tabSetId)))))
-			    
+				  && (( ts == CegoTupleState::INSERTED && tid == _pTM->getTID(_tabSetId)) 
+				      || ( ts == CegoTupleState::DELETED && tid != _pTM->getTID(_tabSetId)))))			    
 			{
-			    _qh.decodeFVL(fl, tp, tlen);			    
+			    _qh.decodeFVL(fl, tp, tlen);
 			    return true;   
 			}
 			else 
@@ -467,8 +450,11 @@ bool CegoTableCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 		}
 		
 		if ( useEntry )
-		{	       
-		    _qh.decodeFVL(fl, tp, tlen);
+		{
+		    
+		    if ( fl.Size() > 0 )
+			_qh.decodeFVL(fl, tp, tlen);
+		    
 		    return true;
 		}
 	    }
@@ -504,10 +490,4 @@ void CegoTableCursor::abort()
     {
 	_pC->abort();
     }
-
-    /* still not clear, if we have to reset this too
-    _idxMatch = CegoAttrCond::INAPP;
-    _idxSetup = false;
-    _idxCached = false;
-    */ 
 }
