@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -20,10 +20,9 @@
 
 // cego includes
 #include "CegoTableObject.h"
-#include "CegoXMLHelper.h"
 #include "CegoTypeConverter.h"
-#include "CegoXMLdef.h"
 
+// needed for memcpy
 #include <string.h>
 #include <stdlib.h>
 
@@ -32,19 +31,10 @@ CegoTableObject::CegoTableObject()
     _subCOList.Insert(this);
 }
 
-CegoTableObject::CegoTableObject(Element *pTO)
-{
-    putElement(pTO);
-    _subCOList.Insert(this);
-}
-
 CegoTableObject::CegoTableObject(const CegoTableObject& oe) : CegoContentObject(oe)
 {
-    _fileId = oe._fileId;
     _pageId = oe._pageId;
-    _lastFileId = oe._lastFileId;
     _lastPageId = oe._lastPageId;
-    _pLastFileId = oe._pLastFileId;
     _pLastPageId = oe._pLastPageId;
     _maxFid = oe._maxFid;
     _subCOList.Insert(this);
@@ -52,24 +42,18 @@ CegoTableObject::CegoTableObject(const CegoTableObject& oe) : CegoContentObject(
 
 CegoTableObject::CegoTableObject(int tabSetId, CegoObject::ObjectType type, const Chain& objName ) : CegoContentObject(tabSetId, type, objName)
 {
-    _fileId = 0;
     _pageId = 0;
-    _lastFileId = 0;
     _lastPageId = 0;
     _pLastPageId=0;
-    _pLastFileId=0;
     _maxFid = 0;
     _subCOList.Insert(this);
 }
 
 CegoTableObject::CegoTableObject(int tabSetId, CegoObject::ObjectType type,  const Chain& objName, const ListT<CegoField>& schema, const Chain& tabName) : CegoContentObject(tabSetId, type, objName, tabName, schema)
 {
-    _fileId = 0;
     _pageId = 0;    
-    _lastFileId = 0;
     _lastPageId = 0;
     _pLastPageId=0;
-    _pLastFileId=0;
     _maxFid = 0;
     _subCOList.Insert(this);
 }
@@ -83,84 +67,41 @@ void CegoTableObject::setSchema(const ListT<CegoField>& schema)
     _schema = schema;
 }
 
-void CegoTableObject::setDataFileId(int fileId)
-{
-    _fileId = fileId;
-}
-
-int CegoTableObject::getDataFileId() const
-{
-    return _fileId;
-}
-
-void CegoTableObject::setDataPageId(int pageId)
+void CegoTableObject::setDataPageId(PageIdType pageId)
 {
     _pageId = pageId;
 }
 
-int CegoTableObject::getDataPageId() const
+PageIdType CegoTableObject::getDataPageId() const
 {
     return _pageId;
 }
 
-void CegoTableObject::setLastDataFileId(int fileId)
-{
-    _lastFileId = fileId;
-}
-
-int CegoTableObject::getLastDataFileId() const
-{
-    return _lastFileId;
-}
-
-void CegoTableObject::setLastDataPageId(int pageId)
+void CegoTableObject::setLastDataPageId(PageIdType pageId)
 {
     _lastPageId = pageId;
 }
 
-int CegoTableObject::getLastDataPageId() const
+PageIdType CegoTableObject::getLastDataPageId() const
 {
     return _lastPageId;
 }
 
-void CegoTableObject::setRefLastDataFileId(int fileId)
-{
-    _lastFileId = fileId;
-    if (_pLastFileId)
-    {
-	memcpy(_pLastFileId, &fileId, sizeof(int));
-    }
-}
-
-int CegoTableObject::getRefLastDataFileId() const
-{
-    if (_pLastFileId)
-    {
-	int fileId;
-	memcpy(&fileId, _pLastFileId, sizeof(int));
-	return fileId;
-    }
-    else
-    {
-	return _lastFileId;
-    }
-}
-
-void CegoTableObject::setRefLastDataPageId(int pageId)
+void CegoTableObject::setRefLastDataPageId(PageIdType pageId)
 {
     _lastPageId = pageId;
     if (_pLastPageId)
     {
-	memcpy(_pLastPageId, &pageId, sizeof(int));
+	memcpy(_pLastPageId, &pageId, sizeof(PageIdType));
     }
 }
 
-int CegoTableObject::getRefLastDataPageId() const
+PageIdType CegoTableObject::getRefLastDataPageId() const
 {
     if (_pLastPageId)
     {
-	int pageId;
-	memcpy(&pageId, _pLastPageId, sizeof(int));
+	PageIdType pageId;
+	memcpy(&pageId, _pLastPageId, sizeof(PageIdType));
 	return pageId;
     }
     else
@@ -180,22 +121,15 @@ int CegoTableObject::getMaxFid() const
 }
 
 int CegoTableObject::getEntrySize() const
-{
-    
+{    
     int entrySize = CegoContentObject::getBaseContentSize();
-
-    entrySize = entrySize + sizeof(int); // fileId
-    entrySize = entrySize + sizeof(int); // pageId
-        
-    entrySize = entrySize + sizeof(int); // last fileId
-    entrySize = entrySize + sizeof(int); // last pageId
-
-    entrySize = entrySize + sizeof(int); // maxFid
-    
+    entrySize = entrySize + sizeof(PageIdType); // pageId       
+    entrySize = entrySize + sizeof(PageIdType); // last pageId
+    entrySize = entrySize + sizeof(int); // maxFid    
     return entrySize;
 }
 
-void CegoTableObject::encode(char *buf)
+void CegoTableObject::encode(char *buf) const
 {
     char* bufPtr = buf;
 
@@ -204,23 +138,14 @@ void CegoTableObject::encode(char *buf)
     CegoContentObject::encodeBaseContent(bufPtr, entrySize);
     bufPtr += CegoContentObject::getBaseContentSize();
 
-    memcpy(bufPtr, &_fileId, sizeof(int));
-    bufPtr = bufPtr + sizeof(int);
-    
-    memcpy(bufPtr, &_pageId, sizeof(int));
-    bufPtr = bufPtr + sizeof(int);
-    
-    _pLastFileId = (char*)bufPtr;
-    memcpy(bufPtr, &_lastFileId, sizeof(int));
-    bufPtr = bufPtr + sizeof(int);
-    
-    _pLastPageId = (char*)bufPtr;
-    memcpy(bufPtr, &_lastPageId, sizeof(int));
-    bufPtr = bufPtr + sizeof(int);
+    memcpy(bufPtr, &_pageId, sizeof(PageIdType));
+    bufPtr = bufPtr + sizeof(PageIdType);
+
+    memcpy(bufPtr, &_lastPageId, sizeof(PageIdType));
+    bufPtr = bufPtr + sizeof(PageIdType);
 
     memcpy(bufPtr, &_maxFid, sizeof(int));
     bufPtr = bufPtr + sizeof(int);
-
 }
 
 void CegoTableObject::decode(char *buf)
@@ -232,45 +157,29 @@ void CegoTableObject::decode(char *buf)
     CegoContentObject::decodeBaseContent(bufPtr, size);
     bufPtr += CegoContentObject::getBaseContentSize();
     
-    memcpy(&_fileId, bufPtr, sizeof(int));
-    bufPtr = bufPtr + sizeof(int);
+    memcpy(&_pageId, bufPtr, sizeof(PageIdType));
+    bufPtr = bufPtr + sizeof(PageIdType);
 
-    memcpy(&_pageId, bufPtr, sizeof(int));
-    bufPtr = bufPtr + sizeof(int);
-    
-    _pLastFileId = (char*)bufPtr;
-    memcpy(&_lastFileId, bufPtr, sizeof(int));
-    bufPtr = bufPtr + sizeof(int);
-    
     _pLastPageId = (char*)bufPtr;
-    memcpy(&_lastPageId, bufPtr, sizeof(int));
-    bufPtr = bufPtr + sizeof(int);
+    memcpy(&_lastPageId, bufPtr, sizeof(PageIdType));
+    bufPtr = bufPtr + sizeof(PageIdType);
 
     memcpy(&_maxFid, bufPtr, sizeof(int));
     bufPtr = bufPtr + sizeof(int);
-    
 }
 
-
 bool CegoTableObject::isValid()
 {
-    return  _fileId != 0 || _pageId != 0;
+    return _pageId != 0;
 }
 
 CegoTableObject& CegoTableObject::operator = ( const CegoTableObject& oe) 
 {
-
     CegoContentObject::operator=(oe);
 
-    _fileId = oe._fileId;
     _pageId = oe._pageId;
-
-    _lastFileId = oe._fileId;
     _lastPageId = oe._pageId;
-
-    _pLastFileId = oe._pLastFileId;
-    _pLastPageId = oe._pLastPageId;
-    
+    _pLastPageId = oe._pLastPageId;    
     _maxFid = oe._maxFid;
 
     return (*this);
@@ -281,146 +190,21 @@ bool CegoTableObject::operator == ( const CegoTableObject& oe)
     return CegoContentObject::operator==(oe);
 }
 
-Chain CegoTableObject::getId() const
+Chain CegoTableObject::getId(CegoProcBlock *pBlock) const
 {
     return getTabName() + Chain("@") + getTableSet() + Chain("@") + getName();
 }
 
-Chain CegoTableObject::toChain() const
+Chain CegoTableObject::toChain(int defTabSetId) const
 {
-    return getTabName() + Chain("@") + getTableSet() + Chain(" ") + getName();
-}
-
-Element* CegoTableObject::getElement() const
-{
-    Element* pRoot = new Element(XML_OBJ_ELEMENT);
-
-    pRoot->setAttribute(XML_TSID_ATTR, Chain(getTabSetId()));
-    switch ( getType() )
-    {
-    case CegoObject::SYSTEM:
-	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_SYSOBJ_VALUE);	
-	break;
-    case CegoObject::TABLE:
-	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_TABOBJ_VALUE);	
-	break;
-    case CegoObject::PAVLTREE:
-	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_PIXOBJ_VALUE);	
-	break;
-    case CegoObject::UAVLTREE:
-	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_UIXOBJ_VALUE);	
-	break;
-    case CegoObject::AVLTREE:
-	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_IDXOBJ_VALUE);	
-	break;
-    case CegoObject::VIEW:
-	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_VIEWOBJ_VALUE);	
-	break;
-    case CegoObject::FKEY:
-	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_FKEYOBJ_VALUE);	
-	break;
-    case CegoObject::PROCEDURE:
-	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_PROCOBJ_VALUE);	
-	break;
-    case CegoObject::JOIN:
-	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_JOINOBJ_VALUE);	
-	break;
-    default:
-	break;
-    }
-
-    pRoot->setAttribute(XML_OBJNAME_ATTR, getName());
-
-    CegoField *pF = _schema.First();
-    while ( pF ) 
-    {
-	Element *pColElement = new Element(XML_SCHEMA_ELEMENT);
-
-	CegoXMLHelper xh;
-	xh.setColInfo(pColElement, pF);
-
-	pRoot->addContent(pColElement);
-	pF = _schema.Next();
-    }
-
-    return pRoot;
-}
-
-void CegoTableObject::putElement(Element* pElement)
-{
-    Element *pRoot = pElement;
-    
-    if ( pRoot )
-    {
-	
-	Chain objName = pRoot->getAttributeValue(XML_OBJNAME_ATTR); 
-	int tabSetId = pRoot->getAttributeValue(XML_TSID_ATTR).asInteger();
-	
-	setName(objName);
-	setTabName(objName);
-	setTabSetId(tabSetId);
-
-	Chain objType = pRoot->getAttributeValue(XML_OBJTYPE_ATTR);
-	if ( objType == Chain(XML_SYSOBJ_VALUE))
-	    setType(CegoObject::SYSTEM);
-	else if ( objType == Chain(XML_TABOBJ_VALUE))
-	    setType(CegoObject::TABLE);
-	else if ( objType == Chain(XML_PIXOBJ_VALUE))
-	    setType(CegoObject::PAVLTREE);
-	else if ( objType == Chain(XML_UIXOBJ_VALUE))
-	    setType(CegoObject::UAVLTREE);
-	else if ( objType == Chain(XML_IDXOBJ_VALUE))
-	    setType(CegoObject::AVLTREE);
-	else if ( objType == Chain(XML_VIEWOBJ_VALUE))
-	    setType(CegoObject::VIEW);
-	else if ( objType == Chain(XML_FKEYOBJ_VALUE))
-	    setType(CegoObject::FKEY);
-	else if ( objType == Chain(XML_PROCOBJ_VALUE))
-	    setType(CegoObject::PROCEDURE);
-	
-	ListT<Element*> colList = pRoot->getChildren(XML_SCHEMA_ELEMENT);
-	
-	ListT<CegoField> fl;
-	Element **pCol = colList.First();
-	while ( pCol ) 
-	{
-	    Chain colName = (*pCol)->getAttributeValue(XML_COLNAME_ATTR);
-	    Chain colType = (*pCol)->getAttributeValue(XML_COLTYPE_ATTR);
-	    Chain colSize = (*pCol)->getAttributeValue(XML_COLSIZE_ATTR);
-
-	    Chain colNullable = (*pCol)->getAttributeValue(XML_COLNULLABLE_ATTR);
-	    Chain colDefValue = (*pCol)->getAttributeValue(XML_COLDEFVALUE_ATTR);
-
-
-	    bool isNullable;
-	    if ( colNullable == Chain(XML_TRUE_VALUE) ) 
-		isNullable = true;
-	    else
-		isNullable = false;
-
-	    
-	    CegoTypeConverter tc;
-	    CegoDataType dataType = tc.getTypeId(colType);
-
-	    CegoFieldValue defValue;
-	    if ( colDefValue != Chain("") )
-	    {
-		defValue = CegoFieldValue(dataType, colDefValue);
-	    }
-
-	    CegoField f(CegoField(objName, objName, colName, dataType, colSize.asInteger(), defValue, isNullable));		
-	    fl.Insert(f);
-
-	    pCol = colList.Next();
-	}
-	
-	setSchema(fl);
-    }
+    if ( defTabSetId == getTabSetId() && defTabSetId != 0 )
+	return getTabName() + Chain(" ") + getName();
+    else
+	return getTabName() + Chain("@") + getTableSet() + Chain(" ") + getName();
 }
 
 Chain CegoTableObject::getFormatted() const
 {
-
     Chain s;
 	
     int maxAttrLen = 12;
@@ -443,6 +227,9 @@ Chain CegoTableObject::getFormatted() const
     s += Chain("| ObjectType : ");
     switch ( getType() )
     {
+    case CegoObject::SYSTEM:
+	s += Chain("system table    "); 
+	break;
     case CegoObject::TABLE:
 	s += Chain("table           "); 
 	break;
@@ -456,9 +243,18 @@ Chain CegoTableObject::getFormatted() const
 	s += Chain("primary avltree ");
 	break;
     case CegoObject::VIEW:
-	s += Chain("view            ");
-	break;
-    default:
+    case CegoObject::RBSEG:
+    case CegoObject::CHECK:
+    case CegoObject::BTREE:
+    case CegoObject::PBTREE:
+    case CegoObject::UBTREE:
+    case CegoObject::ALIAS:
+    case CegoObject::TRIGGER:
+    case CegoObject::FKEY:
+    case CegoObject::PROCEDURE:
+    case CegoObject::JOIN:
+    case CegoObject::UNDEFINED:	
+	throw Exception(EXLOC, "Invalid object type");
 	break;
     }
 
@@ -470,12 +266,10 @@ Chain CegoTableObject::getFormatted() const
 	
     pF = _schema.First();
     while (pF)
-    {
-	
+    {	
 	int num = maxAttrLen - pF->getAttrName().length() ;
 	s += Chain("| ") + pF->getAttrName() + fill(" ", num) + Chain(" |");
-	    
-	    
+	    	    
 	switch (pF->getType())
 	{
 	case INT_TYPE:
@@ -526,12 +320,6 @@ Chain CegoTableObject::getFormatted() const
 	    s += Chain("  decimal[") + l + Chain("]") + fill(" ", 5 - l.length()) + Chain("|");
 	    break;
 	}
-	case FIXED_TYPE:
-	{
-	    Chain l(pF->getLength());
-	    s += Chain("  fixed[") + l + Chain("]") + fill(" ", 7 - l.length()) + Chain("|");
-	    break;
-	}
 	case FLOAT_TYPE:
 	{
 	    s += Chain(" float         |");
@@ -552,7 +340,8 @@ Chain CegoTableObject::getFormatted() const
 	    s += Chain(" clob          |");
 	    break;
 	}
-	default:
+	case NULL_TYPE:
+	case PAGEID_TYPE:
 	    break;
 	}
 	
@@ -572,13 +361,11 @@ Chain CegoTableObject::getFormatted() const
     s += Chain("+") + fill("-", maxAttrLen + 1) + Chain("+---------------+") + fill("-", maxDefLen + 1) + Chain("+----------+\n");
 
     return s;
-
 }
 
 Chain CegoTableObject::fill(const Chain& s, int num) const
 {
-    
-    Chain fs = Chain("");
+    Chain fs = Chain();
     while (num > 0)
     {
 	fs = fs + s;
