@@ -1,12 +1,12 @@
 ///////////////////////////////////////////////////////////////////////////////
 //                                                         
 // CegoTerm.cc
-// ------------
+// -----------
 // Cego procedure term class implementation
 //
 // Design and Implementation by Bjoern Lemke
 //               
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -31,20 +31,12 @@ CegoTerm::CegoTerm()
     _pFactor=0;
 }
 
-CegoTerm::CegoTerm(char* buf, CegoDistManager *pGTM, int tabSetId)
+CegoTerm::CegoTerm(char* buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId)
 {
     _pTerm = 0;
     _pFactor = 0;
 
-    decode(buf, pGTM, tabSetId);
-}
-
-CegoTerm::CegoTerm(Element* pTermElement, CegoDistManager *pGTM)
-{
-    _pTerm = 0;
-    _pFactor = 0;
-
-    fromElement(pTermElement, pGTM);
+    decode(buf, pGTM, pBlock, tabSetId);
 }
 
 CegoTerm::CegoTerm(CegoTerm *pTerm, CegoFactor *pFactor, TermType termType)
@@ -69,6 +61,43 @@ CegoTerm::~CegoTerm()
 	delete _pFactor;
 }
 
+void CegoTerm::cleanUp()
+{
+    if ( _pTerm )
+	_pTerm->cleanUp();
+    if ( _pFactor )
+	_pFactor->cleanUp();
+}
+
+void CegoTerm::setTabSetId(int tabSetId)
+{
+    if ( _pTerm )
+	_pTerm->setTabSetId(tabSetId);
+    if ( _pFactor )
+	_pFactor->setTabSetId(tabSetId);
+}
+
+
+void CegoTerm::clearAttrCache()
+{
+    if ( _pTerm )
+	_pTerm->clearAttrCache();
+    if ( _pFactor )
+	_pFactor->clearAttrCache();
+}
+
+bool CegoTerm::checkConst() const
+{
+    if ( _termType == FACTOR)
+    {
+	return _pFactor->checkConst();
+    }
+    else
+    {
+	return _pTerm->checkConst() && _pFactor->checkConst();
+    }
+}
+
 bool CegoTerm::checkVar(Chain& var) const
 {
     if ( _pFactor )
@@ -87,6 +116,15 @@ CegoAttrDesc* CegoTerm::checkAttr() const
     return 0;
 }
 
+bool CegoTerm::checkLob(CegoFieldValue& fv, CegoProcBlock *pBlock)
+{
+    if ( _pFactor && _termType == FACTOR )
+    {
+	return _pFactor->checkLob(fv, pBlock);
+    }
+    return false;
+}
+
 CegoTerm::TermType CegoTerm::getType() const
 {
     return _termType;
@@ -102,30 +140,6 @@ CegoTerm* CegoTerm::getTerm()
     return _pTerm;
 }
 
-void CegoTerm::setFieldListArray(ListT<CegoField> **pFLA)
-{
-    if ( _pTerm )
-	_pTerm->setFieldListArray(pFLA);
-    if ( _pFactor )
-	_pFactor->setFieldListArray(pFLA);
-}
-
-void CegoTerm::setBlock(CegoProcBlock *pBlock)
-{
-    if ( _pTerm )
-	_pTerm->setBlock(pBlock);
-    if ( _pFactor )
-	_pFactor->setBlock(pBlock);
-}
-
-void CegoTerm::clearAttrCache()
-{
-    if ( _pTerm )
-	_pTerm->clearAttrCache();
-    if ( _pFactor )
-	_pFactor->clearAttrCache();
-}
-
 void CegoTerm::getFunctionList(SetT<CegoObject>& funcList)
 {
     if ( _pTerm )
@@ -147,25 +161,23 @@ void CegoTerm::getSelectQueryList(ListT<CegoSelect*>& queryList)
     }
 }
 
-ListT<CegoField> CegoTerm::getFieldList() const
+void CegoTerm::getFieldList(ListT<CegoField>& fl, ListT<CegoField> **pFLA) const
 {
-    ListT<CegoField> fl;
-
     switch ( _termType )
     {
     case MUL:
     case DIV:
     {
-	fl = _pTerm->getFieldList() + _pFactor->getFieldList();
+	_pTerm->getFieldList(fl, pFLA);
+	_pFactor->getFieldList(fl, pFLA);
 	break;
     }
     case FACTOR:
     {
-	fl = _pFactor->getFieldList();
+	_pFactor->getFieldList(fl, pFLA);
 	break;
     }
     }
-    return fl;
 }
 
 ListT<CegoAttrDesc*> CegoTerm::getAttrRefList() const
@@ -208,41 +220,108 @@ int CegoTerm::evalReferences(CegoContentObject *pCO, const ListT<CegoField>& fl)
 	break;
     }
     }
-
     return refCount;
-
 }
 
-CegoField CegoTerm::evalField() const
+CegoField CegoTerm::evalField(const ListT<CegoField>& fl, bool graceful) const
 {
     CegoField f;
 
     switch ( _termType )
     {
     case MUL:
+    {
+	CegoField f1 =  _pTerm->evalField(fl, graceful);
+	CegoField f2 =  _pFactor->evalField(fl, graceful);
+
+	// the type conversion refers to method : CegoFieldValue operator * ( const CegoFieldValue& fv1, const CegoFieldValue& fv2 )
+	CegoDataType dt;
+	int len;
+	int dim;
+	
+	if ( f1.getType() == DECIMAL_TYPE )
+	{
+	    dt = f1.getType();
+	    len = f1.getLength();
+	    dim = f1.getDim();	   
+	}
+	else if ( f2.getType() == DECIMAL_TYPE )
+	{
+	    dt = f2.getType();
+	    len = f2.getLength();
+	    dim = f2.getDim();	  
+	}
+	else
+	{	    	   
+	    dt =  f1.getLength() > f2.getLength() ? f1.getType() : f2.getType();
+	    len = f1.getLength() > f2.getLength() ? f1.getLength() : f2.getLength();
+	    dim = f1.getLength() > f2.getLength() ? f1.getDim() : f2.getDim();
+	}
+	
+	f = CegoField( Chain("TERM"),
+		       Chain("TERM"),
+		       f1.getAttrName() + Chain("*") + f2.getAttrName(),
+		       dt, len, dim);
+		       
+	break;	
+    }
     case DIV:
-	f = CegoField( Chain("TERM"), Chain("TERM"), toChain(), VARCHAR_TYPE, 20 );
-	break;
-    case FACTOR:
-	f =  _pFactor->evalField();
+    {
+	CegoField f1 =  _pTerm->evalField(fl, graceful);
+	CegoField f2 =  _pFactor->evalField(fl, graceful);
+
+	// the type conversion refers to method : CegoFieldValue operator / ( const CegoFieldValue& fv1, const CegoFieldValue& fv2 )
+	CegoDataType dt;
+	int len;
+	int dim;
+	
+	if ( f1.getType() == DECIMAL_TYPE )
+	{
+	    dt = f1.getType();
+	    len = f1.getLength();
+	    dim = f1.getDim();
+	}
+	else if ( f2.getType() == DECIMAL_TYPE )
+	{
+	    dt = f2.getType();
+	    len = f2.getLength();
+	    dim = f2.getDim();
+	}
+	else
+	{
+	    dt =  f1.getLength() > f2.getLength() ? f1.getType() : f2.getType();
+	    len = f1.getLength() > f2.getLength() ? f1.getLength() : f2.getLength();
+	    dim = f1.getLength() > f2.getLength() ? f1.getDim() : f2.getDim();
+	}
+	
+	f = CegoField( Chain("TERM"),
+		       Chain("TERM"),
+		       f1.getAttrName() + Chain("/") + f2.getAttrName(),
+		       dt, len, dim );     
 	break;
     }
-    
+    case FACTOR:
+    {
+	f =  _pFactor->evalField(fl, graceful);
+	break;
+    }
+    }
     return f;
 }
 
-CegoFieldValue CegoTerm::evalFieldValue() const
-{
+CegoFieldValue CegoTerm::evalFieldValue(ListT<CegoField> **pFLA, CegoProcBlock *pBlock) const
+{	
     switch ( _termType )
     {
     case MUL:
-	return ( _pTerm->evalFieldValue() * _pFactor->evalFieldValue() );	
+	return ( _pTerm->evalFieldValue(pFLA, pBlock) * _pFactor->evalFieldValue(pFLA, pBlock) );
     case DIV:
-	return ( _pTerm->evalFieldValue() / _pFactor->evalFieldValue() );
+	return ( _pTerm->evalFieldValue(pFLA, pBlock) / _pFactor->evalFieldValue(pFLA, pBlock) );
     case FACTOR:
-	return ( _pFactor->evalFieldValue() );
+	return ( _pFactor->evalFieldValue(pFLA, pBlock) );
+    default:
+        throw Exception(EXLOC, Chain("Unknown term type"));
     }
-
 }
 
 ListT<CegoAggregation*> CegoTerm::getAggregationList()
@@ -254,6 +333,8 @@ ListT<CegoAggregation*> CegoTerm::getAggregationList()
 	return ( _pTerm->getAggregationList() + _pFactor->getAggregationList() );	 
     case FACTOR:
 	return ( _pFactor->getAggregationList() );
+    default:
+        throw Exception(EXLOC, Chain("Unknown term type"));
     }
 }
 
@@ -267,7 +348,6 @@ CegoTerm& CegoTerm::operator = ( const CegoTerm& e)
 
 CegoTerm* CegoTerm::clone(bool isAttrRef)
 {
-    
     switch ( _termType )
     {
     case MUL:
@@ -275,13 +355,13 @@ CegoTerm* CegoTerm::clone(bool isAttrRef)
 	return ( new CegoTerm ( _pTerm->clone(isAttrRef), _pFactor->clone(isAttrRef), _termType ) );
     case FACTOR:
 	return ( new CegoTerm ( _pFactor->clone(isAttrRef) ) );
-    }
-    
+    default:
+        throw Exception(EXLOC, Chain("Unknown term type"));
+    }   
 }
 
-void CegoTerm::encode(char *buf)
+void CegoTerm::encode(char *buf, CegoProcBlock *pBlock)
 {
-    
     char* pE = (char*)buf;
 
     memcpy( pE, &_termType, sizeof(CegoTerm::TermType));
@@ -292,24 +372,23 @@ void CegoTerm::encode(char *buf)
     case CegoTerm::MUL:
     case CegoTerm::DIV:
     {
-	_pTerm->encode(pE);
-	pE = pE + _pTerm->getEncodingLength();
-	_pFactor->encode(pE);
-	pE = pE + _pFactor->getEncodingLength();
+	_pTerm->encode(pE, pBlock);
+	pE = pE + _pTerm->getEncodingLength(pBlock);
+	_pFactor->encode(pE, pBlock);
+	pE = pE + _pFactor->getEncodingLength(pBlock);
 	break;
     }
     case CegoTerm::FACTOR:
     {
-	_pFactor->encode(pE);
-	pE = pE + _pFactor->getEncodingLength();
+	_pFactor->encode(pE, pBlock);
+	pE = pE + _pFactor->getEncodingLength(pBlock);
 	break;
     }
     }
 }
 
-void CegoTerm::decode(char *buf, CegoDistManager *pGTM, int tabSetId)
+void CegoTerm::decode(char *buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId)
 {
-
     char* pE = (char*)buf;
 
     memcpy( &_termType, pE, sizeof(CegoTerm::TermType));
@@ -320,25 +399,24 @@ void CegoTerm::decode(char *buf, CegoDistManager *pGTM, int tabSetId)
     case CegoTerm::MUL:
     case CegoTerm::DIV:
     {
-	_pTerm = new CegoTerm(pE, pGTM, tabSetId);
-	pE = pE + _pTerm->getEncodingLength();
-	_pFactor = new CegoFactor(pE, pGTM, tabSetId);
-	pE = pE + _pFactor->getEncodingLength();
+	_pTerm = new CegoTerm(pE, pGTM, pBlock, tabSetId);
+	pE = pE + _pTerm->getEncodingLength(pBlock);
+	_pFactor = new CegoFactor(pE, pGTM, pBlock, tabSetId);
+	pE = pE + _pFactor->getEncodingLength(pBlock);
 	break;
     }
     case CegoTerm::FACTOR:
     {
 	_pTerm=0;
-	_pFactor = new CegoFactor(pE, pGTM, tabSetId);
-	pE = pE + _pFactor->getEncodingLength();
+	_pFactor = new CegoFactor(pE, pGTM, pBlock, tabSetId);
+	pE = pE + _pFactor->getEncodingLength(pBlock);
 	break;
     }
     }
 }
 
-int CegoTerm::getEncodingLength() const
+int CegoTerm::getEncodingLength(CegoProcBlock *pBlock) const
 {
-
     int len = 0;
 
     len += sizeof(CegoTerm::TermType);
@@ -348,13 +426,13 @@ int CegoTerm::getEncodingLength() const
     case CegoTerm::MUL:
     case CegoTerm::DIV:
     {
-	len += _pTerm->getEncodingLength();
-	len += _pFactor->getEncodingLength();
+	len += _pTerm->getEncodingLength(pBlock);
+	len += _pFactor->getEncodingLength(pBlock);
 	break;
     }
     case CegoTerm::FACTOR:
     {
-	len += _pFactor->getEncodingLength();
+	len += _pFactor->getEncodingLength(pBlock);
 	break;
     }
     }
@@ -362,37 +440,37 @@ int CegoTerm::getEncodingLength() const
     return len;
 }
 
-Chain CegoTerm::getId() const
+Chain CegoTerm::getId(CegoProcBlock *pBlock) const
 {
     Chain s;
     switch ( _termType )
     {
     case CegoTerm::MUL:
-	s = _pTerm->getId() + "*" + _pFactor->getId();
+	s = _pTerm->getId(pBlock) + "*" + _pFactor->getId(pBlock);
 	break;
     case CegoTerm::DIV:
-	s = _pTerm->getId() + "/" + _pFactor->getId();
+	s = _pTerm->getId(pBlock) + "/" + _pFactor->getId(pBlock);
 	break;
     case CegoTerm::FACTOR:
-	s = _pFactor->getId();
+	s = _pFactor->getId(pBlock);
 	break;
     }
     return s;    
 }
 
-Chain CegoTerm::toChain() const
+Chain CegoTerm::toChain(int defTabSetId, const Chain& indent) const
 {
     Chain s;
     switch ( _termType )
     {
     case CegoTerm::MUL:
-	s = _pTerm->toChain() + " * " + _pFactor->toChain();
+	s = _pTerm->toChain(defTabSetId, indent) + " * " + _pFactor->toChain(defTabSetId, indent);
 	break;
     case CegoTerm::DIV:
-	s = _pTerm->toChain() + " / " + _pFactor->toChain();
+	s = _pTerm->toChain(defTabSetId, indent) + " / " + _pFactor->toChain(defTabSetId, indent);
 	break;
     case CegoTerm::FACTOR:
-	s = _pFactor->toChain();
+	s = _pFactor->toChain(defTabSetId, indent);
 	break;
     }
     return s;    
@@ -403,92 +481,8 @@ Chain CegoTerm::dbFormat(CegoDatabaseFormater *pForm) const
     return pForm->formatTerm(_termType, _pTerm, _pFactor);
 }
 
-Element* CegoTerm::toElement() const
-{
-    Element* pTermElement = new Element(XML_TERM_ELEMENT);
-
-    switch ( _termType )
-    {
-    case CegoTerm::MUL:	
-	pTermElement->setAttribute( XML_TERM_ATTR, XML_MUL_VALUE ); 
-	pTermElement->addContent( _pTerm->toElement() );
-	pTermElement->addContent( _pFactor->toElement() );
-	break;
-    case CegoTerm::DIV:
-	pTermElement->setAttribute( XML_TERM_ATTR, XML_DIV_VALUE ); 
-	pTermElement->addContent( _pTerm->toElement() );
-	pTermElement->addContent( _pFactor->toElement() );
-	break;
-    case CegoTerm::FACTOR:
-	pTermElement->setAttribute( XML_TERM_ATTR, XML_FACTOR_VALUE );
- 	pTermElement->addContent( _pFactor->toElement() );
-	break;
-    }
-    return pTermElement;    
-}
-
-void CegoTerm::fromElement(Element *pTermElement, CegoDistManager *pGTM)
-{
-
-    if ( _pTerm ) 
-	delete _pTerm;
-    if ( _pFactor )
-	delete _pFactor;
-
-    _pTerm = 0;
-    _pFactor = 0;
-
-    Chain termTypeString = pTermElement->getAttributeValue( XML_TERM_ATTR );
-
-    if ( termTypeString == Chain(XML_MUL_VALUE) )
-    {
-	_termType = CegoTerm::MUL;
-	ListT<Element*> tl = pTermElement->getChildren(XML_TERM_ELEMENT);
-	Element **pTE = tl.First();
-	if ( pTE )
-	{
-	    _pTerm = new CegoTerm(*pTE, pGTM);
-	}
-	ListT<Element*> fl = pTermElement->getChildren(XML_FACTOR_ELEMENT);
-	Element **pFE = fl.First();
-	if ( pFE )
-	{
-	    _pFactor = new CegoFactor(*pFE, pGTM);
-	}
-    } 
-    else if ( termTypeString == Chain(XML_DIV_VALUE) )
-    {
-	_termType = CegoTerm::DIV;
-	ListT<Element*> tl = pTermElement->getChildren(XML_TERM_ELEMENT);
-	Element **pTE = tl.First();
-	if ( pTE )
-	{
-	    _pTerm = new CegoTerm(*pTE, pGTM);
-	}
-	ListT<Element*> fl = pTermElement->getChildren(XML_FACTOR_ELEMENT);
-	Element **pFE = fl.First();
-	if ( pFE )
-	{
-	    _pFactor = new CegoFactor(*pFE, pGTM);
-	}	
-    }
-    else if ( termTypeString == Chain(XML_FACTOR_VALUE) )
-    {
-	_termType = CegoTerm::FACTOR;
-	ListT<Element*> fl = pTermElement->getChildren(XML_FACTOR_ELEMENT);
-	Element **pFE = fl.First();
-	if ( pFE )
-	{
-	    _pFactor = new CegoFactor(*pFE, pGTM);
-	}
-    }
-}
-
 void CegoTerm::getPlanList(ListT<Element*>& planList)
 {
-
-    Element *pPlan = new Element(XML_PLAN_ELEMENT);
-
     if ( _pTerm )
     {
 	_pTerm->getPlanList(planList);
