@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -32,69 +32,55 @@
 class CegoDistManager;
 
 class CegoTerm {
-
- public:
-
+    
+public:
+    
     enum TermType { MUL, DIV, FACTOR  };
-
+    
     CegoTerm();
-    CegoTerm(char* buf, CegoDistManager *pGTM, int tabSetId);
-    CegoTerm(Element* pTermElement, CegoDistManager *pGTM);
+    CegoTerm(char* buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId);
     CegoTerm(CegoTerm *pTerm, CegoFactor *pFac, TermType termType);
     CegoTerm(CegoFactor *pFactor);
     ~CegoTerm();
 
-    void setFieldListArray(ListT<CegoField> **pFLA);
-    void setBlock(CegoProcBlock *pBlock);
-
+    void cleanUp();
+    void setTabSetId(int tabSetId);
     void clearAttrCache();
-
     void getFunctionList(SetT<CegoObject>& funcList);
-	
     void getSelectQueryList(ListT<CegoSelect*>& queryList);
-
+    bool checkConst() const;
     bool checkVar(Chain& var) const;
     CegoAttrDesc* checkAttr() const;
-
-    ListT<CegoField> getFieldList() const;
+    bool checkLob(CegoFieldValue& fv, CegoProcBlock *pBlock);
+    void getFieldList(ListT<CegoField>& fl, ListT<CegoField> **pFLA) const;
     ListT<CegoAttrDesc*> getAttrRefList() const;
     int evalReferences(CegoContentObject *pCO, const ListT<CegoField>& fl);
+    CegoField evalField(const ListT<CegoField>& fl, bool graceful = false) const;
 
-    CegoField evalField() const;
-    CegoFieldValue evalFieldValue() const;
-
+    CegoFieldValue evalFieldValue(ListT<CegoField> **pFLA, CegoProcBlock *pBlock) const;
+    
     TermType getType() const;
     CegoFactor* getFactor();
     CegoTerm* getTerm();
-
     ListT<CegoAggregation*> getAggregationList();
-
     CegoTerm* clone(bool isAttrRef = false);
-
-    CegoTerm& operator = ( const CegoTerm& e);
-
-    Chain getId() const;
-    Chain toChain() const;
+    Chain getId(CegoProcBlock *pBlock) const;
+    Chain toChain(int defTabSetId, const Chain& indent) const;
     Chain dbFormat(CegoDatabaseFormater *pForm) const;
-    
-    Element* toElement() const;
-    void fromElement(Element *pTermElement, CegoDistManager *pGTM);
 
-    friend ostream& operator << (ostream& s, const CegoTerm& e);
-
-    void encode(char *buf);
-    void decode(char *buf, CegoDistManager *pGTM, int tabSetId);
-    int getEncodingLength() const;
+    void encode(char *buf, CegoProcBlock *pBlock);
+    void decode(char *buf, CegoDistManager *pGTM, CegoProcBlock *pBlock, int tabSetId);
+    int getEncodingLength(CegoProcBlock *pBlock) const;
 
     void getPlanList(ListT<Element*>& planList);
 
- private:
+    CegoTerm& operator = ( const CegoTerm& e);
+    friend ostream& operator << (ostream& s, const CegoTerm& e);
+
+private:
 
     TermType _termType;
-
     CegoTerm *_pTerm;
     CegoFactor *_pFactor;
-
 };
-
 #endif
