@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -37,14 +37,10 @@ public:
     CegoTypeConverter();
     ~CegoTypeConverter();
 
-    Chain getTypeString(CegoDataType type) const;
-    CegoDataType getTypeId(const Chain& typeString);
-    int getTypeLen(CegoDataType type, const Chain& val = Chain("")) const;
-
-    Chain getObjectTypeString(CegoObject::ObjectType type) const;
-    CegoObject::ObjectType getObjectTypeId(const Chain& typeString);
-
+    static Chain getTypeString(CegoDataType type);
+    static CegoDataType getTypeId(const Chain& typeString);
+    static int getTypeLen(CegoDataType type, const Chain& val = Chain(""));
+    static Chain getObjectTypeString(CegoObject::ObjectType type);
+    static CegoObject::ObjectType getObjectTypeId(const Chain& typeString);
 };
-
-
 #endif
