@@ -1,12 +1,12 @@
 ///////////////////////////////////////////////////////////////////////////////
 //                                                         
 // CegoViewObject.cc 
-// ------------------
+// -----------------
 // Cego view object entry implementation
 //     
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -23,7 +23,6 @@
 
 // cego includes
 #include "CegoViewObject.h"
-#include "CegoXMLHelper.h"
 #include "CegoXMLdef.h"
 #include "CegoTypeConverter.h"
 
@@ -35,12 +34,6 @@ CegoViewObject::CegoViewObject()
     _subCOList.Insert(this);
 }
 
-CegoViewObject::CegoViewObject(Element *pVO)
-{
-    putElement(pVO);
-    _subCOList.Insert(this);
-}
-
 CegoViewObject::CegoViewObject(const CegoViewObject& vo) : CegoContentObject(vo)
 {
     _viewStmt = vo._viewStmt;
@@ -74,7 +67,7 @@ int CegoViewObject::getEntrySize() const
     return entrySize;
 }
 
-void CegoViewObject::encode(char *buf)
+void CegoViewObject::encode(char *buf) const
 {
     char* bufPtr = buf;
     int entrySize = getEntrySize();
@@ -114,95 +107,23 @@ bool CegoViewObject::operator == ( const CegoViewObject& vo)
     return CegoContentObject::operator==(vo);
 }
 
-Chain CegoViewObject::getId() const
+Chain CegoViewObject::getId(CegoProcBlock *pBlock) const
 {
-    return getTabName() + Chain("@") + getName();
+    // since it can not be ad hoc decided, if the tuple result of the view objects has been changed,
+    // we have to throw MOD_QUERY_ID
+    throw Exception(EXLOC, Chain(MOD_QUERY_ID));
 }
 
-Chain CegoViewObject::toChain() const
+Chain CegoViewObject::toChain(int defTabSetId) const
 {
-    return getTabName() + Chain(" ") + getName();
-}
-
-Element* CegoViewObject::getElement() const
-{
-    Element* pRoot = new Element(XML_OBJ_ELEMENT);
-
-    pRoot->setAttribute(XML_TSID_ATTR, Chain(getTabSetId()));
-    pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_VIEWOBJ_VALUE);	
-    pRoot->setAttribute(XML_OBJNAME_ATTR, getName());
-
-    CegoField *pF = _schema.First();
-    while ( pF ) 
-    {
-	Element *pColElement = new Element(XML_SCHEMA_ELEMENT);
-
-
-	CegoXMLHelper xh;
-	xh.setColInfo(pColElement, pF);
-
-	pRoot->addContent(pColElement);
-	pF = _schema.Next();
-    }
-
-    pRoot->setAttribute(XML_VIEWSTMT_ATTR, _viewStmt);
-
-    return pRoot;
-}
-
-void CegoViewObject::putElement(Element* pElement)
-{
-
-    Element *pRoot = pElement;
-    
-    if ( pRoot )
-    {
-	
-	Chain objName = pRoot->getAttributeValue(XML_OBJNAME_ATTR); 
-	int tabSetId = pRoot->getAttributeValue(XML_TSID_ATTR).asInteger();
-	
-	setName(objName);
-	setTabName(objName);
-	setTabSetId(tabSetId);
-
-	setType(CegoObject::VIEW);
-	
-	ListT<Element*> colList = pRoot->getChildren(XML_SCHEMA_ELEMENT);
-	
-	ListT<CegoField> fl;
-	Element **pCol = colList.First();
-	while ( pCol ) 
-	{
-	    Chain colName = (*pCol)->getAttributeValue(XML_COLNAME_ATTR);
-	    Chain colType = (*pCol)->getAttributeValue(XML_COLTYPE_ATTR);
-	    Chain colSize = (*pCol)->getAttributeValue(XML_COLSIZE_ATTR);
-	    Chain colNullable = (*pCol)->getAttributeValue(XML_COLNULLABLE_ATTR);
-	    bool isNullable;
-	    if ( colNullable == Chain(XML_TRUE_VALUE) ) 
-		isNullable = true;
-	    else
-		isNullable = false;
-	    
-	    CegoFieldValue defValue;
-
-	    CegoTypeConverter tc;
-	    CegoDataType dataType = tc.getTypeId(colType);
-	    
-	    CegoField f(CegoField(objName, objName, colName, dataType, colSize.asInteger(), defValue, isNullable));		
-	    fl.Insert(f);
-
-	    pCol = colList.Next();
-	}
-	
-	setSchema(fl);
-
-	_viewStmt = pRoot->getAttributeValue(XML_VIEWSTMT_ATTR);
-    }    
+    if ( defTabSetId == getTabSetId() && defTabSetId != 0)
+	return getTabName() + Chain(" ") + getName();
+    else
+	return getTabName() + Chain("@") + getTableSet() + Chain(" ") + getName();
 }
 
 Chain CegoViewObject::getFormatted() const
 {
-
     Chain s;
 	
     int maxAttrLen = 12;
@@ -241,8 +162,7 @@ Chain CegoViewObject::getFormatted() const
 	
     pF = _schema.First();
     while (pF)
-    {
-	
+    {	
 	int num = maxAttrLen - pF->getAttrName().length() ;
 	s += Chain("| ") + pF->getAttrName() + fill(" ", num) + Chain(" |");
 
@@ -296,12 +216,6 @@ Chain CegoViewObject::getFormatted() const
 	    s += Chain("  decimal[") + l + Chain("]") + fill(" ", 5 - l.length()) + Chain("|");
 	    break;
 	}
-	case FIXED_TYPE:
-	{
-	    Chain l(pF->getLength());
-	    s += Chain("  fixed[") + l + Chain("]") + fill(" ", 7 - l.length()) + Chain("|");
-	    break;
-	}
 	case FLOAT_TYPE:
 	{
 	    s += Chain(" float         |");
@@ -322,8 +236,9 @@ Chain CegoViewObject::getFormatted() const
 	    s += Chain(" clob          |");
 	    break;
 	}
-
-	default:
+	case NULL_TYPE:
+	case PAGEID_TYPE:
+	    // not formatted
 	    break;
 	}
 	if ( pF->isNullable() )
@@ -346,13 +261,11 @@ Chain CegoViewObject::getFormatted() const
     s += Chain("+") + fill("-", maxFillLen) + Chain("+\n");
 
     return s;
-
 }
 
 Chain CegoViewObject::fill(const Chain& s, int num) const
 {
-    
-    Chain fs = Chain("");
+    Chain fs = Chain();
     while (num > 0)
     {
 	fs = fs + s;
